%% LOBSTER - Matlab Demo Code

%% Introduction
% -------------------------------------------------------------------------
% 
%   Welcome to the LOBSTER Matlab demo.
% 
%                                         http://LOBSTER.wiwi.hu-berlin.de
% -------------------------------------------------------------------------
%
% The code provided below might help you get started with your LOBSTER
% data. The demo focuses on the two LOBSTER output files 'orderbook' and
% 'message'.
%
% You can find a detailed description of the LOBSTER data structure at
% http://goo.gl/m1iGK (link to the LOBSTER forum)
%
% Data used: AMZN - 2009-Jul-01 - 10 Levels
%
% Please go through the demo cell by cell and view the figures in full
% screen.
%
%
%                                               This version: 2011-Oct-12
% -------------------------------------------------------------------------

%% Modifications by Peter Malec, 2011-Nov-21

% -  data loading (paths)

%% Set up the Basics


% Clear system
clear all;
clc;

% Stock name
ticker      = 'AMZN';

% Levels
lvl         = 10;

% File names
nameBook    = '../Data/Raw/AMZN_20090701_orderbook_10.csv';
nameMess    = '../Data/Raw/AMZN_20090701_message_10.csv';
    % Note: The files must be in the same folder as the
    %       LOBSTER_demo.m file.

% Date of files
demoDate    = [2009,7,1];      % year, month, day



%% Load Order Book File


% Load data
book = dlmread(nameBook);
    % Note: The file contains more than 300 000 entries. It takes a few
    %       seconds to load.

% Order book file information:
% -------------------------------------------------------------------------
%   
%   - Dimension:    Matrix of size (NumberEvents X (NumberLevels*4+1))
%
%   - Structure:    Each row:
%                   timeStamp (Ms after midnight), Ask price 1, 
%                   Ask volume 1, Bid price 1, Bid volume 1, Ask price 2, 
%                   Ask volume 2, Bid price 2, Bid volume 2, ...   
% 
%   - Note:         Unoccupied bid (ask) price levels are set to -1 
%                   (99999999) with volume -1 
%
% -------------------------------------------------------------------------



%% Data Preparation - Order Book File


% Remove observations outside the official trading hours
% -------------------------------------------------------------------------

% Trading hours (start & end)
startTrad   = 9.5*60*60*1000;       % 9:30:00.000 in ms after midnight
endTrad     = 16*60*60*1000;        % 16:00:00.000 in ms after midnight

% Get index of observations 
timeIdx = book(:,1) >= startTrad & book(:,1) <= endTrad;

% Remove observations
book = book(timeIdx,:);

% Convert prices into dollars
% -------------------------------------------------------------------------
book(:,[2:2:(4*lvl+1)]) = book(:,[2:2:(4*lvl+1)])./10000;
    % Note: LOBSTER stores prices in dollar price times 10000


    
%% Plot - Snapshot of the Limit Order Book


% Select index of event to be displayed
eventIdx = unidrnd(length(book));
    % Note: Pick a random row/event from the order book
    
% Postions of variables in the book
askPricePos = 2:4:(lvl*4);
askVolPos   = askPricePos+1;
bidPricePos = askPricePos+2;
bidVolPos   = bidPricePos+1;             
vol         = 3:2:(lvl*4+1);

% Plot limits ...
% Price 
maxPrice =  book(eventIdx,askPricePos(lvl)) + 0.01;
minPrice =  book(eventIdx,bidPricePos(lvl)) - 0.01;

% Volume
maxVol =  max(book(eventIdx,vol));

% Mid quote
mid = 0.5*(sum(book(eventIdx,[2,4]),2));

% Plot
figure('Name','Limit Order Book - Snapshot');

    % Ask side
    barh(book(eventIdx,askPricePos)',book(eventIdx,askVolPos)','green');
        title({[ticker ' // ' datestr(datenum(demoDate),'yyyy-mmm-dd')] ...
            [' LOB Snapshot - Time: ' num2str(book(eventIdx,1)) ' ms']});
        xlabel('Volume');
        ylabel('Price ($)');
        ylim([minPrice,maxPrice]);
        xlim([0,maxVol]);

    hold on;

    % Bid side
    barh(book(eventIdx,bidPricePos)',book(eventIdx,bidVolPos)','red');

    % Mid 
    plot(50,mid,'<k','LineWidth', 2);

    % Legend
    legend('Ask','Bid','Mid','Location','East');
    
    hold off


    
%% Plot - Relative Depth in the Limit Order Book


% Relative volume ...
% Ask
bookVolAsk = cumsum(book(eventIdx,askVolPos),2);
bookVolAsk = bookVolAsk./bookVolAsk(end);

% Bid
bookVolBid = cumsum(book(eventIdx,bidVolPos),2);
bookVolBid = bookVolBid./bookVolBid(end);

% Plot
figure('Name','Limit Order Book - Relative Depth');

    % Ask side
    stairs((1:lvl)',bookVolAsk','LineWidth',1.5,'Color','g');
        title({[ticker ' // ' datestr(datenum(demoDate),'yyyy-mmm-dd')] ...
        ['LOB Relative Depth - Time: ' num2str(book(eventIdx,1)) ' ms']});
        ylabel('% of Volume');
        xlabel('Level');
        xlim([0,lvl]);
        set(gca,'YTickLabel',{'1','0,8','0.6','0,4','0,2','','0,2', ...
            '0,4','0,6','0,8','1'},...
            'YTick',[-1 -0.8 -0.6 -0.4 -0.2 0 0.2 0.4 0.6 0.8 1]);
    hold on

    % Bid side
    stairs((1:lvl)',-1*bookVolBid','LineWidth',1.5,'Color','r');
   
    % Legend
    legend('Ask','Bid','Location','SouthWest');
    
    hold off



%% Plot - Intraday Evolution of Depth 


% Display only the first 3 levels
maxLvl = 3;

% Postion of volumes in the book
askVolPos = 3:4:maxLvl*4;

% Depth
bookVolAsk  = cumsum(book(:,askVolPos)./100,2);
bookVolBid  = cumsum(book(:,askVolPos+2)./100,2);

% Max depth
maxVol = max(max(bookVolAsk(:,end)),max(bookVolBid(:,end)));
maxVol = floor(maxVol*1.1/10)*10;

% LegendStr
legendStr = [repmat('Ask',maxLvl,1),blanks(maxLvl)',...
    num2str((1:maxLvl)'); repmat('Bid',maxLvl,1),blanks(maxLvl)',...
    num2str((1:maxLvl)')];

% Plot
figure('Name','Limit Order Book - Intraday Depth');

    plot(book(:,1)./(1000*60*60),[bookVolAsk, -1*bookVolBid]);
        title({[ticker ' // ' datestr(datenum(demoDate),'yyyy-mmm-dd')] ...
             ['Intraday Evolution of Depth']});
        ylim([-maxVol,maxVol]);
        xlim([9,16.5]);
        xlabel('Time');
        ylabel(['Bid ' blanks(12) ' Number of Shares (x100) ' ...
            blanks(12) ' Ask ']);
        
        % Tick labels
        set(gca,'YTickLabel',{' ', num2str(maxVol/2), 0, ... 
            num2str(maxVol/2), ' '},'YTick', ...
            [-maxVol -maxVol/2 0 maxVol/2 maxVol]);
        
   % Legend
    legend(legendStr,'Location','SouthOutside','Orientation','horizontal');

%   Note:   The plot of the intraday evolution of the volume in the order 
%           book highlights the speed and magnitude with which the posted
%           volume changes. Take, for example, the time interval between
%           10:00 and 11:00. For seconds, both the ask and the bid side
%           posted volume spike from below 25,000 shares to well above
%           150,000 shares. 
        
        

%% Load Messsage File


% Load data
mess = dlmread(nameMess);

% Message file information:
% -------------------------------------------------------------------------
%   
%   - Dimension:    Matrix of size (NumberEvents X 6)
%
%   - Structure:    Each row:
%                   timeStamp (Ms after midnight), EventType,
%                   Message ID, Size (# of shares), Price, Direction
%
%                   EventTypes: 
%                       - '1'   Submitted new limit order
%                       - '2'   Cancellation (partial)
%                       - '3'   Deletion (total order)
%                       - '4'   Execution against visible liquidity
%                       - '5'   Execution against hidden liquidity
%                       - '100' Other
%
%                   Direction:
%                       - '-1'  Sell limit order
%                       - '-2'  Buy limit order
%                       - NOTE: Execution of a sell (buy) limit order
%                               corresponds to a buyer-(seller-)initiated  
%                               trade, i.e a BUY (SELL) trade. 
%
% -------------------------------------------------------------------------



%% Data Preparation - Message File


% Remove observations outside the official trading hours
mess = mess(timeIdx,:);
    % Note: Time index of the order book file can be used as the rows of 
    %       both files - order book and message - correspond to each other.
    
        
    
%% Set Bounds for Intraday Intervals


% Define interval length
freq = 5*60*1000;   % Interval length in ms

% Set interval bounds
bounds = (startTrad:freq:endTrad)';

% Number of intervals
bl = size(bounds,1);

% Indices for intervals
boundIdx = zeros(bl,1);

k1 = 1; 
for k2 = 1:size(mess,1)
    if mess(k2,1) >= bounds(k1);
        boundIdx(k1,1) = k2;
        k1 = k1+1;
    end
end

% Get index of observation relevant for bound
boundIdx(2:end-1) = boundIdx(2:end-1)-1;
boundIdx(end) = length(mess);
    % Note: This is a rather rough procedure for obtaining interval bounds! 
    %       It is specially error prone if there are only few observations 
    %       per interval! Be careful!
    
    
    
%% Plot - Number of Executions and Trade Volume by Interval

% Note: What is the difference between trades and executions?
%
%       The LOBSTER output records trade executions and not what one might
%       intuitively consider trades. 
%
%       Imagine a volume of 1000 is posted at the best ask price. Further, 
%       an incoming market buy order of volume 1000 is executed against the
%       quote.
%
%       The LOBSTER output of this trade depends on the composition of the
%       volume at the best ask price. Take the following two scenarios with
%       the best ask volume consisting of ...
%       (a) 1 sell limit order with volume 1000
%       (b) 5 sell limit orders with volume 200 each (ordered according to 
%           time of submission)
%
%       The LOBSTER output for case ...
%       (a) shows one execution of volume 1000. If the incoming market 
%           order is machted with one standing limit order, execution and
%           trade coincide.
%       (b) shows 5 executions of volume 200 each with the same time stamp.
%           The incoming order is matched with 5 standing limit orders and
%           triggers 5 executions. 
%
%       Bottom line: 
%       LOBSTER records the exact limit orders against which incoming
%       market orders are executed. What might be called 'economic' trade
%       size has to be inferred from the executions.
%
%       A detailed disscussion of this issue can be found at:
% http://lobster.wiwi.hu-berlin.de/forum/viewtopic.php?f=3&t=19&sid=1be0c0b
% 4de4e390d37fd6686311ece23
%

% Collection matrix
tradesInfo = zeros(bl-1,4);
    % Note: Number visible executions, volume visible trades, number hidden
    %       executions, volume hidden trades
    

for k1 = 1:(bl-1)
    
    % Get intraday mess window
    temp   = mess(boundIdx(k1):boundIdx(k1+1),[2,4]);
    
    % Visible
    tempVis = temp(temp(:,1)==4,2);
    
    % Hidden 
    tempHid = temp(temp(:,1)==5,2);
    
    % Collect information
    tradesInfo(k1,:) = [ length(tempVis), sum(tempVis), ...
                        length(tempHid), sum(tempHid)];
                    
    clear temp tempVis tempHid;
end


% Plot number of executions
figure('Name','Number of Executions');    

    % Visible ...
    bar(tradesInfo(:,1),'red');
        title({[ticker ' // ' datestr(datenum(demoDate),'yyyy-mmm-dd')] ...
            ['Number of Executions per ' num2str(freq./(1000*60)) ...
            ' min Interval ']});
        xlabel('Interval');
        ylabel(['Number of Executions']);

    hold on
    
    % Hidden ...
    bar(-tradesInfo(:,3),'blue');
    
    % Legend
    legend('Visible','Hidden','Location','North');
    
    hold off;

    
% Plot volume of trades
figure('Name','Volume of Trades');    

    % Visible ...
    bar(tradesInfo(:,2)./100,'red');
        title({[ticker ' // ' datestr(datenum(demoDate),'yyyy-mmm-dd')] ...
            ['Volume of Trades per ' num2str(freq./(1000*60)) ...
            ' min Interval ']});
        xlabel('Interval');
        ylabel(['Volume of Trades (x100 Shares)']);

    hold on
    
    % Hidden ...
    bar(-tradesInfo(:,4)./100,'blue');
    
   % Legend
    legend('Visible','Hidden','Location','North');
    
    hold off;

% Notes: 
%
%   -   When considering cumulative volume in an interval the distinction 
%       between trades and executions is not necessary. However, when, for 
%       example, investigating average trade size one can not rely on the
%       computations performed here. In this case the exectutions first 
%       have to be aggregated to trades.
    
%   -   The figures illustrate that both, number of trades and volume 
%       of trades executed against visible and hidden liquidity, show the 
%       characteristic U-shaped intraday pattern. Activity is higher at
%       opening and closing than during the rest of the day.
%
    


%% Feedback
% -------------------------------------------------------------------------
%
% Should you have questions or suggestions regarding this demo code, please
% contact us via our forum 
% http://lobster.wiwi.hu-berlin.de/forum 
% or send us a messeage via 
% http://lobster.wiwi.hu-berlin.de/Lobster/contact.jsp
%
% Thank you - The LOBSTER Team
