        function [keep_ind] = QuoteClean_Core(quotes,hwin,thresh_gl,thresh_loc)
%% QuoteClean_Core
%--------------------------------------------------------------
% Cleans quote data using the procedure outlined in 
% Barndorff-Nielsen et al. (2009) with two checks:
% 1. -> Delete entries where spread is more than thresh_gl
% times median of the day
% 2. -> Delete entries where midquote deviates by more than thresh_loc
% (local) mean absolute deviations from local window median
%--------------------------------------------------------------
% INPUT:
% quotes - matrix of bid and ask quotes for given day with columns
%   1       -> time
%   2       -> ask price
%   3       -> bid price
% hwin - 1/2 of local window around each obs for 
% which median is computed
% thresh_gl -> threshold for global check
% thresh_loc -> threshold for local window check
%--------------------------------------------------------------
% OUTPUT:
% keep_ind - vector of same size as quotes -> 1: keep obs, 0 del obs
% -------------------------------------------------------------------------
%
% Peter Malec, November 29, 2011
% Based on the (awesome!!!) function "AggExecutions_Core" by Gustav Haitz
% -------------------------------------------------------------------------
%       
 %% Preparations     
 
 % number of obs in quotes
  bl = size(quotes,1);
           
% Initialize Output Matrix
 keep_ind=ones(bl,1);
 
 %Compute median spread for the day
 medd_spread=nanmedian((quotes(:,2)-quotes(:,3)));
 
% -------------------------------------------------------------------------
%     

 %% Check for first and last hwin obs (no local window possible)
 
 %global check
 keep_ind(1:hwin)=(quotes(1:hwin,2)-quotes(1:hwin,3)) <=  (thresh_gl*medd_spread);
 
 keep_ind((end-hwin+1):end)=(quotes((end-hwin+1):end,2)-quotes((end-hwin+1):end,3)) <=  (thresh_gl*medd_spread);
% -------------------------------------------------------------------------
%     
 %% Loop for checking rest of sample

for i=(hwin+1):(length(keep_ind)-hwin)   
    
    %Compute median and mean abs deviation for local window
    %----------------------------------------------------------------------
    
    %Create Local Window of midquotes (excluding i)
    loc_win=[0.5*(quotes((i-hwin):(i-1),2)+quotes((i-hwin):(i-1),3));0.5*(quotes((i+1):(i+hwin),2)+quotes((i+1):(i+hwin),3))];
    
    %compute local median
    loc_med=nanmedian(loc_win);
    
    %compute local mean absolute deviation
    loc_mad=nanmedian(abs(loc_win-loc_med));
    
    %compute abs deviation of midquote from local median
    loc_dev=abs(0.5*(quotes(i,2)+quotes(i,3))-loc_med); 
    
    %Global and local check
    %----------------------------------------------------------------------
    keep_ind(i)=((quotes(i,2)-quotes(i,3)) <=  (thresh_gl*medd_spread)) & ...
        (loc_dev <= (thresh_loc*loc_mad));
                            % global for spreads, local for midquotes
    
    
end

