function [GMWw_mat,GMWwVar_mat]=GMW_weights(SCov_mat,Covmat_mat,Z_tilde)
%------------------------------------------------------------------
%% GMW_weights
%  Estimates GMW weights and associated Standard Errors 
% based on spot covariance matrices and their vec transformations'
% covariance matrices at specific points in time
%------------------------------------------------------------------
%
% INPUTS
% SCov_mat-> (n_inxm(m+1)/2) matrix of spot cov matrices for each point of
%                               estimation
%                    each row: vech(\hat Sigma_t)'
% Covmat_mat ->(m^2(m^2+1)/2xn_in) matrix: Stacked Covariance matrices of smoothed spot (co-)volatilities
%                       under the local parametric experiment      (optional)
%                    each column:vech(Cov(\hat Sigma_t))
% Z_tilde -> (m^2 x m^2)  matrix of fourth moments of d-dimensional
%                   standard Gaussian vector (optional)
%------------------------------------------------------------------
% OUTPUTS
% GMWw_mat -> (n_inxm) matrix of GMW weights for specified points
%                                       in time
% GMWwVar_mat ->(n_inxm) matrix of standard errors of
%                                        GMW weights for specified points in time
%------------------------------------------------------------------
%
%   Peter Malec, January 23, 2014
%
%------------------------------------------------------------------
%
%% Preparations

% no of assets
m= (sqrt(1+size(SCov_mat,2)*8)-1)/2;

%------------------------------------------------------------------
%
%% Compute Portfolio Volatilities and Standard Errors

%number of inputs
nin=size(SCov_mat,1);

%loop across input times
%-------------------------------------------------

if nargin>1
    
    % empty output matrices
    GMWw_mat=NaN*zeros(nin,m);
    GMWwVar_mat=NaN*zeros(nin,m);
    
    % compute Z_tilde???
    if (nargin<3)
        Z_tilde=[];
    end
    if (isempty(Z_tilde))||(size(Z_tilde,1)~=m^2)||(size(Z_tilde,2)~=m^2)
        disp('Z_tilde has to be computed!')
        Z_tilde = constrZ_tilde(m);
    end

    for i=1:nin
        % extract spot covariance matrix estimates and estimated avar
        % of its vec transformation
        SCov_mat_temp=ivech(SCov_mat(i,:)');
        Covmat_temp=ivech(Covmat_mat(:,i))*Z_tilde;
        
        %compute GMW weights
        GMWw_mat(i,:)=(sum(SCov_mat_temp,2)/sum(sum(SCov_mat_temp)))';
        
        % compute standard errors
        GMWwVar_mat(i,:)=(GMW_weights_var(GMWw_mat(i,:)',SCov_mat_temp,Covmat_temp))';
        
    end

else
    
    % empty output matrices
    GMWw_mat=NaN*zeros(nin,m);

    for i=1:nin
        % extract spot covariance matrix estimates 
        SCov_mat_temp=ivech(SCov_mat(i,:)');
        
        %compute GMW weights
        GMWw_mat(i,:)=(sum(SCov_mat_temp,2)/sum(sum(SCov_mat_temp)))';
        
    end
     
    % empty output
    GMWwVar_mat=NaN*zeros(nin,m);
    
end

