function [pfvol_vec,pfvol_var_vec]=pf_vol(weight_m,SCov_mat,Covmat_mat,Z_tilde)
%------------------------------------------------------------------
%% pf_vol
%  Estimates portfolio volatilities and associated Standard Errors 
% for a vector of portfolio weights and a covariance matrices for
% specific points in time
%------------------------------------------------------------------
%
% INPUTS
% weight_m       -> (mxn_in) matrix of portfolio weights for assets
% SCov_mat-> (n_inxm(m+1)/2) matrix of spot cov matrices for each point of
%                               estimation
%                    each row: vech(\hat Sigma_t)'
% Covmat_mat ->(m^2(m^2+1)/2xn_in) matrix: Stacked Covariance matrices of smoothed spot (co-)volatilities
%                       under the local parametric experiment      (optional)
%                    each column:vech(Cov(\hat Sigma_t))
% Z_tilde -> (m^2 x m^2)  matrix of fourth moments of d-dimensional
%                   standard Gaussian vector (optional)
%------------------------------------------------------------------
% OUTPUTS
%pfvol_vec -> (n_inx1) vector of portfolio variances
%pfvol_var_vec ->(n_inx1) vector of variance standard errors
%------------------------------------------------------------------
%
%   Peter Malec, January 15, 2014
%
%------------------------------------------------------------------
%
%% Preparations

% no of assets
m= (sqrt(1+size(SCov_mat,2)*8)-1)/2;

% check dimensions
if size(SCov_mat,1)~=size(weight_m,2)
    error('No. of rows of Spot Covariance matrix matrix has to be equal to no. of columns of weight matrix!')
end
%------------------------------------------------------------------
%
%% Compute Portfolio Volatilities and Standard Errors

%number of inputs
nin=size(SCov_mat,1);

%loop across input times
%-------------------------------------------------

if nargin>2
    
    % empty output matrices
    pfvol_vec=NaN*zeros(nin,1);
    pfvol_var_vec=NaN*zeros(nin,1);
    
    % compute Z_tilde???
    if (nargin<4)
        Z_tilde=[];
    end
    if (isempty(Z_tilde))||(size(Z_tilde,1)~=m^2)||(size(Z_tilde,2)~=m^2)
        disp('Z_tilde has to be computed!')
        Z_tilde = constrZ_tilde(m);
    end

    for i=1:nin
        % extract spot covariance matrix estimates and estimated avar
        % of its vec transformation
        SCov_mat_temp=ivech(SCov_mat(i,:)');
        Covmat_temp=ivech(Covmat_mat(:,i))*Z_tilde;
        
        %compute portfolio vola
        pfvol_vec(i)=sqrt(weight_m(:,i)'*SCov_mat_temp*weight_m(:,i));
        
        % compute standard errors
        d_vec=weight_m(:,i)*weight_m(:,i)'/(2*pfvol_vec(i));
        pfvol_var_vec(i)=d_vec(:)'*Covmat_temp*d_vec(:);
        
    end

else
    
    % empty output matrices
    pfvol_vec=NaN*zeros(nin,1);

    for i=1:nin
        % extract spot covariance matrix estimates and estimated avar
        % of its vec transformation
        SCov_mat_temp=ivech(SCov_mat(i,:)');
        
        %compute portfolio vola
        pfvol_vec(i)=sqrt(weight_m(:,i)'*SCov_mat_temp*weight_m(:,i));
        
    end
     
    % empty output matrix
    pfvol_var_vec=NaN*zeros(nin,1);
    
end

