function [param_out,reg_fit]=reg_fourier_gen_optD_Ret(y,time,maxD,spec)
%% reg_fourier_gen_optD_Ret
%--------------------------------------------------------------
% estimates parameters of Fourier transform with or w/o polynomial
% terms and intercept for 
% determinisctic intraday volatility function by OLS using optimal 
% number of sinusoids according to BIC
% based on transformed returns
%--------------------------------------------------------------
% INPUT:
% svar ->  (Tx1) vector of transformed returns
% time -> (Tx1) vector of observation times as 
% fraction of trading day (time \in [0,1])
% maxD     -> maximum order of the fourier transform
% spec  -> specification indicator: =0 -> no polynomial and intercept
%                                                 =1  ->polynomial, but no
%                                                 intercept
%                                                 = 2   -> intercept, but
%                                                 no polynomial
%                                                  =3  -> intercept and
%                                                  polynomial
%--------------------------------------------------------------
% OUTPUT:
% param -> ((D*2)x1) vector of OLS estimates of Fourier parameters
% reg_fit  -> vector of measures of goodness-of-fit: R2, adjusted R2,
%                   squared sample correlation between observed and fitted                    
% -------------------------------------------------------------------------
%
% Peter Malec, April 7, 2014
%
% -------------------------------------------------------------------------
%       
%% Preparations

% remove NaNs
y=y(~isnan(y));
time=time(~isnan(y));

% default specification
if nargin<4
    spec=0;
end

% nobs
n_obs=length(y);

% number parameters (constant always included)
if isequal(spec,2)||isequal(spec,0)
    n_param=(1+maxD*2);  
else
   n_param=(3+maxD*2);
end
% -------------------------------------------------------------------------
%       
%% Loop over no of sinusoids 

% initialize collection matrices
param_m=NaN*zeros(n_param,maxD);
BIC_v=NaN*zeros(maxD,1);

% matrix of trigonometric terms
trig=[];

% Loop
%------------------------------------

for d=1:maxD

    %trigonometric terms
     trig=[trig cos(time*2*pi*d) sin(time*2*pi*d)];

    % construct regressor matrix
    if isequal(spec,1)||isequal(spec,3)
        x=[time time.^2 trig];
    else
        x=trig;
    end

    % OLS estimation with constant
    [param,yhat]= ols(y,x,1);
    
    % residual variance
    sigma2_hat=mean((y-yhat).^2);
    
    % compute BIC
    BIC_v(d)=log(sigma2_hat)+length(param)*log(n_obs)/n_obs;
    
    % collect parameter estimates
    param_m(1:length(param),d)=param;
    
end
% -------------------------------------------------------------------------
%       
%% Select "optimal" specification

% find minimum d
[min_val, D_min]=min(BIC_v);

% select parameters
n_par_d=n_param-2*(maxD-D_min);
param=param_m(1:n_par_d,D_min);

% drop constant or not
if isequal(spec,0)||isequal(spec,1)
    param_out=param(2:end);%constant is estimate of Expected stochastic component
else
    param_out=param;
end
% -------------------------------------------------------------------------
%       
%% goodness of fit

if nargout>1

% construct regressor matrix
x=x(:,1:(end-2*(maxD-D_min)));
xc=[ones(size(x,1),1) x];

if isequal(spec,2)||isequal(spec,3)
    x=xc;
end

%compute R2 and R2_hat
R2=sum((xc*param-mean(xc*param)).^2)/sum((y-mean(y)).^2);
R2_hat=1-(1-R2)*(n_obs-1)/(n_obs-length(param));

% squared correlation between true and fitted values
scorr_tf=corr(y,(x*param_out)).^2;

%collect
reg_fit=[R2,R2_hat,scorr_tf];

end
