function [corrtf,var_tot,var_exp]=vola_fit(time,param_var1,param_abs_cov,param_var2,var1,abs_cov,var2,spec,beta)
%% vola_fit
%--------------------------------------------------------------
% checks in-sample fit of FFF-based volatility estimates
% Note: observation times are the same for each trading day
%--------------------------------------------------------------
% INPUT:
% time -> (Nx1) vector of observation times as 
%       fraction of trading day (time \in [0,1]): 
% param_var1 ->vector of Fourier parameters
% var1 -> (NTx1) vector of spot variance estimates:
%               T is number of days
% spec  -> specification indicator: =0 -> no polynomial and intercept
%                                                 =1  ->polynomial, but no
%                                                 intercept
%                                                 = 2   -> intercept, but
%                                                 no polynomial
%                                                  =3  -> intercept and
%                                                  polynomial
% beta  -> logical: if ==1, fit for beta is computed
%--------------------------------------------------------------
% OUTPUT:
% corrtf -> (1x4) vector of correlations between true and fitted
%                values
% var_tot -> (1x4) vector of total variations
% var_exp   -> (1x4) vector of explained variations  
% -------------------------------------------------------------------------
%
% Peter Malec, March 17, 2014
%
% -------------------------------------------------------------------------
%       
%% Input Check

if (~isequal(length(var1),length( abs_cov)))||(~isequal(length(var2),length(abs_cov)))||...
        (~isequal(length(var1),length(var2)))
    error('Vectors of estimates must have same length')
end

if (abs(length(var1)/length(time)-floor(length(var1)/length(time)))>0)||(abs(length(abs_cov)/length(time)-floor(length(abs_cov)/length(time)))>0)||...
         (abs(length(var2)/length(time)-floor(length(var2)/length(time)))>0)
     error('Number of estimates and times inconsistent')
end

if nargin<9
    beta=false(1);
end
% -------------------------------------------------------------------------
%   
%% Compute Estimates


%var 1
var1_fit=vola_fourier_gen(time,param_var1,spec);

%abs_cov
abs_cov_fit=vola_fourier_gen(time,param_abs_cov,spec);

%var 2
var2_fit=vola_fourier_gen(time,param_var2,spec);

% corr
if ~beta
    abs_corr_fit=abs_cov_fit./sqrt(var1_fit.*var2_fit);
else
    abs_corr_fit=abs_cov_fit./var2_fit;
end

% compute explained variation
var_exp=[mean((var1_fit-mean(var1_fit)).^2) mean((abs_cov_fit-mean(abs_cov_fit)).^2) mean((var2_fit-mean(var2_fit)).^2)...
    mean((abs_corr_fit-mean(abs_corr_fit)).^2)];

% compute estimated abs correlation
if ~beta
    abs_corr=abs_cov./sqrt(var1.*var2);
else
    abs_corr=abs_cov./var2;
end

% expand fitted values
%--------------------------------------------------------
var1_fit=reshape(var1_fit(:,ones(length(var1)/length(var1_fit),1)),length(var1),1);
abs_cov_fit=reshape(abs_cov_fit(:,ones(length(abs_cov)/length(abs_cov_fit),1)),length(abs_cov),1);
var2_fit=reshape(var2_fit(:,ones(length(var2)/length(var2_fit),1)),length(var2),1);
abs_corr_fit=reshape(abs_corr_fit(:,ones(length(abs_corr)/length(abs_corr_fit),1)),length(abs_corr),1);
% -------------------------------------------------------------------------
%   
%% Correct

%remove infinite correlations
if ~beta
    abs_corr((var1<=0)|(var2<=0))=NaN;
else
    abs_corr(var2<=0)=NaN;
end
abs_corr=abs_corr(~isnan(abs_corr));

% remove exact zeros and negative obs from original observations
var1(var1<=0)=NaN;
var1=var1(~isnan(var1));
abs_cov(abs_cov<=0)=NaN;
abs_cov=abs_cov(~isnan(abs_cov));
var2(var2<=0)=NaN;
var2=var2(~isnan(var2));

%correct
var1_fit=var1_fit(~isnan(var1));
abs_cov_fit=abs_cov_fit(~isnan(abs_cov));
var2_fit=var2_fit(~isnan(var2));
abs_corr_fit=abs_corr_fit(~isnan(abs_corr));
% -------------------------------------------------------------------------
%   
%% Compute Measures of Fit

% correlation
corrtf=[corr(var1,var1_fit) corr(abs_cov,abs_cov_fit) corr(var2,var2_fit) corr(abs_corr,abs_corr_fit)];

% total variation
var_tot=[mean((var1-mean(var1)).^2) mean((abs_cov-mean(abs_cov)).^2) mean((var2-mean(var2)).^2)...
     mean((abs_corr-mean(abs_corr)).^2)];





