function sigma2=vola_fourier_gen(time,param,spec,time_norm)
%% vola_fourier_gen
%--------------------------------------------------------------
% computes deterministic intraday volatility function
% based on Fourier form with or without polyn and constant
% NOTE: Including normalization ensuring integration to one
%--------------------------------------------------------------
% INPUT:
% time -> (Tx1) vector of observation times as 
% fraction of trading day (time \in [0,1])
% param -> ((D*2)x1) vector of Fourier parameters
% spec  -> specification indicator: =0 -> no polynomial and intercept
%                                                 =1  ->polynomial, but no
%                                                 intercept
%                                                 = 2   -> intercept, but
%                                                 no polynomial
%                                                  =3  -> intercept and
%                                                  polynomial
% time_norm -> times (with time \in [0,1]) used for normalization
%                       (optional)
%--------------------------------------------------------------
% OUTPUT:
% sigma2 -> (Tx1) vector of intraday variances
% -------------------------------------------------------------------------
%
% Peter Malec, March 17, 2014
%
% -------------------------------------------------------------------------
%       
% -------------------------------------------------------------------------
%       
%% Input Check

% default specification
if isempty(spec)
    spec=0;
end

% separate times for normalization
if nargin<4
    norm_sep=false(1);
else
    norm_sep=true(1);
end

% number parameters
D=getD(param,spec);
% -------------------------------------------------------------------------
%      
%% Compute fitted Intraday variances and normalize

% regressors: 
%------------------------------
%-----------------------------------

%trigonometric terms
%--------------------------------------
trig=NaN*zeros(length(time),(2*D));

for i=1:D
    trig(:,[(2*i-1)  2*i])=[cos(time*2*pi*i) sin(time*2*pi*i)];
end

% construct regressor matrix
if isequal(spec,1)||isequal(spec,3)
    x=[time time.^2 trig];
else
    x=trig;
end

if isequal(spec,2)||isequal(spec,3)
    x=[ones(length(time),1) x];
end

% fitted variances
sigma2=exp(x*param);

% separate normalization?
if ~norm_sep

    % Normalization
    sigma2=sigma2/mean(sigma2);
    
else
    
    %trigonometric terms
    %--------------------------------------
    trig=NaN*zeros(length(time_norm),(2*D));

    for i=1:D
        trig(:,[(2*i-1)  2*i])=[cos(time_norm*2*pi*i) sin(time_norm*2*pi*i)];
    end
    
    % construct regressor matrix
    if isequal(spec,1)||isequal(spec,3)
        x=[time_norm time_norm.^2 trig];
    else
        x=trig;
    end

    if isequal(spec,2)||isequal(spec,3)
        x=[ones(length(time_norm),1) x];
    end
    
    % fitted variances for normalization
    sigma2_norm=exp(x*param);
    
    % Normalization
    sigma2=sigma2/mean(sigma2_norm);
    
end

