function [Sigma_out,Sigma_kh,h,hs,B,J_max,K]=lle_cov_nonoise(data_a1,data_a2,startt,endt,s_block,diag_fish,K_fac,h_fac,J_choice)
%% lle_cov
%--------------------------------------------------------------
% local likelihood estimation of covolatility matrix  in noiseless case
% with spectral estimator as special case
%--------------------------------------------------------------
% INPUT:
% data_a1 -> (n_a x 2) matrix of synchronous data for asset 1
%       - 1st  col: observation times  (sec after midnight)
%       - 2nd col: prices 
% startt -> start of trading (in hours)
% endt   -> end of trading (in hours)
% s_block -> determines which block is shorter: 'first' or 'last' (default)
% diag_fish -> if =1, diagonal fisher info matrix: spectral estimator
% K_fac   -> constant factor for multiplication with "opt" bandwidth
% h_fac   -> constant factor for multiplication with "opt" block length
% J_choice -> choice of spectral cutoff: 'optim' follows Reiss (2011),
% numerical value yields fixed cutoff, otherwise log(n) (default)
%--------------------------------------------------------------
% OUTPUT:
% Sigma_out -> (2 x 2) matrix containg estimates of integrated (co-)
% volatilities
% Sigma_kh -> (2 x 2 x num(blocks)) array of spot (co-volatilities) at 
% lower bounds of blocks 
% h -> block length
% h -> length of shorter block
% B -> # blocks
% J_max -> spectral cutoff
% K  -> bandwidth for spot volatility estimation
% -------------------------------------------------------------------------
%
% Peter Malec, December 6, 2011
%
% January 3, 2012
% Note: adjusted for one shorter block (smaller h for first or last block)!!
% -------------------------------------------------------------------------
%       
%% Prepare Data

%price _rft is (n_rft x 2) matrix
price_rft=[data_a1(:,2) data_a2(:,2)];
                                    
% standardize times (time_rft in [0,1])                                    
time_rft_st= (data_a1(:,1)-startt*3600)/((endt-startt)*3600);          

% number of obs
n_rft=size(price_rft,1)-1;      %adjust for returns
% -------------------------------------------------------------------------
%       
%% Determine block length and spectral cutoff

% block length (rough choice)
h=min([(h_fac*log(n_rft)/sqrt(n_rft)) 1]);


% spectral cutoff
%-------------------------------------------------

% choose method
if nargin <9
    J_ind=2;
else
    if isnumeric(J_choice)
        J_ind=3;
    else
       J_ind=2;
    end
end

if J_ind==3 % fixed value 
    
    J_max=J_choice;

else % default method
    
    J_max=max(floor(min(log(n_rft),n_rft*h)),1);     
    
end
% -------------------------------------------------------------------------
%       
%%  Determine block bounds

% start and end of each block (# of elements gives # blocks)-
%-----------------------------------------------------------------------

% which block ist shorter?

if isequal(s_block,'first')  % first block
    
    % vector of bounds
    block_v=(fliplr((1:-h:0)))';
    
    if block_v(1)==0         % check if unit interval can be divided in equal blocks of length h
        block_st=block_v(1:end-1);
        block_end=block_v(2:end);
    else                                % first block is shorter !!
     block_st=[0; block_v(1:end-1)];
     block_end=block_v;
    end

    hs=block_end(1)-block_st(1);    %length of shorter block

else        % last block
    
    % vector of bounds
    block_v=(0:h:1)';

    if block_v(end)==1         % check if unit interval can be divided in equal blocks of length h
        block_st=block_v(1:end-1);
        block_end=block_v(2:end);
    else                                % last block is shorter !!
     block_st=block_v;
     block_end=[block_v(2:end); 1];
    end
    
    hs=block_end(end)-block_st(end);    %length of shorter block

end

% -------------------------------------------------------------------------
%       
%% Compute Local Blockwise Averages of Log-Returns

% number of blocks
B=length(block_st);

% frequencies up to spectral cutoff
j=(1:J_max)';

% log-returns
lret=diff(log(price_rft));

% local blockwise averages
%-----------------------------------------------------

% check if number of blocks is one
if B~=1
    
    [lret1_avb,lret2_avb]=lret_avb(time_rft_st(2:end),block_st,block_end,h,hs,s_block,J_max,lret);  %(J_max x B) matrix
   
else% special case if K=B=1
    
    numer= sin(pi*time_rft_st(2:end)*j');        % (n x J_max) matrix
    Phi_numer=numer.*repmat((time_rft_st(2:end) <1),1,J_max);
    denom=sqrt(2)*n_rft*sin(j*pi/(2*n_rft));        % (J_max x 1) vector
    Phi=Phi_numer./repmat(denom',n_rft,1);
    
    lret1_avb=(nansum(repmat(lret(:,1),1,J_max).*Phi))';     % (J_max x 1) vector
    lret2_avb=(nansum(repmat(lret(:,2),1,J_max).*Phi))';
    
end
% -------------------------------------------------------------------------
%       
%% Compute Pilot Estimates of Spot Volatility at Beginning of Blocks

% vector of empirical norms
Phi_norm=(2*n_rft*sin(j*pi/(2*n_rft*h))).^(-2);                     % (J_max x 1) vector
Phi_norms=(2*n_rft*sin(j*pi/(2*n_rft*hs))).^(-2);                     % for shorter block

% set noise to zero
H_mat=zeros(2,2);

if J_max>1       % if J=1, no pilot estimates are needed

    % bandwidth
    K=min(max(floor(K_fac*n_rft^((3/16)/(2*3/16+1))),2),B);   % alpha=3/16 (3/8 * 0.5), upper bound in appendix of Bibinger & Reiss (2011)
                      %min value K=2: not K=1 because even value needed for
                      %t at boundary. max value =# blocks
                      
    % spot volatility
    if K>1
        Sigma_kh=spot_vola(lret1_avb(1,:)',lret2_avb(1,:)',Phi_norm(1),Phi_norms(1),s_block,H_mat,K,n_rft);          % use lowest possible frequency j=1: outp is (2 x 2 x length(block_st)) array
    else % special case if K=B=1
        Sigma_kh=([lret1_avb(1);lret2_avb(1)]*[lret1_avb(1) lret2_avb(1)]-H_mat/n_rft)/Phi_norm(1);
    end
    
    % regularize spot covolatility matrices
    Sigma_kh=spot_vola_reg(Sigma_kh);
    
else
    
    K=0;
    Sigma_kh=0;
    
end
    
% -------------------------------------------------------------------------
%       
%% Compute spectral estimator of integrated covariance

if diag_fish ~=1  % local likelihood estimator

    if Sigma_kh ~=0  
        Sigma_out=lle_cov_core_nonoise(lret1_avb,lret2_avb,Phi_norm,Phi_norms,Sigma_kh,h,hs,s_block);         % (2 x 2) matrix
    else   % for J=1 lle and spectral estimator are equal
        Sigma_out=spectral_est_cov_core(lret1_avb,lret2_avb,Phi_norm,Phi_norms,H_mat,Sigma_kh,n_rft,h,hs,s_block);         % (2 x 2) matrix
    end
    
else % diagonal fisher info matrices: spectral estimator
    
    Sigma_out=spectral_est_cov_core(lret1_avb,lret2_avb,Phi_norm,Phi_norms,H_mat,Sigma_kh,n_rft,h,hs,s_block);         % (2 x 2) matrix
    
end











