function Sigma_out=lle_cov_oracle_faster_adj(data_a1,data_a2,H,startt,endt,i_I_jk,Phi_norm_m,h_v,block_st,block_end,J_maxs)
%% lle_cov_oracle_faster_adj
%--------------------------------------------------------------
% local likelihood estimation of covolatility matrix 
% with spectral estimator as special case
% fastwe oracle version
%--------------------------------------------------------------
% INPUT:
% data_a1 -> (n_a x 2) matrix of synchronous data for asset 1
%       - 1st  col: observation times  (sec after midnight)
%       - 2nd col: prices 
% H -> (2x2) covariance matrix of noise
% startt -> start of trading (in hours)
% endt   -> end of trading (in hours)
% i_I_jk -> structure containing elements of inverse fisher informations
% (each J x B), =0 if J=1
% Phi_norm_m -> (JxB) matrix of norms of spectral weights
% h_v -> (Bx1) vector of block lengths
% h -> block length
% hs -> length of shorter block
% block_st -> (Bx1) vector of block beginnings
% block_end -> (Bx1) vector of block ends
% J_maxs -> spectral cutoff for shorter block
%--------------------------------------------------------------
% OUTPUT:
% Sigma_out -> (2 x 2) matrix containg estimates of integrated (co-)
% volatilities
% -------------------------------------------------------------------------
%
% Peter Malec, February 11, 2012
%
% -------------------------------------------------------------------------
%       
%% Prepare Data and Inputs

%price  is (n_rft x 2) matrix
price=[data_a1(:,2) data_a2(:,2)];
                                    
% standardize times (time_rft in [0,1])                                    
time_st= (data_a1(:,1)-startt*3600)/((endt-startt)*3600);         

% log-returns
lret=diff(log(price));

% number of obs and times
n=size(price,1)-1;      %adjust for returns
t=time_st(2:end);

% number of blocks and freqs
[J B]=size(i_I_jk.i_I_jk_11);

% extract noise (co-)variances
nu2_x=H(1,1);
nu2_y=H(2,2);
nu_xy=H(1,2);

% elements of inverse fisher information
i_I_jk_11=i_I_jk.i_I_jk_11.*Phi_norm_m.^(-2);     % (J x B) matrix
i_I_jk_22 =i_I_jk.i_I_jk_22.*Phi_norm_m.^(-2);   % (J x B) matrix
i_I_jk_33=i_I_jk.i_I_jk_33.*Phi_norm_m.^(-2);     % (J x B) matrix
i_I_jk_12=i_I_jk.i_I_jk_12.*Phi_norm_m.^(-2);   % (J x B) matrix
i_I_jk_23 = i_I_jk.i_I_jk_23.*Phi_norm_m.^(-2);   % (J x B) matrix
i_I_jk_13= i_I_jk.i_I_jk_13.*Phi_norm_m.^(-2);   % (J x B) matrix

% position of shorter block
%-------------------------------------
[hs, inds]=min(h_v);   % length and position of shorter block

if (sum(h_v==hs)>1) % check if all blocks are the same
    inds=NaN;
end

 % -------------------------------------------------------------------------
%       
%% Compute Local Likelihood Estimator

if J~=1

%initialize matrix of spot (co-)volatility on each block
SCov_k_m=NaN*zeros(3,B);

% treat shorter block first
%-------------------------------------------------------
if ~isnan(inds)
    k=inds;     % set index
  
    % Compute Local Blockwise Averages of Log-Returns
    %------------------------------------------------------------------

    % spectral weights for given block
    Phi=spectr_weights_k(t,block_st(k),block_end(k),h_v(k),J_maxs);  % (n x J_maxs) matrix
    
    % local blockwise averages
    lret1_avb=(nansum(repmat(lret(:,1),1,J_maxs).*Phi))';     % (J_maxsx1) vector
    lret2_avb=(nansum(repmat(lret(:,2),1,J_maxs).*Phi))';
           %each freq: multiply return by weight, then sum over returns
           
    if J_maxs~=1       
           
    % Compute Fisher Information matrices for each frequency
    %------------------------------------------------------------------
    
    % matrix of distinct elements of inverse Fisher infos for all freqs
    i_I_jk_m=[i_I_jk_11(1:J_maxs,k)';i_I_jk_12(1:J_maxs,k)';i_I_jk_13(1:J_maxs,k)';i_I_jk_22(1:J_maxs,k)'...
        ;i_I_jk_23(1:J_maxs,k)';i_I_jk_33(1:J_maxs,k)'];      % (6 x J_maxs) matrix
    
    % create array of (symmetric) inverse fisher info matrices for all freqs
     i_I_jk_a=repmat(tril(ones(3)),[1 1 J_maxs]);        % (3x3xJ_maxs) array
     i_I_jk_a(logical(i_I_jk_a))=permute(i_I_jk_m,[1 3 2]);
     i_I_jk_d=i_I_jk_a(logical(repmat(eye(3),[1 1 J_maxs])));
     i_I_jk_a=permute(i_I_jk_a,[2 1 3])+i_I_jk_a;
     i_I_jk_a(logical(repmat(eye(3),[1 1 J_maxs])))=i_I_jk_d;
     
     % compute array of fisher info matrices by inversion
     I_jk_a=nd_inv_3x3sym(i_I_jk_a); %inv3(i_I_jk_a);
     
     % compute sum of fisher info matrices across freqs on block
     I_k_m=nansum(I_jk_a,3);                    % (3x3) matrix
     
     % estimate spot covola for given block as weighted average acr freqs
    %----------------------------------------------------------------------
     
    % matrix of dist. elements of outer poduct matrices of local ret avgs of for freqs
    S_jk_m=[(lret1_avb.^2)';(lret1_avb.*lret2_avb)';(lret2_avb.^2)'];   % (3 x J_maxs) matrix
    
    % matrix of dist. elements of bias-corr. spot vola matrix estimator on block for all freqs
    C_jk_m=(S_jk_m-repmat([nu2_x;nu_xy;nu2_y],1,J_maxs)/n)./repmat(Phi_norm_m(1:J_maxs,k)',3,1);   % (3 x J_maxs) matrix
    
    % array of spot covola estimates for given block as weighted average acr freqs
    C_jk_a=repmat(permute(C_jk_m,[3 1 2]),[3 1 1]);             % (3x3xJ_maxs) array
    
    % sum of prod of fisher info and spot vola estimates based on diff freqs
    Sum_IC_k=nansum(nansum(I_jk_a.*C_jk_a,2),3);   % (3x1) vector
                                                % inner sum: I_jk*C_jk for each j (yields (3x1) vector for j)
                                                 %outer sum: Sum_j I_jk*C_jk (yields (3x1) vector )
                                                 
    % compute spot vola estimates on block
    SCov_k_m(:,k)=I_k_m\Sum_IC_k; % (3x1) vector
    
    else % J_maxs=1
        
    % matrix of dist. elements of outer poduct matrices of local ret avgs of for freqs
    S_jk_m=[(lret1_avb.^2)';(lret1_avb.*lret2_avb)';(lret2_avb.^2)'];   % (3 x 1) vector
    
    % matrix of dist. elements of bias-corr. spot vola matrix estimator on block for all freqs
    SCov_k_m(:,k)=(S_jk_m-([nu2_x;nu_xy;nu2_y])/n)./repmat(Phi_norm_m(1,k),3,1);   % (3 x 1) vector

   end
    
end

% start loop over blocks
%-------------------------------------
for k=setdiff(1:B,inds)
    
    % Compute Local Blockwise Averages of Log-Returns
    %------------------------------------------------------------------

    % spectral weights for given block
    Phi=spectr_weights_k(t,block_st(k),block_end(k),h_v(k),J);  % (n x J) matrix
    
    % local blockwise averages
    lret1_avb=(nansum(repmat(lret(:,1),1,J).*Phi))';     % (Jx1) vector
    lret2_avb=(nansum(repmat(lret(:,2),1,J).*Phi))';
           %each freq: multiply return by weight, then sum over returns
           
    % Compute Fisher Information matrices for each frequency
    %------------------------------------------------------------------
    
    % matrix of distinct elements of inverse Fisher infos for all freqs
    i_I_jk_m=[i_I_jk_11(:,k)';i_I_jk_12(:,k)';i_I_jk_13(:,k)';i_I_jk_22(:,k)'...
        ;i_I_jk_23(:,k)';i_I_jk_33(:,k)'];      % (6 x J) matrix
    
    % create array of (symmetric) inverse fisher info matrices for all freqs
     i_I_jk_a=repmat(tril(ones(3)),[1 1 J]);        % (3x3xJ) array
     i_I_jk_a(logical(i_I_jk_a))=permute(i_I_jk_m,[1 3 2]);
     i_I_jk_d=i_I_jk_a(logical(repmat(eye(3),[1 1 J])));
     i_I_jk_a=permute(i_I_jk_a,[2 1 3])+i_I_jk_a;
     i_I_jk_a(logical(repmat(eye(3),[1 1 J])))=i_I_jk_d;
     
     % compute array of fisher info matrices by inversion
     I_jk_a=nd_inv_3x3sym(i_I_jk_a);
     
     % compute sum of fisher info matrices across freqs on block
     I_k_m=nansum(I_jk_a,3);                    % (3x3) matrix
     
     % estimate spot covola for given block as weighted average acr freqs
    %----------------------------------------------------------------------
     
    % matrix of dist. elements of outer poduct matrices of local ret avgs of for freqs
    S_jk_m=[(lret1_avb.^2)';(lret1_avb.*lret2_avb)';(lret2_avb.^2)'];   % (3 x J) matrix
    
    % matrix of dist. elements of bias-corr. spot vola matrix estimator on block for all freqs
    C_jk_m=(S_jk_m-repmat([nu2_x;nu_xy;nu2_y],1,J)/n)./repmat(Phi_norm_m(:,k)',3,1);   % (3 x J) matrix
    
    % array of spot covola estimates for given block as weighted average acr freqs
    C_jk_a=repmat(permute(C_jk_m,[3 1 2]),[3 1 1]);             % (3x3xJ) array
    
    % sum of prod of fisher info and spot vola estimates based on diff freqs
    Sum_IC_k=nansum(nansum(I_jk_a.*C_jk_a,2),3);   % (3x1) vector
                                                % inner sum: I_jk*C_jk for each j (yields (3x1) vector for j)
                                                 %outer sum: Sum_j I_jk*C_jk (yields (3x1) vector )
                                                 
    % compute spot vola estimates on block
    SCov_k_m(:,k)=I_k_m\Sum_IC_k; % (3x1) vector

end
 
% sum up blockwise integr. covola estimates 
lle_cov=SCov_k_m*h_v;   % (3xB)*(Bx1) >>> (3x1) vector

% create (co-volatility) matrix
Sigma_out=[lle_cov(1) lle_cov(2);lle_cov(2) lle_cov(3)];

%-----------------------------------------------------

else  % if J=1
    
% matrices of inverses of empirical norms of spectral weights 
i_Phi_norm=Phi_norm_m.^(-1);                  % (J x B) matrix

%empty matrices for local blockwise averages
lret1_avb=NaN*zeros(B,J);
lret2_avb=lret1_avb;

% start loop over blocks to compute local blockwise averages
%--------------------------------------------------------------------------------------------------
 for i=1:B

    % spectral weights for given block
    Phi=spectr_weights_k(t,block_st(i),block_end(i),h_v(i),J);  % (n x J) matrix
    
    % local blockwise averages
    lret1_avb(i,:)=nansum(repmat(lret(:,1),1,J).*Phi);     % (J x B) matrix
    lret2_avb(i,:)=nansum(repmat(lret(:,2),1,J).*Phi);
           %each freq: multiply return by weight, then sum over returns

 end
 
 % transpose matrices of local blockwise averages
 lret1_avb=lret1_avb';
 lret2_avb=lret2_avb';

%empty output matrix
Sigma_out=NaN*zeros(2,2);

% Var of asset 1
Sigma_out(1,1)=(sum(lret1_avb.^(2).*i_Phi_norm,1)-nu2_x/n*sum(i_Phi_norm,1))*h_v;          

%Covar of asset 1,2
Sigma_out(1,2)=(sum(lret1_avb.*lret2_avb.*i_Phi_norm,1)-nu_xy/n*sum(i_Phi_norm,1))*h_v;  
Sigma_out(2,1)=Sigma_out(1,2);

% Var of asset 2
Sigma_out(2,2)=(sum(lret2_avb.^(2).*i_Phi_norm,1)-nu2_y/n*sum(i_Phi_norm,1))*h_v;          
    
end










