function Sigma_mat=lle_cov_oracle_faster_const1_AllJ(data_a1,data_a2,H,startt,endt,I_jk_m,Phi_norm)
%% lle_cov_oracle_faster_const
%--------------------------------------------------------------
% local likelihood estimation of covolatility matrix 
%  faster oracle version for constant vola case and one block
% computes estimator using frequencies 1 to n
%--------------------------------------------------------------
% INPUT:
% data_a1 -> (n_a x 2) matrix of synchronous data for asset 1
%       - 1st  col: observation times  (sec after midnight)
%       - 2nd col: prices 
% H -> (2x2) covariance matrix of noise
% startt -> start of trading (in hours)
% endt   -> end of trading (in hours)
% I_jk_m-> (nx6) matrix of distinct elements of fisher infos for all freqs
% Phi_norm_m -> (nx1) matrix of norms of spectral weights for all freqs
%--------------------------------------------------------------
% OUTPUT:
% Sigma_mat -> (n x 3) matrix containg estimates of integrated (co-)
% volatilities: ith row contains (IV1,IC,IV2) estimates using freqs up to i
% -------------------------------------------------------------------------
%
% Peter Malec, March 24, 2012
%
% -------------------------------------------------------------------------
%       
%% Prepare Data and Inputs

%price  is (n_rft x 2) matrix
price=[data_a1(:,2) data_a2(:,2)];
                                    
% standardize times (time_rft in [0,1])                                    
time_st= (data_a1(:,1)-startt*3600)/((endt-startt)*3600);         

% log-returns
lret=diff(log(price));

% number of obs and times
n=size(price,1)-1;      %adjust for returns
t=time_st(2:end);

% extract noise (co-)variances
nu2_x=H(1,1);
nu2_y=H(2,2);
nu_xy=H(1,2);

% initialize output matrix
Sigma_mat=NaN*zeros(n,3);


% -------------------------------------------------------------------------
%       
%% Compute Local Likelihood Estimator using j=1,...,n

%empty matrix of product of fisher info and cjk
    IC_jk_m=NaN*zeros(n,3);

    for j=1:n
        
     % spectral weights
     numer= sin(j*pi*t(:));        % (n x 1) matrix
     denom=sqrt(2)*n*sin(j*pi/(2*n));       
     Phi=numer./denom;                          % (n x 1) vector
     
     % local blockwise averages
    lret1_avb=lret(:,1)'*Phi;     
    lret2_avb=lret(:,2)'*Phi;
    

    % vector of dist. elements of outer poduct matrices of local ret avgs 
    S_jk=[(lret1_avb^2); (lret1_avb*lret2_avb); (lret2_avb^2)];          %(3x1)
 
    %vector of noise (co-)variances divided by sample size
    nu2_v=[nu2_x; nu_xy ;nu2_y]/n;                                                %(3x1)
    
    % vector spot vola estimates using only given freq
    C_jk=(S_jk-nu2_v)/Phi_norm(j);     %(3x1)
    
     % Fisher Info Matrix
    I_jk=ivech(I_jk_m(j,:));
    
    % product of fisher info and cjk
    IC_jk_m(j,:)=(I_jk*C_jk)';
    
    %sum of fisher infos up to j
    I_k=ivech(sum(I_jk_m(1:j,:)));
    
    %LLE using freqs up to j
    Sigma_mat(j,:)=(I_k\(sum(IC_jk_m(1:j,:)))')';
    
    end 
    
    
   











