function Sigma_out=lle_cov_oracle_fasterff(data_a1,data_a2,H,startt,endt,I_jk_a_in,Phi_norm_m,h_v,block_st,block_end,J_maxs)
%% lle_cov_oracle_fasterff
%--------------------------------------------------------------
% local likelihood estimation of covolatility matrix 
% with spectral estimator as special case
% faster oracle version preselecting
% observations on each block
%--------------------------------------------------------------
% INPUT:
% data_a1 -> (n_a x 2) matrix of synchronous data for asset 1
%       - 1st  col: observation times  (sec after midnight)
%       - 2nd col: prices 
% H -> (2x2) covariance matrix of noise
% startt -> start of trading (in hours)
% endt   -> end of trading (in hours)
% W_in -> structure if (J_opt x B) matrices I_jk containing (i,i) elements
% of Fisher information matrices for frequency j and block k
% Phi_norm_m -> (JxB) matrix of norms of spectral weights
% h_v -> (Bx1) vector of block lengths
% h -> block length
% hs -> length of shorter block
% block_st -> (Bx1) vector of block beginnings
% block_end -> (Bx1) vector of block ends
% J_maxs -> spectral cutoff for shorter block
%--------------------------------------------------------------
% OUTPUT:
% Sigma_out -> (2 x 2) matrix containg estimates of integrated (co-)
% volatilities
% -------------------------------------------------------------------------
%
% Peter Malec, April 19, 2012
%
% May 5, 2012 
% correction in computation of weights
% -------------------------------------------------------------------------
%       
%% Prepare Data and Inputs

%price  is (n_rft x 2) matrix
price=[data_a1(:,2) data_a2(:,2)];
                                    
% standardize times (time_rft in [0,1])                                    
time_st= (data_a1(:,1)-startt*3600)/((endt-startt)*3600);         

% log-returns
lret=diff(log(price));

% number of obs and times
n=size(price,1)-1;      %adjust for returns
t=time_st(2:end);

% number of blocks and freqs
[J B]=size(Phi_norm_m);

% extract noise (co-)variances
nu2_x=H(1,1);
nu2_y=H(2,2);
nu_xy=H(1,2);

% create noise matrices (replicated
noise_v=[nu2_x;nu_xy;nu2_y];
noise_m=noise_v(:,ones(J,1));
noise_ms=noise_v(:,ones(J_maxs,1));

% elements of inverse fisher information
I_jk_11=I_jk_a_in.I_jk_11;     % (J x B) matrix
I_jk_12=I_jk_a_in.I_jk_12;     % (J x B) matrix
I_jk_13=I_jk_a_in.I_jk_13;     % (J x B) matrix
I_jk_22=I_jk_a_in.I_jk_22;     % (J x B) matrix
I_jk_23=I_jk_a_in.I_jk_23;     % (J x B) matrix
I_jk_33=I_jk_a_in.I_jk_33;     % (J x B) matrix

% position of shorter block
%-------------------------------------
[hs, inds]=min(h_v);   % length and position of shorter block

if (sum(h_v==hs)>1) % check if all blocks are the same
    inds=NaN;
end

 % -------------------------------------------------------------------------
%       
%% Compute Local Likelihood Estimator

if J~=1

%initialize matrix of spot (co-)volatility on each block
SCov_k_m=NaN*zeros(3,B);

% treat shorter block first
%-------------------------------------------------------
if ~isnan(inds)
    k=inds;     % set index
  
    % Compute Local Blockwise Averages of Log-Returns
    %------------------------------------------------------------------
    
    %empty matrices for local blockwise averages
    lret1_avb=NaN*zeros(J_maxs,1);   % (J_maxsx1) vector
    lret2_avb=lret1_avb;
    
    
    % index of observations on each block
     t_ind=logical(((t >= block_st(k)) & (t <block_end(k))));  
     
     % observations times on each block (since begining)
     tb=t(t_ind)-block_st(k);
     
     % log returns on block
     lret_b=lret(t_ind,:);
        
        for j=1:J_maxs
            
            % spectral weights for given block and frequency
            Phi=spectr_weights_j_onbl(tb,n,h_v(k),j);
            
            % local blockwise averages using frequency j
            lret1_avb(j)=lret_b(:,1)'*Phi; 
            lret2_avb(j)=lret_b(:,2)'*Phi;
            
        end
    
    if J_maxs~=1       
           
    % Create arrays of weights
    %------------------------------------------------------------------
    
    % matrix of distinct elements of Fisher info matrices for all frequs
    I_jk_m=[I_jk_11(1:J_maxs,k)';I_jk_12(1:J_maxs,k)';I_jk_13(1:J_maxs,k)';I_jk_22(1:J_maxs,k)'...
        ;I_jk_23(1:J_maxs,k)';I_jk_33(1:J_maxs,k)'];      % (6 x J_maxs) matrix
    
    % create array of weight matrices for all freqs
    I_jk_a=ivech3(I_jk_m);
    
    % compute sum of fisher info matrices across freqs on block
    I_k_m=nansum(I_jk_a,3);                    % (3x3) matrix
    
    % compute array of weights
    W_a=inv3(I_k_m,I_jk_a);
     
     % estimate spot covola for given block as weighted average acr freqs
    %----------------------------------------------------------------------
    
    % select empirical norms of spectral weights
    Phi_norm_v=Phi_norm_m(1:J_maxs,k);
     
    % matrix of dist. elements of outer poduct matrices of local ret avgs of for freqs
    S_jk_m=[(lret1_avb.^2)';(lret1_avb.*lret2_avb)';(lret2_avb.^2)'];   % (3 x J_maxs) matrix
    
    % matrix of dist. elements of bias-corr. spot vola matrix estimator on block for all freqs
    C_jk_m=(S_jk_m-noise_ms/n)./(Phi_norm_v(:,ones(3,1)))'; % (3 x J_maxs) matrix
    
    %  spot vola estimates on block: sum of prod of fisher info and spot vola estimates based on diff freq
    SCov_k_m(:,k)=nansum(mtimesx(W_a,permute(C_jk_m,[1 3 2])),3);   % (3x1) vector
   
    
    else % J_maxs=1
        
    % select empirical norms of spectral weights
    Phi_norm_v=Phi_norm_m(1,k);
        
    % matrix of dist. elements of outer poduct matrices of local ret avgs of for freqs
    S_jk_m=[(lret1_avb.^2)';(lret1_avb.*lret2_avb)';(lret2_avb.^2)'];   % (3 x 1) vector
    
    % matrix of dist. elements of bias-corr. spot vola matrix estimator on block for all freqs
    SCov_k_m(:,k)=(S_jk_m-noise_ms/n)./(Phi_norm_v(:,ones(3,1)))'; % (3 x J_maxs) matrix

   end
    
end

% start loop over blocks
%-------------------------------------
for k=setdiff(1:B,inds)
    
    % Compute Local Blockwise Averages of Log-Returns
    %------------------------------------------------------------------

    %empty matrices for local blockwise averages
    lret1_avb=NaN*zeros(J,1);   % (Jx1) vector
    lret2_avb=lret1_avb;
    
    
    % index of observations on each block
     t_ind=logical(((t >= block_st(k)) & (t <block_end(k))));  
     
     % observations times on each block (since begining)
     tb=t(t_ind)-block_st(k);
     
     % log returns on block
     lret_b=lret(t_ind,:);
        
        for j=1:J
            
            % spectral weights for given block and frequency
            Phi=spectr_weights_j_onbl(tb,n,h_v(k),j);
            
            % local blockwise averages using frequency j
            lret1_avb(j)=lret_b(:,1)'*Phi; 
            lret2_avb(j)=lret_b(:,2)'*Phi;
            
        end
           
    % Create arrays of weights
    %------------------------------------------------------------------
    
    % matrix of distinct elements of Fisher info matrices for all frequs
    I_jk_m=[I_jk_11(:,k)';I_jk_12(:,k)';I_jk_13(:,k)';I_jk_22(:,k)'...
        ;I_jk_23(:,k)';I_jk_33(:,k)'];      % (6 x J) matrix
    
    % create array of weight matrices for all freqs
    I_jk_a=ivech3(I_jk_m);
    
    % compute sum of fisher info matrices across freqs on block
    I_k_m=nansum(I_jk_a,3);                    % (3x3) matrix
    
    % compute array of weights
    W_a=inv3(I_k_m,I_jk_a);
    
    
    % estimate spot covola for given block as weighted average acr freqs
    %----------------------------------------------------------------------
     
    % select empirical norms of spectral weights
    Phi_norm_v=Phi_norm_m(:,k);
     
    % matrix of dist. elements of outer poduct matrices of local ret avgs of for freqs
    S_jk_m=[(lret1_avb.^2)';(lret1_avb.*lret2_avb)';(lret2_avb.^2)'];   % (3 x J) matrix
    
    % matrix of dist. elements of bias-corr. spot vola matrix estimator on block for all freqs
    C_jk_m=(S_jk_m-noise_m/n)./(Phi_norm_v(:,ones(3,1)))'; % (3 x J) matrix
    
    %  spot vola estimates on block: sum of prod of fisher info and spot vola estimates based on diff freq
    SCov_k_m(:,k)=nansum(mtimesx(W_a,permute(C_jk_m,[1 3 2])),3);   % (3x1) vector

end
 
% sum up blockwise integr. covola estimates 
lle_cov=SCov_k_m*h_v;   % (3xB)*(Bx1) >>> (3x1) vector

% create (co-volatility) matrix
Sigma_out=[lle_cov(1) lle_cov(2);lle_cov(2) lle_cov(3)];

%-----------------------------------------------------

else  % if J=1
    
% matrices of inverses of empirical norms of spectral weights 
i_Phi_norm=Phi_norm_m.^(-1);                  % (J x B) matrix


% Compute Local Blockwise Averages of Log-Returns
%------------------------------------------------------------------
%empty matrices for local blockwise averages
lret1_avb=NaN*zeros(B,1);       % (B x1)
lret2_avb=lret1_avb;

% -- loop --
for k=1:B

 % index of observations on each block
     t_ind=logical(((t >= block_st(k)) & (t <block_end(k))));  
    % observations times on each block (since begining)
     tb=t(t_ind)-block_st(k);
    % log returns on block
     lret_b=lret(t_ind,:);

     % spectral weights for given block and frequency
      Phi=spectr_weights_j_onbl(tb,n,h_v(k),1);
            
      % local blockwise averages using frequency 1
      lret1_avb(k,:)=lret_b(:,1)'*Phi; 
      lret2_avb(k,:)=lret_b(:,2)'*Phi;
            
end

 % transpose matrices of local blockwise averages
 lret1_avb=lret1_avb';
 lret2_avb=lret2_avb';

%empty output matrix
Sigma_out=NaN*zeros(2,2);

% Var of asset 1
Sigma_out(1,1)=(lret1_avb.^(2).*i_Phi_norm-nu2_x/n*i_Phi_norm)*h_v;          

%Covar of asset 1,2
Sigma_out(1,2)=(lret1_avb.*lret2_avb.*i_Phi_norm-nu_xy/n*i_Phi_norm)*h_v;  
Sigma_out(2,1)=Sigma_out(1,2);

% Var of asset 2
Sigma_out(2,2)=(lret2_avb.^(2).*i_Phi_norm-nu2_y/n*i_Phi_norm)*h_v;          
    
end










