function [SCov_mat,block_st,block_end,J_opt,J_p,K,K_p,noisev_m,nobs_v,Covmat_mat]=lle_spot_cov_md_adpt_mult(varargin)
%% lle_spot_cov_md_adpt_mult
%--------------------------------------------------------------
% local likelihood estimation of spot covolatility matrices on specified
% number of blocks adapted to observed price process
% for a set of points in time
% only for cross-sectionally independent noise in m-dimensional case
%
% Note:    
%             * uses data for [0,t] to estimate SCov at t, i.e. at end of last block
%             * Accordingly, smoothing is done only over current and K-1 previous blocks
%             * blocks with equal lengths
%              *inputs either fixed or based on continuous approximations
%             * norms of spectral weights uncorrected (based on continuous approximations)
%            * spectral weights based on cont approximations
%
% USAGE:
% [SCov_mat,block_st,block_end,J_optv,J_p,K,K_p,Covmat_arr]=lle_spot_cov_md_adpt_mult(input_v,PRICETIME,startt,endt,...
%  nkex,phi_ctr,B,J,J_p,K,K_p,theta_h,theta_J,theta_K,K_thr,K_p_thr,twst,twstc,ccorr)
%
%--------------------------------------------------------------
% INPUT:
% input_v -> (n_inx1) vector of estimation points (normalized trading day, i.e. between 0 and 1)
% PRICETIME -> (mx1) cell array with ith cell containing (n0ix2) matrix:
%                   1st col  -  observation times for asset i (in sec after midnight)
%                   2nd col    - high frequency prices for asset i
% noise_v -> (mx1) vector of noise variance estimates
% start -> start of trading (in hours) 
% endt   -> time at which spot cov matrix is to be estimated (in hours)
% nkex -> if >0, exact number of obs on each block is used to determine
%               cutoff
% phi_ctr -> if >0, center of return spells used to compute spectral
%               weights
% B      -> no of equally-sized blocks
% J  -> spectral cutoff for final estimates of spot covariances
% J_p-> spectral cutoff for pilot estimates of spot covariances
% K   -> bandwidth for smoothing of final spot cov estimates
% K_p  -> bandwidth for smoothing of pilot estimates
% theta_h   -> proportionality parameter for block length (relevant if B
%                           not specified)
% theta_J     -> proportionaliy parameter for cutoff (relevant if J
%                           or J_p not specified)
% theta_k    -> proportionality parameter for smoothing bandwidth (relevant if K
%                           or K_p not specified)
% K_thr    -> lower threshold for pilot smoothing bandwidth (relevant if
%                           K not specified): default=1
% K_p_thr    -> lower threshold for pilot smoothing bandwidth (relevant if
%                           K_p not specified): default=1
% twst     -> if >0: twostep estimation is used (estimates from first step
%                   used as pilots for second step
% twstc      -> if >0: Covariance matrices of smoothed spot (co-)volatilities
%   under the local parametric experiment   are computed using 2ndstep spot
%   covariance estimates (mattters only if 8 output arguments)
% ccorr -> if >0, correlation and variance estimates are corrected
%--------------------------------------------------------------
% OUTPUT:
% SCov_mat-> (n_inxm(m+1)/2) matrix of spot cov matrices for each point of
%                               estimation
%                    each row: vech(\hat Sigma_t)'
% block_st -> (n_inx1) cell array containing (Bx1) vectors of block beginnings
% block_end -> (n_inx1) cell array containing (Bx1) vectors  of block ends
% J_opt ->  (n_inx1) cell array containing (Bx1) vectors of optimal spectral cutoffs for each block 
% noisev_m -> (n_inxm) matrix of noise variance estimates for each point in time
% nobs_v  -> (n_inx1) vector of minimum number of observations up to now for each point in time
% Covmat_mat ->(m^2(m^2+1)/2xn_in) matrix: Stacked Covariance matrices of smoothed spot (co-)volatilities
%                       under the local parametric experiment      (optional)
%                    each column:vech(Cov(\hat Sigma_t))
%   NOTE: covariance matrices still need to be postmultiplied by Z_tilde,
%   i.e. the (m^2 x m^2)  matrix of fourth moments of a m-dimensional
%                   standard Gaussian vector
% -------------------------------------------------------------------------
%
% Peter Malec, January 22, 2013 
%
% -------------------------------------------------------------------------
%       
%% Input check

% Check no of inputs
Q = length(varargin);
if Q<19
    error('19 inputs required.') 
end

% Get data and determine number of assets
PRICETIME=varargin{2};
m=length(PRICETIME);

% Get remaining inputs
%-----------------------------------------
input_v=varargin{1};
startt=varargin{Q-16};
endt=varargin{Q-15};
nkex=varargin{Q-14};
phi_ctr=varargin{Q-13};
B=varargin{Q-12};
J=varargin{Q-11};
J_p=varargin{Q-10};
K=varargin{Q-9};
K_p=varargin{Q-8};
theta_h=varargin{Q-7};
theta_J=varargin{Q-6};
theta_K=varargin{Q-5};
K_thr=varargin{Q-4};
K_p_thr=varargin{Q-3};
twst=varargin{Q-2};
twstc=varargin{Q-1};
ccorr=varargin{Q};
%-------------------------------------------------------------------------
      
%% Compute Spot Covariance Matrix Estimates for each Input Time

%empty output arrays
SCov_mat=NaN*zeros(length(input_v),m*(m+1)/2); 
if nargout>9
    Covmat_mat=NaN*zeros(m^2*(m^2+1)/2,length(input_v)); 
end
block_st=cell(length(input_v),1);
block_end=cell(length(input_v),1);
J_opt=cell(length(input_v),1);
J_p_v=NaN*zeros(length(input_v),1);
K_v=NaN*zeros(length(input_v),1);
K_p_v=NaN*zeros(length(input_v),1);
noisev_m=NaN*zeros(length(input_v),m);
nobs_v=NaN*zeros(length(input_v),1);
 
% loop across input times and compute estimates
%-----------------------------------------------------------------------
parfor j=1:length(input_v)
    
    % obtain input in h
    input_cal=input_v(j)*(endt-startt)+startt;
    
    %empty arrays
     PRICETIME_temp=cell(m,1);
     noisev=NaN*zeros(m,1);
     n_obs=NaN*zeros(m,1);
    
     %loop across assets and construct necessary series
     for i=1:m
         
         %data for asset
         data_temp=PRICETIME{i};
         
         %take only data before time of estimation
         PRICETIME_temp{i}=data_temp(data_temp(:,1)<=(input_cal*3600),:);
         n_obs(i)=length(PRICETIME_temp{i})-1;
         
         %estimate noise variances
         data_temp_n=PRICETIME_temp{i};
         eta2=noise_cov_est(data_temp_n(:,2),'cov');
         if eta2<=0
            eta2=noise_cov_est(data_temp_n(:,2),'var');
         end
        noisev(i)=eta2;   
     end
     
     % assign no of observations and noise variances
     nobs_v(j)=min(n_obs);
     noisev_m(j,:)=noisev';
    
     if nargout>9
     
        % compute spot covariance matrix estimate
        [SCov_mat_i,block_st_i,block_end_i,J_opt_i,J_p_i,K_i,K_p_i,Covmat_mat_i]=lle_spot_cov_md_adpt(PRICETIME_temp,noisev,startt,input_cal,...
        nkex,phi_ctr,B,J,J_p,K,K_p,theta_h,theta_J,theta_K,K_thr,K_p_thr,twst,twstc,ccorr);

        %assign outputs
        SCov_mat(j,:)=SCov_mat_i'; 
        Covmat_mat(:,j)=Covmat_mat_i; 
        block_st{j}=block_st_i;
        block_end{j}=block_end_i;
        J_opt{j}=J_opt_i;
        J_p_v(j)=J_p_i;
        K_v(j)=K_i;
        K_p_v(j)=K_p_i;
    
     else
         
         % compute spot covariance matrix estimate
        [SCov_mat_i,block_st_i,block_end_i,J_opt_i,J_p_i,K_i,K_p_i]=lle_spot_cov_md_adpt(PRICETIME_temp,noisev,startt,input_cal,...
        nkex,phi_ctr,B,J,J_p,K,K_p,theta_h,theta_J,theta_K,K_thr,K_p_thr,twst,twstc,ccorr);

        %assign outputs
        SCov_mat(j,:)=SCov_mat_i'; 
        block_st{j}=block_st_i;
        block_end{j}=block_end_i;
        J_opt{j}=J_opt_i;
        J_p_v(j)=J_p_i;
        K_v(j)=K_i;
        K_p_v(j)=K_p_i;
         
     end
end










