function [Sigma_out,h_v,h,hs,block_st,block_end,J_optv]=spec_cov_oracle_async(data_a1,data_a2,H,startt,endt,s_block,SCov_mat,h_fac,nkex,cv)
%% spec_cov_oracle_async
%--------------------------------------------------------------
% spectral estimation of covolatility matrix (asynchr)
% faster oracle version preselecting observations on each block
% only for cross-sectionally independent noise
%
% Note:   *  comprehensive procedure!
%            * equal block lengths
%            * spectral weights based on cont approximations
%            * block length determined using average of #obs/noise
%--------------------------------------------------------------
% INPUT:
% data_a1 -> (n_a x 2) matrix of synchronous data for asset 1
%       - 1st  col: observation times  (sec after midnight)
%       - 2nd col: prices 
% H -> (2x2) diagonal covariance matrix of noise
% startt -> start of trading (in hours)
% endt   -> end of trading (in hours)
% s_block -> determines which block is shorter: 'first' or 'last' (default)
% SCov_mat -> (N x 3) matrix of spot (co-) variances ([SVar1 SVar1 Scov]),
% where N such that times=1/N:1/N:1.
% Note: each SCov matrix has to be positive definite
% h_fac   -> constant factor for multiplication with "opt" block length
% nkex -> if >0, exact number of obs on each block is used to determine
% cutoff
% cv-> critical value for choice of spectral cutoff (frobenius norm of
% "last" weights <=cv). If numeric: all frequencies are used. If integer or vector:
% prespecified vector of spectral cutoffs for each block
%  Default: cv=1e-6
%--------------------------------------------------------------
% OUTPUT:
% Sigma_out -> (2 x 2) matrix containg estimates of integrated (co-)
% volatilities
% h_v -> (Bx1) vector of block lengths
% h -> block length
% hs -> length of shorter block
% block_st -> (Bx1) vector of block beginnings
% block_end -> (Bx1) vector of block ends
% J_opt -> vector of optimal spectral cutoffs for each block 
% -------------------------------------------------------------------------
%
% Peter Malec, August 20, 2012 
%
% -------------------------------------------------------------------------
%       
%% Prepare Data and Inputs

% Input value of cv
%---------------------------------------
if nargin<10
    cv=1e-6;
end

if isnumeric(cv)
    J_all=0;
    if ((length(cv)==1)&&(cv>=1))||length(cv)>1
        J_optv_in=cv;% vector of optimal cutoffs (B prespecified)
    else
        J_optv_in=0;
    end
else
    J_all=1;
end

% log-returns
lret1=diff(log(data_a1(:,2)));
lret2=diff(log(data_a2(:,2)));

% standardize times (time_rft in [0,1])                                    
time_st1= (data_a1(:,1)-startt*3600)/((endt-startt)*3600);         
time_st2= (data_a2(:,1)-startt*3600)/((endt-startt)*3600); 

% times, centered times and differences
%-------------------------------------------------------------------------
t1=time_st1(2:end);
t2=time_st2(2:end);

avgt1=0.5*(time_st1(2:end)+time_st1(1:end-1));
avgt2=0.5*(time_st2(2:end)+time_st2(1:end-1));

dt1=time_st1(2:end)-time_st1(1:end-1);
dt2=time_st2(2:end)-time_st2(1:end-1);

% extract noise (co-)variances
nu2_x=H(1,1);
nu2_y=H(2,2);

% number of obs
nx=length(t1);
ny=length(t2);
nrft=min([nx ny]);
% -------------------------------------------------------------------------
%       
%% Determine block length and bounds

% block length
%-------------------------------------------------

% sample size relative to noise
n_nu=[nx; ny]./mean(diag(H)); %use average 


% block length (rough choice)
h=min([(h_fac*log(n_nu)/sqrt(n_nu)) 1]);

% start and end of each block
[block_st,block_end,hs,ks]=block_start_end(h,s_block);

% number of blocks
B=length(block_st);

% create block length vector accouting for shorter block and B=1
%------------------------------------------------------------------------------
if B~=1
    if isequal(s_block,'first')  % first block
        h_v=[hs;repmat(h,B-1,1)];                                      % (Bx1) vector
    else
        h_v=[repmat(h,B-1,1) ;hs];                                      % (Bx1) vector
    end
else   %special case if B=1 
        h_v=h;
        ks=NaN;
end
% -------------------------------------------------------------------------
%       
%% Compute LLE

% create noise vector (replicated)
noise_v=[nu2_x;0;nu2_y];

% number of Scov obs
N=size(SCov_mat,1);

%times at which Spot covolatilities are "observed"
t_scov=1/N:1/N:1;
    
%initialize matrix of spot (co-)volatility on each block
SCov_k_m=NaN*zeros(3,B);

% vector for cutoffs
if J_optv_in==0
    J_optv=NaN*zeros(B,1);
else
    J_optv=J_optv_in;
end
      
% set of "normal" blocks
normB=setdiff(1:B,ks);   

% Loop over normal blocks
%---------------------------------------------------------------------

for k=normB
    
    % compute spot covolatility estimate on block k
     [SCov_k_v,J_opt_k]=spot_cov_block_spectral(k,block_st(k),block_end(k),h,nrft,avgt1,avgt2,dt1,dt2,lret1,lret2,t_scov,SCov_mat,...
    noise_v,nkex,cv,J_optv_in,J_all);

    % assign estimates to matrix
    SCov_k_m(:,k)=SCov_k_v;

     %assign optimal cutoff to vector
     J_optv(k)=J_opt_k;
    
    
end

% short block 
%-------------------------------------------------
if B>1

% compute spot covolatility estimate on block k
     [SCov_k_v,J_opt_k]=spot_cov_block_spectral(ks,block_st(ks),block_end(ks),hs,nrft,avgt1,avgt2,dt1,dt2,lret1,lret2,t_scov,SCov_mat,...
    noise_v,nkex,cv,J_optv_in,J_all);

    % assign estimates to matrix
    SCov_k_m(:,ks)=SCov_k_v;

     %assign optimal cutoff to vector
     J_optv(ks)=J_opt_k;
end
    
%-------------------------------------------------------------------------
%-------------------------------------------------------------------------

% sum up blockwise integr. covola estimates 
lle_cov=SCov_k_m*h_v;   % (3xB)*(Bx1) >>> (3x1) vector

% create (co-volatility) matrix
Sigma_out=[lle_cov(1) lle_cov(2);lle_cov(2) lle_cov(3)];







     











