function Sigma_out=spectral_cov_est_oracle_faster(data_a1,data_a2,H,startt,endt,W,i_Phi_norm,h_v,block_st,block_end,J_maxs)
%% lle_cov_oracle_faster
%--------------------------------------------------------------
% spectral estimation of covolatility matrix 
% faster oracle version
%--------------------------------------------------------------
% INPUT:
% data_a1 -> (n_a x 2) matrix of synchronous data for asset 1
%       - 1st  col: observation times  (sec after midnight)
%       - 2nd col: prices 
% H -> (2x2) covariance matrix of noise
% startt -> start of trading (in hours)
% endt   -> end of trading (in hours)
% W -> structure containing weights of spectral freqs for each block
% (each J x B), =0 if J=1
% i_Phi_norm -> (JxB) matrix of inverse norms of spectral weights
% h_v -> (Bx1) vector of block lengths
% h -> block length
% hs -> length of shorter block
% block_st -> (Bx1) vector of block beginnings
% block_end -> (Bx1) vector of block ends
% J_maxs -> spectral cutoff for shorter block
%--------------------------------------------------------------
% OUTPUT:
% Sigma_out -> (2 x 2) matrix containg estimates of integrated (co-)
% volatilities
% -------------------------------------------------------------------------
%
% Peter Malec, February 28, 2012
%
% -------------------------------------------------------------------------
%       
%% Prepare Data and Inputs

%price  is (n_rft x 2) matrix
price=[data_a1(:,2) data_a2(:,2)];
                                    
% standardize times (time_rft in [0,1])                                    
time_st= (data_a1(:,1)-startt*3600)/((endt-startt)*3600);         

% log-returns
lret=diff(log(price));

% number of obs and times
n=size(price,1)-1;      %adjust for returns
t=time_st(2:end);

% number of blocks and freqs
[J B]=size(W.W_x);

% extract noise (co-)variances
nu2_x=H(1,1);
nu2_y=H(2,2);
nu_xy=H(1,2);

% weights of spectral freqs for each block
W_x=W.W_x;     % (J x B) matrix
W_xy=W.W_xy;     % (J x B) matrix
W_y=W.W_y;     % (J x B) matrix

% position of shorter block
%-------------------------------------
[hs, inds]=min(h_v);   % length and position of shorter block

if (sum(h_v==hs)>1) % check if all blocks are the same
    inds=NaN;
end

 % -------------------------------------------------------------------------
%       
%% Compute Spectral Estimator

%empty matrices for local blockwise averages
lret1_avb=NaN*zeros(B,J);
lret2_avb=lret1_avb;

% start loop over blocks to compute local blockwise averages
%--------------------------------------------------------------------------------------------------
 for i=1:B

    % spectral weights for given block
    Phi=spectr_weights_k(t,block_st(i),block_end(i),h_v(i),J);  % (n x J) matrix
    
    % local blockwise averages
    lret1_avb(i,:)=nansum(repmat(lret(:,1),1,J).*Phi);     % (J x B) matrix
    lret2_avb(i,:)=nansum(repmat(lret(:,2),1,J).*Phi);
           %each freq: multiply return by weight, then sum over returns

 end
 
 % transpose matrices of local blockwise averages
 lret1_avb=lret1_avb';
 lret2_avb=lret2_avb';
 
 % check if number of frequencies is lower for shorter block
 if J_maxs<J
     lret1_avb((J_maxs+1):end,inds)=zeros(J-J_maxs,1);
     lret2_avb((J_maxs+1):end,inds)=zeros(J-J_maxs,1);
     i_Phi_norm((J_maxs+1):end,inds)=zeros(J-J_maxs,1);
     W_x((J_maxs+1):end,inds)=zeros(J-J_maxs,1);
     W_xy((J_maxs+1):end,inds)=zeros(J-J_maxs,1);
     W_y((J_maxs+1):end,inds)=zeros(J-J_maxs,1);
 end
 
 

%empty output matrix
Sigma_out=NaN*zeros(2,2);

% Var of asset 1
Sigma_out(1,1)=(sum(W_x.*lret1_avb.^(2).*i_Phi_norm,1)-nu2_x/n*sum(W_x.*i_Phi_norm,1))*h_v;          

%Covar of asset 1,2
Sigma_out(1,2)=(sum(W_xy.*lret1_avb.*lret2_avb.*i_Phi_norm,1)-nu_xy/n*sum(W_xy.*i_Phi_norm,1))*h_v;  
Sigma_out(2,1)=Sigma_out(1,2);

% Var of asset 2
Sigma_out(2,2)=(sum(W_y.*lret2_avb.^(2).*i_Phi_norm,1)-nu2_y/n*sum(W_y.*i_Phi_norm,1))*h_v;                  
    











