function [Sigma_out]=spectral_est_cov_core(lret1_avb,lret2_avb,Phi_norm,Phi_norms,H,Sigma_kh,n,h,hs,s_block)
%% spectral_est_cov_core
%--------------------------------------------------------------
% Core function for spectral estimation of covolatility following 
% Bibinger & Reiss (2011) based on pilot estimates of spot covariance 
% at beginning of each block
%--------------------------------------------------------------
% INPUT:
% lret1_avb -> (J x B) matrix of  local Blockwise Averages of Log-Returns
%                     for asset 1 for different spectral frequencies
% Phi_norm -> (J x 1) vector empirical norm of spectral weights  for different spectral
%                   frequencies
% Phi_norms -> (J x 1) vector empirical norm of spectral weights  for shorter block and chosen frequency j
% H             -> (2 x 2) matrix being estimate of noise covariance
% Sigma_kh -> (2 x 2 x B) array of spot covariance matrices at beginning of
%                     each block
% n              -> # of observations
% h              -> block length
% hs            -> length of shorter block
% s_block -> determines which block is shorter: 'first' or 'last' (default)
%--------------------------------------------------------------
% OUTPUT:
% Sigma_out -> (2 x 2) matrix containg estimates of integrated (co-)
% volatilities
% -------------------------------------------------------------------------
%
% Peter Malec, December 7, 2011
%
% January 4, 2012
% adjusted for one shorter block (smaller h for first or last block)!!
%
% -------------------------------------------------------------------------
%       
%% Preparations

%number of frequencies and blocks
J=size(lret1_avb,1);
B=size(lret1_avb,2);

% extract noise (co-)variances
nu2_x=H(1,1);
nu2_y=H(2,2);
nu_xy=H(1,2);

% extract spot (co-) volatility estimates
if Sigma_kh ~=0             % check if J=1 (no weights needed)
    sigma2_x=squeeze(Sigma_kh(1,1,:));      % (B x 1) vector
    sigma2_y=squeeze(Sigma_kh(2,2,:)); 
    sigma_xy=squeeze(Sigma_kh(1,2,:)); 
end


% create Phi_norm matrix and block length vector accouting for shorter
% block and B=1
if B~=1
    if isequal(s_block,'first')  % first block
        Phi_norm_m=[Phi_norms repmat(Phi_norm,1,B-1)]; % (J x B) matrix
        h_v=[hs;repmat(h,B-1,1)];                                      % (Bx1) vector
    else
        Phi_norm_m=[repmat(Phi_norm,1,B-1) Phi_norms];  % (J x B) matrix
        h_v=[repmat(h,B-1,1); hs];                                      % (Bx1) vector
    end
else   %special case if B=1 
        Phi_norm_m=Phi_norm;
        h_v=h;
end


% matrices of inverses and squared inverses of empirical norms of spectral weights 
i_Phi_norm=Phi_norm_m.^(-1);                  % (J x B) matrix
i2_Phi_norm=Phi_norm_m.^(-2);                 % (J x B) matrix
% -------------------------------------------------------------------------
%       
%% Compute weights of spectral freqs for each block

if Sigma_kh ~=0  % check if J=1 (no weights needed)
       
    % Var of asset 1
    Num_x=(i_Phi_norm*(nu2_x/n)+repmat(sigma2_x',J,1)).^(-2);    % (J x B) matrix
    W_x=Num_x./repmat(sum(Num_x),J,1);           % sum over freqs in denominator (J x B) matrix    

    %Covar of asset 1,2
    Num_xy=(i2_Phi_norm*(nu2_x*nu2_y+nu_xy^2)/(n^2)+repmat((sigma2_x'.*sigma2_y'+(sigma_xy').^2),J,1)+...
           i_Phi_norm.*repmat((sigma2_x'*nu2_y/n+sigma2_y'*nu2_x/n+2*sigma_xy'*nu_xy/n),J,1)).^(-1);          % (J x B) matrix
    W_xy=Num_xy./repmat(sum(Num_xy),J,1);           % sum over freqs in denominator (J x B) matrix    

    % Var of asset 2
    Num_y=(i_Phi_norm*(nu2_y/n)+repmat(sigma2_y',J,1)).^(-2);          % (J x B) matrix
    W_y=Num_y./repmat(sum(Num_y),J,1);           % sum over freqs in denominator (J x B) matrix    

else
    
    W_x=ones(1,B);
    W_xy=W_x;
    W_y=W_x;
    
end
    
% -------------------------------------------------------------------------
%       
%% Compute Estimates

%empty output matrix
Sigma_out=NaN*zeros(2,2);

% Var of asset 1
Sigma_out(1,1)=(sum(W_x.*lret1_avb.^(2).*i_Phi_norm,1)-nu2_x/n*sum(W_x.*i_Phi_norm,1))*h_v;          

%Covar of asset 1,2
Sigma_out(1,2)=(sum(W_xy.*lret1_avb.*lret2_avb.*i_Phi_norm,1)-nu_xy/n*sum(W_xy.*i_Phi_norm,1))*h_v;  
Sigma_out(2,1)=Sigma_out(1,2);

% Var of asset 2
Sigma_out(2,2)=(sum(W_y.*lret2_avb.^(2).*i_Phi_norm,1)-nu2_y/n*sum(W_y.*i_Phi_norm,1))*h_v;          






