function [Sigma_out,Avg_frq_weight]=lle_cov_core(lret1_avb,lret2_avb,Phi_norm,Phi_norms,H,Sigma_kh,n,h,hs,s_block)
%% lle_cov_core
%--------------------------------------------------------------
% Core function for local likelihood estimation of covolatility matrix 
% based on pilot estimates of spot covariance 
% at beginning of each block. Note: use spectral_est_cov_core for J=1!!
%--------------------------------------------------------------
% INPUT:
% lret1_avb -> (J x B) matrix of  local Blockwise Averages of Log-Returns
%                     for asset 1 for different spectral frequencies
% Phi_norm -> (J x 1) vector empirical norm of spectral weights  for different spectral
%                   frequencies
% Phi_norms -> (J x 1) vector empirical norm of spectral weights  for shorter block and chosen frequency j
% H             -> (2 x 2) matrix being estimate of noise covariance
% Sigma_kh -> (2 x 2 x B) array of spot covariance matrices at beginning of
%                     each block
% n              -> # of observations
% h              -> block length
% hs            -> length of shorter block
% s_block -> determines which block is shorter: 'first' or 'last' (default)
%--------------------------------------------------------------
% OUTPUT:
% Sigma_out -> (2 x 2) matrix containg estimates of integrated (co-)
% volatilities
% Avg_frq_weight -> (3 x 3 x 5) array containing weight matrices (upper
% triangular elements) for first 5 frequencies averaged across blocks
% -------------------------------------------------------------------------
%
% Peter Malec, January 13, 2012
%
% -------------------------------------------------------------------------
%       
%% Preparations

%number of frequencies and blocks
J=size(lret1_avb,1);
B=size(lret1_avb,2);

% extract noise (co-)variances
nu2_x=H(1,1);
nu2_y=H(2,2);
nu_xy=H(1,2);

% extract spot (co-) volatility estimates

sigma2_x=squeeze(Sigma_kh(1,1,:));      % (B x 1) vector
sigma2_y=squeeze(Sigma_kh(2,2,:)); 
sigma_xy=squeeze(Sigma_kh(1,2,:)); 



% create Phi_norm matrix and block length vector accouting for shorter
% block and B=1
if B~=1
    if isequal(s_block,'first')  % first block
        Phi_norm_m=[Phi_norms repmat(Phi_norm,1,B-1)]; % (J x B) matrix
        h_v=[hs;repmat(h,B-1,1)];                                      % (Bx1) vector
    else
        Phi_norm_m=[repmat(Phi_norm,1,B-1) Phi_norms];  % (J x B) matrix
        h_v=[repmat(h,B-1,1) ;hs];                                      % (Bx1) vector
    end
else   %special case if B=1 
        Phi_norm_m=Phi_norm;
        h_v=h;
end

% matrices of squared empirical norms of spectral weights
Phi_norm_m_2=Phi_norm_m.^(2);                 % (J x B) matrix
% -------------------------------------------------------------------------
%       
%% Compute elements of inverse fisher information

i_I_jk_11=2*(nu2_x/n+repmat(sigma2_x',J,1).*Phi_norm_m).^2;     % (J x B) matrix

i_I_jk_22=(nu2_x*nu2_y+nu_xy^2)/(n^2)+repmat((sigma2_x'.*sigma2_y'+(sigma_xy').^2),J,1).*Phi_norm_m_2+...
    (Phi_norm_m/n).*repmat(sigma2_x'*nu2_y+sigma2_y'*nu2_x+2*sigma_xy'*nu_xy,J,1);   % (J x B) matrix

i_I_jk_33=2*(nu2_y/n+repmat(sigma2_y',J,1).*Phi_norm_m).^2;     % (J x B) matrix

i_I_jk_12=2*Phi_norm_m_2.*repmat(sigma2_x'.*sigma_xy',J,1)+2*nu2_x*nu_xy/(n^2)+...
    2*(Phi_norm_m/n).*repmat(sigma2_x'*nu_xy+sigma_xy'*nu2_x,J,1);   % (J x B) matrix

i_I_jk_23=2*Phi_norm_m_2.*repmat(sigma2_y'.*sigma_xy',J,1)+2*nu2_y*nu_xy/(n^2)+...
    2*(Phi_norm_m/n).*repmat(sigma2_y'*nu_xy+sigma_xy'*nu2_y,J,1);   % (J x B) matrix

i_I_jk_13=2*Phi_norm_m_2.*repmat((sigma_xy').^2,J,1)+2*(nu_xy/n)^2+...
    4*(Phi_norm_m/n).*repmat(sigma_xy'*nu_xy,J,1);   % (J x B) matrix
% -------------------------------------------------------------------------
%       
%% Compute Estimates of spot covolatilities on each block

%initialize matrix of spot (co-)volatility on each block
SCov_k_m=NaN*zeros(3,B);

% initialize matrices for elements of weight matrices 
if nargout==2
    weight_m1=NaN*zeros(B,6);
    weight_m2=weight_m1;
    weight_m3=weight_m1;
    weight_m4=weight_m1;
    weight_m5=weight_m1;
end

% Loop over blocks
%-------------------------------------
%-------------------------------------
for k=1:B
    
    % Compute Fisher Information matrices for each frequency
    %------------------------------------------------------------------
    
    % matrix of distinct elements of inverse Fisher infos for all freqs
    i_I_jk_m=[i_I_jk_11(:,k)';i_I_jk_12(:,k)';i_I_jk_13(:,k)';i_I_jk_22(:,k)'...
        ;i_I_jk_23(:,k)';i_I_jk_33(:,k)'];      % (6 x J) matrix
    
    % create array of (symmetric) inverse fisher info matrices for all freqs
     i_I_jk_a=repmat(tril(ones(3)),[1 1 J]);        % (3x3xJ) array
     i_I_jk_a(logical(i_I_jk_a))=permute(i_I_jk_m,[1 3 2]);
     i_I_jk_d=i_I_jk_a(logical(repmat(eye(3),[1 1 J])));
     i_I_jk_a=permute(i_I_jk_a,[2 1 3])+i_I_jk_a;
     i_I_jk_a(logical(repmat(eye(3),[1 1 J])))=i_I_jk_d;
     
     % compute array of fisher info matrices by inversion
     I_jk_a=nd_inv_3x3sym(i_I_jk_a);
     
     % compute sum of fisher info matrices across freqs on block
     I_k_m=nansum(I_jk_a,3);                    % (3x3) matrix
     
     % estimate spot covola for given block as weighted average acr freqs
    %----------------------------------------------------------------------
     
    % matrix of dist. elements of outer poduct matrices of local ret avgs of for freqs
    S_jk_m=[(lret1_avb(:,k).^2)';(lret1_avb(:,k).*lret2_avb(:,k))';(lret2_avb(:,k).^2)'];   % (3 x J) matrix
    
    % matrix of dist. elements of bias-corr. spot vola matrix estimator on block for all freqs
    C_jk_m=(S_jk_m-repmat([nu2_x;nu_xy;nu2_y],1,J)/n)./repmat(Phi_norm_m(:,k)',3,1);   % (3 x J) matrix
    
    % array of spot covola estimates for given block as weighted average acr freqs
    C_jk_a=repmat(permute(C_jk_m,[3 1 2]),[3 1 1]);             % (3x3xJ) array
    
    % sum of prod of fisher info and spot vola estimates based on diff freqs
    Sum_IC_k=nansum(nansum(I_jk_a.*C_jk_a,2),3);   % (3x1) vector
                                                % inner sum: I_jk*C_jk for each j (yields (3x1) vector for j)
                                                 %outer sum: Sum_j I_jk*C_jk (yields (3x1) vector )
                                                 
    % compute spot vola estimates on block
    SCov_k_m(:,k)=I_k_m\Sum_IC_k; % (3x1) vector
    
    % weight matrices for 1st 5  frequencies
    if nargout==2
        weight_m1_temp=I_k_m\I_jk_a(:,:,1);
        weight_m1(k,:)=weight_m1_temp(logical(triu(ones(3))))';
        weight_m2_temp=I_k_m\I_jk_a(:,:,2);
        weight_m2(k,:)=weight_m2_temp(logical(triu(ones(3))))';
        weight_m3_temp=I_k_m\I_jk_a(:,:,3);
        weight_m3(k,:)=weight_m3_temp(logical(triu(ones(3))))';
        weight_m4_temp=I_k_m\I_jk_a(:,:,4);
        weight_m4(k,:)=weight_m4_temp(logical(triu(ones(3))))';
        weight_m5_temp=I_k_m\I_jk_a(:,:,5);
        weight_m5(k,:)=weight_m5_temp(logical(triu(ones(3))))';
    end
   
end
% -------------------------------------------------------------------------
%       
%% Compute estimate of integrated (co-)volatity matrix

% sum up blockwise integr. covola estimates 
lle_cov=SCov_k_m*h_v;   % (3xB)*(Bx1) >>> (3x1) vector

% create (co-volatility) matrix
Sigma_out=[lle_cov(1) lle_cov(2);lle_cov(2) lle_cov(3)];
% -------------------------------------------------------------------------
%       
%% Averages of weight matrices for 1st 5  frequencies

if nargout==2

    % compute averages and put them into upper triangular matrices
    Avg_frq_weight1=zeros(3);
    Avg_frq_weight1(logical(triu(ones(3))))=mean(abs(weight_m1))';
    Avg_frq_weight2=zeros(3);
    Avg_frq_weight2(logical(triu(ones(3))))=mean(abs(weight_m2))';
    Avg_frq_weight3=zeros(3);
    Avg_frq_weight3(logical(triu(ones(3))))=mean(abs(weight_m3))';
    Avg_frq_weight4=zeros(3);
    Avg_frq_weight4(logical(triu(ones(3))))=mean(abs(weight_m4))';
    Avg_frq_weight5=zeros(3);
    Avg_frq_weight5(logical(triu(ones(3))))=mean(abs(weight_m5))';
    
    % output array
    Avg_frq_weight=cat(3,Avg_frq_weight1,Avg_frq_weight2,Avg_frq_weight3,Avg_frq_weight4,Avg_frq_weight5);
    
    
end




