function [Sigma_out]=lle_cov_core_nonoise(lret1_avb,lret2_avb,Phi_norm,Phi_norms,Sigma_kh,h,hs,s_block)
%% lle_cov_core_nonoise
%--------------------------------------------------------------
% Core function for local likelihood estimation of covolatility matrix 
% based on pilot estimates of spot covariance in noiseless case
% at beginning of each block. Note: use spectral_est_cov_core for J=1!!
%--------------------------------------------------------------
% INPUT:
% lret1_avb -> (J x B) matrix of  local Blockwise Averages of Log-Returns
%                     for asset 1 for different spectral frequencies
% Phi_norm -> (J x 1) vector empirical norm of spectral weights  for different spectral
%                   frequencies
% Phi_norms -> (J x 1) vector empirical norm of spectral weights  for shorter block and chosen frequency j
% Sigma_kh -> (2 x 2 x B) array of spot covariance matrices at beginning of
%                     each block
% h              -> block length
% hs            -> length of shorter block
% s_block -> determines which block is shorter: 'first' or 'last' (default)
%--------------------------------------------------------------
% OUTPUT:
% Sigma_out -> (2 x 2) matrix containg estimates of integrated (co-)
% volatilities
% -------------------------------------------------------------------------
%
% Peter Malec, January 13, 2012
%
%
% -------------------------------------------------------------------------
%       
%% Preparations

%number of frequencies and blocks
J=size(lret1_avb,1);
B=size(lret1_avb,2);


% extract spot (co-) volatility estimates

sigma2_x=squeeze(Sigma_kh(1,1,:));      % (B x 1) vector
sigma2_y=squeeze(Sigma_kh(2,2,:)); 
sigma_xy=squeeze(Sigma_kh(1,2,:)); 



% create Phi_norm matrix and block length vector accouting for shorter
% block and B=1
if B~=1
    if isequal(s_block,'first')  % first block
        Phi_norm_m=[Phi_norms repmat(Phi_norm,1,B-1)]; % (J x B) matrix
        h_v=[hs;repmat(h,B-1,1)];                                      % (Bx1) vector
    else
        Phi_norm_m=[repmat(Phi_norm,1,B-1) Phi_norms];  % (J x B) matrix
        h_v=[repmat(h,B-1,1) ;hs];                                      % (Bx1) vector
    end
else   %special case if B=1 
        Phi_norm_m=Phi_norm;
        h_v=h;
end

% matrices of squared empirical norms of spectral weights
Phi_norm_m_2=Phi_norm_m.^(2);                 % (J x B) matrix


% -------------------------------------------------------------------------
%       
%% Compute elements of inverse fisher information w/o empirical norms

i_I_jk_11=sigma2_x.^2;     % (Bx1) vector

i_I_jk_22=(sigma2_x.*sigma2_y+(sigma_xy).^2)/2;   

i_I_jk_33=sigma2_y.^2;    

i_I_jk_12=sigma2_x.*sigma_xy;   

i_I_jk_23=sigma2_y.*sigma_xy;   

i_I_jk_13=sigma_xy.^2;  
% -------------------------------------------------------------------------
%       
%% Compute Estimates of spot covolatilities on each block

%initialize matrix of spot (co-)volatility on each block
SCov_k_m=NaN*zeros(3,B);

% Loop over blocks
%-------------------------------------
%-------------------------------------
for k=1:B
    
    % Compute Fisher Information matrices for each frequency
    %------------------------------------------------------------------
    
    % matrix of distinct elements of inverse Fisher infos for all freqs
    i_I_jk_m=[i_I_jk_11(k);i_I_jk_12(k);i_I_jk_13(k);i_I_jk_22(k)...
        ;i_I_jk_23(k);i_I_jk_33(k)];      % (6 x 1) vector
    
    % create (symmetric) inverse fisher info matrix
     i_I_jk_a=tril(ones(3));        % (3x3) matrix
     i_I_jk_a(logical(i_I_jk_a))=i_I_jk_m;
     i_I_jk_d=diag(i_I_jk_a);
     i_I_jk_a=i_I_jk_a+i_I_jk_a';
     i_I_jk_a(logical(eye(3)))=i_I_jk_d;
     
     
     
     % compute array of fisher info matrices by inversion 
     [ I_jk,i_I_jk_a_s, eig_s ,v_s]=nd_inv_3x3sym_sing(i_I_jk_a);
     I_jk_a=0.5*repmat(I_jk,[1 1 J])./repmat(permute(Phi_norm_m_2(:,k)',[1 3 2]),[3 3 1]);
                                                                                    %squared empirical norm of Phi was left out before
                                                                                    
     % compute sum of fisher info matrices across freqs on block
     I_k_m=nansum(I_jk_a,3);                    % (3x3) matrix
     
%      if (abs(det(I_k_m))<=eps)||(abs(det(I_k_m))>1/eps)
%          disp(['Block ' int2str(k) ' of ' int2str(B)])
%          disp('I_k_m')
%          disp(I_k_m)
%          disp('rho_kh')
%          disp(Sigma_kh(1,2,k)/sqrt(Sigma_kh(1,1,k)*Sigma_kh(2,2,k)))
%          
%          
%      end

     
%      if isempty(v_s)~=1
%          disp(['Block ' int2str(k) ' of ' int2str(B)])
%          disp('i_I_jk_a: ')
%          disp(i_I_jk_a)
%          disp(' Eigenvalues of i_I_jk_a: ')
%         disp(eig_s)
%         disp('Sigma_kh')
%         disp(squeeze(Sigma_kh(:,:,k)))
%          disp('rho_kh')
%         disp(Sigma_kh(1,2,k)/sqrt(Sigma_kh(1,1,k)*Sigma_kh(2,2,k)))
%      end
     
    % estimate spot covola for given block as weighted average acr freqs
    %----------------------------------------------------------------------
     
    % matrix of dist. elements of outer poduct matrices of local ret avgs of for freqs
    S_jk_m=[(lret1_avb(:,k).^2)';(lret1_avb(:,k).*lret2_avb(:,k))';(lret2_avb(:,k).^2)'];   % (3 x J) matrix
    
    % matrix of dist. elements of bias-corr. spot vola matrix estimator on block for all freqs
    C_jk_m=S_jk_m./repmat(Phi_norm_m(:,k)',3,1);   % (3 x J) matrix
    
    % array of spot covola estimates for given block as weighted average acr freqs
    C_jk_a=repmat(permute(C_jk_m,[3 1 2]),[3 1 1]);             % (3x3xJ) array
    
    % sum of prod of fisher info and spot vola estimates based on diff freqs
    Sum_IC_k=nansum(nansum(I_jk_a.*C_jk_a,2),3);   % (3x1) vector
                                                % inner sum: I_jk*C_jk for each j (yields (3x1) vector for j)
                                                 %outer sum: Sum_j I_jk*C_jk (yields (3x1) vector )
                                                 
    % compute spot vola estimates on block
    if (abs(det(I_k_m))<=eps)||(abs(det(I_k_m))>1/eps)
        I_k_m=I_k_m*det(I_k_m)^(-1/3);
        SCov_k_m(:,k)=(I_k_m\Sum_IC_k)*det(I_k_m)^(-1/3);   % (3x1) vector
    else
        SCov_k_m(:,k)=I_k_m\Sum_IC_k;
    end
   
end
% -------------------------------------------------------------------------
%       
%% Compute estimate of integrated (co-)volatity matrix

% sum up blockwise integr. covola estimates 
lle_cov=SCov_k_m*h_v;   % (3xB)*(Bx1) >>> (3x1) vector

% create (co-volatility) matrix
Sigma_out=[lle_cov(1) lle_cov(2);lle_cov(2) lle_cov(3)];





