function [Sigma_out,h,B,J_max,Avg_frq_weight]=lle_cov_oracle(data_a1,data_a2,H_mat,startt,endt,s_block,diag_fish,SCov_mat,h_fac,J_choice)
%% lle_cov
%--------------------------------------------------------------
% local likelihood estimation of covolatility matrix 
% with spectral estimator as special case
% oracle version: true spot covolatility known
%--------------------------------------------------------------
% INPUT:
% data_a1 -> (n_a x 2) matrix of synchronous data for asset 1
%       - 1st  col: observation times  (sec after midnight)
%       - 2nd col: prices 
% H_mat -> (2x2) covariance matrix of noise
% startt -> start of trading (in hours)
% endt   -> end of trading (in hours)
% s_block -> determines which block is shorter: 'first' or 'last' (default)
% diag_fish -> if =1, diagonal fisher info matrix: spectral estimator
% SCov_mat -> (N x 3) matrix of spot (co-) variances ([SVar1 SVar1 Scov]),
% where N such that times=1/N:1/N:1.
% Note: each SCov matrix has to be positive definite
% h_fac   -> constant factor for multiplication with "opt" block length
% J_choice -> choice of spectral cutoff.
% numerical value yields fixed cutoff, otherwise log(n) (default)
%--------------------------------------------------------------
% OUTPUT:
% Sigma_out -> (2 x 2) matrix containg estimates of integrated (co-)
% volatilities
% h -> block length
% B -> # blocks
% J_max -> spectral cutoff
% Avg_frq_weight -> (3 x 3 x 5) array containing weight matrices (upper
% triangular elements) for first 5 frequencies averaged across blocks
% -------------------------------------------------------------------------
%
% Peter Malec, January 30, 2012
%
% -------------------------------------------------------------------------
%       
%% Prepare Data

%price _rft is (n_rft x 2) matrix
price_rft=[data_a1(:,2) data_a2(:,2)];
                                    
% standardize times (time_rft in [0,1])                                    
time_rft_st= (data_a1(:,1)-startt*3600)/((endt-startt)*3600);          

% number of obs
n_rft=size(price_rft,1)-1;      %adjust for returns
% -------------------------------------------------------------------------
%       
%% Determine block length and spectral cutoff

% block length
%-------------------------------------------------

% sample size relative to noise
n_nu=n_rft/mean(diag(H_mat)); %use average noise variance

% block length (rough choice)
h=min([(h_fac*log(n_nu)/sqrt(n_nu)) 1]);


% spectral cutoff
%-------------------------------------------------

% choose method
if nargin <10
    J_ind=2;
else
    if isnumeric(J_choice)
        J_ind=3;
    else
       J_ind=2;
    end
end

if J_ind==3 % fixed value 
    
    J_max=J_choice;

else % default method
    
    J_max=max(min(ceil(log(n_rft)),floor(n_rft*h)),1);     
    
end
% -------------------------------------------------------------------------
%       
%%  Determine block bounds

% start and end of each block (# of elements gives # blocks)-
%-----------------------------------------------------------------------

% which block ist shorter?

if isequal(s_block,'first')  % first block
    
    % vector of bounds
    block_v=(fliplr((1:-h:0)))';
    
    if block_v(1)==0         % check if unit interval can be divided in equal blocks of length h
        block_st=block_v(1:end-1);
        block_end=block_v(2:end);
    else                                % first block is shorter !!
     block_st=[0; block_v(1:end-1)];
     block_end=block_v;
    end

    hs=block_end(1)-block_st(1);    %length of shorter block

else        % last block
    
    % vector of bounds
    block_v=(0:h:1)';

    if block_v(end)==1         % check if unit interval can be divided in equal blocks of length h
        block_st=block_v(1:end-1);
        block_end=block_v(2:end);
    else                                % last block is shorter !!
     block_st=block_v;
     block_end=[block_v(2:end); 1];
    end
    
    hs=block_end(end)-block_st(end);    %length of shorter block

end

% -------------------------------------------------------------------------
%       
%% Compute Local Blockwise Averages of Log-Returns

% number of blocks
B=length(block_st);

% frequencies up to spectral cutoff
j=(1:J_max)';

% log-returns
lret=diff(log(price_rft));

% local blockwise averages
%-----------------------------------------------------

% check if number of blocks is one
if B~=1
    
    [lret1_avb,lret2_avb]=lret_avb(time_rft_st(2:end),block_st,block_end,h,hs,s_block,J_max,lret);  %(J_max x B) matrix
   
else% special case if K=B=1
    
    numer= sin(pi*time_rft_st(2:end)*j');        % (n x J_max) matrix
    Phi_numer=numer.*repmat((time_rft_st(2:end) <1),1,J_max);
    denom=sqrt(2)*n_rft*sin(j*pi/(2*n_rft));        % (J_max x 1) vector
    Phi=Phi_numer./repmat(denom',n_rft,1);
    
    lret1_avb=(nansum(repmat(lret(:,1),1,J_max).*Phi))';     % (J_max x 1) vector
    lret2_avb=(nansum(repmat(lret(:,2),1,J_max).*Phi))';
    
end

% -------------------------------------------------------------------------
%       
%% Create Array of Spot Volatility at Beginning of Blocks



if J_max>1       % if J=1, no pilot estimates are needed
    
    % number of Scov obs
    N=size(SCov_mat,1);

    %times at which Spot covolatilities are "observed"
     t_scov=1/N:1/N:1;
    
    % Spot Volatility at Beginning of Blocks taking last value
    %------------------------------------------------------------------
    
    % initialize
    Sigma_kh=NaN*zeros(2,2,B);
    
    % first block
    Sigma_kh(:,:,1)=[SCov_mat(1,1) SCov_mat(1,3);SCov_mat(1,3) SCov_mat(1,2)];
   
    % loop for other blocks
    if B>1
        for k=2:B
            t_scov_ind=find(t_scov<= block_st(k),1,'last');
            Sigma_kh(:,:,k)=[SCov_mat(t_scov_ind,1) SCov_mat(t_scov_ind,3);SCov_mat(t_scov_ind,3) SCov_mat(t_scov_ind,2)];
        end
    end
    
else
    
   
    Sigma_kh=0;
    
end
    
% -------------------------------------------------------------------------
%       
%% Compute spectral estimator of integrated covariance

% vector of empirical norms
Phi_norm=(2*n_rft*sin(j*pi/(2*n_rft*h))).^(-2);                     % (J_max x 1) vector
Phi_norms=(2*n_rft*sin(j*pi/(2*n_rft*hs))).^(-2);                     % for shorter block


if diag_fish ~=1  % local likelihood estimator

    if J_max>1  
        if nargout==5 %compute average weights
            [Sigma_out,Avg_frq_weight]=lle_cov_core(lret1_avb,lret2_avb,Phi_norm,Phi_norms,H_mat,Sigma_kh,n_rft,h,hs,s_block);         % (2 x 2) matrix
        else
            Sigma_out=lle_cov_core(lret1_avb,lret2_avb,Phi_norm,Phi_norms,H_mat,Sigma_kh,n_rft,h,hs,s_block);         % (2 x 2) matrix
        end
     else   % for J=1 lle and spectral estimator are equal
        Sigma_out=spectral_est_cov_core(lret1_avb,lret2_avb,Phi_norm,Phi_norms,H_mat,Sigma_kh,n_rft,h,hs,s_block);         % (2 x 2) matrix
    end
    
else % diagonal fisher info matrices: spectral estimator
    
    Sigma_out=spectral_est_cov_core(lret1_avb,lret2_avb,Phi_norm,Phi_norms,H_mat,Sigma_kh,n_rft,h,hs,s_block);         % (2 x 2) matrix
    
end











