function [Sigma_out,h_v,h,hs,block_st,block_end,J_opt]=lle_cov_oracle_all(data_a1,data_a2,H,startt,endt,s_block,SCov_mat,h_fac,J_all)
%% lle_cov_oracle_prep
%--------------------------------------------------------------
% local likelihood estimation of covolatility matrix (synchr)
% faster oracle version preselecting
% observations on each block
% Note: comprehensive procedure!
%--------------------------------------------------------------
% INPUT:
% data_a1 -> (n_a x 2) matrix of synchronous data for asset 1
%       - 1st  col: observation times  (sec after midnight)
%       - 2nd col: prices 
% H -> (2x2) covariance matrix of noise
% startt -> start of trading (in hours)
% endt   -> end of trading (in hours)
% s_block -> determines which block is shorter: 'first' or 'last' (default)
% SCov_mat -> (N x 3) matrix of spot (co-) variances ([SVar1 SVar1 Scov]),
% where N such that times=1/N:1/N:1.
% Note: each SCov matrix has to be positive definite
% h_fac   -> constant factor for multiplication with "opt" block length
% J_all -> if >0, all frequencies are used, else optimal number (default)
%--------------------------------------------------------------
% OUTPUT:
% Sigma_out -> (2 x 2) matrix containg estimates of integrated (co-)
% volatilities
% h_v -> (Bx1) vector of block lengths
% h -> block length
% hs -> length of shorter block
% block_st -> (Bx1) vector of block beginnings
% block_end -> (Bx1) vector of block ends
% J_opts -> spectral cutoff for shorter block
% -------------------------------------------------------------------------
%
% Peter Malec, May 9, 2012 
%
% -------------------------------------------------------------------------
%       
%% Prepare Data and Inputs

% Input value of J_all
if nargin<9
    J_all=0;
end

%price  is (n_rft x 2) matrix
price=[data_a1(:,2) data_a2(:,2)];
                                    
% standardize times (time_rft in [0,1])                                    
time_st= (data_a1(:,1)-startt*3600)/((endt-startt)*3600);         

% log-returns
lret=diff(log(price));

% number of obs and times
n_rft=size(price,1)-1;      %adjust for returns
t=time_st(2:end);

% extract noise (co-)variances
nu2_x=H(1,1);
nu2_y=H(2,2);
nu_xy=H(1,2);

% -------------------------------------------------------------------------
%       
%% Determine block length 

% block length
%-------------------------------------------------

% sample size relative to noise
n_nu=n_rft/mean(diag(H)); %use average noise variance


% block length (rough choice)
h=min([(h_fac*log(n_nu)/sqrt(n_nu)) 1]);


% -------------------------------------------------------------------------
%       
%%  Determine block bounds and spectral cutoff (pilot)

% start and end of each block (# of elements gives # blocks)-
%-----------------------------------------------------------------------

% which block ist shorter?

if isequal(s_block,'first')  % first block
    
    % vector of bounds
    block_v=(fliplr((1:-h:0)))';
    
    if block_v(1)==0         % check if unit interval can be divided in equal blocks of length h
        block_st=block_v(1:end-1);
        block_end=block_v(2:end);
    else                                % first block is shorter !!
     block_st=[0; block_v(1:end-1)];
     block_end=block_v;
    end

    hs=block_end(1)-block_st(1);    %length of shorter block
    
    % block index for short block
    ks=1;

else        % last block
    
    % vector of bounds
    block_v=(0:h:1)';

    if block_v(end)==1         % check if unit interval can be divided in equal blocks of length h
        block_st=block_v(1:end-1);
        block_end=block_v(2:end);
    else                                % last block is shorter !!
     block_st=block_v;
     block_end=[block_v(2:end); 1];
    end
    
    hs=block_end(end)-block_st(end);    %length of shorter block
    
    % block index for short block
    ks=length(block_st);

end

% spectral cutoff 
%-------------------------------------------------

% pilot value
J_max=max(floor(n_rft*h),1);    
       
% frequencies up to spectral cutoff
j=(1:J_max)';

% number of blocks
B=length(block_st);
% -------------------------------------------------------------------------
%       
%% Compute LLE

% vector of empirical norms
Phi_norm=(2*n_rft*sin(j*pi/(2*n_rft*h))).^(-2);                     % (J_max x 1) vector
Phi_norms=(2*n_rft*sin(j*pi/(2*n_rft*hs))).^(-2);                     % for shorter block

% vectors of inverse & squared inv empirical norms of spectral weights 
i_Phi_norm=Phi_norm.^(-1);  % (J x 1) vector
i_Phi_norm_2=i_Phi_norm.^(2);

% create block length vector accouting for shorter block and B=1
if B~=1
    if isequal(s_block,'first')  % first block
        h_v=[hs;repmat(h,B-1,1)];                                      % (Bx1) vector
    else
        h_v=[repmat(h,B-1,1) ;hs];                                      % (Bx1) vector
    end
else   %special case if B=1 
        h_v=h;
        ks=NaN;
end

% rename number of obs
n=n_rft;
    
% number of Scov obs
N=size(SCov_mat,1);

%times at which Spot covolatilities are "observed"
t_scov=1/N:1/N:1;
    
% vector for cutoffs
J_optv=NaN*zeros(B,1);
     
%initialize matrix of spot (co-)volatility on each block
SCov_k_m=NaN*zeros(3,B);
      
% create noise vector (replicated)
noise_v=[nu2_x;nu_xy;nu2_y];
noise_m=noise_v(:,ones(J_max,1));
    
% set of "normal" blocks
normB=setdiff(1:B,ks);   
    

% Loop over normal blocks
%---------------------------------------------------------------------
%---------------------------------------------------------------------
     
for k=normB
    
    % extract spot (co-) volatility estimates
    %--------------------------------------------------------

    if k~=1
       t_scov_ind=find(t_scov<= block_st(k),1,'last');
       sigma2_x=SCov_mat(t_scov_ind,1);
       sigma2_y=SCov_mat(t_scov_ind,2);
       sigma_xy=SCov_mat(t_scov_ind,3);
   else
       sigma2_x=SCov_mat(1,1);
       sigma2_y=SCov_mat(1,2);
       sigma_xy=SCov_mat(1,3);  %first block
   end
    
    % Compute elements of inverse fisher information and put into structure
    %-----------------------------------------------------------------------------------
    
    i_I_jk_11=2*(i_Phi_norm.*nu2_x/n+sigma2_x).^2;     % (J x 1) matrix

    i_I_jk_22=i_Phi_norm_2.*(nu2_x*nu2_y+nu_xy^2)/(n^2)+(sigma2_x.*sigma2_y+(sigma_xy).^2)+...
    (i_Phi_norm/n).*(sigma2_x*nu2_y+sigma2_y*nu2_x+2*sigma_xy*nu_xy);   % (J x 1) matrix

    i_I_jk_33=2*(i_Phi_norm.*nu2_y/n+sigma2_y).^2;     % (J x 1) matrix

    i_I_jk_12=2*sigma2_x.*sigma_xy+i_Phi_norm_2.*2*nu2_x*nu_xy/(n^2)+...
    2*(i_Phi_norm/n).*(sigma2_x*nu_xy+sigma_xy*nu2_x);   % (J x 1) matrix

    i_I_jk_23=2*sigma2_y.*sigma_xy+i_Phi_norm_2.*2*nu2_y*nu_xy/(n^2)+...
    2*(i_Phi_norm/n).*(sigma2_y*nu_xy+sigma_xy*nu2_y);   % (J x 1) matrix

    i_I_jk_13=2*(sigma_xy).^2+i_Phi_norm_2.*2*(nu_xy/n)^2+...
    4*(i_Phi_norm/n).*sigma_xy*nu_xy;   % (J x 1) matrix

    % Compute elements of inverse fisher information and weights
    %----------------------------------------------------------------------
    
    % matrix of distinct elements of inverse Fisher infos for all freqs
    i_I_jk_m=[i_I_jk_11';i_I_jk_12';i_I_jk_13';i_I_jk_22'...
        ;i_I_jk_23';i_I_jk_33'];      % (6 x J) matrix
    
    % create array of (symmetric) inverse fisher info matrices for all freqs
     i_I_jk_a=ivech3(i_I_jk_m);             % (3x3xJ) array
     
     % compute array of fisher info matrices by inversion
     I_jk_a=inv3(i_I_jk_a);
     
     % compute sum of fisher info matrices across freqs on block
     I_k_m=nansum(I_jk_a,3);                    % (3x3) matrix
   
     % compute array of weights
     W_k_a=inv3(I_k_m,I_jk_a);
     
    % find cutoff
    if J_all<=0
        J_opt_k=find_maxJ_const(W_k_a);
    else
       J_opt_k=J_max;
    end
    J_optv(k)=J_opt_k;
    
    % adjust weights 
   if J_opt_k<J_max
     I_jk_a=I_jk_a(:,:,1:J_opt_k);
     I_k_m=nansum(I_jk_a,3);                    % (3x3) matrix
     W_k_a=inv3(I_k_m,I_jk_a);
   end
   
     % Compute Local Blockwise Averages of Log-Returns
    %------------------------------------------------------------------

    %empty matrices for local blockwise averages
    lret1_avb=NaN*zeros(J_opt_k,1);   % (J_opt_kx1) vector
    lret2_avb=lret1_avb;
    
    
    % index of observations on each block
     t_ind=logical(((t >= block_st(k)) & (t <=block_end(k))));  
     
     % observations times on each block (since begining)
     tb=t(t_ind)-block_st(k);
     
     % log returns on block
     lret_b=lret(t_ind,:);
        
        for j=1:J_opt_k
            
            % spectral weights for given block and frequency
            Phi=spectr_weights_j_onbl(tb,n,h,j);
            
            % local blockwise averages using frequency j
            lret1_avb(j)=lret_b(:,1)'*Phi; 
            lret2_avb(j)=lret_b(:,2)'*Phi;
            
        end
     
    % estimate spot covola for given block as weighted average acr freqs
    %----------------------------------------------------------------------
     
    % select empirical norms of spectral weights
    Phi_norm_v=Phi_norm(1:J_opt_k);
     
    % matrix of dist. elements of outer poduct matrices of local ret avgs of for freqs
    S_jk_m=[(lret1_avb.^2)';(lret1_avb.*lret2_avb)';(lret2_avb.^2)'];   % (3 x J_opt_k) matrix
    
    % matrix of dist. elements of bias-corr. spot vola matrix estimator on block for all freqs
    C_jk_m=(S_jk_m-noise_m(:,1:J_opt_k)/n)./(Phi_norm_v(:,ones(3,1)))'; % (3 x J_opt_k) matrix
    
    %  spot vola estimates on block: sum of prod of fisher info and spot vola estimates based on diff freq
    SCov_k_m(:,k)=nansum(mtimesx(W_k_a,permute(C_jk_m,[1 3 2])),3);   % (3xJ_opt_k) vector

end
%-------------------------------------------------------------------------

% short block
%-------------------------------------------------
%-------------------------------------------------

% global optimal cutoff and cutoff of short block
J_opt=nanmax(J_optv);
J_opts=min([J_opt floor(n_rft*hs)]);

if B>1

% vectors of inverse & squared inv empirical norms of spectral weights 
i_Phi_norms=Phi_norms(1:J_opts).^(-1);  % (J_opts x 1) vector
i_Phi_norms_2=i_Phi_norms.^(2);


% extract spot (co-) volatility estimates
    %--------------------------------------------------------

    if ks~=1
       t_scov_ind=find(t_scov<= block_st(ks),1,'last');
       sigma2_x=SCov_mat(t_scov_ind,1);
       sigma2_y=SCov_mat(t_scov_ind,2);
       sigma_xy=SCov_mat(t_scov_ind,3);
   else
       sigma2_x=SCov_mat(1,1);
       sigma2_y=SCov_mat(1,2);
       sigma_xy=SCov_mat(1,3);  %first block
   end
    
    % Compute elements of inverse fisher information and put into structure
    %-----------------------------------------------------------------------------------
    
    i_I_jk_11=2*(i_Phi_norms.*nu2_x/n+sigma2_x).^2;     % (J x 1) matrix

    i_I_jk_22=i_Phi_norms_2.*(nu2_x*nu2_y+nu_xy^2)/(n^2)+(sigma2_x.*sigma2_y+(sigma_xy).^2)+...
    (i_Phi_norms/n).*(sigma2_x*nu2_y+sigma2_y*nu2_x+2*sigma_xy*nu_xy);   % (J x 1) matrix

    i_I_jk_33=2*(i_Phi_norms.*nu2_y/n+sigma2_y).^2;     % (J x 1) matrix

    i_I_jk_12=2*sigma2_x.*sigma_xy+i_Phi_norms_2.*2*nu2_x*nu_xy/(n^2)+...
    2*(i_Phi_norms/n).*(sigma2_x*nu_xy+sigma_xy*nu2_x);   % (J x 1) matrix

    i_I_jk_23=2*sigma2_y.*sigma_xy+i_Phi_norms_2.*2*nu2_y*nu_xy/(n^2)+...
    2*(i_Phi_norms/n).*(sigma2_y*nu_xy+sigma_xy*nu2_y);   % (J x 1) matrix

    i_I_jk_13=2*(sigma_xy).^2+i_Phi_norms_2.*2*(nu_xy/n)^2+...
    4*(i_Phi_norms/n).*sigma_xy*nu_xy;   % (J x 1) matrix

    % Compute elements of inverse fisher information and weights
    %----------------------------------------------------------------------
    
    % matrix of distinct elements of inverse Fisher infos for all freqs
    i_I_jk_m=[i_I_jk_11';i_I_jk_12';i_I_jk_13';i_I_jk_22'...
        ;i_I_jk_23';i_I_jk_33'];      % (6 x J) matrix
    
    % create array of (symmetric) inverse fisher info matrices for all freqs
     i_I_jk_a=ivech3(i_I_jk_m);             % (3x3xJ) array
     
     % compute array of fisher info matrices by inversion
     I_jk_a=inv3(i_I_jk_a);
     
     % compute sum of fisher info matrices across freqs on block
     I_k_m=nansum(I_jk_a,3);                    % (3x3) matrix
   
     % compute array of weights
     W_k_a=inv3(I_k_m,I_jk_a);
     
     
      % adjust weights 
   if J_opt_k<J_max
     I_jk_a=I_jk_a(:,:,1:J_opt_k);
     I_k_m=nansum(I_jk_a,3);                    % (3x3) matrix
     W_k_a=inv3(I_k_m,I_jk_a);
   end
     
    
     % Compute Local Blockwise Averages of Log-Returns
    %------------------------------------------------------------------

    %empty matrices for local blockwise averages
    lret1_avb=NaN*zeros(J_opts,1);   % (J_opts x1) vector
    lret2_avb=lret1_avb;
    
    
    % index of observations on each block
     t_ind=logical(((t >= block_st(ks)) & (t <block_end(ks))));  
     
     % observations times on each block (since begining)
     tb=t(t_ind)-block_st(ks);
     
     % log returns on block
     lret_b=lret(t_ind,:);
        
        for j=1:J_opts
            
            % spectral weights for given block and frequency
            Phi=spectr_weights_j_onbl(tb,n,hs,j);
            
            % local blockwise averages using frequency j
            lret1_avb(j)=lret_b(:,1)'*Phi; 
            lret2_avb(j)=lret_b(:,2)'*Phi;
            
        end
     
    % estimate spot covola for given block as weighted average acr freqs
    %----------------------------------------------------------------------
     
    % select empirical norms of spectral weights
    Phi_norm_v=Phi_norms(1:J_opts);
     
    % matrix of dist. elements of outer poduct matrices of local ret avgs of for freqs
    S_jk_m=[(lret1_avb.^2)';(lret1_avb.*lret2_avb)';(lret2_avb.^2)'];   % (3 x J_opts) matrix
    
    % matrix of dist. elements of bias-corr. spot vola matrix estimator on block for all freqs
    C_jk_m=(S_jk_m-noise_m(:,1:J_opts)/n)./(Phi_norm_v(:,ones(3,1)))'; % (3 x J_opts) matrix
    
    %  spot vola estimates on block: sum of prod of fisher info and spot vola estimates based on diff freq
    SCov_k_m(:,ks)=nansum(mtimesx(W_k_a,permute(C_jk_m,[1 3 2])),3);   % (3xJ_opts) vector

end

%-------------------------------------------------------------------------
%-------------------------------------------------------------------------

% sum up blockwise integr. covola estimates 
lle_cov=SCov_k_m*h_v;   % (3xB)*(Bx1) >>> (3x1) vector

% create (co-volatility) matrix
Sigma_out=[lle_cov(1) lle_cov(2);lle_cov(2) lle_cov(3)];







     











