function [Sigma_out,h_v,h,hs,block_st,block_end,J_optv]=lle_cov_oracle_async(data_a1,data_a2,H,startt,endt,s_block,SCov_mat,h_fac,nkex,cv)
%% lle_cov_oracle_async
%--------------------------------------------------------------
% local likelihood estimation of covolatility matrix (asynchr)
% faster oracle version preselecting observations on each block
% only for cross-sectionally independent noise
%
% Note:   *  comprehensive procedure!
%            * equal block lengths
%            * spectral weights based on cont approximations
%            * block length determined using average of #obs/noise
%--------------------------------------------------------------
% INPUT:
% data_a1 -> (n_a x 2) matrix of synchronous data for asset 1
%       - 1st  col: observation times  (sec after midnight)
%       - 2nd col: prices 
% H -> (2x2) diagonal covariance matrix of noise
% startt -> start of trading (in hours)
% endt   -> end of trading (in hours)
% s_block -> determines which block is shorter: 'first' or 'last' (default)
% SCov_mat -> (N x 3) matrix of spot (co-) variances ([SVar1 SVar1 Scov]),
% where N such that times=1/N:1/N:1.
% Note: each SCov matrix has to be positive definite
% h_fac   -> constant factor for multiplication with "opt" block length
% nkex -> if >0, exact number of obs on each block is used to determine
% cutoff
% cv-> critical value for choice of spectral cutoff (frobenius norm of
% "last" weight matrix <=cv). If numeric: all frequencies are used.
%  Default: cv=1e-6
%--------------------------------------------------------------
% OUTPUT:
% Sigma_out -> (2 x 2) matrix containg estimates of integrated (co-)
% volatilities
% h_v -> (Bx1) vector of block lengths
% h -> block length
% hs -> length of shorter block
% block_st -> (Bx1) vector of block beginnings
% block_end -> (Bx1) vector of block ends
% J_opt -> vector of optimal spectral cutoffs for each block 
% -------------------------------------------------------------------------
%
% Peter Malec, August 8, 2012 
%
% -------------------------------------------------------------------------
%       
%% Prepare Data and Inputs

% Input value of cv
if nargin<10
    cv=1e-6;
end

if isnumeric(cv)
    J_all=0;
else
    J_all=1;
end



% log-returns
lret1=diff(log(data_a1(:,2)));
lret2=diff(log(data_a2(:,2)));

% standardize times (time_rft in [0,1])                                    
time_st1= (data_a1(:,1)-startt*3600)/((endt-startt)*3600);         
time_st2= (data_a2(:,1)-startt*3600)/((endt-startt)*3600); 

% times, centered times and differences
%-------------------------------------------------------------------------
t1=time_st1(2:end);
t2=time_st2(2:end);

avgt1=0.5*(time_st1(2:end)+time_st1(1:end-1));
avgt2=0.5*(time_st2(2:end)+time_st2(1:end-1));

dt1=time_st1(2:end)-time_st1(1:end-1);
dt2=time_st2(2:end)-time_st2(1:end-1);

% extract noise (co-)variances
nu2_x=H(1,1);
nu2_y=H(2,2);

% number of obs
nx=length(t1);
ny=length(t2);
nrft=max([nx ny]);
% -------------------------------------------------------------------------
%       
%% Determine block length 

% block length
%-------------------------------------------------

% sample size relative to noise
n_nu=mean([nx; ny]./diag(H)); %use average 


% block length (rough choice)
h=min([(h_fac*log(n_nu)/sqrt(n_nu)) 1]);
% -------------------------------------------------------------------------
%       
%%  Determine block bounds and spectral cutoff (pilot)

% start and end of each block (# of elements gives # blocks)-
%-----------------------------------------------------------------------

% which block ist shorter?

if isequal(s_block,'first')  % first block
    
    % vector of bounds
    block_v=(fliplr((1:-h:0)))';
    
    if block_v(1)==0         % check if unit interval can be divided in equal blocks of length h
        block_st=block_v(1:end-1);
        block_end=block_v(2:end);
    else                                % first block is shorter !!
     block_st=[0; block_v(1:end-1)];
     block_end=block_v;
    end

    hs=block_end(1)-block_st(1);    %length of shorter block
    
    % block index for short block
    ks=1;

else        % last block
    
    % vector of bounds
    block_v=(0:h:1)';

    if block_v(end)==1         % check if unit interval can be divided in equal blocks of length h
        block_st=block_v(1:end-1);
        block_end=block_v(2:end);
    else                                % last block is shorter !!
     block_st=block_v;
     block_end=[block_v(2:end); 1];
    end
    
    hs=block_end(end)-block_st(end);    %length of shorter block
    
    % block index for short block
    ks=length(block_st);

end

% number of blocks
B=length(block_st);
% -------------------------------------------------------------------------
%       
%% Compute LLE

% create block length vector accouting for shorter block and B=1
if B~=1
    if isequal(s_block,'first')  % first block
        h_v=[hs;repmat(h,B-1,1)];                                      % (Bx1) vector
    else
        h_v=[repmat(h,B-1,1) ;hs];                                      % (Bx1) vector
    end
else   %special case if B=1 
        h_v=h;
        ks=NaN;
end

% number of Scov obs
N=size(SCov_mat,1);

%times at which Spot covolatilities are "observed"
t_scov=1/N:1/N:1;
    
% vector for cutoffs
J_optv=NaN*zeros(B,1);
     
%initialize matrix of spot (co-)volatility on each block
SCov_k_m=NaN*zeros(3,B);
      
% set of "normal" blocks
normB=setdiff(1:B,ks);   

% create noise vector (replicated)
noise_v=[nu2_x;0;nu2_y];


% Loop over normal blocks
%---------------------------------------------------------------------
%---------------------------------------------------------------------
     
for k=normB
    
    % extract spot (co-) volatility estimates
    %--------------------------------------------------------

    if k~=1
       t_scov_ind=find(t_scov<= block_st(k),1,'last');
       sigma2_x=SCov_mat(t_scov_ind,1);
       sigma2_y=SCov_mat(t_scov_ind,2);
       sigma_xy=SCov_mat(t_scov_ind,3);
   else
       sigma2_x=SCov_mat(1,1);
       sigma2_y=SCov_mat(1,2);
       sigma_xy=SCov_mat(1,3);  %first block
    end
   
    % Compute number of obs etc on block
    %----------------------------------------------------------------------
    
    % index of observations on each block (based on centers)
     t_ind1=logical(((avgt1 >= block_st(k)) & (avgt1 <block_end(k))));  
     t_ind2=logical(((avgt2 >= block_st(k)) & (avgt2 <block_end(k))));  
     
    % centered observations times on block (since begining)
     avgtb1=avgt1(t_ind1)-block_st(k);
     avgtb2=avgt2(t_ind2)-block_st(k);
     
     % quadratic variation of time on block 
     qvtbx=sum(dt1(t_ind1).^2);
     qvtby=sum(dt2(t_ind2).^2);
     
     % log returns on block
     lret_b1=lret1(t_ind1,:);
     lret_b2=lret2(t_ind2,:);
     
     % Compute weight matrices
    %----------------------------------------------------------------------
     
    % pilot value of cutoff
    if nkex>0
        n_max=max([length(lret_b1) length(lret_b2)]);
    else
        n_max=floor(nrft*h);
    end
    J_max=max(n_max,1);
    
    % array of weight matrices for each freq j
    [W_k_a,J_opt_k,i_Phi_norm]=lle_cov_weight_array(J_max,cv,h,nu2_x,qvtbx,sigma2_x,nu2_y,qvtby,sigma2_y,sigma_xy,J_all);
     
    %assign optimal cutoff to vector
     J_optv(k)=J_opt_k;
     
     % Compute Local Blockwise Averages of Log-Returns
    %------------------------------------------------------------------

    %empty matrices for local blockwise averages
    lret1_avb=NaN*zeros(J_opt_k,1);   % (J_opt_kx1) vector
    lret2_avb=lret1_avb;

    
        % loop
        for j=1:J_opt_k
            
            % spectral weights for given block and frequency
            Phi1=spectr_weights_j_onbl_cont(avgtb1,h,j);
            Phi2=spectr_weights_j_onbl_cont(avgtb2,h,j);
            
            % local blockwise averages using frequency j
            lret1_avb(j)=lret_b1'*Phi1; 
            lret2_avb(j)=lret_b2'*Phi2;
            
        end
     
    % estimate spot covola for given block as weighted average acr freqs
    %----------------------------------------------------------------------
    
    % create local noise vector (replicated)
    lnoise_v=noise_v.*[qvtbx;0;qvtby]/h;  % (3 x 1) vector
     
    % select empirical norms of spectral weights
    i_Phi_norm_v=(i_Phi_norm(1:J_opt_k))';                 
    
    % matrix of dist. elements of outer poduct matrices of local ret avgs of for freqs
    S_jk_m=[(lret1_avb.^2)';(lret1_avb.*lret2_avb)';(lret2_avb.^2)'];   % (3 x J_opt_k) matrix
    
    % matrix of dist. elements of bias-corr. spot vola matrix estimator on 
    C_jk_m=(S_jk_m-lnoise_v(:,ones(J_opt_k,1))).*i_Phi_norm_v(ones(3,1),:); % (3 x J_opt_k) matrix
   
    
    %  spot vola estimates on block: sum of prod of fisher info and spot vola estimates based on diff freq
    SCov_k_m(:,k)=nansum(mtimesx(W_k_a,permute(C_jk_m,[1 3 2])),3);   % (3xJ_opt_k) vector

end

%-------------------------------------------------------------------------

% short block   (continue here !!!!)
%-------------------------------------------------
%-------------------------------------------------

if B>1

% extract spot (co-) volatility estimates
    %--------------------------------------------------------

    if ks~=1
       t_scov_ind=find(t_scov<= block_st(ks),1,'last');
       sigma2_x=SCov_mat(t_scov_ind,1);
       sigma2_y=SCov_mat(t_scov_ind,2);
       sigma_xy=SCov_mat(t_scov_ind,3);
   else
       sigma2_x=SCov_mat(1,1);
       sigma2_y=SCov_mat(1,2);
       sigma_xy=SCov_mat(1,3);  %first block
    end
    
    % Compute number of obs on block
    %----------------------------------------------------------------------
    
   % index of observations on each block (based on centers)
     t_ind1=logical(((avgt1 >= block_st(ks)) & (avgt1 <block_end(ks))));  
     t_ind2=logical(((avgt2 >= block_st(ks)) & (avgt2 <block_end(ks))));  
     
   % centered observations times on block (since begining)
     avgtb1=avgt1(t_ind1)-block_st(ks);
     avgtb2=avgt2(t_ind2)-block_st(ks);
     
     % quadratic variation of time on block 
     qvtbx=sum(dt1(t_ind1).^2);
     qvtby=sum(dt2(t_ind2).^2);
     
     % log returns on block
     lret_b1=lret1(t_ind1,:);
     lret_b2=lret2(t_ind2,:);
     
     % Compute weight matrices
    %----------------------------------------------------------------------
   
    % pilot value of cutoff
    if nkex>0
        n_max=max([length(lret_b1) length(lret_b2)]);
    else
        n_max=floor(nrft*hs);
    end
    J_maxs=max(n_max,1);
    
     % array of weight matrices for each freq j
    [W_k_a,J_opt_s,i_Phi_norm]=lle_cov_weight_array(J_maxs,cv,hs,nu2_x,qvtbx,sigma2_x,nu2_y,qvtby,sigma2_y,sigma_xy,J_all);
     
     %assign optimal cutoff to vector
     J_optv(ks)=J_opt_s;
    
     % Compute Local Blockwise Averages of Log-Returns
    %------------------------------------------------------------------

   %empty matrices for local blockwise averages
    lret1_avb=NaN*zeros(J_opt_s,1);   % (J_opt_sx1) vector
    lret2_avb=lret1_avb;

    
        % loop
        for j=1:J_opt_s
            
            % spectral weights for given block and frequency
            Phi1=spectr_weights_j_onbl_cont(avgtb1,hs,j);
            Phi2=spectr_weights_j_onbl_cont(avgtb2,hs,j);
            
            % local blockwise averages using frequency j
            lret1_avb(j)=lret_b1'*Phi1; 
            lret2_avb(j)=lret_b2'*Phi2;
            
        end
   
    % estimate spot covola for given block as weighted average acr freqs
    %----------------------------------------------------------------------
     
   % create local noise vector (replicated)
    lnoise_v=noise_v.*[qvtbx;0;qvtby]/hs;  % (3 x 1) vector
     
    % select empirical norms of spectral weights
    i_Phi_norm_v=(i_Phi_norm(1:J_opt_s))';                 
    
    % matrix of dist. elements of outer poduct matrices of local ret avgs of for freqs
    S_jk_m=[(lret1_avb.^2)';(lret1_avb.*lret2_avb)';(lret2_avb.^2)'];   % (3 x J_opt_s) matrix
    
    % matrix of dist. elements of bias-corr. spot vola matrix estimator on 
    C_jk_m=(S_jk_m-lnoise_v(:,ones(J_opt_s,1))).*i_Phi_norm_v(ones(3,1),:); % (3 x J_opt_s) matrix
   
    
    %  spot vola estimates on block: sum of prod of fisher info and spot vola estimates based on diff freq
    SCov_k_m(:,ks)=nansum(mtimesx(W_k_a,permute(C_jk_m,[1 3 2])),3);   % (3xJ_opt_s) vector
end
    
%-------------------------------------------------------------------------
%-------------------------------------------------------------------------

% sum up blockwise integr. covola estimates 
lle_cov=SCov_k_m*h_v;   % (3xB)*(Bx1) >>> (3x1) vector

% create (co-volatility) matrix
Sigma_out=[lle_cov(1) lle_cov(2);lle_cov(2) lle_cov(3)];







     











