function [SCov_mat,h,hs,block_st,block_end,J_optv,b,Covmat_arr]=lle_spot_cov_beta_corrd(input,data_a1,data_a2,H,startt,endt,s_block,...
    h_fac,nkex,phi_ctr,J_fac,J_pilot,Kpval,Kval,b,modi,twst,twstc,cv)
%% lle_spot_cov_beta_corrd
%--------------------------------------------------------------
% local likelihood estimation of spot covolatility matrices on specified
% number of blocks based on beta kernel smoothing of final estimates
% only for cross-sectionally independent noise
%
% Note:    
%           * spot variances are x10000
%            * pilot estimation of spot covs is performed using the "old"
%            approach
%            * norms of spectral weights corrected for discr sampling
%               (only in bias correction -> save comp time)
%            * equal block lengths
%            * spectral weights based on cont approximations
%            * block length determined using average of #obs/noise
%--------------------------------------------------------------
% INPUT:
% input    -> grid of estimation points for spot covariances
% data_a1 -> (n_a x 2) matrix of synchronous data for asset 1
%       - 1st  col: observation times  (sec after midnight)
%       - 2nd col: prices 
% H -> (2x2) diagonal covariance matrix of noise
% startt -> start of trading (in hours)
% endt   -> end of trading (in hours)
% s_block -> determines which block is shorter: 'first' or 'last' (default)
%                 If numeric: Number of blocks fixed
% h_fac   -> constant factor for multiplication with "opt" block length
% nkex -> if >0, exact number of obs on each block is used to determine
%               cutoff
% phi_ctr -> if >0, center of return spells used to compute spectral
%               weights
% J_fac -> factor by which "optimal" cutoff (based on frob norm of weights)
%               is divided
% J_pilot -> spectral cutoff for pilot estimates of spot covariances
% Kpval    -> bandwidth for smoothing of pilot estimates
% Kval    -> bandwidth for smoothing of spot cov estimates
% b          -> smoothing bandwidth for beta kernel
% modi -> if >0, modified beta kernel is used, else
%             standard kernel
% twst     -> if >0: twostep estimation is used (estimates from first step
%                   used as pilots for second step
% twstc      -> if >0: Covariance matrices of smoothed spot (co-)volatilities
%   under the local parametric experiment   are computed using 2ndstep spot
%   covariance estimates (mattters only if 8 output arguments)
% cv-> critical value for choice of spectral cutoff (frobenius norm of
%           "last" weights <=cv). If not numeric: all frequencies are used. If integer or vector:
%         prespecified vector of spectral cutoffs for each block
%           Default: cv=1e-6
%--------------------------------------------------------------
% OUTPUT:
% SCov_mat-> (Bx3) matrix of smoothed spot (co-)volatility estimates on
% blocks (kth row: [sigma_(x,k)^2 sigma_(xy,k) sigma_(y,k)^2])
% h -> block length
% hs -> length of shorter block
% block_st -> (Bx1) vector of block beginnings
% block_end -> (Bx1) vector of block ends
% J_opt -> vector of optimal spectral cutoffs for each block 
% Covmat_arr -> (3x3xB) array: Covariance matrices of smoothed spot (co-)volatilities
%   under the local parametric experiment      (optional)
% -------------------------------------------------------------------------
%
% Peter Malec, October 6, 2012 
%
% -------------------------------------------------------------------------
%       
%% Prepare Data and Inputs

% inputs for pilot estimator
J_pilot=max(1,round(J_pilot));
Kval=max(1,round(Kval));

% fixed # blocks
if isnumeric(s_block)
        B_f=max(1,round(s_block));
else
     B_f=[];
end

% Input value of cv
%-------------------------------
if nargin<19
    cv=1e-6;
end

if isnumeric(cv)
    J_all=0;
    if ((length(cv)==1)&&(cv>=1))||length(cv)>1
        J_optv_in=cv;% vector of optimal cutoffs (B prespecified)
        if (~isempty(B_f))&&(length(cv)~=B_f)
            error('Dimension of vector of cutoffs has to be equal to no. of blocks!!!')
        end
    else
        J_optv_in=0;
        if twst>0
            disp('Two-step approach works only for pre-specified cutoffs: switch to one-step')
            twst=0;     %NOTE: could be relaxed>>> no change of cutoffs after est step one!
        end
    end
else
    J_all=1;
     if twst>0
            disp('Two-step approach works only for pre-specified cutoffs: switch to one-step')
            twst=0;
     end
end

% bandwidth choice for final smoothing
if ~isnumeric(b)
    lscv=1;
else
    lscv=0;
end

% log-returns
lret1=diff(log(data_a1(:,2)));
lret2=diff(log(data_a2(:,2)));

% standardize times (time_rft in [0,1])                                    
time_st1= (data_a1(:,1)-startt*3600)/((endt-startt)*3600);         
time_st2= (data_a2(:,1)-startt*3600)/((endt-startt)*3600); 

% times, lagged times, centered times 
%-------------------------------------------------------------------------
t1=time_st1(2:end);
t2=time_st2(2:end);

tl1=time_st1(1:end-1);
tl2=time_st2(1:end-1);

if phi_ctr>0
    avgt1=0.5*(t1+tl1);
    avgt2=0.5*(t2+tl2);
else
     avgt1=t1;
     avgt2=t2;
end

% extract noise (co-)variances
nu2_x=H(1,1);
nu2_y=H(2,2);

% number of obs
nx=length(t1);
ny=length(t2);
nrft=min([nx ny]);
% -------------------------------------------------------------------------
%       
%% Determine block length and bounds

% check if no blocks prespecified
if isempty(B_f)

    % sample size relative to noise
    n_nu=mean([nx; ny]./diag(H)); %use average 

    % block length (rough choice)
    h=min([(h_fac*log(n_nu)/sqrt(n_nu)) 1]);

     % start and end of each block
    [block_st,block_end,hs,ks]=block_start_end(h,s_block);

    % number of blocks
    B=length(block_st);

    % account for B=1
     if B==1
        ks=NaN;
     end
    
else  % fixed no blocks
    B=B_f;
    h=1/B;
    hs=h;
    block_st=0:(1/B):(1-1/B);
    block_end=(1/B):(1/B):1;
    ks=NaN;
end
% -------------------------------------------------------------------------
%       
%% Compute Pilot Estimates of Spot (Co-variances) on blocks

% create noise vector (replicated)
noise_v=[nu2_x;0;nu2_y];

% set of "normal" blocks
normB=setdiff(1:B,ks);   

% pilot estimates
[SCov_mat,lret1_avb_m,lret2_avb_m,Phi_norm_k_x_m,Phi_norm_k_xy_m,Phi_norm_k_y_m]...
    =spot_covs_pilot(normB,ks,block_st,block_end,h,hs,avgt1,avgt2,t1,t2,tl1,tl2,lret1,lret2,noise_v,Kpval,J_pilot);

% -------------------------------------------------------------------------
%       
%% Compute LLE (step one)

% vector for cutoffs
if J_optv_in==0
    J_optv=NaN*zeros(B,1);
else
    J_optv=J_optv_in;
end

%initialize matrix of spot (co-)volatility on each block
SCov_k_m=NaN*zeros(3,B);


% initialize collection vector for qvt 
if (twst>0)||((nargout>7)&&(twstc>0))
    qvtbx_v=NaN*zeros(B,1);
    qvtby_v=NaN*zeros(B,1);
end

% initialize collection matrices for local spectral means
% initialize matrices for empirical norms of phis
if twst>0
    lnoise_v_m=NaN*zeros(3,B);
    J_optv_in_max=max(J_optv_in);
    lret1_avb_mn=NaN*zeros(B,J_optv_in_max);
    lret2_avb_mn=NaN*zeros(B,J_optv_in_max);
    Phi_norm_k_x_mn=NaN*zeros(B,J_optv_in_max);
    Phi_norm_k_xy_mn=NaN*zeros(B,J_optv_in_max);
    Phi_norm_k_y_mn=NaN*zeros(B,J_optv_in_max);
end


% array for inverse of sum of fisher infos
i_I_k_m_arr=NaN*zeros(3,3,B);

% Loop over normal blocks
%---------------------------------------------------------------------
for k=normB
    
    % compute spot covolatility estimate on block k
     [SCov_k_v,J_opt_k,I_k_m,Phi_norm_k_m,lret1_avb,lret2_avb,qvtbx,qvtby,lnoise_v,~]=spot_cov_block_adapt_corrd(block_st(k),block_end(k),h,nrft,avgt1,avgt2,t1,t2,tl1,tl2,lret1,lret2,...
    lret1_avb_m(k,:),lret2_avb_m(k,:),Phi_norm_k_x_m(k,:),Phi_norm_k_xy_m(k,:),Phi_norm_k_y_m(k,:),SCov_mat(k,:),...
    noise_v,nkex,cv,J_fac,J_optv_in(k),J_all);
   
    % assign estimates to matrix
    SCov_k_m(:,k)=SCov_k_v;
    
    %assign inputs for 2nd step
    if twst>0
        qvtbx_v(k)=qvtbx;
        qvtby_v(k)=qvtby;
        lnoise_v_m(:,k)=lnoise_v;
        lret1_avb_mn(k,1:length(lret1_avb))=lret1_avb';
        lret2_avb_mn(k,1:length(lret2_avb))=lret2_avb';
        Phi_norm_k_x_mn(k,1:length(lret1_avb))=Phi_norm_k_m(1,:);
        Phi_norm_k_xy_mn(k,1:length(lret1_avb))=Phi_norm_k_m(2,:);
        Phi_norm_k_y_mn(k,1:length(lret1_avb))=Phi_norm_k_m(3,:);
    else
        % assign inverse of sum of fisher infos
        if (nargout>7)&&(twstc<=0)
            i_I_k_m_arr(:,:,k)=inv(I_k_m);
        elseif (nargout>7)&&(twstc>0)
            qvtbx_v(k)=qvtbx;
            qvtby_v(k)=qvtby;
        end
    end
    
     %assign optimal cutoff to vector
     J_optv(k)=J_opt_k;

end

% short block   
%-------------------------------------------------
if ~isnan(ks)

 % compute spot covolatility estimate on block k
    [SCov_k_v,J_opt_k,I_k_m,Phi_norm_k_m,lret1_avb,lret2_avb,qvtbx,qvtby,lnoise_v,~]=spot_cov_block_adapt_corrd(block_st(ks),block_end(ks),hs,nrft,avgt1,avgt2,t1,t2,tl1,tl2,lret1,lret2,...
    lret1_avb_m(ks,:),lret2_avb_m(ks,:),Phi_norm_k_x_m(ks,:),Phi_norm_k_xy_m(ks,:),Phi_norm_k_y_m(ks,:),SCov_mat(ks,:),...
    noise_v,nkex,cv,J_fac,J_optv_in(ks),J_all);
    
    % assign estimates to matrix
    SCov_k_m(:,ks)=SCov_k_v;
    
    %assign inputs for 2nd step
    if twst>0
        qvtbx_v(ks)=qvtbx;
        qvtby_v(ks)=qvtby;
        lnoise_v_m(:,ks)=lnoise_v;
        lret1_avb_mn(ks,1:length(lret1_avb))=lret1_avb';
        lret2_avb_mn(ks,1:length(lret2_avb))=lret2_avb';
        Phi_norm_k_x_mn(ks,1:length(lret1_avb))=Phi_norm_k_m(1,:);
        Phi_norm_k_xy_mn(ks,1:length(lret1_avb))=Phi_norm_k_m(2,:);
        Phi_norm_k_y_mn(ks,1:length(lret1_avb))=Phi_norm_k_m(3,:);
    else
        % assign inverse of sum of fisher infos
        if (nargout>7)&&(twstc<=0)
            i_I_k_m_arr(:,:,ks)=inv(I_k_m);
        elseif (nargout>7)&&(twstc>0)
            qvtbx_v(ks)=qvtbx;
            qvtby_v(ks)=qvtby;
        end
    end
    
     %assign optimal cutoff to vector
     J_optv(ks)=J_opt_k;
     
end
% -------------------------------------------------------------------------
%       
%% Compute LLE (step two)

if twst>0   % was it requested?
    
% obtain pilot spot cov estimates by smoothing
SCov_mat=spot_vola_smooth(SCov_k_m',Kval);

%initialize matrix of spot (co-)volatility on each block
SCov_k_m=NaN*zeros(3,B);

% Loop over normal blocks
%---------------------------------------------------------------------
for k=normB
    
    % compute spot covolatility estimate on block k
    [SCov_k_v,I_k_m]=spot_cov_block_adapt_rweights_corrd(h,SCov_mat(k,:),...
    noise_v,cv,J_fac,J_optv(k),J_all,[Phi_norm_k_x_mn(k,1:J_optv(k));Phi_norm_k_xy_mn(k,1:J_optv(k));Phi_norm_k_y_mn(k,1:J_optv(k))],...
    lret1_avb_mn(k,1:J_optv(k)),lret2_avb_mn(k,1:J_optv(k)),qvtbx_v(k),qvtby_v(k),lnoise_v_m(:,k),J_optv(k));
    
    % assign estimates to matrix
    SCov_k_m(:,k)=SCov_k_v;
    
    % assign inverse of sum of fisher infos
      if (nargout>7)&&(twstc<=0)
           i_I_k_m_arr(:,:,k)=inv(I_k_m);
      end

end

% short block   
%-------------------------------------------------
if ~isnan(ks)

    % compute spot covolatility estimate on block k
    [SCov_k_v,I_k_m]=spot_cov_block_adapt_rweights_corrd(hs,SCov_mat(ks,:),...
    noise_v,cv,J_fac,J_optv(ks),J_all,[Phi_norm_k_x_mn(ks,1:J_optv(ks));Phi_norm_k_xy_mn(ks,1:J_optv(ks));Phi_norm_k_y_mn(ks,1:J_optv(ks))],...
    lret1_avb_mn(ks,1:J_optv(ks)),lret2_avb_mn(ks,1:J_optv(ks)),qvtbx_v(ks),qvtby_v(ks),lnoise_v_m(:,ks),J_optv(ks));
    
    % assign estimates to matrix
    SCov_k_m(:,ks)=SCov_k_v;

     % assign inverse of sum of fisher infos
     if (nargout>7)&&(twstc<=0)
           i_I_k_m_arr(:,:,ks)=inv(I_k_m);
     end
     
end

end
% -------------------------------------------------------------------------
%       
%% smooth and compute covariance matrices of estimates under local param exp

% choose bandwidth for beta kernel by lscv
if  lscv>0
     b=spot_vola_smooth_beta_lscvb(block_st,(SCov_k_m*10000)',b,modi);
end

if nargout>7
    
    % use final estimates for info matrices(else pilot)
    %------------------------------------------------------------------
    
    if twstc>0
        
        % smoothed pilot estimates of spot covs
        SCov_mat=spot_vola_smooth(SCov_k_m',Kval);
        
        % normal blocks
        for k=normB
            % compute info mat on block k
            I_k_m=info_mat_block_adapt(h,SCov_mat(k,:),noise_v,qvtbx_v(k),qvtby_v(k),J_optv(k)); 
            i_I_k_m_arr(:,:,k)=inv(I_k_m);
        end
        % short block   
        if ~isnan(ks)
            % compute info mat on block k
            I_k_m=info_mat_block_adapt(hs,SCov_mat(ks,:),noise_v,qvtbx_v(ks),qvtby_v(ks),J_optv(ks)); 
            i_I_k_m_arr(:,:,ks)=inv(I_k_m);
        end
        
    end
    
    % spot cov estimates and corresponding cov matrices
        [SCov_mat,Covmat_arr]=spot_vola_smooth_beta(input,b,block_st,(SCov_k_m*10000)',modi,(i_I_k_m_arr*10000^2));
    
 else
    
    [SCov_mat,Covmat_arr]=spot_vola_smooth_beta(input,b,block_st,(SCov_k_m*10000)',modi);
    
end

 
    








     











