function [SCov_mat,block_st,block_end,J_optv,J_p,K,K_p,Covmat_arr]=lle_spot_cov_corrd_simp(input_v,data_a1,data_a2,H,startt,endt,...
    nkex,phi_ctr,B,J,J_p,K,K_p,theta_h,theta_J,theta_K,K_thr,K_p_thr,twst,twstc,lKb,ccorr)
%% lle_spot_cov_corrd_simp
%--------------------------------------------------------------
% local likelihood estimation of spot covolatility matrices on specified
% number of blocks
% only for cross-sectionally independent noise
%
% Note:    
%             * blocks with equal lengths
%              *inputs either fixed or based on continuous approximations
%             * norms of spectral weights corrected for discr sampling
%               (only in bias correction -> save comp time)
%            * spectral weights based on cont approximations
%--------------------------------------------------------------
% INPUT:
% input_v -> (n_inx1) vector of estimation points
% data_a1 -> (n_a x 2) matrix of synchronous data for asset 1
%       - 1st  col: observation times  (sec after midnight)
%       - 2nd col: prices 
% H -> (2x2) diagonal covariance matrix of noise
% startt -> start of trading (in hours)
% endt   -> end of trading (in hours)
% nkex -> if >0, exact number of obs on each block is used to determine
%               cutoff
% phi_ctr -> if >0, center of return spells used to compute spectral
%               weights
% B      -> no of equally-sized blocks
% J  -> spectral cutoff for final estimates of spot covariances
% J_p-> spectral cutoff for pilot estimates of spot covariances
% K   -> bandwidth for smoothing of final spot cov estimates
% K_p  -> bandwidth for smoothing of pilot estimates
% theta_h   -> proportionality parameter for block length (relevant if B
%                           not specified)
% theta_J     -> proportionaliy parameter for cutoff (relevant if J
%                           or J_p not specified)
% theta_k    -> proportionality parameter for smoothing bandwidth (relevant if K
%                           or K_p not specified)
% K_thr    -> lower threshold for pilot smoothing bandwidth (relevant if
%                           K not specified): default=1
% K_p_thr    -> lower threshold for pilot smoothing bandwidth (relevant if
%                           K_p not specified): default=1
% twst     -> if >0: twostep estimation is used (estimates from first step
%                   used as pilots for second step
% twstc      -> if >0: Covariance matrices of smoothed spot (co-)volatilities
%   under the local parametric experiment   are computed using 2ndstep spot
%   covariance estimates (mattters only if 8 output arguments)
% lkB         -> if >0, shorter smoothing windows at beginning and end of
%                   day are used for spot cov estim (NOT PILOT!)
% ccorr -> if >0, correlation and variance estimates are corrected
%--------------------------------------------------------------
% OUTPUT:
% SCov_mat-> (ninpx3) matrix of smoothed spot (co-)volatility estimates at
%                       estimation points (kth row: [sigma_(x,k)^2 sigma_(xy,k) sigma_(y,k)^2])
% block_st -> (Bx1) vector of block beginnings
% block_end -> (Bx1) vector of block ends
% J_opt -> vector of optimal spectral cutoffs for each block 
% Covmat_arr -> (3x3xninp) array: Covariance matrices of smoothed spot (co-)volatilities
%                       under the local parametric experiment      (optional)
% -------------------------------------------------------------------------
%
% Peter Malec, May 8, 2013 
%
% -------------------------------------------------------------------------
%       
%% Input check

if isempty(B)
    compB=1;
    if isempty(theta_h)
        error('Specifiy no of blocks or proportionality parameter for block length!')
    end
else
    compB=0;
end

if isempty(J)
    compJ=1;
    if isempty(theta_J)
        error('Specifiy spectral cutoff or corresponding proportionality parameter!')
    end
else
    compJ=0;
end

if isempty(J_p)
    compJ_p=1;
    if isempty(theta_J)
        error('Specifiy pilot spectral cutoff or corresponding proportionality parameter!')
    end
else
    compJ_p=0;
end

if isempty(K)
    compK=1;
    if isempty(theta_K)
        error('Specifiy smoothing bandwidth or corresponding proportionality parameter!')
    end
    if isempty(K_thr)
        K_thr=1;
    end
else
    compK=0;
end

if isempty(K_p)
    compK_p=1;
    if isempty(theta_K)
        error('Specifiy pilot smoothing bandwidth or corresponding proportionality parameter!')
    end
    if isempty(K_p_thr)
        K_p_thr=1;
    end
else
    compK_p=0;
end
% -------------------------------------------------------------------------
%       
%% Prepare Data 

% log-returns
lret1=diff(log(data_a1(:,2)));
lret2=diff(log(data_a2(:,2)));

% standardize times (time_rft in [0,1])                                    
time_st1= (data_a1(:,1)-startt*3600)/((endt-startt)*3600);         
time_st2= (data_a2(:,1)-startt*3600)/((endt-startt)*3600); 

% times, lagged times, centered times 
%-------------------------------------------------------------------------
t1=time_st1(2:end);
t2=time_st2(2:end);

tl1=time_st1(1:end-1);
tl2=time_st2(1:end-1);

if phi_ctr>0
    avgt1=0.5*(t1+tl1);
    avgt2=0.5*(t2+tl2);
else
     avgt1=t1;
     avgt2=t2;
end

% extract noise (co-)variances
nu2_x=H(1,1);
nu2_y=H(2,2);

% number of obs
nx=length(t1);
ny=length(t2);
n_min=min([nx ny]);
% -------------------------------------------------------------------------
%       
%% Determine block length and bounds

% number of Blocks, block length, start & end
if compB
    B=ceil((sqrt(n_min)/log(n_min))/theta_h);
end
B=max([B 1]);
h=1/B;
block_st=0:(1/B):(1-1/B);
block_end=(1/B):(1/B):1;

% cutoff
if compJ
    J=floor((theta_J*log(n_min)));
end
J=max([J 1]);

% pilot cutoff
if compJ_p
    J_p=floor((theta_J*log(n_min)));
end
J_p=max([J_p 1]);

% bandwidth
if compK
    K=ceil(theta_K*(n_min^(1/4)));
end
K=min([max([K K_thr]) B]);

% pilot bandwidth
if compK_p
    K_p=ceil(theta_K*(n_min^(1/4)));
end
K_p=min([max([K_p K_p_thr]) B]);

% -------------------------------------------------------------------------
%       
%% Compute Pilot Estimates of Spot (Co-variances) on blocks

% create noise vector 
noise_v=[nu2_x;0;nu2_y];

% pilot estimates
[SCov_mat,lret1_avb_m,lret2_avb_m,Phi_norm_k_x_m,Phi_norm_k_xy_m,Phi_norm_k_y_m]...
    =spot_covs_pilot_simp(block_st,block_end,h,avgt1,avgt2,t1,t2,tl1,tl2,lret1,lret2,noise_v,K_p,J_p,ccorr);
% -------------------------------------------------------------------------
%       
%% Compute LLE (step one)

% vector for cutoffs
J_optv=NaN*zeros(B,1);


%initialize matrix of spot (co-)volatility on each block
SCov_k_m=NaN*zeros(3,B);


% initialize collection vector for qvt 
if (twst>0)||((nargout>6)&&(twstc>0))
    qvtbx_v=NaN*zeros(B,1);
    qvtby_v=NaN*zeros(B,1);
end

% initialize collection matrices for local spectral means
% initialize matrices for empirical norms of phis
if twst>0
    lnoise_v_m=NaN*zeros(3,B);
    lret1_avb_mn=NaN*zeros(B,J);
    lret2_avb_mn=NaN*zeros(B,J);
    Phi_norm_k_x_mn=NaN*zeros(B,J);
    Phi_norm_k_xy_mn=NaN*zeros(B,J);
    Phi_norm_k_y_mn=NaN*zeros(B,J);
end


% array for inverse of sum of fisher infos
i_I_k_m_arr=NaN*zeros(3,3,B);

% Loop over blocks
%---------------------------------------------------------------------
for k=1:B
    
    % compute spot covolatility estimate on block k
     [SCov_k_v,J_opt_k,I_k_m,Phi_norm_k_m,lret1_avb,lret2_avb,qvtbx,qvtby,lnoise_v]=spot_cov_block_adapt_corrd_simp(block_st(k),block_end(k),h,n_min,avgt1,avgt2,t1,t2,tl1,tl2,lret1,lret2,...
    lret1_avb_m(k,:),lret2_avb_m(k,:),Phi_norm_k_x_m(k,:),Phi_norm_k_xy_m(k,:),Phi_norm_k_y_m(k,:),SCov_mat(k,:),...
    noise_v,nkex,J,ccorr);
   
    % assign estimates to matrix
    SCov_k_m(:,k)=SCov_k_v;
    
    %assign inputs for 2nd step
    if twst>0
        qvtbx_v(k)=qvtbx;
        qvtby_v(k)=qvtby;
        lnoise_v_m(:,k)=lnoise_v;
        lret1_avb_mn(k,1:length(lret1_avb))=lret1_avb';
        lret2_avb_mn(k,1:length(lret2_avb))=lret2_avb';
        Phi_norm_k_x_mn(k,1:length(lret1_avb))=Phi_norm_k_m(1,:);
        Phi_norm_k_xy_mn(k,1:length(lret1_avb))=Phi_norm_k_m(2,:);
        Phi_norm_k_y_mn(k,1:length(lret1_avb))=Phi_norm_k_m(3,:);
    else
        % assign inverse of sum of fisher infos
        if (nargout>6)&&(twstc<=0)
            if (sum(sum(isnan(I_k_m)))==0)
                i_I_k_m_arr(:,:,k)=inv(I_k_m);
            else
                i_I_k_m_arr(:,:,k)=zeros(3,3);
            end
        elseif (nargout>6)&&(twstc>0)
            qvtbx_v(k)=qvtbx;
            qvtby_v(k)=qvtby;
        end
    end
    
     %assign optimal cutoff to vector
     J_optv(k)=J_opt_k;

end
% -------------------------------------------------------------------------
%       
%% Compute LLE (step two)

if twst>0   % was it requested?
    
% obtain pilot spot cov estimates by smoothing
if K_p>1
    SCov_mat=spot_vola_smooth(SCov_k_m',K_p);
else
    SCov_mat=SCov_k_m';
 end

%initialize matrix of spot (co-)volatility on each block
SCov_k_m=NaN*zeros(3,B);

% Loop over blocks
%---------------------------------------------------------------------
for k=1:B
    
    % compute spot covolatility estimate on block k
    [SCov_k_v,I_k_m]=spot_cov_block_adapt_rweights_corrd_simp(h,SCov_mat(k,:),...
    noise_v,cv,[Phi_norm_k_x_mn(k,1:J_optv(k));Phi_norm_k_xy_mn(k,1:J_optv(k));Phi_norm_k_y_mn(k,1:J_optv(k))],...
    lret1_avb_mn(k,1:J_optv(k)),lret2_avb_mn(k,1:J_optv(k)),qvtbx_v(k),qvtby_v(k),lnoise_v_m(:,k),J_optv(k),ccorr);
    
    % assign estimates to matrix
    SCov_k_m(:,k)=SCov_k_v;
    
    % assign inverse of sum of fisher infos
      if (nargout>6)&&(twstc<=0)
          if (sum(sum(isnan(I_k_m)))==0)
                i_I_k_m_arr(:,:,k)=inv(I_k_m);
           else
                i_I_k_m_arr(:,:,k)=zeros(3,3);
           end
      end

end

 end
% -------------------------------------------------------------------------
%       
%% smooth, compute covariance matrices of estimates under local param exp and select


if nargout>6
    
    % use final estimates for info matrices(else pilot)
    %------------------------------------------------------------------
    
    if twstc>0
        
        % obtain pilot spot cov estimates by smoothing
        if K_p>1
            SCov_mat=spot_vola_smooth(SCov_k_m',K_p);
        else
            SCov_mat=SCov_k_m';
        end
        
        % normal blocks
        for k=1:B
            % compute info mat on block k
            I_k_m=info_mat_block_adapt(h,SCov_mat(k,:),noise_v,qvtbx_v(k),qvtby_v(k),J_optv(k)); 
            if (sum(sum(isnan(I_k_m)))==0)
                i_I_k_m_arr(:,:,k)=inv(I_k_m);
            else
                i_I_k_m_arr(:,:,k)=zeros(3,3);
            end
        end
        
    end
    
    % smoothed (final) estimates and covariance matrices of smoothed estimates
    %------------------------------------------------------------------
        
        if K>1
            if lKb>0
                SCov_mat=spot_vola_smooth_lKb(SCov_k_m',K);
                Covmat_arr=spot_vola_smooth_cov_lKb(i_I_k_m_arr,K);
            else
                SCov_mat=spot_vola_smooth(SCov_k_m',K);
                Covmat_arr=spot_vola_smooth_cov(i_I_k_m_arr,K);
            end
        else
             SCov_mat=SCov_k_m';
            Covmat_arr=i_I_k_m_arr;
        end     % adjust lkb
        
        % select estimates and covariances
        [SCov_mat,Covmat_arr]=select_block_vola(input_v,block_st,block_end, SCov_mat,Covmat_arr);
        
else
    
    % smoothed (final) estimates
        %------------------------------------------------------------------
        if K>1
            if lKb>0
                SCov_mat=spot_vola_smooth_lKb(SCov_k_m',K);
            else
                SCov_mat=spot_vola_smooth(SCov_k_m',K);
            end
        else
            SCov_mat=SCov_k_m';
        end 
    
    % select estimates 
        [SCov_mat,~]=select_block_vola(input_v,block_st,block_end, SCov_mat);
    
 end

 
    








     











