function [SCov_mat,block_st,block_end,J_optv,J_p,K,K_p,Covmat_mat]=lle_spot_cov_md_kern(varargin)
%% lle_spot_cov_md_kern
%--------------------------------------------------------------
% local likelihood estimation of spot covolatility matrices on specified
% number of blocks with final smoothing using kernel weights
% only for cross-sectionally independent noise in m-dimensional case
%
% Note:    
%             * blocks with equal lengths
%              *inputs either fixed or based on continuous approximations
%             * norms of spectral weights uncorrected (based on continuous approximations)
%            * spectral weights based on cont approximations
%
% USAGE:
% [SCov_mat,block_st,block_end,J_optv,J_p,K,K_p,Covmat_arr]=lle_spot_cov_md(input_v,PRICETIME,noisev,startt,endt,...
%  nkex,phi_ctr,B,J,J_p,K,K_p,theta_h,theta_J,theta_K,K_thr,K_p_thr,twst,twstc,kern,ccorr,ccorrlast)
%
%--------------------------------------------------------------
% INPUT:
% input_v -> (n_inx1) vector of estimation points
% PRICETIME -> (mx1) cell array with ith cell containing (n0ix2) matrix:
%                   1st col  -  observation times for asset i (in sec after midnight)
%                   2nd col    - high frequency prices for asset i
% noise_v -> (mx1) vector of noise variance estimates
% startt -> start of trading (in hours)
% endt   -> end of trading (in hours)
% nkex -> if >0, exact number of obs on each block is used to determine
%               cutoff
% phi_ctr -> if >0, center of return spells used to compute spectral
%               weights
% B      -> no of equally-sized blocks
% J  -> spectral cutoff for final estimates of spot covariances
% J_p-> spectral cutoff for pilot estimates of spot covariances
% K   -> bandwidth for smoothing of final spot cov estimates
% K_p  -> bandwidth for smoothing of pilot estimates
% theta_h   -> proportionality parameter for block length (relevant if B
%                           not specified)
% theta_J     -> proportionaliy parameter for cutoff (relevant if J
%                           or J_p not specified)
% theta_k    -> proportionality parameter for smoothing bandwidth (relevant if K
%                           or K_p not specified)
% K_thr    -> lower threshold for pilot smoothing bandwidth (relevant if
%                           K not specified): default=0 (+eps)
% K_p_thr    -> lower threshold for pilot smoothing bandwidth (relevant if
%                           K_p not specified): default=0 (+eps)
% twst     -> if >0: twostep estimation is used (estimates from first step
%                   used as pilots for second step
% twstc      -> if >0: Covariance matrices of smoothed spot (co-)volatilities
%   under the local parametric experiment   are computed using 2ndstep spot
%   covariance estimates (mattters only if 8 output arguments)
% kern   -> string, type of kernel:
%                   -- 'gauss' (Gaussian)
%                   -- 'beta'   (Beta)
%                   -- 'betamod'   (modified Beta)
% ccorr -> if >0, correlation and variance estimates are corrected
% ccorrlast -> if >0, correlation and variance estimates are corrected (last estimation step)
%--------------------------------------------------------------
% OUTPUT:
% SCov_mat-> (n_inxm(m+1)/2) matrix of spot cov matrices for each point of
%                               estimation
%                    each row: vech(\hat Sigma_t)'
% block_st -> (Bx1) vector of block beginnings
% block_end -> (Bx1) vector of block ends
% J_opt -> vector of optimal spectral cutoffs for each block 
% Covmat_arr ->(m^2(m^2+1)/2xn_in) matrix: Stacked Covariance matrices of smoothed spot (co-)volatilities
%                       under the local parametric experiment      (optional)
%                    each column:vech(Cov(\hat Sigma_t))
%   NOTE: covariance matrices still need to be postmultiplied by Z_tilde,
%   i.e. the (m^2 x m^2)  matrix of fourth moments of a m-dimensional
%                   standard Gaussian vector
% -------------------------------------------------------------------------
%
% Peter Malec, May 16, 2016 
%
% -------------------------------------------------------------------------
%       
%% Input check


% Check no of inputs
Q = length(varargin);
if Q<21 % 22 all in all (see below)
    error('21 inputs required.') 
end

% Get inputs except data
%-----------------------------------------
if isequal(Q,22)
    input_v=varargin{1};
    noise_v=varargin{Q-19};
    startt=varargin{Q-18};
    endt=varargin{Q-17};
    nkex=varargin{Q-16};
    phi_ctr=varargin{Q-15};
    B=varargin{Q-14};
    J=varargin{Q-13};
    J_p=varargin{Q-12};
    K=varargin{Q-11};
    K_p=varargin{Q-10};
    theta_h=varargin{Q-9};
    theta_J=varargin{Q-8};
    theta_K=varargin{Q-7};
    K_thr=varargin{Q-6};
    K_p_thr=varargin{Q-5};
    twst=varargin{Q-4};
    twstc=varargin{Q-3};
    kern=varargin{Q-2};
    ccorr=varargin{Q-1};
    ccorrlast=varargin{Q};
else
    input_v=varargin{1};
    noise_v=varargin{Q-18};
    startt=varargin{Q-17};
    endt=varargin{Q-16};
    nkex=varargin{Q-15};
    phi_ctr=varargin{Q-14};
    B=varargin{Q-13};
    J=varargin{Q-12};
    J_p=varargin{Q-11};
    K=varargin{Q-10};
    K_p=varargin{Q-9};
    theta_h=varargin{Q-8};
    theta_J=varargin{Q-7};
    theta_K=varargin{Q-6};
    K_thr=varargin{Q-5};
    K_p_thr=varargin{Q-4};
    twst=varargin{Q-3};
    twstc=varargin{Q-2};
    kern=varargin{Q-1};
    ccorr=varargin{Q};
    ccorrlast=ccorr;
end


% Get data and determine number of assets
PRICETIME=varargin{2};
m=length(PRICETIME);
if m>20
    disp('Warning: Computation of estimator may take a while!')
elseif m>31
    error('Too many assets! Info matrices would become too huge!') 
end


% Assign price and time obs into cell arrays
%------------------------------------------------------
% empty cell arrays
price = cell(m,1);
time = cell(m,1);

% Loop across assets
for i=1:m
    data_temp=PRICETIME{i};
    price{i} = data_temp(:,2);
    time{i} =data_temp(:,1);  
    if any(diff(time{i})<0)
        error(['TIME' num2str(i) ' must be sorted and increasing.'])
    end
    time{i} = double(time{i});
end

% check tuning parameters
%----------------------------------------
if isempty(B)
    compB=1;
    if isempty(theta_h)
        error('Specifiy no of blocks or proportionality parameter for block length!')
    end
else
    compB=0;
end

if isempty(J)
    compJ=1;
    if isempty(theta_J)
        error('Specifiy spectral cutoff or corresponding proportionality parameter!')
    end
else
    compJ=0;
end

if isempty(J_p)
    compJ_p=1;
    if isempty(theta_J)
        error('Specifiy pilot spectral cutoff or corresponding proportionality parameter!')
    end
else
    compJ_p=0;
end

if isempty(K)
    compK=1;
    if isempty(theta_K)
        error('Specifiy smoothing bandwidth or corresponding proportionality parameter!')
    end
    if isempty(K_thr)
        K_thr=0;
    end
else
    compK=0;
end

if isempty(K_p)
    compK_p=1;
    if isempty(theta_K)
        error('Specifiy pilot smoothing bandwidth or corresponding proportionality parameter!')
    end
    if isempty(K_p_thr)
        K_p_thr=0;
    end
else
    compK_p=0;
end
%-------------------------------------------------------------------------
      
%% Prepare Data 

% empty cell arrays
time_l=cell(m,1);
avgtime=cell(m,1);
lret=cell(m,1);
n_obs_v=NaN*zeros(m,1);


%loop across assets and construct series
for i=1:m
    
    %log-returns
    lret{i}=diff(log(price{i}));
    
    % standardized times (time in [0,1])                                    
    time_st= (time{i}-startt*3600)/((endt-startt)*3600);   
    
    % times, lagged times, centered times 
    time{i}=time_st(2:end);
    time_l{i}=time_st(1:end-1);
    if phi_ctr>0
        avgtime{i}=0.5*(time{i}+time_l{i});
    else
        avgtime{i}=time{i};
    end
    
    % number of obs
    n_obs_v(i)=length(time{i});
end

% minimum number of obs
n_min=min(n_obs_v);
% -------------------------------------------------------------------------
%      
%% Determine block length and bounds

% number of Blocks, block length, start & end
if compB
    B=ceil((sqrt(n_min)/log(n_min))/theta_h);
end
B=max([B 1]);
h=1/B;
block_st=0:(1/B):(1-1/B);
block_end=(1/B):(1/B):1;

% cutoff
if compJ
    J=floor((theta_J*log(n_min)));
end
J=max([J 1]);

% pilot cutoff
if compJ_p
    J_p=floor((theta_J*log(n_min)));
end
J_p=max([J_p 1]);

% bandwidth
if compK
    K=theta_K*(n_min^(1/4));
end
K=max([K (K_thr+1e-6)]);

% pilot bandwidth
if compK_p
    K_p=theta_K*(n_min^(1/4));
end
K_p=max([K_p (K_p_thr+1e-6)]);
%
% -------------------------------------------------------------------------
%       
%% Compute Pilot Estimates of Spot (Co-variances) on blocks

% pilot estimates
[Sigma_k_sm,lret_avb_arr,lnoise_m]...
    =spot_covs_pilot_md_kern(block_st,block_end,h,avgtime,time,time_l,lret,noise_v,K_p,J_p,ccorr,kern);
% -------------------------------------------------------------------------
%       
%% Compute LLE (step one)

% vector for cutoffs
J_optv=NaN*zeros(B,1);

% number of unique cov mat elements
elu=m*(m+1)/2;

%initialize matrix of spot (co-)volatility on each block
SCov_k_m=NaN*zeros(elu,B);

% initialize collection matrices for local spectral means
if twst>0
   lret_avb_arrn=NaN*zeros(J,m,B);
 end

% matrix for inverse of sum of fisher infos
nc_el=m^2*(m^2+1)/2;
i_I_k_mat=NaN*zeros(nc_el,B); 

% determine which correction indicator necessary
if twst>0
    ccorrLLE1=ccorr;
else
    ccorrLLE1=ccorrlast;
end

% Loop over blocks
%---------------------------------------------------------------------
for k=1:B
    
    % compute spot covolatility estimate on block k
    [Sigma_k,I_k_m,lret_avb_m,J_opt_k]=spot_cov_block_adapt_md(block_st(k),block_end(k),h,n_min,avgtime,...
    lret,squeeze(lret_avb_arr(:,:,k)),ivech(Sigma_k_sm(k,:)'),lnoise_m(:,k),nkex,J,ccorrLLE1);

     % assign estimates to matrix
     SCov_k_m(:,k)=vech(Sigma_k);

    %assign inputs for 2nd step
    if twst>0
        lret_avb_arrn(1:size(lret_avb_m,1),:,k)=lret_avb_m;
    else
        % assign inverse of sum of fisher infos
        if (nargout>7)&&(twstc<=0)
            if (sum(sum(isnan(I_k_m)))==0)
               i_I_k_mat(:,k)=vech(inv(I_k_m));  
            else
                i_I_k_mat(:,k)=zeros(nc_el,1);   
            end        
        end
    end
    
     %assign optimal cutoff to vector
     J_optv(k)=J_opt_k;
     
end
% -------------------------------------------------------------------------
%       
%% Compute LLE (step two)

if twst>0   % was it requested?
    
% obtain pilot spot cov estimates by smoothing
[SCov_mat,~]=spot_vola_smooth_kern(block_st,K_p,block_st,SCov_k_m',kern);

%initialize matrix of spot (co-)volatility on each block
SCov_k_m=NaN*zeros(elu,B);

% Loop over blocks
%---------------------------------------------------------------------
for k=1:B
    
    % compute spot covolatility estimate on block k
    [Sigma_k,I_k_m]=spot_cov_block_adapt_rweights_md(h,squeeze(lret_avb_arrn(1:J_optv(k),:,k)),ivech(SCov_mat(k,:)'),lnoise_m(:,k),J_optv(k),ccorrlast);

    % assign estimates to matrix
    SCov_k_m(:,k)=vech(Sigma_k);
    
    % assign inverse of sum of fisher infos
      if (nargout>7)&&(twstc<=0)
          if (sum(sum(isnan(I_k_m)))==0)
               i_I_k_mat(:,k)=vech(inv(I_k_m));  
            else
                i_I_k_mat(:,k)=zeros(nc_el,1);   
          end        
      end   

end

 end
% -------------------------------------------------------------------------
%       
%% smooth, compute covariance matrices of estimates under local param exp and select


if nargout>7
    
    % use final estimates for info matrices(else pilot)
    %------------------------------------------------------------------
    
    if twstc>0
        
        % obtain pilot spot cov estimates by smoothing
        [SCov_mat,~]=spot_vola_smooth_kern(block_st,K_p,block_st,SCov_k_m',kern);
        
        % normal blocks
        for k=1:B
            % compute info mat on block k
            I_k_m=info_mat_block_adapt_md(J_optv(k),h,lnoise_m(:,k),ivech(SCov_mat(k,:)'));
            if (sum(sum(isnan(I_k_m)))==0)
                i_I_k_mat(:,k)=vech(inv(I_k_m));
            else   
                i_I_k_mat(:,k)=zeros(nc_el,1);
            end
        end

    end
    
    % smoothed (final) estimates and covariance matrices of smoothed estimates
    [SCov_mat,Covmat_mat]=spot_vola_smooth_kern(input_v,K,block_st,SCov_k_m',kern,i_I_k_mat);
                                                         
else
    
    % smoothed (final) estimates
    [SCov_mat,Covmat_mat]=spot_vola_smooth_kern(input_v,K,block_st,SCov_k_m',kern);
       
 end



     











