function [Sigma_out,B,J_max,H_mat]=spectral_est_cov_oracle(data_a1,data_a2,startt,endt,s_block,J_choice,Sigma_m)
%% spectral_est_cov
%--------------------------------------------------------------
% Performs spectral estimation of covolatility following 
% Bibinger & Reiss (2011) using oracle version of estim
%--------------------------------------------------------------
% INPUT:
% data_a1 -> (n_a x 2) matrix of synchronous data for asset 1
%       - 1st  col: observation times  (sec after midnight)
%       - 2nd col: prices 
% startt -> start of trading (in hours)
% endt   -> end of trading (in hours)
% s_block -> determines which block is shorter: 'first' or 'last' (default)
% J_choice -> choice of spectral cutoff: 'optim' follows Reiss (2011),
% numerical value yields fixed cutoff, otherwise log(n) (default)
% Sigma_m -> ((N+1) x 3) matrix of spot volatilities and (co-volatilities),
% where N is number of seconds per trading day (N+1 incl starting value)
%--------------------------------------------------------------
% OUTPUT:
% Sigma_out -> (2 x 2) matrix containg estimates of integrated (co-)
% volatilities
% Sigma_kh -> (2 x 2 x num(blocks)) array of spot (co-volatilities) at 
% lower bounds of blocks 
% B -> number of blocks 
% J_max -> spectral cutoff
% K  -> bandwidth for spot volatility estimation
% H_mat -> (2x2) covariance matrix of noise
% -------------------------------------------------------------------------
%
% Peter Malec, January 3, 2012
%
% -------------------------------------------------------------------------
%       
%% Prepare Data

%price _rft is (n_rft x 2) matrix
price_rft=[data_a1(:,2) data_a2(:,2)];
                                    
% standardize times (time_rft in [0,1])                                    
time_rft_st= (time_rft-startt*3600)/((endt-startt)*3600);          

% number of obs
n_rft=size(price_rft,1);
% -------------------------------------------------------------------------
%       
%% Determine block length noise level and spectral cutoffs

% block length (rough choice)
h=log(n_rft)/sqrt(n_rft);

% estimate noise (co-)variance
H_mat=noise_cov_est(price_rft,'cov');       % use covariance based estimator

% spectral cutoff
%-------------------------------------------------

% choose method
if nargin <6
    J_ind=2;
else
    if isequal(J_choice,'optim')
     J_ind=1;
    elseif isnumeric(J_choice)
        J_ind=3;
    else
       J_ind=2;
    end
end

if J_ind==1 % "optimal" choice
    
    % compute low frequency pilot estimator of integrated covariance
    Price1_PTLF=realized_price_filter(data_a1(:,2),data_a1(:,1),'seconds','CalendarTime',300);       % Previous tick 5Min for asset1
    Price2_PTLF=realized_price_filter(data_a2(:,2),data_a2(:,1),'seconds','CalendarTime',300);       % Previous tick 5Min for asset1
    Sigma_bar=diff([log(Price1_PTLF) log(Price2_PTLF)])'*diff([log(Price1_PTLF) log(Price2_PTLF)]); %realized covariance

    % determine spectral cutoffs
    J1=max(floor(min(2*sqrt(Sigma_bar(1,1))*h/(pi*sqrt(H_mat(1,1))),n_rft*h)),1);           % following Reiss (2011)
    J12=max(floor(min(2*sqrt(abs(Sigma_bar(1,2)))*h/(pi*sqrt(abs(H_mat(1,1)))),n_rft*h)),1);
    J2=max(floor(min(2*sqrt(Sigma_bar(2,2))*h/(pi*sqrt(H_mat(2,2))),n_rft*h)),1);

    % max. spectral cutoff
    J_max=max([J1 J12 J2]);
    
elseif J_ind==3 % fixed value 
    
    J_max=J_choice;

else % default method
    
    J_max=max(floor(min(log(n_rft),n_rft*h)),1);     
    
end
% -------------------------------------------------------------------------
%       
%% Compute Spectral Weights

% determine block bounds
%--------------------------------
%--------------------------------


% start and end of each block (# of elements gives # blocks)-
%-----------------------------------------------------------------------

% which block ist shorter?

if isequal(s_block,'first')  % first block
    
    % vector of bounds
    block_v=(fliplr((1:-h:0)))';
    
    if block_v(1)==0         % check if unit interval can be divided in equal blocks of length h
        block_st=block_v(1:end-1);
        block_end=block_v(2:end);
    else                                % first block is shorter !!
     block_st=[0; block_v(1:end-1)];
     block_end=block_v;
    end

    hs=block_end(1)-block_st(1);    %length of shorter block

else        % last block
    
    % vector of bounds
    block_v=(0:h:1)';

    if block_v(end)==1         % check if unit interval can be divided in equal blocks of length h
        block_st=block_v(1:end-1);
        block_end=block_v(2:end);
    else                                % last block is shorter !!
     block_st=block_v;
     block_end=[block_v(2:end); 1];
    end
    
    hs=block_end(end)-block_st(end);    %length of shorter block

end

% number of blocks
B=length(block_st);

% determine spectral weights 
%--------------------------------

% spectral weights
Phi=spectr_weights(time_rft_st(2:end),block_st,block_end,h,hs,s_block,J_max);     % (n_rft x J_max x length(block_st)) array
                    %times in rows, freqs in columns,blocks in third dim
                    
% frequencies up to spectral cutoff
j=(1:J_max)';
                    
% vector of empirical norms 
Phi_norm=(2*n_rft*sin(j*pi/(2*n_rft*h))).^(-2);                     % (J_max x 1) vector
Phi_norms=(2*n_rft*sin(j*pi/(2*n_rft*hs))).^(-2);                     % for shorter block
% -------------------------------------------------------------------------
%       
%% Compute Local Blockwise Averages of Log-Returns
 
% log-returns
lret=diff(log(price_rft));

%compute local averages
lret1_avb=squeeze(nansum(repmat(lret(:,1),[1 J_max length(block_st)]).*Phi));     % (J_max x length(block_st)) matrix
lret2_avb=squeeze(nansum(repmat(lret(:,2),[1 J_max length(block_st)]).*Phi));
           %each freq, block: multiply return by weight, then sum over
           %returns
% -------------------------------------------------------------------------
%       
%% Determine Spot Volatilities at Beginning of Blocks

if J_max>1       % if J=1, no pilot estimates are needed

    % Find relevant volatilities
    Sigma_m    % some kind of interpolation needed

else
    
   
    Sigma_kh=0;
    
end
    
% -------------------------------------------------------------------------
%       
%% Compute spectral estimator of integrated covariance

Sigma_out=spectral_est_cov_core(lret1_avb,lret2_avb,Phi_norm,Phi_norms,H_mat,Sigma_kh,n_rft,h,hs,s_block);         % (2 x 2) matrix











