function [Sigma_out,h_v,h,hs,block_st,block_end,J_opt]=lle_cov_oracle_all_async_eql(data_a1,data_a2,H,startt,endt,s_block,SCov_mat,h_fac,bc,nkt,J_all)
%% lle_cov_oracle_all_async
%--------------------------------------------------------------
% local likelihood estimation of covolatility matrix (asynchr)
% faster oracle version preselecting
% observations on each block
%
% Note:   *  comprehensive procedure!
%            * equal block lengths
%            * average number of obs used in weights
%            * spectral weights based on cont approximations
%--------------------------------------------------------------
% INPUT:
% data_a1 -> (n_a x 2) matrix of synchronous data for asset 1
%       - 1st  col: observation times  (sec after midnight)
%       - 2nd col: prices 
% H -> (2x2) covariance matrix of noise
% startt -> start of trading (in hours)
% endt   -> end of trading (in hours)
% s_block -> determines which block is shorter: 'first' or 'last' (default)
% SCov_mat -> (N x 3) matrix of spot (co-) variances ([SVar1 SVar1 Scov]),
% where N such that times=1/N:1/N:1.
% Note: each SCov matrix has to be positive definite
% h_fac   -> constant factor for multiplication with "opt" block length
% bc -> if >0, bias correction, =0 bias correction of vars, <0: no bias
% correction
% nkt > if >0, number of obs on blocks computed as average obs, otherwise
% inverse of average duration
% J_all -> if >0, all frequencies are used, else optimal number (default)
%--------------------------------------------------------------
% OUTPUT:
% Sigma_out -> (2 x 2) matrix containg estimates of integrated (co-)
% volatilities
% h_v -> (Bx1) vector of block lengths
% h -> block length
% hs -> length of shorter block
% block_st -> (Bx1) vector of block beginnings
% block_end -> (Bx1) vector of block ends
% J_opts -> spectral cutoff for shorter block
% -------------------------------------------------------------------------
%
% Peter Malec, May 10, 2012 
%
% -------------------------------------------------------------------------
%       
%% Prepare Data and Inputs

% Input value of J_all
if nargin<11
    J_all=0;
end

% log-returns
lret1=diff(log(data_a1(:,2)));
lret2=diff(log(data_a2(:,2)));

% standardize times (time_rft in [0,1])                                    
time_st1= (data_a1(:,1)-startt*3600)/((endt-startt)*3600);         
time_st2= (data_a2(:,1)-startt*3600)/((endt-startt)*3600); 

% number of obs and times
t1=time_st1(2:end);
t2=time_st2(2:end);

% extract noise (co-)variances
nu2_x=H(1,1);
nu2_y=H(2,2);
nu_xy=H(1,2);

% noise vector for bias correction


% -------------------------------------------------------------------------
%       
%% Determine block length 

% block length
%-------------------------------------------------

% average sample size
n_rft=mean([size(t1,1) size(t2,1)]);

% sample size relative to noise
n_nu=n_rft/mean(diag(H)); %use average noise variance


% block length (rough choice)
h=min([(h_fac*log(n_nu)/sqrt(n_nu)) 1]);


% -------------------------------------------------------------------------
%       
%%  Determine block bounds and spectral cutoff (pilot)

% start and end of each block (# of elements gives # blocks)-
%-----------------------------------------------------------------------

% which block ist shorter?

if isequal(s_block,'first')  % first block
    
    % vector of bounds
    block_v=(fliplr((1:-h:0)))';
    
    if block_v(1)==0         % check if unit interval can be divided in equal blocks of length h
        block_st=block_v(1:end-1);
        block_end=block_v(2:end);
    else                                % first block is shorter !!
     block_st=[0; block_v(1:end-1)];
     block_end=block_v;
    end

    hs=block_end(1)-block_st(1);    %length of shorter block
    
    % block index for short block
    ks=1;

else        % last block
    
    % vector of bounds
    block_v=(0:h:1)';

    if block_v(end)==1         % check if unit interval can be divided in equal blocks of length h
        block_st=block_v(1:end-1);
        block_end=block_v(2:end);
    else                                % last block is shorter !!
     block_st=block_v;
     block_end=[block_v(2:end); 1];
    end
    
    hs=block_end(end)-block_st(end);    %length of shorter block
    
    % block index for short block
    ks=length(block_st);

end

% number of blocks
B=length(block_st);
% -------------------------------------------------------------------------
%       
%% Compute LLE

% create block length vector accouting for shorter block and B=1
if B~=1
    if isequal(s_block,'first')  % first block
        h_v=[hs;repmat(h,B-1,1)];                                      % (Bx1) vector
    else
        h_v=[repmat(h,B-1,1) ;hs];                                      % (Bx1) vector
    end
else   %special case if B=1 
        h_v=h;
        ks=NaN;
end

% number of Scov obs
N=size(SCov_mat,1);

%times at which Spot covolatilities are "observed"
t_scov=1/N:1/N:1;
    
% vector for cutoffs
J_optv=NaN*zeros(B,1);
     
%initialize matrix of spot (co-)volatility on each block
SCov_k_m=NaN*zeros(3,B);
      
% set of "normal" blocks
normB=setdiff(1:B,ks);   

% create noise vector (replicated)

if bc>0
    noise_v=[nu2_x;nu_xy;nu2_y];
elseif bc==0
    noise_v=[nu2_x;0;nu2_y];
end

% Loop over normal blocks
%---------------------------------------------------------------------
%---------------------------------------------------------------------
     
for k=normB
    
    % extract spot (co-) volatility estimates
    %--------------------------------------------------------

    if k~=1
       t_scov_ind=find(t_scov<= block_st(k),1,'last');
       sigma2_x=SCov_mat(t_scov_ind,1);
       sigma2_y=SCov_mat(t_scov_ind,2);
       sigma_xy=SCov_mat(t_scov_ind,3);
   else
       sigma2_x=SCov_mat(1,1);
       sigma2_y=SCov_mat(1,2);
       sigma_xy=SCov_mat(1,3);  %first block
    end
   
    % Compute number of obs on block
    %----------------------------------------------------------------------
    
    % index of observations on each block
     t_ind1=logical(((t1 >= block_st(k)) & (t1 <block_end(k))));  
     t_ind2=logical(((t2 >= block_st(k)) & (t2 <block_end(k))));  
     
    % observations times on each block (since begining)
     tb1=t1(t_ind1)-block_st(k);
     tb2=t2(t_ind2)-block_st(k);
     
     % log returns on block
     lret_b1=lret1(t_ind1,:);
     lret_b2=lret2(t_ind2,:);
     
      if nkt<=0
        % times between obs
        delta_k1=mean(diff(tb1));
        delta_k2=mean(diff(tb2));
        % inverse of average of deltas
        n_k=1/mean([delta_k1 delta_k2]);
     else
         n_k=mean([size(lret_b1,1) size(lret_b2,1)])+1;
     end
     
    % pilot value of cutoff
    %n_max=max([size(lret_b1,1) size(lret_b2,1)])+1;
    n_max=n_k*h;
    J_max=max(floor(n_max),1);
    
    % frequencies up to spectral cutoff
    j=(1:J_max)';
    
    
    % Compute elements of inverse fisher information and put into structure
    %-----------------------------------------------------------------------------------
    
    % vector of (inverse) empirical norms  (continuous approximations)
    i_Phi_norm=h^(-2)*(pi^2)*j(:).^2;
    i_Phi_norm_2=i_Phi_norm.^(2);
    
    % weights
     i_I_jk_11=2*(i_Phi_norm.*nu2_x/n_k+sigma2_x).^2;     % (J x 1) matrix

    i_I_jk_22=i_Phi_norm_2.*(nu2_x*nu2_y+nu_xy^2)/(n_k^2)+(sigma2_x.*sigma2_y+(sigma_xy).^2)+...
    (i_Phi_norm/n_k).*(sigma2_x*nu2_y+sigma2_y*nu2_x+2*sigma_xy*nu_xy);   % (J x 1) matrix

    i_I_jk_33=2*(i_Phi_norm.*nu2_y/n_k+sigma2_y).^2;     % (J x 1) matrix

    i_I_jk_12=2*sigma2_x.*sigma_xy+i_Phi_norm_2.*2*nu2_x*nu_xy/(n_k^2)+...
    2*(i_Phi_norm/n_k).*(sigma2_x*nu_xy+sigma_xy*nu2_x);   % (J x 1) matrix

    i_I_jk_23=2*sigma2_y.*sigma_xy+i_Phi_norm_2.*2*nu2_y*nu_xy/(n_k^2)+...
    2*(i_Phi_norm/n_k).*(sigma2_y*nu_xy+sigma_xy*nu2_y);   % (J x 1) matrix

    i_I_jk_13=2*(sigma_xy).^2+i_Phi_norm_2.*2*(nu_xy/n_k)^2+...
    4*(i_Phi_norm/n_k).*sigma_xy*nu_xy;   % (J x 1) matrix

    % Compute elements of inverse fisher information and weights
    %----------------------------------------------------------------------
    
    % matrix of distinct elements of inverse Fisher infos for all freqs
    i_I_jk_m=[i_I_jk_11';i_I_jk_12';i_I_jk_13';i_I_jk_22'...
        ;i_I_jk_23';i_I_jk_33'];      % (6 x J) matrix
    
    % create array of (symmetric) inverse fisher info matrices for all freqs
     i_I_jk_a=ivech3(i_I_jk_m);             % (3x3xJ) array
     
     % compute array of fisher info matrices by inversion
     I_jk_a=inv3(i_I_jk_a);
     
     % compute sum of fisher info matrices across freqs on block
     I_k_m=nansum(I_jk_a,3);                    % (3x3) matrix
   
     % compute array of weights
     W_k_a=inv3(I_k_m,I_jk_a);
     
    % find cutoff
    if J_all<=0
        J_opt_k=find_maxJ_const(W_k_a);
    else
       J_opt_k=J_max;
    end
    J_optv(k)=J_opt_k;
    
    % adjust weights 
     W_k_a=W_k_a(:,:,1:J_opt_k);
     
     % Compute Local Blockwise Averages of Log-Returns
    %------------------------------------------------------------------

    %empty matrices for local blockwise averages
    lret1_avb=NaN*zeros(J_opt_k,1);   % (J_opt_kx1) vector
    lret2_avb=lret1_avb;

    
        % loop
        for j=1:J_opt_k
            
            % spectral weights for given block and frequency
            Phi1=spectr_weights_j_onbl_cont(tb1,h,j);
            Phi2=spectr_weights_j_onbl_cont(tb2,h,j);
            
            % local blockwise averages using frequency j
            lret1_avb(j)=lret_b1'*Phi1; 
            lret2_avb(j)=lret_b2'*Phi2;
            
        end
     
    % estimate spot covola for given block as weighted average acr freqs
    %----------------------------------------------------------------------
     
    % select empirical norms of spectral weights
    Phi_norm_v=(i_Phi_norm(1:J_opt_k).^(-1))';                 
    
    % matrix of dist. elements of outer poduct matrices of local ret avgs of for freqs
    S_jk_m=[(lret1_avb.^2)';(lret1_avb.*lret2_avb)';(lret2_avb.^2)'];   % (3 x J_opt_k) matrix
    
    % matrix of dist. elements of bias-corr. spot vola matrix estimator on 
    if bc>=0
        C_jk_m=(S_jk_m-noise_v(:,ones(J_opt_k,1))/n_k)./Phi_norm_v(ones(3,1),:); % (3 x J_opt_k) matrix
    else
        C_jk_m=S_jk_m./Phi_norm_v(ones(3,1),:); % (3 x J_opt_k) matrix
    end
    
    %  spot vola estimates on block: sum of prod of fisher info and spot vola estimates based on diff freq
    SCov_k_m(:,k)=nansum(mtimesx(W_k_a,permute(C_jk_m,[1 3 2])),3);   % (3xJ_opt_k) vector

end

% global optimal cutoff and cutoff of short block
J_opt=nanmax(J_optv);

%-------------------------------------------------------------------------

% short block
%-------------------------------------------------
%-------------------------------------------------

if B>1

% extract spot (co-) volatility estimates
    %--------------------------------------------------------

    if ks~=1
       t_scov_ind=find(t_scov<= block_st(ks),1,'last');
       sigma2_x=SCov_mat(t_scov_ind,1);
       sigma2_y=SCov_mat(t_scov_ind,2);
       sigma_xy=SCov_mat(t_scov_ind,3);
   else
       sigma2_x=SCov_mat(1,1);
       sigma2_y=SCov_mat(1,2);
       sigma_xy=SCov_mat(1,3);  %first block
    end
    
    % Compute number of obs on block
    %----------------------------------------------------------------------
    
    % index of observations on each block
     t_ind1=logical(((t1 >= block_st(ks)) & (t1 <block_end(ks))));  
     t_ind2=logical(((t2 >= block_st(ks)) & (t2 <block_end(ks))));  
     
    % observations times on each block (since begining)
     tb1=t1(t_ind1)-block_st(ks);
     tb2=t2(t_ind2)-block_st(ks);
     
     % log returns on block
     lret_b1=lret1(t_ind1,:);
     lret_b2=lret2(t_ind2,:);
     
      if nkt<=0
        % times between obs
        delta_k1=mean(diff(tb1));
        delta_k2=mean(diff(tb2));
        % inverse of average of deltas
        n_k=1/mean([delta_k1 delta_k2]);
     else
         n_k=mean([size(lret_b1,1) size(lret_b2,1)])+1;
     end
     
    % pilot value of cutoff
    %n_max=max([size(lret_b1,1) size(lret_b2,1)])+1;
    n_max=n_k*hs;
    J_opts=min([J_opt floor(n_max)]);
    
    % frequencies up to spectral cutoff
    j=(1:J_opts)';
    
    % Compute elements of inverse fisher information and put into structure
    %-----------------------------------------------------------------------------------
    
     % vector of (inverse) empirical norms  (continuous approximations)
    i_Phi_norms=hs^(-2)*(pi^2)*j(:).^2;
    i_Phi_norms_2=i_Phi_norms.^(2);
    
    % weights 
    i_I_jk_11=2*(i_Phi_norms.*nu2_x/n_k+sigma2_x).^2;     % (J x 1) matrix

    i_I_jk_22=i_Phi_norms_2.*(nu2_x*nu2_y+nu_xy^2)/(n_k^2)+(sigma2_x.*sigma2_y+(sigma_xy).^2)+...
    (i_Phi_norms/n_k).*(sigma2_x*nu2_y+sigma2_y*nu2_x+2*sigma_xy*nu_xy);   % (J x 1) matrix

    i_I_jk_33=2*(i_Phi_norms.*nu2_y/n_k+sigma2_y).^2;     % (J x 1) matrix

    i_I_jk_12=2*sigma2_x.*sigma_xy+i_Phi_norms_2.*2*nu2_x*nu_xy/(n_k^2)+...
    2*(i_Phi_norms/n_k).*(sigma2_x*nu_xy+sigma_xy*nu2_x);   % (J x 1) matrix

    i_I_jk_23=2*sigma2_y.*sigma_xy+i_Phi_norms_2.*2*nu2_y*nu_xy/(n_k^2)+...
    2*(i_Phi_norms/n_k).*(sigma2_y*nu_xy+sigma_xy*nu2_y);   % (J x 1) matrix

    i_I_jk_13=2*(sigma_xy).^2+i_Phi_norms_2.*2*(nu_xy/n_k)^2+...
    4*(i_Phi_norms/n_k).*sigma_xy*nu_xy;   % (J x 1) matrix

    % Compute elements of inverse fisher information and weights
    %----------------------------------------------------------------------
    
    % matrix of distinct elements of inverse Fisher infos for all freqs
    i_I_jk_m=[i_I_jk_11';i_I_jk_12';i_I_jk_13';i_I_jk_22'...
        ;i_I_jk_23';i_I_jk_33'];      % (6 x J) matrix
    
    % create array of (symmetric) inverse fisher info matrices for all freqs
     i_I_jk_a=ivech3(i_I_jk_m);             % (3x3xJ) array
     
     % compute array of fisher info matrices by inversion
     I_jk_a=inv3(i_I_jk_a);
     
     % compute sum of fisher info matrices across freqs on block
     I_k_m=nansum(I_jk_a,3);                    % (3x3) matrix
   
     % compute array of weights
     W_k_a=inv3(I_k_m,I_jk_a);
     
    
     % Compute Local Blockwise Averages of Log-Returns
    %------------------------------------------------------------------

    %empty matrices for local blockwise averages
    lret1_avb=NaN*zeros(J_opts,1);   % (J_opts x1) vector
    lret2_avb=lret1_avb;
    
    
    % loop
        for j=1:J_opts
            
            % spectral weights for given block and frequency
            Phi1=spectr_weights_j_onbl_cont(tb1,hs,j);
            Phi2=spectr_weights_j_onbl_cont(tb2,hs,j);
            
            % local blockwise averages using frequency j
            lret1_avb(j)=lret_b1'*Phi1; 
            lret2_avb(j)=lret_b2'*Phi2;
            
        end
   
    % estimate spot covola for given block as weighted average acr freqs
    %----------------------------------------------------------------------
     
    % select empirical norms of spectral weights
    Phi_norm_v=(i_Phi_norms(1:J_opts).^(-1))';         

     % matrix of dist. elements of outer poduct matrices of local ret avgs of for freqs
    S_jk_m=[(lret1_avb.^2)';(lret1_avb.*lret2_avb)';(lret2_avb.^2)'];   % (3 x J_opts) matrix
    
    % matrix of dist. elements of bias-corr. spot vola matrix estimator on block for all freqs
    if bc>=0
        C_jk_m=(S_jk_m-noise_v(:,ones(J_opts,1))/n_k)./Phi_norm_v(ones(3,1),:); % (3 x J_opts) matrix
    else
        C_jk_m=S_jk_m./Phi_norm_v(ones(3,1),:); % (3 x J_opts) matrix
    end
    
    %  spot vola estimates on block: sum of prod of fisher info and spot vola estimates based on diff freq
    SCov_k_m(:,ks)=nansum(mtimesx(W_k_a,permute(C_jk_m,[1 3 2])),3);   % (3xJ_opts) vector

end
    
%-------------------------------------------------------------------------
%-------------------------------------------------------------------------

% sum up blockwise integr. covola estimates 
lle_cov=SCov_k_m*h_v;   % (3xB)*(Bx1) >>> (3x1) vector

% create (co-volatility) matrix
Sigma_out=[lle_cov(1) lle_cov(2);lle_cov(2) lle_cov(3)];







     











