function Sigma_out=lle_cov_oracle_faster_const_old(data_a1,data_a2,H,startt,endt,W_k_a,W_k_as,Phi_norm_m,h_v,block_st,block_end,J_maxs)
%% lle_cov_oracle_faster_const
%--------------------------------------------------------------
% local likelihood estimation of covolatility matrix 
%  fastwe oracle version for constant vola case
%--------------------------------------------------------------
% INPUT:
% data_a1 -> (n_a x 2) matrix of synchronous data for asset 1
%       - 1st  col: observation times  (sec after midnight)
%       - 2nd col: prices 
% H -> (2x2) covariance matrix of noise
% startt -> start of trading (in hours)
% endt   -> end of trading (in hours)
%W_k_a -> (3x3xJ) array of weight matrices for each frequency J
% W_k_as -> (3x3xJ_maxs) array of weight matrices for each frequency J
% (shorter block)
% Phi_norm_m -> (JxB) matrix of norms of spectral weights
% h_v -> (Bx1) vector of block lengths
% h -> block length
% hs -> length of shorter block
% block_st -> (Bx1) vector of block beginnings
% block_end -> (Bx1) vector of block ends
% J_maxs -> spectral cutoff for shorter block
%--------------------------------------------------------------
% OUTPUT:
% Sigma_out -> (2 x 2) matrix containg estimates of integrated (co-)
% volatilities
% -------------------------------------------------------------------------
%
% Peter Malec, March 7, 2012
%
% -------------------------------------------------------------------------
%       
%% Prepare Data and Inputs

%price  is (n_rft x 2) matrix
price=[data_a1(:,2) data_a2(:,2)];
                                    
% standardize times (time_rft in [0,1])                                    
time_st= (data_a1(:,1)-startt*3600)/((endt-startt)*3600);         

% log-returns
lret=diff(log(price));

% number of obs and times
n=size(price,1)-1;      %adjust for returns
t=time_st(2:end);

% number of blocks and freqs
[J B]=size(Phi_norm_m);

% extract noise (co-)variances
nu2_x=H(1,1);
nu2_y=H(2,2);
nu_xy=H(1,2);


% position of shorter block
%-------------------------------------
[hs, inds]=min(h_v);   % length and position of shorter block

if (sum(h_v==hs)>1)||(B==1) % check if all blocks are the same (or onyl one Block)
    inds=NaN;
end

% -------------------------------------------------------------------------
%       
%% Compute Local Blockwise Averages

%empty matrices for local blockwise averages
lret1_avb=NaN*zeros(B,J);
lret2_avb=lret1_avb;

% start loop over blocks to compute local blockwise averages
%--------------------------------------------------------------------------------------------------
 for i=1:B

    % spectral weights for given block
    Phi=spectr_weights_k(t,block_st(i),block_end(i),h_v(i),J);  % (n x J) matrix
    
    % local blockwise averages
    lret1_avb(i,:)=nansum(repmat(lret(:,1),1,J).*Phi);     % (B x J) matrix
    lret2_avb(i,:)=nansum(repmat(lret(:,2),1,J).*Phi);
           %each freq: multiply return by weight, then sum over returns

 end
 
 % -------------------------------------------------------------------------
%       
%% Compute Local Likelihood Estimator

if J~=1

% create arrays of local blockwise averages
 lret1_avb_a=permute(lret1_avb,[2 3 1]);                    % (J x 1 x B)
 lret2_avb_a=permute(lret2_avb,[2 3 1]);
 
 %arrays of dist. elements of outer poduct matrices of local ret avgs of for freqs
 S_jk_a=[(lret1_avb_a.^2) (lret1_avb_a.*lret2_avb_a) (lret2_avb_a.^2)];          %(Jx3xB)
 
 %array of noise (co-)variances divided by sample size
 nu2_a=repmat([nu2_x nu_xy nu2_y],[J 1 B])/n;                                                %(Jx3xB)
 
 % arrays of various rows of weight matrices
 W_k_a1=repmat(permute(W_k_a(1,:,:),[3 2 1]),[1 1 B]);                                  %(Jx3xB)
 W_k_a2=repmat(permute(W_k_a(2,:,:),[3 2 1]),[1 1 B]);  
 W_k_a3=repmat(permute(W_k_a(3,:,:),[3 2 1]),[1 1 B]);  
 
 % array of empirical norms of spectral weights
 Phi_norm_a=repmat(permute(Phi_norm_m,[1 3 2]),[1 3 1]);
 
 if ~isnan(inds)
    k=inds;     % set index
    normB=setdiff(1:B,k);   % set of "normal" blocks
    
    % treat normal blocks
    %----------------------------------------------------------
    
    % array of spot vola estimates
    C_jk_a=(S_jk_a(:,:,normB)-nu2_a(:,:,normB))./Phi_norm_a(:,:,normB);     %(Jx3xB-1)
    
    % Var1
    LLeV1_n=sum(sum(sum(W_k_a1(:,:,normB).*C_jk_a,2)),3)*h_v(normB(1));
    % cov
    LLeC_n=sum(sum(sum(W_k_a2(:,:,normB).*C_jk_a,2)),3)*h_v(normB(1));
    % Var2
    LLeV2_n=sum(sum(sum(W_k_a3(:,:,normB).*C_jk_a,2)),3)*h_v(normB(1));
    
    % treat short block
    %----------------------------------------------------------
    
     % array of spot vola estimates
     C_jk_as=(S_jk_a(1:J_maxs,:,k)-nu2_a(1:J_maxs,:,k))./Phi_norm_a(1:J_maxs,:,k);     %(J_maxsx3x1)
     
     % arrays of various rows of weight matrices
     W_k_as1=permute(W_k_as(1,:,:),[3 2 1]);                                  %(J_maxs x3x1)
     W_k_as2=permute(W_k_as(2,:,:),[3 2 1]);  
     W_k_as3=permute(W_k_as(3,:,:),[3 2 1]);  
    
    if J_maxs~=1     
  
        % Var1
        LLeV1_s=sum(sum(W_k_as1.*C_jk_as,2))*h_v(k);
        % Cov
        LLeC_s=sum(sum(W_k_as2.*C_jk_as,2))*h_v(k);
        % Var2
        LLeV2_s=sum(sum(W_k_as3.*C_jk_as,2))*h_v(k);
        
    else
        % Var1
        LLeV1_s=C_jk_as(1,1,1)*h_v(k);
        % Cov
        LLeC_s=C_jk_as(1,2,1)*h_v(k);
        % Var2
        LLeV2_s=C_jk_as(1,3,1)*h_v(k);
      
    end
    
    % add estimates from blocks
    %----------------------------------------------------------
    LLeV1=LLeV1_s+LLeV1_n;
    LLeC=LLeC_s+LLeC_n;
    LLeV2=LLeV2_s+LLeV2_n;
    
 else
     
     % treat all blocks
    %----------------------------------------------------------
    
    % array of spot vola estimates
    C_jk_a=(S_jk_a-nu2_a)./Phi_norm_a;     %(Jx3xB)
    
    % Var1
    LLeV1=sum(sum(sum(W_k_a1.*C_jk_a,2)),3)*h_v(1);
    % cov
    LLeC=sum(sum(sum(W_k_a2.*C_jk_a,2)),3)*h_v(1);
    % Var2
    LLeV2=sum(sum(sum(W_k_a3.*C_jk_a,2)),3)*h_v(1);
     
 end
 
 
 % create output matrix
 %----------------------------------------------------------
 Sigma_out=[LLeV1 LLeC;LLeC LLeV2];

%-----------------------------------------------------
%-----------------------------------------------------

else  % if J=1
    
% transpose matrices of local blockwise averages
lret1_avb=lret1_avb';
lret2_avb=lret2_avb';
    
% matrices of inverses of empirical norms of spectral weights 
i_Phi_norm=Phi_norm_m.^(-1);                  % (J x B) matrix

%empty output matrix
Sigma_out=NaN*zeros(2,2);

% Var of asset 1
Sigma_out(1,1)=(sum(lret1_avb.^(2).*i_Phi_norm,1)-nu2_x/n*sum(i_Phi_norm,1))*h_v;          

%Covar of asset 1,2
Sigma_out(1,2)=(sum(lret1_avb.*lret2_avb.*i_Phi_norm,1)-nu_xy/n*sum(i_Phi_norm,1))*h_v;  
Sigma_out(2,1)=Sigma_out(1,2);

% Var of asset 2
Sigma_out(2,2)=(sum(lret2_avb.^(2).*i_Phi_norm,1)-nu2_y/n*sum(i_Phi_norm,1))*h_v;          
    
end










