function [SCov_k_v,J_opt_k]=spot_cov_block_corrd(k,block_st,block_end,h,nrft,avgt1,avgt2,t1,t2,tl1,tl2,lret1,lret2,t_scov,SCov_mat,...
    noise_v,nkex,noise_discr,cv,J_fac,J_all)
%% spot_cov_block_corrd
%--------------------------------------------------------------
% computes spot covolatility estimate on block k
%
% Note:      * norms of spectral weights corrected for discr sampling
%               (only in bias correction -> save comp time)
%               * spectral weights based on cont approximations
%               % only for cross-sectionally independent noise
%--------------------------------------------------------------
% INPUT:
% k-> block index
% block_st -> block start
% block_end -> block end
% h -> block length
% n_rft -> max number of obs of series 1 and 2
% avgt1 -> vector of centered obs times of returns (or times at end of return spells)
% t1-> vector of observation times at end of return spells
% tl1-> vector of lags of t1 (same length)
% lret1   -> vector of log-returns
% t_scov -> (Nx1) vector of times at which spot (co-) variances ([SVar1 SVar1 Scov])
% are observed (N such that times=1/N:1/N:1.)
% SCov_mat -> (N x 3) matrix of spot (co-) variances ([SVar1 SVar1 Scov])
% noise_v -> vector of noise (co-)variances ([nu2_x;0;nu2_y])
% nkex -> if >0, exact number of obs on each block is used to determine
% cutoff
% noise_discr -> if >0, bias correction is done based on finite sample
% cv-> critical value for choice of spectral cutoff (frobenius norm of
% "last" weight matrix <=cv). 
% J_fac -> factor by which "optimal" cutoff (based on frob norm of weights)
%               is divided
% J_all -> if>0: all frequencies j are used 
%--------------------------------------------------------------
% OUTPUT:
% SCov_k_v -> (3x1) vector of spot (co-)volatility estimates on block 
% ([sigma_(x,k)^2;sigma_(xy,k);sigma_(y,k)^2])
% J_opt_k -> optimal spectral cutoff on block
% -------------------------------------------------------------------------
%
% Peter Malec, October 4, 2012 
%
% -------------------------------------------------------------------------
%       
%% extract spot (co-) volatility estimates and noise
    
% (co-) volatility
 if k~=1
       t_scov_ind=find(t_scov<= block_st,1,'last');
       sigma2_x=SCov_mat(t_scov_ind,1);
       sigma2_y=SCov_mat(t_scov_ind,2);
       sigma_xy=SCov_mat(t_scov_ind,3);
   else
       sigma2_x=SCov_mat(1,1);
       sigma2_y=SCov_mat(1,2);
       sigma_xy=SCov_mat(1,3);  %first block
 end
  
 %noise
 nu2_x=noise_v(1);
 nu2_y=noise_v(end);
% -------------------------------------------------------------------------
%     
 %%  Compute number of obs etc on block
  
 % index of observations on each block (based on centers)
     t_ind1=logical(((avgt1 >= block_st) & (avgt1 <=block_end)));  
     t_ind2=logical(((avgt2 >= block_st) & (avgt2 <=block_end)));  
     
    % centered observations times on block (since beginning)
     avgtb1=avgt1(t_ind1)-block_st;
     avgtb2=avgt2(t_ind2)-block_st;
     
     % time and lagged times on block  (since beginning)
     tb1=t1(t_ind1)-block_st;
     tlb1=tl1(t_ind1)-block_st;
      tb2=t2(t_ind2)-block_st;
     tlb2=tl2(t_ind2)-block_st;
     
     % quadratic variation of time on block 
     qvtbx=nansum((tb1-tlb1).^2);
     qvtby=nansum((tb2-tlb2).^2);
     
     % log returns on block
     lret_b1=lret1(t_ind1,:);
     lret_b2=lret2(t_ind2,:);
% -------------------------------------------------------------------------
%        
%% Compute weight matrices
   
    % pilot value of cutoff
    if nkex>0
        n_max=min([length(lret_b1) length(lret_b2)]);
    else
        n_max=floor(nrft*h);
    end
    J_max=max(n_max,1);
    
    % array of weight matrices for each freq j
    [W_k_a,J_opt_k,~]=lle_cov_weight_array(J_max,cv,h,nu2_x,qvtbx,sigma2_x,nu2_y,qvtby,sigma2_y,sigma_xy,J_fac,J_all);
    
%% Compute Local Blockwise Averages of Log-Returns and Emp Phi Norms
   

    %empty matrices for local blockwise averages
    lret1_avb=NaN*zeros(J_opt_k,1);   % (J_opt_kx1) vector
    lret2_avb=lret1_avb;
    
    %empty matrix of empirical norms of Phis 
    Phi_norm_k_m=NaN*zeros(3,J_opt_k);   % (3xJ_opt_k) matrix
    
    %length of overlap of return spells of x,y (else zero)
    mat_ind=return_overlap_faster(tb1,tlb1,tb2,tlb2);
    
    % check if bias correction based on finite sample expr
    if noise_discr>0
        
        %empty matrix of sum of squared weights
        Phi_sum_sq=zeros(3,J_opt_k);   % (3xJ_opt_k) matrix
        
        % loop
        for j=1:J_opt_k
            
            % spectral weights for given block and frequency
            Phi1=spectr_weights_j_onbl_cont(avgtb1,h,j);
            Phi2=spectr_weights_j_onbl_cont(avgtb2,h,j);
            
            % local blockwise averages using frequency j
            lret1_avb(j)=lret_b1'*Phi1; 
            lret2_avb(j)=lret_b2'*Phi2;
            
            % empirical norms of Phis 
            [Phi_norm_jk_x,Phi_norm_jk_y,Phi_norm_jk_xy]=Phi_norm_emp_block_faster(Phi1,Phi2,tb1,tlb1,tb2,tlb2,mat_ind);
            Phi_norm_k_m(:,j)=[Phi_norm_jk_x;Phi_norm_jk_xy;Phi_norm_jk_y];
            
            % sums of squared weights
            Phi_sum_sq(1,j)=sum(Phi1.^2)-sum(Phi1(1:end-1).*Phi1(2:end));
            Phi_sum_sq(3,j)=sum(Phi2.^2)-sum(Phi2(1:end-1).*Phi2(2:end));
            
        end
        
        % create local noise vector for bias correction
        lnoise_v=noise_v(:,ones(J_opt_k,1)).*Phi_sum_sq*2;% (3 x J_opt_k) matrix
        
    else
    
        % loop
        for j=1:J_opt_k
            
            % spectral weights for given block and frequency
            Phi1=spectr_weights_j_onbl_cont(avgtb1,h,j);
            Phi2=spectr_weights_j_onbl_cont(avgtb2,h,j);
            
            % local blockwise averages using frequency j
            lret1_avb(j)=lret_b1'*Phi1; 
            lret2_avb(j)=lret_b2'*Phi2;
            
            % empirical norms of Phis 
            [Phi_norm_jk_x,Phi_norm_jk_y,Phi_norm_jk_xy]=Phi_norm_emp_block_faster(Phi1,Phi2,tb1,tlb1,tb2,tlb2,mat_ind);
            Phi_norm_k_m(:,j)=[Phi_norm_jk_x;Phi_norm_jk_xy;Phi_norm_jk_y];
            
        end
        
        % create local noise vector for bias correction
        lnoise_v=noise_v.*[qvtbx;0;qvtby]/h;  % (3 x 1) vector
        lnoise_v=lnoise_v(:,ones(J_opt_k,1));  % (3 x J_opt_k) matrix
        
    end
% -------------------------------------------------------------------------
%         
%% estimate spot covola for given block as weighted average acr freqs
    
    % matrix of dist. elements of outer poduct matrices of local ret avgs of for freqs
    S_jk_m=[(lret1_avb.^2)';(lret1_avb.*lret2_avb)';(lret2_avb.^2)'];   % (3 x J_opt_k) matrix
    
    % matrix of dist. elements of bias-corr. spot vola matrix estimator on 
    C_jk_m=(S_jk_m-lnoise_v)./Phi_norm_k_m; % (3 x J_opt_k) matrix
   
    %  spot vola estimates on block: sum of prod of fisher info and spot vola estimates based on diff freq
    SCov_k_v=nansum(mtimesx(W_k_a,permute(C_jk_m,[1 3 2])),3);   % (3x1) vector
    
    
    