function I_k_m=info_mat_block_adapt(h,SCov_vec,noise_v,qvtbx,qvtby,J_val)
%% info_mat_block_adapt
%--------------------------------------------------------------
% computes information matrix on block k using pilot estimate
%
% Note:      
%               * spectral weights based on cont approximations
%               % only for cross-sectionally independent noise
%--------------------------------------------------------------
% INPUT:
% h -> block length
% SCov_vec -> (1x3) vector of smoothed spot (co-)volatility estimates on
% blocks (kth row: [sigma_(x,k)^2 sigma_(xy,k) sigma_(y,k)^2])
% noise_v -> vector of noise (co-)variances ([nu2_x;0;nu2_y])
% qvtbx -> quadratic variation of time on block
% J_val -> spectral cutoff
%--------------------------------------------------------------
% OUTPUT:
%  I_k_m  -> (3x3) matrix: sum of fisher info matrices across freqs on
%  block 
% -------------------------------------------------------------------------
%
% Peter Malec, April 19, 2013 
%
% mod May 16, 2013: correction for too few observations on block
%                               included
% -------------------------------------------------------------------------
%       
%% check if there were too few observations on blocks

if  (~isnan(qvtbx))&&(~isnan(qvtby))%normal  
% -------------------------------------------------------------------------
%  
%% extract spot (co-) volatility estimates and noise
    
% spot (co-) variances
sigma2_x=SCov_vec(1);
sigma2_y=SCov_vec(3);
sigma_xy=SCov_vec(2);

% check if corr to close to one
rho_xy=sigma_xy/sqrt(sigma2_x*sigma2_y);
if abs(rho_xy)>(1-1e-5)
    %disp(['Spot correlation on block ' int2str(k) ' too close to one. Correction necessary!'])
    sigma_xy=sign(rho_xy)*(1-1e-5)*sqrt(sigma2_x*sigma2_y);
end                             %replace rho by threshold
    
%noise
 nu2_x=noise_v(1);
 nu2_y=noise_v(end);
 % -------------------------------------------------------------------------
%       
%% Compute elements of inverse fisher information and put into structure

    % frequencies up to spectral cutoff
    j=(1:J_val)';

    % vector of (inverse) empirical norms  (continuous approximations)
    i_Phi_norm=h^(-2)*(pi^2)*j(:).^2;
    i_Phi_norm_2=i_Phi_norm.^(2);
    
    % weights
     i_I_jk_11=2*(i_Phi_norm.*nu2_x*qvtbx/h+sigma2_x).^2;     % (J x 1) matrix

    i_I_jk_22=i_Phi_norm_2.*nu2_x*nu2_y*qvtbx*qvtby/(h^2)+(sigma2_x.*sigma2_y+(sigma_xy).^2)+...
    (i_Phi_norm/h).*(sigma2_x*nu2_y*qvtby+sigma2_y*nu2_x*qvtbx);   % (J x 1) matrix

    i_I_jk_33=2*(i_Phi_norm.*nu2_y*qvtby/h+sigma2_y).^2;     % (J x 1) matrix

    i_I_jk_12=2*sigma2_x.*sigma_xy+...
    2*i_Phi_norm.*sigma_xy*qvtbx*nu2_x/h;   % (J x 1) matrix

    i_I_jk_23=2*sigma2_y.*sigma_xy+...
    2*i_Phi_norm.*sigma_xy*qvtby*nu2_y/h;   % (J x 1) matrix

    i_I_jk_13=2*(sigma_xy).^(2)*ones(J_val,1); % (J x 1) matrix

%% Compute elements of inverse fisher information and weights
   
   % matrix of distinct elements of inverse Fisher infos for all freqs
    i_I_jk_m=[i_I_jk_11';i_I_jk_12';i_I_jk_13';i_I_jk_22'...
        ;i_I_jk_23';i_I_jk_33'];      % (6 x J) matrix
    
    % create array of (symmetric) inverse fisher info matrices for all freqs
     i_I_jk_a=ivech3(i_I_jk_m);             % (3x3xJ) array
     
     % compute array of fisher info matrices by inversion
     I_jk_a=inv3(i_I_jk_a);
     
     % compute sum of fisher info matrices across freqs on block
     I_k_m=nansum(I_jk_a,3);                    % (3x3) matrix
 % -------------------------------------------------------------------------
%         
%% if too few observations
    
else
     I_k_m=NaN*zeros(3,3);
 end   
