function [I_jk_a_out,Phi_norm_m,h_v,h,hs,block_st,block_end,J_opts]=lle_cov_oracle_prep_Jopt(n_rft,H,s_block,SCov_mat,h_fac,J_all)
%% lle_cov_oracle_prep
%--------------------------------------------------------------
% computes inputs for oracle versoin of 
% local likelihood estimator of covolatility matrix 
% based on "optimal" J
%--------------------------------------------------------------
% INPUT:
% n_rft -> number of return observations
% H -> (2x2) covariance matrix of noise
% s_block -> determines which block is shorter: 'first' or 'last' (default)
% SCov_mat -> (N x 3) matrix of spot (co-) variances ([SVar1 SVar1 Scov]),
% where N such that times=1/N:1/N:1.
% Note: each SCov matrix has to be positive definite
% h_fac   -> constant factor for multiplication with "opt" block length
% J_all -> if >0, all frequencies are used, else optimal number (default)
%--------------------------------------------------------------
% OUTPUT:
% I_jk_a_outt -> structure of (J_opt x B) matrices I_jk containing (i,i) elements
% of Fisher info matrices for frequency j and block k
% Phi_norm_m -> (JxB) matrix of norms of spectral weights
% h_v -> (Bx1) vector of block lengths
% h -> block length
% hs -> length of shorter block
% block_st -> (Bx1) vector of block beginnings
% block_end -> (Bx1) vector of block ends
% J_opts -> spectral cutoff for shorter block
% -------------------------------------------------------------------------
%
% Peter Malec, April 18, 2012
%
% May 5, 2012 
% correction in computation of weights
% -------------------------------------------------------------------------
%       
%% Determine block length 

% block length
%-------------------------------------------------

% sample size relative to noise
n_nu=n_rft/mean(diag(H)); %use average noise variance


% block length (rough choice)
h=min([(h_fac*log(n_nu)/sqrt(n_nu)) 1]);


% -------------------------------------------------------------------------
%       
%%  Determine block bounds and spectral cutoff (pilot)

% start and end of each block (# of elements gives # blocks)-
%-----------------------------------------------------------------------

% which block ist shorter?

if isequal(s_block,'first')  % first block
    
    % vector of bounds
    block_v=(fliplr((1:-h:0)))';
    
    if block_v(1)==0         % check if unit interval can be divided in equal blocks of length h
        block_st=block_v(1:end-1);
        block_end=block_v(2:end);
    else                                % first block is shorter !!
     block_st=[0; block_v(1:end-1)];
     block_end=block_v;
    end

    hs=block_end(1)-block_st(1);    %length of shorter block
    
    % block index for short block
        ks=1;
        

else        % last block
    
    % vector of bounds
    block_v=(0:h:1)';

    if block_v(end)==1         % check if unit interval can be divided in equal blocks of length h
        block_st=block_v(1:end-1);
        block_end=block_v(2:end);
    else                                % last block is shorter !!
     block_st=block_v;
     block_end=[block_v(2:end); 1];
    end
    
    hs=block_end(end)-block_st(end);    %length of shorter block
    
    % block index for short block
    ks=length(block_st);

end

% spectral cutoff 
%-------------------------------------------------

% pilot value
J_max=max(floor(n_rft*h),1);    
       
% frequencies up to spectral cutoff
j=(1:J_max)';

% number of blocks
B=length(block_st);
% -------------------------------------------------------------------------
%       
%% Compute Weights for LLE

% vector of empirical norms
Phi_norm=(2*n_rft*sin(j*pi/(2*n_rft*h))).^(-2);                     % (J_max x 1) vector
Phi_norms=(2*n_rft*sin(j*pi/(2*n_rft*hs))).^(-2);                     % for shorter block

% create Phi_norm matrix and block length vector accouting for shorter
% block and B=1
if B~=1
    if isequal(s_block,'first')  % first block
        Phi_norm_m=[Phi_norms repmat(Phi_norm,1,B-1)]; % (J x B) matrix
        h_v=[hs;repmat(h,B-1,1)];                                      % (Bx1) vector
    else
        Phi_norm_m=[repmat(Phi_norm,1,B-1) Phi_norms];  % (J x B) matrix
        h_v=[repmat(h,B-1,1) ;hs];                                      % (Bx1) vector
    end
else   %special case if B=1 
        Phi_norm_m=Phi_norm;
        h_v=h;
        ks=NaN;
end

    
    
    n=n_rft;
    
    % vectors of inverse & squared inv empirical norms of spectral weights 
    i_Phi_norm=Phi_norm.^(-1);  % (J x 1) vector
    i_Phi_norm_2=i_Phi_norm.^(2);
    
    % extract noise (co-)variances
    nu2_x=H(1,1);
    nu2_y=H(2,2);
    nu_xy=H(1,2);
    
    
    
    % Loop for computation of weights
    %----------------------------------------------------------------------
    %----------------------------------------------------------------------
    %----------------------------------------------------------------------
    
    % set of "normal" blocks
    normB=setdiff(1:B,ks);   
    
    % number of Scov obs
    N=size(SCov_mat,1);

    %times at which Spot covolatilities are "observed"
     t_scov=1/N:1/N:1;
    
     % vector for cutoffs
     J_optv=NaN*zeros(B,1);
     
     % matrices for distinct elements of weight matrices
      I_jk_a11=NaN*zeros(J_max,B);
      I_jk_a12=NaN*zeros(J_max,B);
      I_jk_a13=NaN*zeros(J_max,B);
      I_jk_a22=NaN*zeros(J_max,B);
      I_jk_a23=NaN*zeros(J_max,B);
      I_jk_a33=NaN*zeros(J_max,B);
     
     % normal blocks
     %---------------------------------------------------------------------
     %---------------------------------------------------------------------
     
    for k=normB
    
    % extract spot (co-) volatility estimates
    %--------------------------------------------------------

    if k~=1
       t_scov_ind=find(t_scov<= block_st(k),1,'last');
       sigma2_x=SCov_mat(t_scov_ind,1);
       sigma2_y=SCov_mat(t_scov_ind,2);
       sigma_xy=SCov_mat(t_scov_ind,3);
   else
       sigma2_x=SCov_mat(1,1);
       sigma2_y=SCov_mat(1,2);
       sigma_xy=SCov_mat(1,3);  %first block
   end
    
    % Compute elements of inverse fisher information and put into structure
    %-----------------------------------------------------------------------------------

    i_I_jk_11=2*(i_Phi_norm.*nu2_x/n+sigma2_x).^2;     % (J x 1) matrix

    i_I_jk_22=i_Phi_norm_2.*(nu2_x*nu2_y+nu_xy^2)/(n^2)+(sigma2_x.*sigma2_y+(sigma_xy).^2)+...
    (i_Phi_norm/n).*(sigma2_x*nu2_y+sigma2_y*nu2_x+2*sigma_xy*nu_xy);   % (J x 1) matrix

    i_I_jk_33=2*(i_Phi_norm.*nu2_y/n+sigma2_y).^2;     % (J x 1) matrix

    i_I_jk_12=2*sigma2_x.*sigma_xy+i_Phi_norm_2.*2*nu2_x*nu_xy/(n^2)+...
    2*(i_Phi_norm/n).*(sigma2_x*nu_xy+sigma_xy*nu2_x);   % (J x 1) matrix

    i_I_jk_23=2*sigma2_y.*sigma_xy+i_Phi_norm_2.*2*nu2_y*nu_xy/(n^2)+...
    2*(i_Phi_norm/n).*(sigma2_y*nu_xy+sigma_xy*nu2_y);   % (J x 1) matrix

    i_I_jk_13=2*(sigma_xy).^2+i_Phi_norm_2.*2*(nu_xy/n)^2+...
    4*(i_Phi_norm/n).*sigma_xy*nu_xy;   % (J x 1) matrix

    % Compute elements of inverse fisher information and weights
    %----------------------------------------------------------------------
    
    % matrix of distinct elements of inverse Fisher infos for all freqs
    i_I_jk_m=[i_I_jk_11';i_I_jk_12';i_I_jk_13';i_I_jk_22'...
        ;i_I_jk_23';i_I_jk_33'];      % (6 x J) matrix
    
    % create array of (symmetric) inverse fisher info matrices for all freqs
     i_I_jk_a=ivech3(i_I_jk_m);             % (3x3xJ) array
     
     % compute array of fisher info matrices by inversion
     I_jk_a=inv3(i_I_jk_a);
     
     % compute sum of fisher info matrices across freqs on block
     I_k_m=nansum(I_jk_a,3);                    % (3x3) matrix
   
     % compute array of weights
     W_k_a=inv3(I_k_m,I_jk_a);
     
     % store elements
      I_jk_a11(:,k)=squeeze(I_jk_a(1,1,:));
      I_jk_a12(:,k)=squeeze(I_jk_a(1,2,:));
      I_jk_a13(:,k)=squeeze(I_jk_a(1,3,:));
      I_jk_a22(:,k)=squeeze(I_jk_a(2,2,:));
      I_jk_a23(:,k)=squeeze(I_jk_a(2,3,:));
      I_jk_a33(:,k)=squeeze(I_jk_a(3,3,:));
     
     % find cutoff
     J_optv(k)=find_maxJ_const(W_k_a);

    end
%-------------------------------------------------------------------------

% optimal cutoff (global)
%-----------------------------------------------  

% Input value
if nargin<6
    J_all=0;
end
% find cutoff
if J_all<=0
    J_opt=nanmax(J_optv);
else
    J_opt=J_max;
end
J_opts=min([J_opt floor(n_rft*hs)]);

% adjust other outputs
%-----------------------------------------------
I_jk_a11=I_jk_a11(1:J_opt,:);
I_jk_a12=I_jk_a12(1:J_opt,:);
I_jk_a13=I_jk_a13(1:J_opt,:);
I_jk_a22=I_jk_a22(1:J_opt,:);
I_jk_a23=I_jk_a23(1:J_opt,:);
I_jk_a33=I_jk_a33(1:J_opt,:);
Phi_norm_m=Phi_norm_m(1:J_opt,:);

% short block
%-------------------------------------------------
%-------------------------------------------------

if B>1

% vectors of inverse & squared inv empirical norms of spectral weights 
   
    i_Phi_norms=Phi_norms(1:J_opts).^(-1);  % (J_opts x 1) vector
    i_Phi_norms_2=i_Phi_norms.^(2);
    
    
    % extract spot (co-) volatility estimates
    %--------------------------------------------------------
  if ks~=1
       t_scov_ind=find(t_scov<= block_st(ks),1,'last');
       sigma2_x=SCov_mat(t_scov_ind,1);
       sigma2_y=SCov_mat(t_scov_ind,2);
       sigma_xy=SCov_mat(t_scov_ind,3);
   else
       sigma2_x=SCov_mat(1,1);
       sigma2_y=SCov_mat(1,2);
       sigma_xy=SCov_mat(1,3);  %first block
   end
    
    
    

    % Compute elements of inverse fisher information and put into structure
    %-----------------------------------------------------------------------------------

    i_I_jk_11s=2*(i_Phi_norms.*nu2_x/n+sigma2_x).^2;     % (J_maxs x 1) matrix

    i_I_jk_22s=i_Phi_norms_2.*(nu2_x*nu2_y+nu_xy^2)/(n^2)+(sigma2_x.*sigma2_y+(sigma_xy).^2)+...
    (i_Phi_norms/n).*(sigma2_x*nu2_y+sigma2_y*nu2_x+2*sigma_xy*nu_xy);   % (J_maxs x 1) matrix

    i_I_jk_33s=2*(i_Phi_norms.*nu2_y/n+sigma2_y).^2;     % (J_maxs x 1) matrix

    i_I_jk_12s=2*sigma2_x.*sigma_xy+i_Phi_norms_2.*2*nu2_x*nu_xy/(n^2)+...
    2*(i_Phi_norms/n).*(sigma2_x*nu_xy+sigma_xy*nu2_x);   % (J_maxs x 1) matrix

    i_I_jk_23s=2*sigma2_y.*sigma_xy+i_Phi_norms_2.*2*nu2_y*nu_xy/(n^2)+...
    2*(i_Phi_norms/n).*(sigma2_y*nu_xy+sigma_xy*nu2_y);   % (J_maxs x 1) matrix

    i_I_jk_13s=2*(sigma_xy).^2+i_Phi_norms_2.*2*(nu_xy/n)^2+...
    4*(i_Phi_norms/n).*sigma_xy*nu_xy;   % (J_maxs x 1) matrix

    
    % Compute elements of inverse fisher information (short block)
    %----------------------------------------------------------------------
    
    % matrix of distinct elements of inverse Fisher infos for all freqs
    i_I_jk_m=[i_I_jk_11s';i_I_jk_12s';i_I_jk_13s';i_I_jk_22s'...
        ;i_I_jk_23s';i_I_jk_33s'];      % (6 x J_maxs) matrix
    
    % create array of (symmetric) inverse fisher info matrices for all freqs
    i_I_jk_a=ivech3(i_I_jk_m);
    
      % compute array of fisher info matrices by inversion
     I_jk_as=inv3(i_I_jk_a);
     
     % store elements
     I_jk_a11(1:J_opts,ks)=squeeze(I_jk_as(1,1,:));
     I_jk_a12(1:J_opts,ks)=squeeze(I_jk_as(1,2,:));
     I_jk_a13(1:J_opts,ks)=squeeze(I_jk_as(1,3,:));
     I_jk_a22(1:J_opts,ks)=squeeze(I_jk_as(2,2,:));
     I_jk_a23(1:J_opts,ks)=squeeze(I_jk_as(2,3,:));
     I_jk_a33(1:J_opts,ks)=squeeze(I_jk_as(3,3,:));
     
end
     
% create structure for weights
%-------------------------------------------------
%-------------------------------------------------
I_jk_a_out.I_jk_11=I_jk_a11;
I_jk_a_out.I_jk_12=I_jk_a12;
I_jk_a_out.I_jk_13=I_jk_a13;
I_jk_a_out.I_jk_22=I_jk_a22;
I_jk_a_out.I_jk_23=I_jk_a23;
I_jk_a_out.I_jk_33=I_jk_a33;











