function [W_k_a,W_k_as,Phi_norm_m,h_v,h,hs,block_st,block_end,J_maxs]=lle_cov_oracle_prep_const(n_rft,H,s_block,SCov_mat,h_fac,J_choice)
%% lle_cov_oracle_prep
%--------------------------------------------------------------
% computes inputs for oracle versoin of 
% local likelihood estimator of covolatility matrix 
% in case of constant spot volatility
%--------------------------------------------------------------
% INPUT:
% n_rft -> number of return observations
% H -> (2x2) covariance matrix of noise
% s_block -> determines which block is shorter: 'first' or 'last' (default)
% SCov_mat -> (1 x 3) vector of spot (co-) variances ([SVar1 SVar1 Scov]),
% Note: each SCov matrix has to be positive definite
% h_fac   -> constant factor for multiplication with "opt" block length
% J_choice -> choice of spectral cutoff.
% numerical value yields fixed cutoff, 'full' the maximum value,
%'resc' log(n) * n^(1/5.5), otherwise log(n) (default)
%--------------------------------------------------------------
% OUTPUT:
% W_k_a -> (3x3xJ) array of weight matrices for each frequency J
% W_k_as -> (3x3xJMaxs) array of weight matrices for each frequency J
% (shorter block)
% Phi_norm_m -> (JxB) matrix of norms of spectral weights
% h_v -> (Bx1) vector of block lengths
% h -> block length
% hs -> length of shorter block
% block_st -> (Bx1) vector of block beginnings
% block_end -> (Bx1) vector of block ends
% J_maxs -> spectral cutoff for shorter block
% -------------------------------------------------------------------------
%
% Peter Malec, March 7, 2012
%
% -------------------------------------------------------------------------
%       
%% Determine block length 

% block length
%-------------------------------------------------

% sample size relative to noise
n_nu=n_rft/mean(diag(H)); %use average noise variance


% block length (rough choice)
h=min([(h_fac*log(n_nu)/sqrt(n_nu)) 1]);


% -------------------------------------------------------------------------
%       
%%  Determine block bounds and spectral cutoff

% start and end of each block (# of elements gives # blocks)-
%-----------------------------------------------------------------------

% which block ist shorter?

if isequal(s_block,'first')  % first block
    
    % vector of bounds
    block_v=(fliplr((1:-h:0)))';
    
    if block_v(1)==0         % check if unit interval can be divided in equal blocks of length h
        block_st=block_v(1:end-1);
        block_end=block_v(2:end);
    else                                % first block is shorter !!
     block_st=[0; block_v(1:end-1)];
     block_end=block_v;
    end

    hs=block_end(1)-block_st(1);    %length of shorter block

else        % last block
    
    % vector of bounds
    block_v=(0:h:1)';

    if block_v(end)==1         % check if unit interval can be divided in equal blocks of length h
        block_st=block_v(1:end-1);
        block_end=block_v(2:end);
    else                                % last block is shorter !!
     block_st=block_v;
     block_end=[block_v(2:end); 1];
    end
    
    hs=block_end(end)-block_st(end);    %length of shorter block

end

% spectral cutoff 
%-------------------------------------------------

% choose method
if nargin <6
  J_max=max(min(ceil(log(n_nu*h)),floor(n_rft*h)),1);    
        J_maxs=max(min(ceil(log(n_nu*h)),floor(n_rft*hs)),1);    
else
   if isnumeric(J_choice)
        J_max=J_choice;
        J_maxs=J_choice;
    elseif isequal(J_choice,'resc_l')
         J_max=max(min(ceil(log(n_nu*h)*(n_nu*h)^(1/4.5)),floor(n_rft*h)),1);    
        J_maxs=max(min(ceil(log(n_nu*h)*(n_nu*h)^(1/4.5)),floor(n_rft*hs)),1);    
   elseif isequal(J_choice,'all')
         J_max=max(floor(n_rft*h),1);    
        J_maxs=max(floor(n_rft*hs),1);          
   elseif isequal(J_choice,'resc')
        J_max=max(min(ceil(log(n_nu*h)*(n_nu*h)^(1/5.5)),floor(n_rft*h)),1);    
        J_maxs=max(min(ceil(log(n_nu*h)*(n_nu*h)^(1/5.5)),floor(n_rft*hs)),1);    
    else % default method
        J_max=max(min(ceil(log(n_nu*h)),floor(n_rft*h)),1);    
        J_maxs=max(min(ceil(log(n_nu*h)),floor(n_rft*hs)),1);    
   end
end

% frequencies up to spectral cutoff
j=(1:J_max)';

% number of blocks
B=length(block_st);
% -------------------------------------------------------------------------
%       
%% Compute Weights for LLE

% vector of empirical norms
Phi_norm=(2*n_rft*sin(j*pi/(2*n_rft*h))).^(-2);                     % (J_max x 1) vector
Phi_norms=(2*n_rft*sin(j*pi/(2*n_rft*hs))).^(-2);                     % for shorter block

% create Phi_norm matrix and block length vector accouting for shorter
% block and B=1
if B~=1
    if isequal(s_block,'first')  % first block
        Phi_norm_m=[Phi_norms repmat(Phi_norm,1,B-1)]; % (J x B) matrix
        h_v=[hs;repmat(h,B-1,1)];                                      % (Bx1) vector
    else
        Phi_norm_m=[repmat(Phi_norm,1,B-1) Phi_norms];  % (J x B) matrix
        h_v=[repmat(h,B-1,1) ;hs];                                      % (Bx1) vector
    end
else   %special case if B=1 
        Phi_norm_m=Phi_norm;
        h_v=h;
end

if J_max>1  
    
    J=J_max;
    n=n_rft;
    
    % vectors of inverse & squared inv empirical norms of spectral weights 
    i_Phi_norm=Phi_norm.^(-1);  % (J x 1) vector
    i_Phi_norm_2=i_Phi_norm.^(2);
    i_Phi_norms=Phi_norms(1:J_maxs).^(-1);  % (J_maxs x 1) vector
    i_Phi_norms_2=i_Phi_norms.^(2);
    
    % extract noise (co-)variances
    nu2_x=H(1,1);
    nu2_y=H(2,2);
    nu_xy=H(1,2);
    
    % extract spot (co-) volatility estimates
    sigma2_x=SCov_mat(1,1);      
    sigma2_y=SCov_mat(1,2); 
    sigma_xy=SCov_mat(1,3); 

    % Compute elements of inverse fisher information and put into structure
    %-----------------------------------------------------------------------------------

    i_I_jk_11=2*(i_Phi_norm.*nu2_x/n+sigma2_x).^2;     % (J x 1) matrix

    i_I_jk_22=i_Phi_norm_2.*(nu2_x*nu2_y+nu_xy^2)/(n^2)+(sigma2_x.*sigma2_y+(sigma_xy).^2)+...
    (i_Phi_norm/n).*(sigma2_x*nu2_y+sigma2_y*nu2_x+2*sigma_xy*nu_xy);   % (J x 1) matrix

    i_I_jk_33=2*(i_Phi_norm.*nu2_y/n+sigma2_y).^2;     % (J x 1) matrix

    i_I_jk_12=2*sigma2_x.*sigma_xy+i_Phi_norm_2.*2*nu2_x*nu_xy/(n^2)+...
    2*(i_Phi_norm/n).*(sigma2_x*nu_xy+sigma_xy*nu2_x);   % (J x 1) matrix

    i_I_jk_23=2*sigma2_y.*sigma_xy+i_Phi_norm_2.*2*nu2_y*nu_xy/(n^2)+...
    2*(i_Phi_norm/n).*(sigma2_y*nu_xy+sigma_xy*nu2_y);   % (J x 1) matrix

    i_I_jk_13=2*(sigma_xy).^2+i_Phi_norm_2.*2*(nu_xy/n)^2+...
    4*(i_Phi_norm/n).*sigma_xy*nu_xy;   % (J x 1) matrix

    i_I_jk_11s=2*(i_Phi_norms.*nu2_x/n+sigma2_x).^2;     % (J_maxs x 1) matrix

    i_I_jk_22s=i_Phi_norms_2.*(nu2_x*nu2_y+nu_xy^2)/(n^2)+(sigma2_x.*sigma2_y+(sigma_xy).^2)+...
    (i_Phi_norms/n).*(sigma2_x*nu2_y+sigma2_y*nu2_x+2*sigma_xy*nu_xy);   % (J_maxs x 1) matrix

    i_I_jk_33s=2*(i_Phi_norms.*nu2_y/n+sigma2_y).^2;     % (J_maxs x 1) matrix

    i_I_jk_12s=2*sigma2_x.*sigma_xy+i_Phi_norms_2.*2*nu2_x*nu_xy/(n^2)+...
    2*(i_Phi_norms/n).*(sigma2_x*nu_xy+sigma_xy*nu2_x);   % (J_maxs x 1) matrix

    i_I_jk_23s=2*sigma2_y.*sigma_xy+i_Phi_norms_2.*2*nu2_y*nu_xy/(n^2)+...
    2*(i_Phi_norms/n).*(sigma2_y*nu_xy+sigma_xy*nu2_y);   % (J_maxs x 1) matrix

    i_I_jk_13s=2*(sigma_xy).^2+i_Phi_norms_2.*2*(nu_xy/n)^2+...
    4*(i_Phi_norms/n).*sigma_xy*nu_xy;   % (J_maxs x 1) matrix

    % Compute elements of inverse fisher information (all blocks)
    %----------------------------------------------------------------------
    
    % matrix of distinct elements of inverse Fisher infos for all freqs
    i_I_jk_m=[i_I_jk_11';i_I_jk_12';i_I_jk_13';i_I_jk_22'...
        ;i_I_jk_23';i_I_jk_33'];      % (6 x J) matrix
    
    % create array of (symmetric) inverse fisher info matrices for all freqs
     i_I_jk_a=repmat(tril(ones(3)),[1 1 J]);        % (3x3xJ) array
     i_I_jk_a(logical(i_I_jk_a))=permute(i_I_jk_m,[1 3 2]);
     i_I_jk_d=i_I_jk_a(logical(repmat(eye(3),[1 1 J])));
     i_I_jk_a=permute(i_I_jk_a,[2 1 3])+i_I_jk_a;
     i_I_jk_a(logical(repmat(eye(3),[1 1 J])))=i_I_jk_d;
     
     % compute array of fisher info matrices by inversion
     I_jk_a=nd_inv_3x3sym(i_I_jk_a);
     
     % compute sum of fisher info matrices across freqs on block
     I_k_m=nansum(I_jk_a,3);                    % (3x3) matrix
%      
     % compute array of weights
     W_k_a=inv3(I_k_m,I_jk_a);
     
     % Compute elements of inverse fisher information (short block)
    %----------------------------------------------------------------------
    
    % matrix of distinct elements of inverse Fisher infos for all freqs
    i_I_jk_m=[i_I_jk_11s';i_I_jk_12s';i_I_jk_13s';i_I_jk_22s'...
        ;i_I_jk_23s';i_I_jk_33s'];      % (6 x J_maxs) matrix
    
    % create array of (symmetric) inverse fisher info matrices for all freqs
     i_I_jk_a=repmat(tril(ones(3)),[1 1 J_maxs]);        % (3x3xJ_maxs) array
     i_I_jk_a(logical(i_I_jk_a))=permute(i_I_jk_m,[1 3 2]);
     i_I_jk_d=i_I_jk_a(logical(repmat(eye(3),[1 1 J_maxs])));
     i_I_jk_a=permute(i_I_jk_a,[2 1 3])+i_I_jk_a;
     i_I_jk_a(logical(repmat(eye(3),[1 1 J_maxs])))=i_I_jk_d;
     
     % compute array of fisher info matrices by inversion
     I_jk_as=nd_inv_3x3sym(i_I_jk_a);
     
     % compute sum of fisher info matrices across freqs on block
     I_k_ms=nansum(I_jk_as,3);                    % (3x3) matrix
%      
     % compute array of weights
     W_k_as=inv3(I_k_ms,I_jk_as);
     
     

else  % if J=1
    
    W_k_a=0;
    W_k_as=0;
end
    












