function [I_jk_m,Phi_norm,J_max]=lle_cov_oracle_prep_const1(n_rft,H,SCov_mat,J_fac)
%% lle_cov_oracle_prep
%--------------------------------------------------------------
% computes inputs for oracle versoin of 
% local likelihood estimator of covolatility matrix 
% in case of constant spot volatility and one block
%--------------------------------------------------------------
% INPUT:
% n_rft -> number of return observations
% H -> (2x2) covariance matrix of noise
% SCov_mat -> (1 x 3) vector of spot (co-) variances ([SVar1 SVar1 Scov]),
% Note: each SCov matrix has to be positive definite
% J_fac -> choice of spectral cutoff.
%--------------------------------------------------------------
% OUTPUT:
% I_jk_m-> (Jx6) matrix of distinct elements of fisher infos
% Phi_norm_m -> (Jx1) matrix of norms of spectral weights
% J_max -> spectral cutoff 
% -------------------------------------------------------------------------
%
% Peter Malec, March 7, 2012
%
% -------------------------------------------------------------------------
%       
%% Determine block length 

% block length
%-------------------------------------------------

% sample size relative to noise
n_nu=n_rft/mean(diag(H));

% -------------------------------------------------------------------------
%       
%%  Determine spectral cutoff


% spectral cutoff 
%-------------------------------------------------

% choose method

   if isequal(J_fac,'full')
        J_max=n_rft;
   elseif isequal(J_fac,'simp')
          J_max=max(min(ceil(log(n_nu)),n_rft),1);     
   else % default method
        J_max=max(min(ceil(log(n_nu)*n_nu^(1/J_fac)),n_rft),1);     
   end


% frequencies up to spectral cutoff
j=(1:J_max)';
% -------------------------------------------------------------------------
%       
%% Compute Weights for LLE

% vector of empirical norms
Phi_norm=(2*n_rft*sin(j*pi/(2*n_rft))).^(-2);                     % (J_max x 1) vector

if J_max>1  
    
    n=n_rft;
    
    % vectors of inverse & squared inv empirical norms of spectral weights 
    i_Phi_norm=Phi_norm.^(-1);  % (J x 1) vector
    i_Phi_norm_2=i_Phi_norm.^(2);
   
     % extract noise (co-)variances
    nu2_x=H(1,1);
    nu2_y=H(2,2);
    nu_xy=H(1,2);
    
    % extract spot (co-) volatility estimates
    sigma2_x=SCov_mat(1,1);      
    sigma2_y=SCov_mat(1,2); 
    sigma_xy=SCov_mat(1,3); 

    % Compute elements of inverse fisher information and put into structure
    %-----------------------------------------------------------------------------------

    i_I_jk_11=2*(i_Phi_norm.*nu2_x/n+sigma2_x).^2;     % (J x 1) matrix

    i_I_jk_22=i_Phi_norm_2.*(nu2_x*nu2_y+nu_xy^2)/(n^2)+(sigma2_x.*sigma2_y+(sigma_xy).^2)+...
    (i_Phi_norm/n).*(sigma2_x*nu2_y+sigma2_y*nu2_x+2*sigma_xy*nu_xy);   % (J x 1) matrix

    i_I_jk_33=2*(i_Phi_norm.*nu2_y/n+sigma2_y).^2;     % (J x 1) matrix

    i_I_jk_12=2*sigma2_x.*sigma_xy+i_Phi_norm_2.*2*nu2_x*nu_xy/(n^2)+...
    2*(i_Phi_norm/n).*(sigma2_x*nu_xy+sigma_xy*nu2_x);   % (J x 1) matrix

    i_I_jk_23=2*sigma2_y.*sigma_xy+i_Phi_norm_2.*2*nu2_y*nu_xy/(n^2)+...
    2*(i_Phi_norm/n).*(sigma2_y*nu_xy+sigma_xy*nu2_y);   % (J x 1) matrix

    i_I_jk_13=2*(sigma_xy).^2+i_Phi_norm_2.*2*(nu_xy/n)^2+...
    4*(i_Phi_norm/n).*sigma_xy*nu_xy;   % (J x 1) matrix

    % Compute elements of inverse fisher information (all blocks)
    %----------------------------------------------------------------------
    
    % empty matrix of distinct elements of fisher infos for each freq
    I_jk_m=NaN*zeros(J_max,6);
    
    for i=1:J_max
        
    % inverse Fisher infos for given frequency
    i_I_jk_v=[i_I_jk_11(i);i_I_jk_12(i);i_I_jk_13(i);i_I_jk_22(i)...
    ;i_I_jk_23(i);i_I_jk_33(i)];      
    i_I_jk=ivech(i_I_jk_v);
    
    % Fisher info
    I_jk=inv(i_I_jk);
    
    %put distinct elements into matrix
    I_jk_m(i,:)=(vech(I_jk))';
    
    end
    
    
else  % if J=1
    
    I_jk_m=0;
  
end
    












