function [W_k_a,J_opt_k,i_Phi_norm,I_k_m]=lle_cov_weight_array(J_max,cv,h,nu2_x,qvtbx,...
    sigma2_x,nu2_y,qvtby,sigma2_y,sigma_xy,J_fac,J_optv_in,J_all)
%% lle_cov_weight_array
%--------------------------------------------------------------
% computes array of weights (based on Fisher Infos) on block k for LLE
%
% Note:      * spectral weights based on cont approximations
%               % only for cross-sectionally independent noise
%--------------------------------------------------------------
% INPUT:
% J_opts -> max value of spectral cutoff
% cv-> critical value for choice of spectral cutoff (frobenius norm of
% "last" weight matrix <=cv). 
% h -> block length
% nu2_x -> noise variance
% qvtbx -> quadratic variation of time (not transformed)
% sigma2_x -> cosntant spot variance on block k
% sigma2_xy -> cosntant spot covariance on block k
% J_fac -> factor by which "optimal" cutoff (based on frob norm of weights)
%               is divided
% J_optv_in -> optimal cutoff
% J_all -> if>0: all frequencies j are used 
%--------------------------------------------------------------
% OUTPUT:
% W_k_a -> (3x3xJ_opt_k) array of weight matrices for block k
% J_opt_k -> optimal spectral cutoff
% i_Phi_norm -> (J_opt_kx1) vector of (inverse) empirical
% norms  (continuous approximations) of spectral weights
% I_k_m  -> (3x3) matrix: sum of fisher info matrices across freqs on block
% -------------------------------------------------------------------------
%
% Peter Malec, October 4, 2012 
%
% -------------------------------------------------------------------------
%       
%% Compute elements of inverse fisher information and put into structure

    % frequencies up to spectral cutoff
    j=(1:J_max)';

    % vector of (inverse) empirical norms  (continuous approximations)
    i_Phi_norm=h^(-2)*(pi^2)*j(:).^2;
    i_Phi_norm_2=i_Phi_norm.^(2);
    
    % weights
     i_I_jk_11=2*(i_Phi_norm.*nu2_x*qvtbx/h+sigma2_x).^2;     % (J x 1) matrix

    i_I_jk_22=i_Phi_norm_2.*nu2_x*nu2_y*qvtbx*qvtby/(h^2)+(sigma2_x.*sigma2_y+(sigma_xy).^2)+...
    (i_Phi_norm/h).*(sigma2_x*nu2_y*qvtby+sigma2_y*nu2_x*qvtbx);   % (J x 1) matrix

    i_I_jk_33=2*(i_Phi_norm.*nu2_y*qvtby/h+sigma2_y).^2;     % (J x 1) matrix

    i_I_jk_12=2*sigma2_x.*sigma_xy+...
    2*i_Phi_norm.*sigma_xy*qvtbx*nu2_x/h;   % (J x 1) matrix

    i_I_jk_23=2*sigma2_y.*sigma_xy+...
    2*i_Phi_norm.*sigma_xy*qvtby*nu2_y/h;   % (J x 1) matrix

    i_I_jk_13=2*(sigma_xy).^(2)*ones(J_max,1); % (J x 1) matrix

%% Compute elements of inverse fisher information and weights
   
   % matrix of distinct elements of inverse Fisher infos for all freqs
    i_I_jk_m=[i_I_jk_11';i_I_jk_12';i_I_jk_13';i_I_jk_22'...
        ;i_I_jk_23';i_I_jk_33'];      % (6 x J) matrix
    
    % create array of (symmetric) inverse fisher info matrices for all freqs
     i_I_jk_a=ivech3(i_I_jk_m);             % (3x3xJ) array
     
     % compute array of fisher info matrices by inversion
     I_jk_a=inv3(i_I_jk_a);
     
     % compute sum of fisher info matrices across freqs on block
     I_k_m=nansum(I_jk_a,3);                    % (3x3) matrix
   
     % compute array of weights
     W_k_a=inv3(I_k_m,I_jk_a);
     
    % find cutoff
    if (isequal(J_optv_in,0))&&(J_all<=0)
        J_opt_k=max(floor(find_maxJ_const(W_k_a,cv)/J_fac),1);
    else
       J_opt_k=J_max;
    end
   
   % adjust weights 
   %--------------------------------
   if J_opt_k<J_max
     I_jk_a=I_jk_a(:,:,1:J_opt_k);
     I_k_m=nansum(I_jk_a,3);                    % (3x3) matrix
     W_k_a=inv3(I_k_m,I_jk_a);
   end