function [lret1_avb,lret2_avb]=lret_avb(t,block_st,block_end,h,hs,s_block,J,lret)
%------------------------------------------------------------------
%% lret_avb
% Computes local blockwise averages of returns
% following Bibinger & Reiss (2001)
%------------------------------------------------------------------
% INPUTS
% t -> (n x 1) vector of observation times (as fraction of daily trading
% period, i.e. t in [0,1]
% block_st -> vector containing lower bounds of blocks: k*h with
% k=0,...,h^{-1}-1 (note: last block may be of different length)
% block_end -> vector containing upper bounds of blocks: (k+1)*h with
% k=0,...,h^{-1}-1 (note: last block may be of different length)
% h -> length of blocks (assuming t_0=0; T=1), h^{-1} is # of blocks in
% equidistant sampling case (if h^-1 is integer)
% hs -> length of shorter block
% s_block -> determines which block is shorter: 'first' or 'last' (default)
% J ->  spectral cutoff: maximum frequency per block
% lret -> (nx2) matrix of log-returns
%----------------------------------------------------------------------
% OUTPUTS
% lret1_avb -> (J x length(block_st)) matrix of local blockwise averages
%------------------------------------------------------------------
%
%Peter Malec, January 15, 2012
%
%% Preparations

% number of blocks
B=length(block_st);

% determine vector of block lengths
if isequal(s_block,'first')  % first block
    h_v=[hs;repmat(h,B-1,1)];                                      % (Bx1) vector
else
    h_v=[repmat(h,B-1,1); hs];                                      % (Bx1) vector
end
%------------------------------------------------------------------
%
%% Loop over Blocks

%empty matrices for local blockwise averages
lret1_avb=NaN*zeros(B,J);
lret2_avb=lret1_avb;

% start loop 
%------------------
%-------------------

 for i=1:B

    % spectral weights for given block
    Phi=spectr_weights_k(t,block_st(i),block_end(i),h_v(i),J);  % (n x J) matrix
    
    % local blockwise averages
    lret1_avb(i,:)=nansum(repmat(lret(:,1),1,J).*Phi);     % (J x B) matrix
    lret2_avb(i,:)=nansum(repmat(lret(:,2),1,J).*Phi);
           %each freq: multiply return by weight, then sum over returns

 end
 
 % transpose output matrices
 lret1_avb=lret1_avb';
 lret2_avb=lret2_avb';
 







