function Phi=spectr_weights(t,block_st,block_end,h,hs,s_block,J)
%------------------------------------------------------------------
%% spectr_weight
% Computes spectral weights for local blockwise averages of returns
% following Bibinger & Reiss (2001)
%------------------------------------------------------------------
%
% INPUTS
% t -> (n x 1) vector of observation times (as fraction of daily trading
% period, i.e. t in [0,1]
% block_st -> vector containing lower bounds of blocks: k*h with
% k=0,...,h^{-1}-1 (note: last block may be of different length)
% block_end -> vector containing upper bounds of blocks: (k+1)*h with
% k=0,...,h^{-1}-1 (note: last block may be of different length)
% h -> length of blocks (assuming t_0=0; T=1), h^{-1} is # of blocks in
% equidistant sampling case (if h^-1 is integer)
% hs -> length of shorter block
% s_block -> determines which block is shorter: 'first' or 'last' (default)
% J ->  spectral cutoff: maximum frequency per block
%------------------------------------------------------------------
% OUTPUTS
% Phi -> (n x J x K) array of spectral weights
%------------------------------------------------------------------
%
%Peter Malec, December 6, 2011
%
% January 3, 2012
% adjusted for one shorter block (smaller h for first or last block)!!
%
%% Preparations

% # obs
n=length(t);

% frequencies up to spectral cutoff
j=(1:J)';
%------------------------------------------------------------------
%
%% Compute spectral weights

%empty array for weights and numerator of weights
Phi_numer=NaN*zeros(n,J,length(block_st));          % (n x J x K) array
Phi=Phi_numer;

% which block ist shorter?
if isequal(s_block,'first') % first block

    % Loop over blocks for computing numerator
    %-----------------------------------------------
    for i=2:length(block_st)

        numer= sin(pi*(t-block_st(i))/(h)*j');        % (n x J) matrix
    
        % set numerator to 0 for t outside given block
        Phi_numer(:,:,i)=numer.*repmat(((t >= block_st(i)) & (t <block_end(i))),1,J);

    end

    % denominator
    denom=sqrt(2*h)*n*sin(j*pi/(2*n*h));        % (J x 1) vector

    % spectral weights
    Phi(:,:,2:end)=Phi_numer(:,:,2:end)./repmat(denom',[n 1 size(Phi_numer(:,:,2:end),3)]);
                        % divide each row of Phi_numer by transp. of denom vector
                        
    %adjust for shorter block
    numer= sin(pi*(t-block_st(1))/(hs)*j');        % (n x J) matrix
    Phi_numer(:,:,1)=numer.*repmat(((t >= block_st(1)) & (t <block_end(1))),1,J);
    denom=sqrt(2*hs)*n*sin(j*pi/(2*n*hs));        % (J x 1) vector
     Phi(:,:,1)=Phi_numer(:,:,1)./repmat(denom',[n 1 size(Phi_numer(:,:,1),3)]);
     
else % last block
    
    % Loop over blocks for computing numerator
    %-----------------------------------------------
    for i=1:(length(block_st)-1)

        numer= sin(pi*(t-block_st(i))/(h)*j');        % (n x J) matrix
    
        % set numerator to 0 for t outside given block
        Phi_numer(:,:,i)=numer.*repmat(((t >= block_st(i)) & (t <block_end(i))),1,J);

    end

    % denominator
    denom=sqrt(2*h)*n*sin(j*pi/(2*n*h));        % (J x 1) vector

    % spectral weights
    Phi(:,:,1:(end-1))=Phi_numer(:,:,1:(end-1))./repmat(denom',[n 1 size(Phi_numer(:,:,1:(end-1)),3)]);
                        % divide each row of Phi_numer by transp. of denom vector
                        
    %adjust for shorter block
    numer= sin(pi*(t-block_st(end))/(hs)*j');        % (n x J) matrix
    Phi_numer(:,:,end)=numer.*repmat(((t >= block_st(end)) & (t <block_end(end))),1,J);
    denom=sqrt(2*hs)*n*sin(j*pi/(2*n*hs));        % (J x 1) vector
     Phi(:,:,end)=Phi_numer(:,:,end)./repmat(denom',[n 1 size(Phi_numer(:,:,end),3)]);
    
end


                        
