function [W,i_Phi_norm,h_v,h,hs,block_st,block_end,J_maxs]=spectral_est_cov_oracle_prep(n_rft,H,s_block,SCov_mat,h_fac,J_choice)
%% lle_cov_oracle_prep
%--------------------------------------------------------------
% computes inputs for oracle versoin of 
% spectral estimator of covolatility matrix 
%--------------------------------------------------------------
% INPUT:
% n_rft -> number of return observations
% H -> (2x2) covariance matrix of noise
% s_block -> determines which block is shorter: 'first' or 'last' (default)
% SCov_mat -> (N x 3) matrix of spot (co-) variances ([SVar1 SVar1 Scov]),
% where N such that times=1/N:1/N:1.
% Note: each SCov matrix has to be positive definite
% h_fac   -> constant factor for multiplication with "opt" block length
% J_choice -> choice of spectral cutoff.
% numerical value yields fixed cutoff, 'full' the maximum value,
%'resc' log(n) * n^(1/6), otherwise log(n) (default)
%--------------------------------------------------------------
% OUTPUT:
% W -> structure containing weights of spectral freqs for each block
% (each J x B), =0 if J=1
% i_Phi_norm -> (JxB) matrix of inverse norms of spectral weights
% h_v -> (Bx1) vector of block lengths
% h -> block length
% hs -> length of shorter block
% block_st -> (Bx1) vector of block beginnings
% block_end -> (Bx1) vector of block ends
% J_maxs -> spectral cutoff for shorter block
% -------------------------------------------------------------------------
%
% Peter Malec, February 28, 2012
%
% -------------------------------------------------------------------------
%       
%% Determine block length 

% block length
%-------------------------------------------------

% sample size relative to noise
%n_nu=n_rft/mean(diag(H)); %use average noise variance
n_nu=n_rft;

% block length (rough choice)
h=min([(h_fac*log(n_nu)/sqrt(n_nu)) 1]);


% -------------------------------------------------------------------------
%       
%%  Determine block bounds and spectral cutoff

% start and end of each block (# of elements gives # blocks)-
%-----------------------------------------------------------------------

% which block ist shorter?

if isequal(s_block,'first')  % first block
    
    % vector of bounds
    block_v=(fliplr((1:-h:0)))';
    
    if block_v(1)==0         % check if unit interval can be divided in equal blocks of length h
        block_st=block_v(1:end-1);
        block_end=block_v(2:end);
    else                                % first block is shorter !!
     block_st=[0; block_v(1:end-1)];
     block_end=block_v;
    end

    hs=block_end(1)-block_st(1);    %length of shorter block

else        % last block
    
    % vector of bounds
    block_v=(0:h:1)';

    if block_v(end)==1         % check if unit interval can be divided in equal blocks of length h
        block_st=block_v(1:end-1);
        block_end=block_v(2:end);
    else                                % last block is shorter !!
     block_st=block_v;
     block_end=[block_v(2:end); 1];
    end
    
    hs=block_end(end)-block_st(end);    %length of shorter block

end

% spectral cutoff 
%-------------------------------------------------

% choose method
if nargin <6
     J_max=max(min(ceil(log(n_rft)),floor(n_rft*h)),1);     
     J_maxs=max(min(ceil(log(n_rft)),floor(n_rft*hs)),1);    
else
   if isnumeric(J_choice)
        J_max=J_choice;
        J_maxs=J_choice;
    elseif isequal(J_choice,'full')
        J_max=max(floor(n_rft*h),1);     
        J_maxs=max(floor(n_rft*hs),1);     
   elseif isequal(J_choice,'resc')
        J_max=max(min(ceil(log(n_rft)*n_rft^(1/6)),floor(n_rft*h)),1);    
        J_maxs=max(min(ceil(log(n_rft)*n_rft^(1/6)),floor(n_rft*hs)),1);    
    else % default method
        J_max=max(min(ceil(log(n_rft)),floor(n_rft*h)),1);    
        J_maxs=max(min(ceil(log(n_rft)),floor(n_rft*hs)),1);    
   end
end

% frequencies up to spectral cutoff
j=(1:J_max)';

% number of blocks
B=length(block_st);
% -------------------------------------------------------------------------
%       
%% Create Array of Spot Volatility at Beginning of Blocks

if J_max>1       % if J=1, no pilot estimates are needed
    
    % number of Scov obs
    N=size(SCov_mat,1);

    %times at which Spot covolatilities are "observed"
     t_scov=1/N:1/N:1;
    
    % Spot Volatility at Beginning of Blocks taking last value
    %------------------------------------------------------------------
    
    % initialize
    Sigma_kh=NaN*zeros(2,2,B);
    
    % first block
    Sigma_kh(:,:,1)=[SCov_mat(1,1) SCov_mat(1,3);SCov_mat(1,3) SCov_mat(1,2)];
   
    % loop for other blocks
    if B>1
        for k=2:B
            t_scov_ind=find(t_scov<= block_st(k),1,'last');
            Sigma_kh(:,:,k)=[SCov_mat(t_scov_ind,1) SCov_mat(t_scov_ind,3);SCov_mat(t_scov_ind,3) SCov_mat(t_scov_ind,2)];
        end
    end
    
else
    
   
    Sigma_kh=0;
    
end
    
% -------------------------------------------------------------------------
%       
%% Compute Weights for LLE

% vector of empirical norms
Phi_norm=(2*n_rft*sin(j*pi/(2*n_rft*h))).^(-2);                     % (J_max x 1) vector
Phi_norms=(2*n_rft*sin(j*pi/(2*n_rft*hs))).^(-2);                     % for shorter block

% create Phi_norm matrix and block length vector accouting for shorter
% block and B=1
if B~=1
    if isequal(s_block,'first')  % first block
        Phi_norm_m=[Phi_norms repmat(Phi_norm,1,B-1)]; % (J x B) matrix
        h_v=[hs;repmat(h,B-1,1)];                                      % (Bx1) vector
    else
        Phi_norm_m=[repmat(Phi_norm,1,B-1) Phi_norms];  % (J x B) matrix
        h_v=[repmat(h,B-1,1) ;hs];                                      % (Bx1) vector
    end
else   %special case if B=1 
        Phi_norm_m=Phi_norm;
        h_v=h;
end

if J_max>1  
    
    J=J_max;
    n=n_rft;
    
    % matrices of inverses and squared inverses of empirical norms of spectral weights 
    i_Phi_norm=Phi_norm_m.^(-1);                  % (J x B) matrix
    i2_Phi_norm=Phi_norm_m.^(-2);                 % (J x B) matrix
    
    % extract noise (co-)variances
    nu2_x=H(1,1);
    nu2_y=H(2,2);
    nu_xy=H(1,2);
    
    % extract spot (co-) volatility estimates
    sigma2_x=squeeze(Sigma_kh(1,1,:));      % (B x 1) vector
    sigma2_y=squeeze(Sigma_kh(2,2,:)); 
    sigma_xy=squeeze(Sigma_kh(1,2,:)); 

    % Compute weights of spectral freqs for each block
    %-----------------------------------------------------------------------------------

    % Var of asset 1
    Num_x=(i_Phi_norm*(nu2_x/n)+repmat(sigma2_x',J,1)).^(-2);    % (J x B) matrix
    W.W_x=Num_x./repmat(sum(Num_x),J,1);           % sum over freqs in denominator (J x B) matrix    

    %Covar of asset 1,2
    Num_xy=(i2_Phi_norm*(nu2_x*nu2_y+nu_xy^2)/(n^2)+repmat((sigma2_x'.*sigma2_y'+(sigma_xy').^2),J,1)+...
           i_Phi_norm.*repmat((sigma2_x'*nu2_y/n+sigma2_y'*nu2_x/n+2*sigma_xy'*nu_xy/n),J,1)).^(-1);          % (J x B) matrix
    W.W_xy=Num_xy./repmat(sum(Num_xy),J,1);           % sum over freqs in denominator (J x B) matrix    

    % Var of asset 2
    Num_y=(i_Phi_norm*(nu2_y/n)+repmat(sigma2_y',J,1)).^(-2);          % (J x B) matrix
    W.W_y=Num_y./repmat(sum(Num_y),J,1);           % sum over freqs in denominator (J x B) matrix    

else  % if J=1
    
    % matrices of inverses of empirical norms of spectral weights 
    i_Phi_norm=Phi_norm_m.^(-1);                  % (J x B) matrix
    
    % weights of spectral freqs for each block
    W.W_x=ones(1,B);
    W.W_xy=W.W_x;
    W.W_y=W.W_x;
    
end
    












