function [Sigma_k,lret_avb_m,lnoise_v]=spot_cov_block_EQW_md(block_st,block_end,h,avgtime,time,time_l,lret,noise_v,J_val,ccorr)
%% spot_cov_block_EQW_md
%--------------------------------------------------------------
% computes spot covolatility estimate on block k with equal weights 
% for each frequency j in m-dimensional case
%
% Note:      * norms of spectral weights uncorrected (based on continuous approximations)
%               * spectral weights based on cont approximations
%               * only for cross-sectionally independent noise
%--------------------------------------------------------------
% INPUT:
% block_st -> block start
% block_end -> block end
% h -> block length
% avgtime -> (mx1) cell array of centered obs times of returns (or times at end of return spells)
% time-> (mx1) cell array of observation times at end of return spells
% time_l-> (mx1) cell array of lags of time (same length)
% lret   -> (mx1) cell array of log-return (NOTE: each of the above arrays contains column vectors of observations)
% noise_v -> (mx1) vector of noise variance estimates
% J_val -> spectral cutoff
% ccorr -> if >0, correlation and variance estimates are corrected
% (default: =0)
%--------------------------------------------------------------
% OUTPUT:
% Sigma_k -> (mxm) matrix of spot (co-)volatility estimates on block 
%  lret_avb_m -> (J_valxm) matrix of local spectral means of assets
% lnoise_v-> (mx1) vector of local noise level estimates
% -------------------------------------------------------------------------
%
% Peter Malec, January 8, 2013 
%
% -------------------------------------------------------------------------
%       
%% Preparations

% check for correction
if nargin<10
    ccorr=0;
end

% number of assets
m=length(lret);
% -------------------------------------------------------------------------
%  
%% create weight array

% weight array
weightm_j=eye(m^2)*(1/J_val);
W_k_a=weightm_j(:,:,ones(J_val,1));
% -------------------------------------------------------------------------
%     
 %%  Compute number of obs etc on block
 
 
 % loop across assets
%------------------------------------------------------------------
time_ind=cell(m,1);
avgtimeb=cell(m,1);
lret_b=cell(m,1);
qvtb=NaN*ones(m,1);
n_obs_b=NaN*ones(m,1);

for i=1:m
    
    %select
    avgtime_sel=avgtime{i};
    lret_sel=lret{i};
    time_sel=time{i};
    time_l_sel=time_l{i};
    
    % index of observations on each block (based on centers)
    time_ind{i}=logical(((avgtime_sel >= block_st) & (avgtime_sel <=block_end)));
    
    % centered observations times on block (since beginning)
    avgtimeb{i}=avgtime_sel(time_ind{i})-block_st;
    
    % quadratic variation of time on block 
    qvtb(i)=nansum((time_sel(time_ind{i})-time_l_sel(time_ind{i})).^2);
    
    % log returns on block
    lret_b{i}=lret_sel(time_ind{i});
    
    %  no obs
    n_obs_b(i)=length(lret_b{i});
    
end
%
% -------------------------------------------------------------------------
    
 %% check number of obs
 
 n_min=min(n_obs_b);
if (n_min>3) %normal
% -------------------------------------------------------------------------
%        
%% Compute Local Blockwise Averages of Log-Returns and Emp Phi Norms
%    
% check if cutoff too large

% if J_val>n_min
%    disp(['Specified pilot cutoff too large on block' num2str(round(block_end/h)) '!'])
% end

%empty matrix for local blockwise averages
lret_avb_m=NaN*zeros(J_val,m); % (J_valxm) matrix

% loop over assets
for i=1:m
        
     %select
      lret_b_sel=lret_b{i};
      avgtimeb_sel=avgtimeb{i};
            
          % loop over frequencies
          for j=1:J_val
                
              % local blockwise averages using frequency j
               lret_avb_m(j,i)=lret_b_sel'*spectr_weights_j_onbl_cont(avgtimeb_sel,h,j);
                
          end
          
end
         
% create local noise vector for bias correction
lnoise_v=noise_v(:).*qvtb(:)/h;  % (m x 1) vector

% vector of (inverse) empirical norms  (continuous approximations)
j=(1:J_val)';
i_Phi_norm=h^(-2)*(pi^2)*j(:).^2;
% -------------------------------------------------------------------------
%         
%% estimate spot covola for given block as weighted average acr freqs

% compute 3d-array containing matrix of products of local blockwise averages
S_arr=permute(lret_avb_m,[2 3 1]); %(mx1xJ) array
SS_arr=mtimesx(S_arr,S_arr,'t');  %(mxmxJ) array

% matrix of dist. elements of bias-corr. spot vola matrix estimator on 
lnoise_arr=diag(lnoise_v); % (mxm)
lnoise_arr=lnoise_arr(:,:,ones(J_val,1)); % (mxmxJ)
i_Phi_norm_arr=permute(i_Phi_norm,[2 3 1]); % (1x1xJ)
V_arr=(SS_arr-lnoise_arr).*i_Phi_norm_arr(ones(m,1),ones(m,1),:);

%  spot vola estimates on block: sum of prod of fisher info and spot vola estimates based on diff freq
Sigma_k=nansum(mtimesx(W_k_a,reshape(V_arr,[m^2 1 J_val])),3);   % (m^2x1) vector
Sigma_k=reshape(Sigma_k,m,m);

% correction of correlation and variance estimates 
if ccorr>0
    [Sigma_k,~]=check_spot_covm_md(Sigma_k); 
end
% -------------------------------------------------------------------------
%         
%% if too few observations
    
else
    
%      disp(['Less than three observations in block' num2str(round(block_end/h)) '!'])
     
     % number of assets
    m=length(lret);
   
     % empty outputs
    Sigma_k=zeros(m,m);
     lret_avb_m=NaN*zeros(J_val,m);
     lnoise_v=NaN*zeros(m,1);
     
end
    
    