function [SCov_k_v,J_opt_k,I_k_m,Phi_norm_k_m,lret1_avb,lret2_avb,qvtbx,qvtby,lnoise_v_out,J_max]=spot_cov_block_adapt_corrd(block_st,block_end,h,nrft,avgt1,avgt2,t1,t2,tl1,tl2,lret1,lret2,...
    lret1_avb_v,lret2_avb_v,Phi_norm_k_x_v,Phi_norm_k_xy_v,Phi_norm_k_y_v,SCov_vec,...
    noise_v,nkex,cv,J_fac,J_optv_in,J_all)
%% spot_cov_block_adapt_corrd
%--------------------------------------------------------------
% computes spot covolatility estimate on block k using pilot estimate
%
% Note:      * norms of spectral weights corrected for discr sampling
%               (only in bias correction -> save comp time)
%               * spectral weights based on cont approximations
%               % only for cross-sectionally independent noise
%--------------------------------------------------------------
% INPUT:
% k-> block index
% block_st -> block start
% block_end -> block end
% h -> block length
% n_rft -> max number of obs of series 1 and 2
% avgt1 -> vector of centered obs times of returns (or times at end of return spells)
% t1-> vector of observation times at end of return spells
% tl1-> vector of lags of t1 (same length)
% lret1   -> vector of log-returns
% SCov_vec -> (1x3) vector of smoothed spot (co-)volatility estimates on
% blocks (kth row: [sigma_(x,k)^2 sigma_(xy,k) sigma_(y,k)^2])
% lret1_avb_v -> (1xJ_pilot) vector of local spectral means of asset 1
% Phi_norm_k_x_v -> (1xJ_pilot) vector of empirical norms of Phis
% SCov_vec-> (1x3) vector of spot (co-)volatility pilot estimates on
%               block ([sigma_(x,k)^2 sigma_(xy,k) sigma_(y,k)^2])
% noise_v -> vector of noise (co-)variances ([nu2_x;0;nu2_y])
% nkex -> if >0, exact number of obs on each block is used to determine
% cutoff
% cv-> critical value for choice of spectral cutoff (frobenius norm of
%       "last" weight matrix <=cv). 
% J_fac -> factor by which "optimal" cutoff (based on frob norm of weights)
%               is divided
% J_optv_in -> optimal cutoff
% J_all -> if>0: all frequencies j are used 
%--------------------------------------------------------------
% OUTPUT:
% SCov_k_v -> (3x1) vector of spot (co-)volatility estimates on block 
% ([sigma_(x,k)^2;sigma_(xy,k);sigma_(y,k)^2])
% J_opt_k -> optimal spectral cutoff on block
%  I_k_m  -> (3x3) matrix: sum of fisher info matrices across freqs on
%  block 
% Phi_norm_k_m -> (3xJ_val) matrix of empirical norms of Phis
%  lret1_avb -> (J_valx1) vector of local spectral means of asset 1
% qvtbx -> quadratic variation of time on block
% lnoise_v -> (3x1) vector of local noise level
% J_max -> pilot value of cutoff
% -------------------------------------------------------------------------
%
% Peter Malec, October 6, 2012 
%
% -------------------------------------------------------------------------
%       
%% extract spot (co-) volatility estimates and noise
    
% spot (co-) variances
sigma2_x=SCov_vec(1);
sigma2_y=SCov_vec(3);
sigma_xy=SCov_vec(2);

% check if corr to close to one
rho_xy=sigma_xy/sqrt(sigma2_x*sigma2_y);
if abs(rho_xy)>(1-1e-5)
    %disp(['Spot correlation on block ' int2str(k) ' too close to one. Correction necessary!'])
    sigma_xy=sign(rho_xy)*(1-1e-5)*sqrt(sigma2_x*sigma2_y);
end                             %replace rho by threshold
    
%noise
 nu2_x=noise_v(1);
 nu2_y=noise_v(end);
% -------------------------------------------------------------------------
%     
 %%  Compute number of obs etc on block
  
 % index of observations on each block (based on centers)
     t_ind1=logical(((avgt1 >= block_st) & (avgt1 <=block_end)));  
     t_ind2=logical(((avgt2 >= block_st) & (avgt2 <=block_end)));  
     
    % centered observations times on block (since beginning)
     avgtb1=avgt1(t_ind1)-block_st;
     avgtb2=avgt2(t_ind2)-block_st;
     
     % time and lagged times on block  (since beginning)
     tb1=t1(t_ind1)-block_st;
     tlb1=tl1(t_ind1)-block_st;
      tb2=t2(t_ind2)-block_st;
     tlb2=tl2(t_ind2)-block_st;
     
     % quadratic variation of time on block 
     qvtbx=nansum((tb1-tlb1).^2);
     qvtby=nansum((tb2-tlb2).^2);
     
     % log returns on block
     lret_b1=lret1(t_ind1,:);
     lret_b2=lret2(t_ind2,:);
% -------------------------------------------------------------------------
%        
%% Compute weight matrices

    % max value of cutoff
    if nkex>0
            n_max=min([length(lret_b1) length(lret_b2)]);
    else
            n_max=floor(nrft*h);
    end
   
    % pilot value of cutoff
    if J_optv_in==0
        J_max=max(n_max,1);
    else
        J_max=min(J_optv_in,max(n_max,1));
    end
    
    % array of weight matrices for each freq j
    [W_k_a,J_opt_k,~,I_k_m]=lle_cov_weight_array(J_max,cv,h,nu2_x,qvtbx,sigma2_x,nu2_y,qvtby,sigma2_y,sigma_xy,J_fac,J_optv_in,J_all);
    
%% Compute Local Blockwise Averages of Log-Returns and Emp Phi Norms

% pilot cutoff
J_pilot=length(lret1_avb_v);
   
% do more computations
%-------------------------------------
if J_opt_k>J_pilot
    J_add=J_opt_k-J_pilot;

    %empty matrices for local blockwise averages
    lret1_avb=[lret1_avb_v(:);NaN*zeros(J_add,1)];   % (J_opt_kx1) vector
   lret2_avb=[lret2_avb_v(:);NaN*zeros(J_add,1)];   % (J_opt_kx1) vector
    
    %empty matrix of empirical norms of Phis 
    Phi_norm_k_m=[[Phi_norm_k_x_v(:) Phi_norm_k_xy_v(:) Phi_norm_k_y_v(:)]'...
        NaN*zeros(3,J_add)];% (3xJ_opt_k) matrix

    %length of overlap of return spells of x,y (else zero)
    mat_ind=return_overlap_faster(tb1,tlb1,tb2,tlb2);
   
    % loop
        for j=(J_pilot+1):J_opt_k
            
            % spectral weights for given block and frequency
            Phi1=spectr_weights_j_onbl_cont(avgtb1,h,j);
            Phi2=spectr_weights_j_onbl_cont(avgtb2,h,j);
            
            % local blockwise averages using frequency j
            lret1_avb(j)=lret_b1'*Phi1; 
            lret2_avb(j)=lret_b2'*Phi2;
            
            % empirical norms of Phis 
            [Phi_norm_jk_x,Phi_norm_jk_y,Phi_norm_jk_xy]=Phi_norm_emp_block_faster(Phi1,Phi2,tb1,tlb1,tb2,tlb2,mat_ind);
            Phi_norm_k_m(:,j)=[Phi_norm_jk_x;Phi_norm_jk_xy;Phi_norm_jk_y];
            
        end
        
    
else  % no more computations
         %-------------------------------------

    
     % vectors of local blockwise averages
    lret1_avb=lret1_avb_v(1:J_opt_k);   % (J_opt_kx1) vector
    lret1_avb=lret1_avb(:);
    lret2_avb=lret2_avb_v(1:J_opt_k);    % (J_opt_kx1) vector
    lret2_avb=lret2_avb(:);
    
    %empty matrix of empirical norms of Phis 
    Phi_norm_k_m=[Phi_norm_k_x_v(:) Phi_norm_k_xy_v(:) Phi_norm_k_y_v(:)]';
                                        % (3xJ_pilot) matrix
    Phi_norm_k_m=Phi_norm_k_m(:,1:J_opt_k);                              
    
    
end

% create local noise vector for bias correction
 lnoise_v_out=noise_v.*[qvtbx;0;qvtby]/h;  % (3 x 1) vector
lnoise_v=lnoise_v_out(:,ones(J_opt_k,1));  % (3 x J_opt_k) matrix
% -------------------------------------------------------------------------
%         
%% estimate spot covola for given block as weighted average acr freqs
    
    % matrix of dist. elements of outer poduct matrices of local ret avgs of for freqs
    S_jk_m=[(lret1_avb.^2)';(lret1_avb.*lret2_avb)';(lret2_avb.^2)'];   % (3 x J_opt_k) matrix
    
    % matrix of dist. elements of bias-corr. spot vola matrix estimator on 
    C_jk_m=(S_jk_m-lnoise_v)./Phi_norm_k_m; % (3 x J_opt_k) matrix
   
    %  spot vola estimates on block: sum of prod of fisher info and spot vola estimates based on diff freq
    SCov_k_v=nansum(mtimesx(W_k_a,permute(C_jk_m,[1 3 2])),3);   % (3x1) vector
    
    
    