function [Sigma_k,I_k_m,lret_avb_m,J_out]=spot_cov_block_adapt_md(block_st,block_end,h,n_min,avgtime,...
    lret,lret_avb_m,Sigma_p,lnoise_v,nkex,J_val,ccorr)
%% spot_cov_block_adapt_md
%--------------------------------------------------------------
% computes spot covolatility estimate on block k using pilot estimate
% in m-dimensional case
%
% Note:      * blocks of equal size and inputs fixed
%               * norms of spectral weights uncorrected (based on continuous approximations)
%               * spectral weights based on cont approximations
%               % only for cross-sectionally independent noise
%--------------------------------------------------------------
% INPUT:
% block_st -> block start
% block_end -> block end
% h -> block length
% n_min -> minimum number of obs of all series
% avgtime -> (mx1) cell array of centered obs times of returns (or times at end of return spells)
% lret   -> (mx1) cell array of log-return (NOTE: each of the above arrays contains column vectors of observations)
% Sigma_p -> pilot estimate of (mxm) spot covariance matrix on block
% lret_avb_m-> (J_pilotxm) matrix of local spectral means of assets
% lnoise_v -> (mx1) vector of local noise level estimates
% nkex -> if >0, exact number of obs on each block is used to determine
% cutoff
% J_val -> input value for cutoff 
% ccorr -> if >0, correlation and variance estimates are corrected
%--------------------------------------------------------------
% OUTPUT:
% Sigma_k -> (mxm) matrix of spot (co-)volatility estimates on block 
%  I_k_m  -> (m^2xm^2) matrix: sum of fisher info matrices across freqs on
%  block 
%  lret_avb_m -> (J_valxm) matrix of local spectral means of assets
% J_out -> cutoff value actually used
% -------------------------------------------------------------------------
%
% Peter Malec, January 8, 2013 
%
% -------------------------------------------------------------------------
%       
%% check if there were too few observations on blocks

if (sum(sum(isnan(lret_avb_m)))==0)%normal
% -------------------------------------------------------------------------
%  
%%  Compute number of obs etc on block

% number of assets
m=length(lret);
   
% loop across assets
%------------------------------------------------------------------
time_ind=cell(m,1);
avgtimeb=cell(m,1);
lret_b=cell(m,1);
n_obs_b=NaN*ones(m,1);

for i=1:m
    
    %select
    avgtime_sel=avgtime{i};
    lret_sel=lret{i};
    
    % index of observations on each block (based on centers)
    time_ind{i}=logical(((avgtime_sel >= block_st) & (avgtime_sel <=block_end)));
    
    % centered observations times on block (since beginning)
    avgtimeb{i}=avgtime_sel(time_ind{i})-block_st;
    
    % log returns on block
    lret_b{i}=lret_sel(time_ind{i});
    
    %  no obs
    n_obs_b(i)=length(lret_b{i});
    
end
% -------------------------------------------------------------------------
%        
%% Compute weight matrices

% max value of cutoff
if nkex>0
     n_max=min(n_obs_b);
else
     n_max=floor(n_min*h);
end
   
% actual value of cutoff
J_out=min(J_val,max(n_max,1));
% if J_out<J_val
%    disp(['Specified cutoff too large on block' num2str(round(block_end/h)) '!'])
% end
    
% array of weight matrices for each freq j
[W_k_a,i_Phi_norm,I_k_m]=lle_cov_weight_array_md(J_out,h,lnoise_v,Sigma_p);
% -------------------------------------------------------------------------
%    
%% Compute Local Blockwise Averages of Log-Returns and Emp Phi Norms

% pilot cutoff
J_pilot=size(lret_avb_m,1);
   
% do more computations
%-------------------------------------
if J_out>J_pilot
    J_add=J_out-J_pilot;

    %empty matrix for local blockwise averages
    lret_avb_m=[lret_avb_m;NaN*zeros(J_add,m)]; % (J_outxm) matrix
    
    % loop over assets
    for i=1:m
        
        %select
        lret_b_sel=lret_b{i};
        avgtimeb_sel=avgtimeb{i};
            
            % loop over frequencies
            for j=(J_pilot+1):J_out
                
                % local blockwise averages using frequency j
                lret_avb_m(j,i)=lret_b_sel'*spectr_weights_j_onbl_cont(avgtimeb_sel,h,j);
                
            end
              
     end
           
else  % no more computations
         %-------------------------------------
    
     % matrix of local blockwise averages
    lret_avb_m=lret_avb_m(1:J_out,:);   % (J_outxm) matrix 
    
end
% -------------------------------------------------------------------------
%  
%% Compute Spot Covariance Estimate on Block as weighted average acr freqs

% compute 3d-array containing matrix of products of local blockwise averages
S_arr=permute(lret_avb_m,[2 3 1]); %(mx1xJ) array
SS_arr=mtimesx(S_arr,S_arr,'t');  %(mxmxJ) array

% matrix of dist. elements of bias-corr. spot vola matrix estimator on 
lnoise_arr=diag(lnoise_v); % (mxm)
lnoise_arr=lnoise_arr(:,:,ones(J_out,1)); % (mxmxJ)
i_Phi_norm_arr=permute(i_Phi_norm,[2 3 1]); % (1x1xJ)
V_arr=(SS_arr-lnoise_arr).*i_Phi_norm_arr(ones(m,1),ones(m,1),:);

%  spot vola estimates on block: sum of prod of fisher info and spot vola estimates based on diff freq
Sigma_k=nansum(mtimesx(W_k_a,reshape(V_arr,[m^2 1 J_out])),3);   % (m^2x1) vector
Sigma_k=reshape(Sigma_k,m,m);

% correction of correlation and variance estimates 
if ccorr>0
    [Sigma_k,~]=check_spot_covm_md(Sigma_k); 
end
% -------------------------------------------------------------------------
%         
%% if too few observations
    
else
    
    % number of assets
    m=length(lret);
   
    % empty outputs
    Sigma_k=zeros(m,m);
     J_out=J_val;
     I_k_m=NaN*zeros(m^2,m^2);
     lret_avb_m=NaN*zeros(J_val,m);
 end
