function [SCov_k_v,I_k_m]=spot_cov_block_adapt_rweights_corrd(h,SCov_vec,...
    noise_v,cv,J_fac,J_optv_in,J_all,Phi_norm_k_m,lret1_avb,lret2_avb,qvtbx,qvtby,lnoise_v,J_max)
%% spot_cov_block_adapt_rweights_corrd
%--------------------------------------------------------------
% computes spot covolatility estimate on block k using pilot estimate
% based on re-evaluated weights
%
% Note:      * norms of spectral weights corrected for discr sampling
%               (only in bias correction -> save comp time)
%               * spectral weights based on cont approximations
%               % only for cross-sectionally independent noise
%               % computation has to be the same as in 
%                   "spot_cov_block_adapt_corrd"
%--------------------------------------------------------------
% INPUT:
% h -> block length
% SCov_vec-> (1x3) vector of spot (co-)volatility pilot estimates on
%               block ([sigma_(x,k)^2 sigma_(xy,k) sigma_(y,k)^2])
% noise_v -> vector of noise (co-)variances ([nu2_x;0;nu2_y])
% cv-> critical value for choice of spectral cutoff (frobenius norm of
%       "last" weight matrix <=cv). 
% J_fac -> factor by which "optimal" cutoff (based on frob norm of weights)
%               is divided
% J_optv_in -> optimal cutoff
% J_all -> if>0: all frequencies j are used 
% Phi_norm_k_m -> (3xJ_val) matrix of empirical norms of Phis
%  lret1_avb -> (J_valx1) vector of local spectral means of asset 1
% qvtbx -> quadratic variation of time on block
% lnoise_v -> (3x1) vector of local noise level
% J_max -> pilot value of cutoff
%--------------------------------------------------------------
% OUTPUT:
% SCov_k_v -> (3x1) vector of spot (co-)volatility estimates on block 
% ([sigma_(x,k)^2;sigma_(xy,k);sigma_(y,k)^2])
%  I_k_m  -> (3x3) matrix: sum of fisher info matrices across freqs on
%  block 
% -------------------------------------------------------------------------
%
% Peter Malec, April 19, 2013 
%
% -------------------------------------------------------------------------
%       
%% extract spot (co-) volatility estimates and noise
    
% spot (co-) variances
sigma2_x=SCov_vec(1);
sigma2_y=SCov_vec(3);
sigma_xy=SCov_vec(2);

% check if corr to close to one
rho_xy=sigma_xy/sqrt(sigma2_x*sigma2_y);
if abs(rho_xy)>(1-1e-5)
    %disp(['Spot correlation on block ' int2str(k) ' too close to one. Correction necessary!'])
    sigma_xy=sign(rho_xy)*(1-1e-5)*sqrt(sigma2_x*sigma2_y);
end                             %replace rho by threshold
    
%noise
 nu2_x=noise_v(1);
 nu2_y=noise_v(end);
% -------------------------------------------------------------------------
%     
%% Compute weight matrices

% array of weight matrices for each freq j
[W_k_a,J_opt_k,~,I_k_m]=lle_cov_weight_array(J_max,cv,h,nu2_x,qvtbx,sigma2_x,nu2_y,qvtby,sigma2_y,sigma_xy,J_fac,J_optv_in,J_all);
% -------------------------------------------------------------------------
%         
%% estimate spot covola for given block as weighted average acr freqs

% replicate local noise vector
lnoise_v=lnoise_v(:,ones(J_opt_k,1));  % (3 x J_opt_k) matrix
    
% matrix of dist. elements of outer poduct matrices of local ret avgs of for freqs
S_jk_m=[(lret1_avb(:).^2)';(lret1_avb(:).*lret2_avb(:))';(lret2_avb(:).^2)'];   % (3 x J_opt_k) matrix

% matrix of dist. elements of bias-corr. spot vola matrix estimator on 
C_jk_m=(S_jk_m-lnoise_v)./Phi_norm_k_m; % (3 x J_opt_k) matrix
   
%  spot vola estimates on block: sum of prod of fisher info and spot vola estimates based on diff freq
SCov_k_v=nansum(mtimesx(W_k_a,permute(C_jk_m,[1 3 2])),3);   % (3x1) vector
    
    
    