function [SCov_k_v,Phi_norm_k_m,lret1_avb,lret2_avb]=spot_cov_block_corrd_EQW(block_st,block_end,h,avgt1,avgt2,t1,t2,tl1,tl2,lret1,lret2,...
    noise_v,J_val,ccorr)
%% spot_cov_block_corrd_EQW
%--------------------------------------------------------------
% computes spot covolatility estimate on block k with equal weights 
% for each frequency j
%
% Note:      * norms of spectral weights corrected for discr sampling
%               (only in bias correction -> save comp time)
%               * spectral weights based on cont approximations
%               % only for cross-sectionally independent noise
%--------------------------------------------------------------
% INPUT:
% k-> block index
% block_st -> block start
% block_end -> block end
% h -> block length
% n_rft -> max number of obs of series 1 and 2
% avgt1 -> vector of centered obs times of returns (or times at end of return spells)
% t1-> vector of observation times at end of return spells
% tl1-> vector of lags of t1 (same length)
% lret1   -> vector of log-returns
% noise_v -> vector of noise (co-)variances ([nu2_x;0;nu2_y])
% nkex -> if >0, exact number of obs on each block is used to determine
% cutoff
% J_val -> spectral cutoff
% ccorr -> if >0, correlation and variance estimates are corrected
% (default: =0)
%--------------------------------------------------------------
% OUTPUT:
% SCov_k_v -> (3x1) vector of spot (co-)volatility estimates on block 
% ([sigma_(x,k)^2;sigma_(xy,k);sigma_(y,k)^2])
% Phi_norm_k_m -> (3xJ_val) matrix of empirical norms of Phis
% lret1_avb -> (J_valx1) vector of local spectral means of asset 1
% -------------------------------------------------------------------------
%
% Peter Malec, October 6, 2012 
%
% mod May 16, 2013: correction for too few observations on block
%                               included
%
% -------------------------------------------------------------------------
%       
%% Preparations

% check for correction
if nargin<14
    ccorr=0;
end
% -------------------------------------------------------------------------
%  
%% create weight array

% weight array
weightm_j=eye(3)*(1/J_val);
W_k_a=weightm_j(:,:,ones(J_val,1));
% -------------------------------------------------------------------------
%     
 %%  Compute number of obs etc on block
  
 % index of observations on each block (based on centers)
     t_ind1=logical(((avgt1 >= block_st) & (avgt1 <=block_end)));  
     t_ind2=logical(((avgt2 >= block_st) & (avgt2 <=block_end)));  
     
    % centered observations times on block (since beginning)
     avgtb1=avgt1(t_ind1)-block_st;
     avgtb2=avgt2(t_ind2)-block_st;
     
     % time and lagged times on block  (since beginning)
     tb1=t1(t_ind1)-block_st;
     tlb1=tl1(t_ind1)-block_st;
      tb2=t2(t_ind2)-block_st;
     tlb2=tl2(t_ind2)-block_st;
     
     % quadratic variation of time on block 
     qvtbx=nansum((tb1-tlb1).^2);
     qvtby=nansum((tb2-tlb2).^2);
     
     % log returns on block
     lret_b1=lret1(t_ind1,:);
     lret_b2=lret2(t_ind2,:);
% -------------------------------------------------------------------------
%     
 %% check number of obs
 
     if (length(lret_b1)>3)&&(length(lret_b2)>3) %normal
 % -------------------------------------------------------------------------
%        
%% Compute Local Blockwise Averages of Log-Returns and Emp Phi Norms
   
    % check if cutoff too large
     n_max=min([length(lret_b1) length(lret_b2)]);
     if J_val>n_max
         disp(['Specified pilot cutoff too large on block' num2str(round(block_end/h)) '!'])
     end

    %empty matrices for local blockwise averages
    lret1_avb=NaN*zeros(J_val,1);   % (J_valx1) vector
    lret2_avb=lret1_avb;
    
    %empty matrix of empirical norms of Phis 
    Phi_norm_k_m=NaN*zeros(3,J_val);   % (3xJ_val) matrix
    
    %length of overlap of return spells of x,y (else zero)
    mat_ind=return_overlap_faster(tb1,tlb1,tb2,tlb2);
    
   
    % loop
        for j=1:J_val
            
            % spectral weights for given block and frequency
            Phi1=spectr_weights_j_onbl_cont(avgtb1,h,j);
            Phi2=spectr_weights_j_onbl_cont(avgtb2,h,j);
            
            % local blockwise averages using frequency j
            lret1_avb(j)=lret_b1'*Phi1; 
            lret2_avb(j)=lret_b2'*Phi2;
            
            % empirical norms of Phis 
            [Phi_norm_jk_x,Phi_norm_jk_y,Phi_norm_jk_xy]=Phi_norm_emp_block_faster(Phi1,Phi2,tb1,tlb1,tb2,tlb2,mat_ind);
            Phi_norm_k_m(:,j)=[Phi_norm_jk_x;Phi_norm_jk_xy;Phi_norm_jk_y];
            
        end
        
        % create local noise vector for bias correction
        lnoise_v=noise_v.*[qvtbx;0;qvtby]/h;  % (3 x 1) vector
        lnoise_v=lnoise_v(:,ones(J_val,1));  % (3 x J_val) matrix
% -------------------------------------------------------------------------
%         
%% estimate spot covola for given block as weighted average acr freqs
    
    % matrix of dist. elements of outer poduct matrices of local ret avgs of for freqs
    S_jk_m=[(lret1_avb.^2)';(lret1_avb.*lret2_avb)';(lret2_avb.^2)'];   % (3 x J_val) matrix
    
    % matrix of dist. elements of bias-corr. spot vola matrix estimator on 
    C_jk_m=(S_jk_m-lnoise_v)./Phi_norm_k_m; % (3 x J_val) matrix
   
    %  spot vola estimates on block: sum of prod of fisher info and spot vola estimates based on diff freq
    SCov_k_v=nansum(mtimesx(W_k_a,permute(C_jk_m,[1 3 2])),3);   % (3x1) vector
    
     % correction of correlation and variance estimates
    if ccorr>0
        [SCov_k_v]=check_spot_covs(SCov_k_v,block_end,h);
    end
% -------------------------------------------------------------------------
%         
%% if too few observations
    
     else
     disp(['Less than three observations in block' num2str(round(block_end/h)) '!'])
     SCov_k_v=zeros(3,1);
     Phi_norm_k_m=NaN*zeros(3,J_val);
     lret1_avb=NaN*zeros(J_val,1);
     lret2_avb=NaN*zeros(J_val,1);
     end
    
    