function [SCov_mat_sm,lret1_avb_m,lret2_avb_m]...
    =spot_covs_pilot_cont(normB,ks,block_st,block_end,h,hs,avgt1,avgt2,t1,t2,tl1,tl2,lret1,lret2,noise_v,Kval,J_val)
%% spot_covs_pilot
%--------------------------------------------------------------
% computes pilot spot covolatility estimates on all blocks with equal weights 
% for each frequency j
%
% Note:     
%               * spectral weights based on cont approximations
%               % only for cross-sectionally independent noise
%--------------------------------------------------------------
% INPUT:
% normB -> set of "normal" blocks
% ks-> short block index (NaN if no shorter block)
% block_st -> block start
% block_end -> block end
% h -> block length
% hs -> block length of shorter block
% n_rft -> max number of obs of series 1 and 2
% avgt1 -> vector of centered obs times of returns (or times at end of return spells)
% t1-> vector of observation times at end of return spells
% tl1-> vector of lags of t1 (same length)
% lret1   -> vector of log-returns
% noise_v -> vector of noise (co-)variances ([nu2_x;0;nu2_y])
% nkex -> if >0, exact number of obs on each block is used to determine
% cutoff
% Kval    -> bandwidth for smoothing of pilot estimates
% J_val -> spectral cutoff
%--------------------------------------------------------------
% OUTPUT:
% SCov_mat_sm -> (Bx3) matrix of smoothed spot (co-)volatility estimates on
% blocks (kth row: [sigma_(x,k)^2 sigma_(xy,k) sigma_(y,k)^2])
% lret1_avb_m -> (BxJ_val) matrix of local spectral means of asset 1
% -------------------------------------------------------------------------
%
% Peter Malec, October 6, 2012 
%
% -------------------------------------------------------------------------
%       
%% Preparations

% no of blocks
B=length(normB);
if ~isnan(ks)
    B=B+1;
end

% empty output arrays
SCov_mat=NaN*zeros(B,3);
lret1_avb_m=NaN*zeros(B,J_val);
lret2_avb_m=NaN*zeros(B,J_val);
% -------------------------------------------------------------------------
%       
%% Compute Pilot Estimates of Spot (Co-variances)

% Loop over normal blocks
%---------------------------------------------------------------------
for k=normB
    
    % compute spot covolatility estimate on block k
    [SCov_k_v,lret1_avb,lret2_avb]=spot_cov_block_cont_EQW(block_st(k),block_end(k),h,avgt1,avgt2,t1,t2,tl1,tl2,lret1,lret2,...
    noise_v,J_val);

    % assign to arrays
    SCov_mat(k,:)=SCov_k_v';
    lret1_avb_m(k,:)=lret1_avb';
    lret2_avb_m(k,:)=lret2_avb';

end

% short block   
%-------------------------------------------------
if ~isnan(ks)

    % compute spot covolatility estimate on block k
    [SCov_k_v,lret1_avb,lret2_avb]=spot_cov_block_cont_EQW(block_st(ks),block_end(ks),hs,avgt1,avgt2,t1,t2,tl1,tl2,lret1,lret2,...
    noise_v,J_val);

    % assign to arrays
    SCov_mat(ks,:)=SCov_k_v';
    lret1_avb_m(ks,:)=lret1_avb';
    lret2_avb_m(ks,:)=lret2_avb';

     
end

% Smooth
%---------------------------------------------------------
if Kval>1
    SCov_mat_sm=spot_vola_smooth(SCov_mat,Kval);
else
    SCov_mat_sm=SCov_mat;
end






