function [Sigma_k_sm,lret_avb_arr,lnoise_m]...
    =spot_covs_pilot_md_kern(block_st,block_end,h,avgtime,time,time_l,lret,noise_v,Kval,J_val,ccorr,kern)
%% spot_covs_pilot_md_kern
%--------------------------------------------------------------
% computes pilot spot covolatility estimates on all blocks with equal weights 
% for each frequency j in m-dimensional case and smooths final estimates
% using kernel weights
%
% Note:      * only for blocks with equal lengths
%               * norms of spectral weights uncorrected (based on continuous approximations)
%               * spectral weights based on cont approximations
%               % only for cross-sectionally independent noise
%--------------------------------------------------------------
% INPUT:
% block_st -> block start
% block_end -> block end
% h -> block length
% avgtime -> (mx1) cell array of centered obs times of returns (or times at end of return spells)
% time-> (mx1) cell array of observation times at end of return spells
% time_l-> (mx1) cell array of lags of time (same length)
% lret   -> (mx1) cell array of log-return (NOTE: each of the above arrays contains column vectors of observations)
% noise_v -> (mx1) vector of noise variance estimates
% Kval    -> bandwidth for smoothing of pilot estimates
% J_val -> spectral cutoff
% ccorr -> if >0, correlation and variance estimates are corrected
% (default: =0)
% kern   -> string, type of kernel:
%                   -- 'gauss' (Gaussian)
%                   -- 'beta'   (Beta)
%                   -- 'betamod'   (modified Beta)
%--------------------------------------------------------------
% OUTPUT:
% Sigma_k_sm-> (Bxm(m+1)/2) matrix of smoothed spot (co-)volatility estimates on
%                           blocks (each row is vech(Sigma_t)')
% lret_avb_arr -> (J_valxmxB) matrix of local spectral means of m different
%                           assets
% lnoise_m       -> (mxB) matrix of local noise level estimates on blocks
% -------------------------------------------------------------------------
%
% Peter Malec, May 16, 2016 
%
% -------------------------------------------------------------------------
%       
%% Preparations

% no of blocks
B=length(block_st);

% number of assets
m=length(lret);

% unique elements of cov mat
elu=m*(m+1)/2;

% check for correction
if nargin<11
    ccorr=0;
end

% empty output arrays
Sigma_k_mat=NaN*zeros(B,elu);
lret_avb_arr=NaN*zeros(J_val,m,B);
lnoise_m=NaN*zeros(m,B);
% -------------------------------------------------------------------------
%       
%% Compute Pilot Estimates of Spot (Co-variances)

% Loop over blocks
%---------------------------------------------------------------------
for k=1:B
    
    % compute spot covolatility estimate on block k
    [Sigma_k,lret_avb_m,lnoise_v]=spot_cov_block_EQW_md(block_st(k),block_end(k),h,avgtime,time,time_l,lret,noise_v,J_val,ccorr);

    % assign to arrays
    Sigma_k_mat(k,:)=vech(Sigma_k)';
    lret_avb_arr(:,:,k)=lret_avb_m;
    lnoise_m(:,k)=lnoise_v;

end


% Smooth
%---------------------------------------------------------
[Sigma_k_sm,~]=spot_vola_smooth_kern(block_st,Kval,block_st,Sigma_k_mat,kern);


