function [SCov_mat_sm,lret1_avb_m,lret2_avb_m,Phi_norm_k_x_m,Phi_norm_k_xy_m,Phi_norm_k_y_m]...
    =spot_covs_pilot_simp(block_st,block_end,h,avgt1,avgt2,t1,t2,tl1,tl2,lret1,lret2,noise_v,Kval,J_val,ccorr)
%% spot_covs_pilot_simp
%--------------------------------------------------------------
% computes pilot spot covolatility estimates on all blocks with equal weights 
% for each frequency j
%
% Note:      * only for blocks with equal lengths
%               * norms of spectral weights corrected for discr sampling
%               (only in bias correction -> save comp time)
%               * spectral weights based on cont approximations
%               % only for cross-sectionally independent noise
%--------------------------------------------------------------
% INPUT:
% block_st -> block start
% block_end -> block end
% h -> block length
% n_rft -> max number of obs of series 1 and 2
% avgt1 -> vector of centered obs times of returns (or times at end of return spells)
% t1-> vector of observation times at end of return spells
% tl1-> vector of lags of t1 (same length)
% lret1   -> vector of log-returns
% noise_v -> vector of noise (co-)variances ([nu2_x;0;nu2_y])
% nkex -> if >0, exact number of obs on each block is used to determine
% cutoff
% Kval    -> bandwidth for smoothing of pilot estimates
% J_val -> spectral cutoff: should not be chosen too large!!!
% ccorr -> if >0, correlation and variance estimates are corrected
% (default: =0)
%--------------------------------------------------------------
% OUTPUT:
% SCov_mat_sm -> (Bx3) matrix of smoothed spot (co-)volatility estimates on
% blocks (kth row: [sigma_(x,k)^2 sigma_(xy,k) sigma_(y,k)^2])
% lret1_avb_m -> (BxJ_val) matrix of local spectral means of asset 1
% Phi_norm_k_x_m -> (BxJ_val) matrix of empirical norms of Phis
% -------------------------------------------------------------------------
%
% Peter Malec, May 8, 2013 
%
% -------------------------------------------------------------------------
%       
%% Preparations

% no of blocks
B=length(block_st);

% check for correction
if nargin<15
    ccorr=0;
end

% empty output arrays
SCov_mat=NaN*zeros(B,3);
lret1_avb_m=NaN*zeros(B,J_val);
lret2_avb_m=NaN*zeros(B,J_val);
Phi_norm_k_x_m=NaN*zeros(B,J_val);
Phi_norm_k_xy_m=NaN*zeros(B,J_val);
Phi_norm_k_y_m=NaN*zeros(B,J_val);
% -------------------------------------------------------------------------
%       
%% Compute Pilot Estimates of Spot (Co-variances)

% Loop over blocks
%---------------------------------------------------------------------
for k=1:B
    
    % compute spot covolatility estimate on block k
    [SCov_k_v,Phi_norm_k_m,lret1_avb,lret2_avb]=spot_cov_block_corrd_EQW(block_st(k),block_end(k),h,avgt1,avgt2,t1,t2,tl1,tl2,lret1,lret2,...
    noise_v,J_val,ccorr);

    % assign to arrays
    SCov_mat(k,:)=SCov_k_v';
    Phi_norm_k_x_m(k,:)=Phi_norm_k_m(1,:);
    Phi_norm_k_xy_m(k,:)=Phi_norm_k_m(2,:);
    Phi_norm_k_y_m(k,:)=Phi_norm_k_m(3,:);
    lret1_avb_m(k,:)=lret1_avb';
    lret2_avb_m(k,:)=lret2_avb';

end


% Smooth
%---------------------------------------------------------
if Kval>1
    SCov_mat_sm=spot_vola_smooth(SCov_mat,Kval);
else
    SCov_mat_sm=SCov_mat;
end






