function Sigma_kh=spot_vola(lret1_avb,lret2_avb,Phi_norm,Phi_norms,s_block,H,K,n)
%------------------------------------------------------------------
%% spot_vola
% Estimates spot (co-) volatility matrix at the beginning of each block
% spectral estimator following Bibinger & Reiss (2001)
%------------------------------------------------------------------
%
% INPUTS
% lret1_avb -> (B x 1) vector of  local Blockwise Averages of Log-Returns for asset 1 (
%                     for chosen frequency j)
% Phi_norm -> empirical norm of spectral weights  for chosen frequency j
%                   (scalar)
% Phi_norms -> empirical norm of spectral weights  for shorter block and chosen frequency j
%                   (scalar)
% s_block -> determines which block is shorter: 'first' or 'last' (default)
% H             -> (2 x 2) matrix being estimate of noise covariance
% K             -> bandwidth of spot volatility estimator: # of blocks to
%                     average over. We build local window around (beginning
%                     of) block k using K/2 blocks before and K/2-1 blocks after 
% n              -> # of observations
%------------------------------------------------------------------
% OUTPUTS
% Sigma_kh -> (2 x 2 x B) array of spot covariance matrices at beginning of
%                     each block
%------------------------------------------------------------------
%
%Peter Malec, December 7, 2011
%
% January 4, 2012
% adjusted for one shorter block (smaller h for first or last block)!!
%
%% Preparations

%number of blocks
B=length(lret1_avb);

% check bandwidth
if K < 2
    error('supplied bandwidth K has to be >= 2!')
else
    if (mod(K,2)~=0)   % has to be even integer!!!!
        K=K-1;
    end
end

% Define Matrix of local Blockwise Averages for both assets
lret_avb_m=[lret1_avb lret2_avb];       % (B x 2) matrix

% Initialize Array of local window inner products of Blockwise Averages 
lret_avb_ip=NaN*zeros(2,2,B);

% Initialize vector of local window average of inverse empirical norms
iPhi_norm_avg=NaN*zeros(B,1);

% create Phi_norm vector accouting for shorter block
%-------------------------------------------------------------
if isequal(s_block,'first')  % first block
    Phi_norm_v=[Phi_norms;repmat(Phi_norm,B-1,1)];
else
   Phi_norm_v=[repmat(Phi_norm,B-1,1);Phi_norms];
end

%------------------------------------------------------------------
%
%% Weighted (Phi_norm) Local Window Inner product of Blockwise Averages  for each block

% "Intermediate" blocks: standard proc
for i=(K/2+1):(B-K/2+1)
    block_ind=(i-K/2):(i+K/2-1);        % local window around (beginning of) block k using K/2 blocks before and K/2-1 blocks after 
    lret_avb_ip(:,:,i)=lret_avb_m(block_ind,:)'*(lret_avb_m(block_ind,:)./[Phi_norm_v(block_ind,:) Phi_norm_v(block_ind,:)]);
    iPhi_norm_avg(i)=nanmean((Phi_norm_v(block_ind,:)).^(-1)); % local window average of inverse empirical norms
end

% "First" blocks: we do not have K/2 blocks before
block_ind=1:K;          % fixed window at beginning
lret_avb_ip(:,:,1:K/2)=repmat(lret_avb_m(block_ind,:)'*(lret_avb_m(block_ind,:)./[Phi_norm_v(block_ind,:) Phi_norm_v(block_ind,:)]),[1 1 K/2]);
iPhi_norm_avg(1:K/2)=nanmean((Phi_norm_v(block_ind,:)).^(-1)); % local window average of inverse empirical norms

% "Last" blocks: we do not have K/2-1 blocks after
block_ind=(B-K+1):B;    % fixed window at end
lret_avb_ip(:,:,(B-K/2+2):B)=repmat(lret_avb_m(block_ind,:)'*(lret_avb_m(block_ind,:)./[Phi_norm_v(block_ind,:) Phi_norm_v(block_ind,:)]),[1 1 (K/2-1)]);
iPhi_norm_avg((B-K/2+2):B)=nanmean((Phi_norm_v(block_ind,:)).^(-1)); % local window average of inverse empirical norms
%------------------------------------------------------------------
%
%%  Compute Spot Covolatility 

Sigma_kh=lret_avb_ip/K-repmat(reshape(iPhi_norm_avg,[1 1 B]),[2 2 1]).*repmat(H,[1 1 B])/n;      % (2 x 2 x B) array: spot covariance matrix at beginning of each block





