function [SCov_mat_sm,CovSCov_mat_sm]=spot_vola_smooth_kern(input,b,block_st,SCov_mat,kern,i_I_k_mat)
%------------------------------------------------------------------
%% spot_vola_smooth_kern
% Smooths Spot (Co-variance) estimates using kernel regression
%------------------------------------------------------------------
%
% INPUTS
% input       -> vector of points of estimation
% b             -> smoothing bandwidth
% block_st    -> (Bx1) vector of block start times
% SCov_mat -> (Bxm(m+1)/2) matrix of spot (co-)volatility estimates on
%                   blocks (each row is vech(Sigma_t)')
% kern   -> string, type of kernel:
%                   -- 'gauss' (Gaussian)
%                   -- 'beta'   (Beta)
%                   -- 'betamod'   (modified Beta)
% i_I_k_mat -> (m^2(m^2+1)/2xB) matrix of inverses of sum of fisher infos on each
%                       block (each column is vech(inv(I_k_m))) (optional)
%------------------------------------------------------------------
% OUTPUTS
% SCov_mat_sm -> (length(input)xm(m+1)/2) matrix of smoothed spot (co-)volatility estimates 
% CovSCov_mat_sm -> (m^2(m^2+1)/2xlength(input)) array: Covariance matrices of smoothed spot (co-)volatilities
%   under the local parametric experiment
%------------------------------------------------------------------
%
%   Peter Malec, May 15, 2016
%
%------------------------------------------------------------------
%
%% Preparations

% make column vector
block_st=block_st(:);
input=input(:);

% number of points of estimation
nin=length(input);

% check bandwidth
if  b<=0   % has to be even integer!!!!
        error('bandwidth has to be positive')
end

% check kernel 
if ~ismember(upper(kern),['GAUSS','BETA','BETAMOD'])
    error('Kern needs to be GAUSS, BETA or BETAMOD!')
end
%------------------------------------------------------------------
%
%% Smoothing

% relevant dimensions
dim_est=size(SCov_mat,2);
if nargin>5
    dim_cov=size(i_I_k_mat,1);
end

% empty output matrix
SCov_mat_sm =NaN*zeros(nin,dim_est);

if nargin>5 % compute Covariance matrices  of estimates
    
    % empty output matrix
    CovSCov_mat_sm =NaN*zeros(dim_cov,nin);
    
    % evaluate Smoothed estimates
    for i=1:nin
        if isequal(upper(kern),'GAUSS')
            kernelv=normpdf((block_st-input(i))/b);
        elseif isequal(upper(kern),'BETA')
            kernelv=beta_kernel(block_st,input(i),b,0);
        elseif isequal(upper(kern),'BETAMOD')
            kernelv=beta_kernel(block_st,input(i),b,1);
        end
        weightv=kernelv/sum(kernelv); %(Bx1 vector)
        SCov_mat_sm(i,:)=sum(SCov_mat.*weightv(:,ones(dim_est,1))); 
        CovSCov_mat_sm(:,i)=sum(i_I_k_mat.*(weightv(:,ones(dim_cov,1))').^2,2);
    end
    
else % do not compute Covariance matrices  of estimates
    
     CovSCov_mat_sm=NaN;
     
     % evaluate Smoothed estimates
    for i=1:nin
        if isequal(upper(kern),'GAUSS')
            kernelv=normpdf((block_st-input(i))/b);
        elseif isequal(upper(kern),'BETA')
            kernelv=beta_kernel(block_st,input(i),b,0);
        elseif isequal(upper(kern),'BETAMOD')
            kernelv=beta_kernel(block_st,input(i),b,1);
        end
        weightv=kernelv/sum(kernelv); %(Bx1 vector)
        SCov_mat_sm(i,:)=sum(SCov_mat.*weightv(:,ones(dim_est,1))); 
    end
       
end







