function [eta_est,stat_val_vec,AC_diff_vec] = test_noise_AC(x,alpha,Q_max)
%%  test_noise_AC
%------------------------------------------------------------
% Determines order of dependence Q, for which
% Cov(\eps_i,\eps_{i+u})=0 with u>=Q .
% Provides resulting estimator of long-run noise variance
% Note: Q=R+1, where R such that
% Cov(\eps_i,\eps_{i+u})=0 with u>R .
%--------------------------------------------------------------
% INPUT:
% x -> ((n+1)x1) vector containing observed log prices
% alpha ->  significance level
% Q_max -> maximum order of dependence allowed for inthe procedure
%--------------------------------------------------------------
% OUTPUT:
% eta_est        -> estimate of long-run noise variance
% stat_val_vec -> (Q+1)x1 vector containing individual test statistics:
%                          last statistic not significant at alpha level
%                          for H_0^Q: Cov(\eps_i,\eps_{i+u})=0 with u>=Q 
% AC_diff_vec -> ((Q+1)x1) vector containing estimates of differences between rth and(r+1)th order autocovariance of
%                       noise process: \hat\eta_r-\hat\eta_{r+1} for
%                       r=0,..., Q
% -------------------------------------------------------------------------
%
% Peter Malec, March 3, 2014
%
% -------------------------------------------------------------------------
%       
%% Preparations

%input check
if (alpha<0)||(alpha>1)
    error('Alpha must be between 0 and 1!')
end

% compute differences
dx=diff(x(:));

% critical value
crit_val=norminv(1-alpha/2,0,1);

% objects necessary for initialization of proc (Q=0)
%----------------------------------------------------------------

% difference between 0th and 1st order autocovariance
AC_diff_vec=ACov(dx,0)/2;

% corresponding variance (times n)
V_est_vec=Est_fourthMom_Noise(dx,0,0,0,AC_diff_vec,[],AC_diff_vec,[])/4;
% -------------------------------------------------------------------------
%       
%% Loop over Q until first non-rejection

% initialize
stat_val_vec=[];
Q=0;
reject=true(1);

%start loop (also end if maximum Q reached)
%----------------------------------------------------------------
while reject&&(Q<Q_max)
    
    %increase counter
    Q=Q+1;
    
    % compute Statistic for H_0^Q
    [stat_val,AC_diff_vec,V_est_vec] = test_noise_AC_it(dx,AC_diff_vec,V_est_vec);
    
    % augment vector of test statistics
    stat_val_vec=[stat_val_vec;stat_val];
    
    % test decision
    reject=abs(stat_val)>crit_val;
    
end
%       
%% Compute Long-Run Variance Estimate

% vector of estimates of noise autocovariances
eta_est_vec=flipud(cumsum(flipud(AC_diff_vec(1:(end-1)))));  %((R+1)x1) vector (R=Q-1): lag order increasing

% LR variance estimate
eta_est=eta_est_vec(1)+2*sum(eta_est_vec(2:end));


