function [stat_val,AC_diff_vec,V_est_vec] = test_noise_AC_it(dx,AC_diff_vec,V_est_vec)
%%  test_noise_AC_it
%------------------------------------------------------------
% Computes test statistic for H0 of all noise autocovariances
% of order u>=Q being equal to zero
%--------------------------------------------------------------
% INPUT:
% dx -> (nx1) vector containing 1st differences of realizeation of observed process
% AC_diff_vec ->(Qx1) vector containing estimates of differences between rth and(r+1)th order autocovariance of
%                       noise process: \hat\eta_r-\hat\eta_{r+1} for
%                       r=0,..., Q-1
% V_est_vec -> (Qx1) vector containing estimates of variances of the above
%                      differences (times n) under the assumption that \eta_u=0
%                      for all u>r
%--------------------------------------------------------------
% OUTPUT:
% stat_val -> value of statistic
% AC_diff_vec -> updated ((Q+1)x1) vector with results for r=0,..., Q
% V_est_vec   -> updated ((Q+1)x1) vector with results for r=0,..., Q
% -------------------------------------------------------------------------
%
% Peter Malec, March 3, 2014
%
% -------------------------------------------------------------------------
%       
%% Preparations

% adjust
dx=dx(:);

% determine order Q
Q=length(AC_diff_vec);
% -------------------------------------------------------------------------
%       
%% Augment estimates of differences between rth and(r+1)th order autocovariance

% Qth order noise autocovariance
eta_Q=AC_diff_vec(end)+ACov(dx,Q);

% augment vector
AC_diff_vec=[AC_diff_vec;eta_Q];            %((Q+1)x1) vector
% -------------------------------------------------------------------------
%       
%% Augment variance estimates of differences between rth and(r+1)th order autocovariance

% Compute relevant fourth moments of noise process
%-----------------------------------------------------------------

Sigma00Q=Est_fourthMom_Noise(dx,0,0,Q,AC_diff_vec(1),[],AC_diff_vec(1),[]);

SigmauQ0_v=NaN*zeros(Q,1);
for i=1:Q;
    SigmauQ0_v(i)=Est_fourthMom_Noise(dx,i-1,Q,0,AC_diff_vec(i),AC_diff_vec(gt0(i-1)),AC_diff_vec(end),AC_diff_vec(end-1));
end

SigmaQu0_v=NaN*zeros(Q,1);
for i=1:Q;
    SigmaQu0_v(i)=Est_fourthMom_Noise(dx,Q,i,0,AC_diff_vec(end),AC_diff_vec(end-1),AC_diff_vec(i+1),AC_diff_vec(i));
end

SigmauQq_m=NaN*zeros(Q-1,Q);
for i=1:(Q-1);
        for j=1:Q;
            SigmauQq_m(i,j)=Est_fourthMom_Noise(dx,j-1,Q,i,AC_diff_vec(j),AC_diff_vec(gt0(j-1)),AC_diff_vec(end),AC_diff_vec(end-1));
        end
end

SigmaQuq_m=NaN*zeros(Q-1,Q);
for i=1:(Q-1);
        for j=1:Q;
            SigmaQuq_m(i,j)=Est_fourthMom_Noise(dx,Q,j,i,AC_diff_vec(end),AC_diff_vec(end-1),AC_diff_vec(j+1),AC_diff_vec(j));
        end
end

SigmauuQ_m=NaN*zeros(Q+1,Q);
for i=1:(Q+1);
        for j=1:Q;
            SigmauuQ_m(i,j)=Est_fourthMom_Noise(dx,i-1,j,Q,AC_diff_vec(i),AC_diff_vec(gt0(i-1)),AC_diff_vec(j+1),AC_diff_vec(j));
        end
end

% Compute Variance of Qth order noise autocovariance(times n)
%-----------------------------------------------------------------
V_est_Q=V_est_vec(end)+0.5*Sigma00Q+sum(SigmauQ0_v)+sum(SigmaQu0_v)+...
    2*sum(sum(SigmauQq_m))+2*sum(sum(SigmaQuq_m))+2*sum(sum(SigmauuQ_m));

% augment vector
V_est_vec=[V_est_vec;V_est_Q];
% -------------------------------------------------------------------------
%  
%% Compute Statistic

% correct variance estimate
V_est_Q=max([V_est_Q eps]);

%statistic
stat_val=sqrt(length(dx)/V_est_Q)*eta_Q;






