function [rho_vec,rho_var_vec]=comp_corr_md(asseti,SCov_mat,Covmat_mat,Z_tilde)
%------------------------------------------------------------------
%% comp_corr_md
% Computes correlations and associated Standard Errors 
% for a specific asset pair from a general m-dimensional
% covariance matrix
%------------------------------------------------------------------
%
% INPUTS
% asseti        -> (2x1) vector of indices of relevant assets: asseti(1)>= asset(2)
% SCov_mat-> (n_inxm(m+1)/2) matrix of spot cov matrices for each point of
%                               estimation
%                    each row: vech(\hat Sigma_t)'
% Covmat_mat ->(m^2(m^2+1)/2xn_in) matrix: Stacked Covariance matrices of smoothed spot (co-)volatilities
%                       under the local parametric experiment      (optional)
%                    each column:vech(Cov(\hat Sigma_t))
% Z_tilde -> (m^2 x m^2)  matrix of fourth moments of d-dimensional
%                   standard Gaussian vector (optional)
%------------------------------------------------------------------
% OUTPUTS
% rho_vec -> (n_inx1) vector of correlations
% rho_var_vec ->(n_inx1) vector of correlation standard errors
%------------------------------------------------------------------
%
%   Peter Malec, January 8, 2014
%
%------------------------------------------------------------------
%
%% Preparations

% no of assets
m= (sqrt(1+size(SCov_mat,2)*8)-1)/2;

% check asset indices
asseti=asseti(:);
if (asseti(2)>m)||(asseti(1)>m)
    error('Asset indices larger than no of assets.')
end
if asseti(2)>asseti(1)
    asseti=flipud(asseti);
end
%------------------------------------------------------------------
%
%% Compute Correlations

%number of inputs
nin=size(SCov_mat,1);

% obtain column indices for SCov_mat
if asseti(2)>1
    indcov=sum(m-((1:(asseti(2)-1))-1))+asseti(1)-asseti(2)+1;%(asseti(2)-1)*(m+1)+asseti(1)-asseti(2)-1-sum(1:(asseti(2)-1));
else
     indcov=asseti(1)-asseti(2)+1;
end
if asseti(1)>1
    indv1=sum(m-((1:(asseti(1)-1))-1))+asseti(1)-asseti(1)+1;
else
    indv1=asseti(1)-asseti(1)+1;
end
if asseti(2)>1
    indv2=sum(m-((1:(asseti(2)-1))-1))+asseti(2)-asseti(2)+1;
else
    indv2=asseti(2)-asseti(2)+1;
end
              
% retrieve necessary elements of SCov_mat
SCov_mat=SCov_mat(:,[indv1 indcov indv2]);

% correlations
rho_vec=SCov_mat(:,2)./sqrt(SCov_mat(:,1).*SCov_mat(:,3));

% correct for 0/0=nan
rho_vec(isnan(rho_vec))=0;
%------------------------------------------------------------------
%
%% Compute Standard errors

if nargin>2
    
    % compute Z_tilde???
    if (nargin<4)
        Z_tilde=[];
    end
    if (isempty(Z_tilde))||(size(Z_tilde,1)~=m^2)||(size(Z_tilde,2)~=m^2)
        disp('Z_tilde has to be computed!')
        Z_tilde = constrZ_tilde(m);
    end
    
    % obtain indices for Covmat_mat
    indcov=(asseti(2)-1)*m+asseti(1);
    indv1=(asseti(1)-1)*m+asseti(1);
    indv2=(asseti(2)-1)*m+asseti(2);
    
    % retrieve necessary elements of Covmat_mat
    %--------------------------------------------------------
    Covmat_arr=NaN*zeros(3,3,nin);
    for i=1:size(Covmat_mat,2)
        Covmat_temp=ivech(Covmat_mat(:,i))*Z_tilde;
        Covmat_arr(:,:,i)=ivech([Covmat_temp(indv1,indv1) Covmat_temp(indv1,indcov) Covmat_temp(indv1,indv2) ...
                                    Covmat_temp(indcov,indcov) Covmat_temp(indcov,indv2) Covmat_temp(indv2,indv2)]');
    end

    
    % compute gradient of corr wrt var1,cov and var2
    %--------------------------------------------------------
    gam_vec=(sqrt(SCov_mat(:,1).*SCov_mat(:,3))).^(-1);
    beta21_vec=SCov_mat(:,2)./SCov_mat(:,1);
    beta12_vec=SCov_mat(:,2)./SCov_mat(:,3);
    drho_mat=gam_vec(:,ones(3,1)).*[((-0.5)*beta21_vec) ones(nin,1) ((-0.5)*beta12_vec)];
                                            %(ninx3) matrix
     % correct for 0/0=nan
    drho_mat(isnan(drho_mat))=0;                                       
                                            
                                            
     % compute variance of correlation estimate
     %--------------------------------------------------
    drho_arr=permute(drho_mat,[3 2 1]);%(1x3xB) array
    drho_arr=drho_arr(ones(3,1),:,:); %(3x3xB) array
    var_drho_arr=sum(Covmat_arr.*drho_arr,2);%(3x1xB) array
    rho_var_vec=sum(permute(drho_arr(1,:,:),[2 1 3]).*var_drho_arr ,1);%(1x1xB) array
    rho_var_vec=squeeze(rho_var_vec);
else
    
    rho_var_vec=NaN*zeros(nin,1);
    
end