function [Avg_frq_weight]=lle_avg_weights_mod(i_I_jk,total)
%% lle_avg_weights
%--------------------------------------------------------------
% Computes weight matrices of lle (upper
% triangular elements) for first 5 frequencies averaged across blocks
%--------------------------------------------------------------
% INPUT:
% i_I_jk -> structure containing elements of inverse fisher informations
% (each J x B)
% total -> if >0 we average across all frequencies
%--------------------------------------------------------------
% OUTPUT:
% Avg_frq_weight -> (3 x 3 x 5) array containing weight matrices (upper
% triangular elements) for first 5 frequencies averaged across blocks
% -------------------------------------------------------------------------
%
% Peter Malec, February 17, 2012
%
% -------------------------------------------------------------------------
%       
%% Prepare Inputs

% number of blocks and cutoff
[J B]=size(i_I_jk.i_I_jk_11);

% elements of inverse fisher information
i_I_jk_11=i_I_jk.i_I_jk_11;     % (J x B) matrix
i_I_jk_22 =i_I_jk.i_I_jk_22;   % (J x B) matrix
i_I_jk_33=i_I_jk.i_I_jk_33;     % (J x B) matrix
i_I_jk_12=i_I_jk.i_I_jk_12;   % (J x B) matrix
i_I_jk_23 = i_I_jk.i_I_jk_23;   % (J x B) matrix
i_I_jk_13= i_I_jk.i_I_jk_13;   % (J x B) matrix

% initialize matrices for elements of weight matrices 

if total>0
    
    weight_m=NaN*zeros(B,6);
    
else

    weight_m1=NaN*zeros(B,6);
    weight_m2=weight_m1;
    weight_m3=weight_m1;
    weight_m4=weight_m1;
    weight_m5=weight_m1;

end
% -------------------------------------------------------------------------
%       
%% Compute Average Weights

% Loop over blocks
%-------------------------------------
%-------------------------------------
for k=1:B
    
    % Compute Fisher Information matrices for each frequency
    %------------------------------------------------------------------
    
    % matrix of distinct elements of inverse Fisher infos for all freqs
    i_I_jk_m=[i_I_jk_11(:,k)';i_I_jk_12(:,k)';i_I_jk_13(:,k)';i_I_jk_22(:,k)'...
        ;i_I_jk_23(:,k)';i_I_jk_33(:,k)'];      % (6 x J) matrix
    
    % create array of (symmetric) inverse fisher info matrices for all freqs
     i_I_jk_a=repmat(tril(ones(3)),[1 1 J]);        % (3x3xJ) array
     i_I_jk_a(logical(i_I_jk_a))=permute(i_I_jk_m,[1 3 2]);
     i_I_jk_d=i_I_jk_a(logical(repmat(eye(3),[1 1 J])));
     i_I_jk_a=permute(i_I_jk_a,[2 1 3])+i_I_jk_a;
     i_I_jk_a(logical(repmat(eye(3),[1 1 J])))=i_I_jk_d;
     
     % compute array of fisher info matrices by inversion
     I_jk_a=nd_inv_3x3sym(i_I_jk_a);
     
     % compute sum of fisher info matrices across freqs on block
     I_k_m=nansum(I_jk_a,3);                    % (3x3) matrix
     
     % weight matrices for 1st 5  frequencies
     
     if total>0
         
         weight_m_temp=mean(abs(inv3(repmat(I_k_m,[1 1 J]),I_jk_a)),3);
         weight_m(k,:)=weight_m_temp(logical(triu(ones(3))))';
         
     else
     
        weight_m1_temp=I_k_m\I_jk_a(:,:,1);
        weight_m1(k,:)=weight_m1_temp(logical(triu(ones(3))))';
        weight_m2_temp=I_k_m\I_jk_a(:,:,2);
        weight_m2(k,:)=weight_m2_temp(logical(triu(ones(3))))';
        weight_m3_temp=I_k_m\I_jk_a(:,:,3);
        weight_m3(k,:)=weight_m3_temp(logical(triu(ones(3))))';
        weight_m4_temp=I_k_m\I_jk_a(:,:,4);
        weight_m4(k,:)=weight_m4_temp(logical(triu(ones(3))))';
        weight_m5_temp=I_k_m\I_jk_a(:,:,5);
        weight_m5(k,:)=weight_m5_temp(logical(triu(ones(3))))';
     
     end
    
end

% Averages of block weight matrices for 1st 5  frequencies
%---------------------------------------------------------------------

if total>0
    
    % compute average and put it into upper triangular matrices
    Avg_frq_weight=zeros(3);
    Avg_frq_weight(logical(triu(ones(3))))=mean(weight_m)';
    
else

    % compute averages and put them into upper triangular matrices
    Avg_frq_weight1=zeros(3);
    Avg_frq_weight1(logical(triu(ones(3))))=mean(abs(weight_m1))';
    Avg_frq_weight2=zeros(3);
    Avg_frq_weight2(logical(triu(ones(3))))=mean(abs(weight_m2))';
    Avg_frq_weight3=zeros(3);
    Avg_frq_weight3(logical(triu(ones(3))))=mean(abs(weight_m3))';
    Avg_frq_weight4=zeros(3);
    Avg_frq_weight4(logical(triu(ones(3))))=mean(abs(weight_m4))';
    Avg_frq_weight5=zeros(3);
    Avg_frq_weight5(logical(triu(ones(3))))=mean(abs(weight_m5))';


    % output array
    Avg_frq_weight=cat(3,Avg_frq_weight1,Avg_frq_weight2,Avg_frq_weight3,Avg_frq_weight4,Avg_frq_weight5);

end