function [SCov_mat,Covmat_arr]=retrieve_cov_el(asseti,SCov_mat,Covmat_mat,Z_tilde)
%------------------------------------------------------------------
%% retrieve_cov_el
% Retrives elements of (2x2) sub-covariance matrix from bigger
% stacked covariance matrix
%------------------------------------------------------------------
%
% INPUTS
% asseti        -> (2x1) vector of indices of relevant assets: asseti(1)>= asset(2)
% SCov_mat-> (n_inxm(m+1)/2) matrix of spot cov matrices for each point of
%                               estimation
%                    each row: vech(\hat Sigma_t)'
% Covmat_mat ->(m^2(m^2+1)/2xn_in) matrix: Stacked Covariance matrices of smoothed spot (co-)volatilities
%                       under the local parametric experiment      (optional)
%                    each column:vech(Cov(\hat Sigma_t))
% Z_tilde -> (m^2 x m^2)  matrix of fourth moments of d-dimensional
%                   standard Gaussian vector (optional)
%------------------------------------------------------------------
% OUTPUTS
% SCov_mat -> (n_inx3) matrix of spot cov matrix elements for each point of
%                       estimation.
%                        each row: vech(\hat Sigma^{sub}_t)', i.e.
 %                         Sigma_11 Sigma_12 Sigma22  
% Covmat_arr ->(3x3x_n_in) array of Covariance matrices of vech trafo of
 %                           spot sub cov matrix estimators
 %                               each page:  Cov(\hat \vech(Sigma^{sub}_t) )              
%------------------------------------------------------------------
%
%   Peter Malec,February 14, 2014
%
%------------------------------------------------------------------
%
%% Preparations

% no of assets
m= (sqrt(1+size(SCov_mat,2)*8)-1)/2;

% check asset indices
asseti=asseti(:);
if (asseti(2)>m)||(asseti(1)>m)
    error('Asset indices larger than no of assets.')
end
if asseti(2)>asseti(1)
    asseti=flipud(asseti);
end
%------------------------------------------------------------------
%
%% Sub covariance matrix

%number of inputs
nin=size(SCov_mat,1);

% obtain column indices for SCov_mat
if asseti(2)>1
    indcov=sum(m-((1:(asseti(2)-1))-1))+asseti(1)-asseti(2)+1;%(asseti(2)-1)*(m+1)+asseti(1)-asseti(2)-1-sum(1:(asseti(2)-1));
else
     indcov=asseti(1)-asseti(2)+1;
end
if asseti(1)>1
    indv1=sum(m-((1:(asseti(1)-1))-1))+asseti(1)-asseti(1)+1;
else
    indv1=asseti(1)-asseti(1)+1;
end
if asseti(2)>1
    indv2=sum(m-((1:(asseti(2)-1))-1))+asseti(2)-asseti(2)+1;
else
    indv2=asseti(2)-asseti(2)+1;
end
              
% retrieve necessary elements of SCov_mat
SCov_mat=SCov_mat(:,[indv1 indcov indv2]);
%------------------------------------------------------------------
%
%% Covariance Matrices of spot estimators

if nargin>2
    
    % compute Z_tilde???
    if (nargin<4)
        Z_tilde=[];
    end
    if (isempty(Z_tilde))||(size(Z_tilde,1)~=m^2)||(size(Z_tilde,2)~=m^2)
        disp('Z_tilde has to be computed!')
        Z_tilde = constrZ_tilde(m);
    end
    
    % obtain indices for Covmat_mat
    indcov=(asseti(2)-1)*m+asseti(1);
    indv1=(asseti(1)-1)*m+asseti(1);
    indv2=(asseti(2)-1)*m+asseti(2);
    
    % retrieve necessary elements of Covmat_mat
    %--------------------------------------------------------
   Covmat_arr=NaN*zeros(3,3,nin);
    for i=1:size(Covmat_mat,2)
        Covmat_temp=ivech(Covmat_mat(:,i))*Z_tilde;
        Covmat_arr(:,:,i)=ivech([Covmat_temp(indv1,indv1) Covmat_temp(indv1,indcov) Covmat_temp(indv1,indv2) ...
                                    Covmat_temp(indcov,indcov) Covmat_temp(indcov,indv2) Covmat_temp(indv2,indv2)]');
    end

    
  
else
    
    Covmat_arr=NaN;
    
end