function [EVal_mat,EVec_arr]=spec_dec_spot_covm(SCov_mat,nf,corrmat,rel)
%% spec_dec_spot_covm
%--------------------------------------------------------------
% computes spectral components of instantaneous covariance
% matrix estimates
%--------------------------------------------------------------
% INPUT:
% SCov_mat-> (n_inxm(m+1)/2) matrix of spot cov matrices for each point of
%                               estimation
%                    each row: vech(\hat Sigma_t)'
% nf             -> no. of factors
% corrmat     -> if ==1, decomposition of correlation matrix (default) 
% rel            -> if ==1, eigenvalues relative to total variation are computed (default) 
%--------------------------------------------------------------
% OUTPUT:
% EVal_mat -> (n_inxnf) matrix of eigenvalues with each row
%                     corresponding to spot covariance at specific point in
%                     time
% EVec_arr      -> (mxnfxn_in) array of (normalized) eigenvectors with each
%                      page corresponding to spot covariance at specific point in
%                     time (optional)
% -------------------------------------------------------------------------
%
% Peter Malec, February 12, 2013 
%
% -------------------------------------------------------------------------
%       
%% Input Check

% no of assets
m= (sqrt(1+size(SCov_mat,2)*8)-1)/2;

%number of inputs
nin=size(SCov_mat,1);

% number of factors
if nf>m
    error('number of factors must not be larger than no. of assets!')
end

% indicators
if isempty(corrmat)
    corrmat=1;
end

if isempty(rel)
    rel=1;
end
% -------------------------------------------------------------------------
%    
%% Obtain Spectral Components

% empty output arrays
EVal_mat=NaN*zeros(nin,nf);
if nargout>1
    EVec_arr=NaN*zeros(m,nf,nin);
end

% Loop over input times
%-----------------------------------------------------
for i=1:nin
    
    % retrieve instantaneous cov mat
    Sigma_k=ivech(SCov_mat(i,:)');
    
    % take correlations?
    if isequal(corrmat,1)
        % Check of variances
        Variances=diag(Sigma_k);
        Variances(Variances<=0)=1e-10;
        Sigma_k(logical(eye(m)))=Variances;
        % compute correlation matrix
        Sigma_k=diag(diag(Sigma_k).^(-.5))*Sigma_k*diag(diag(Sigma_k).^(-.5));
    end
    
    % Spec Decomp
    [U, S] = eig(Sigma_k);
    
    % Put eigenvalues into vector
    lambd=diag(S); 
    
    %replace negative eigenvalues by small positive values
    S_neg_ind=lambd<=0;
    lambd(S_neg_ind)=1e-10*ones(sum(S_neg_ind),1);
    if sum(S_neg_ind)>0
        disp(['Some eigenvalues negative for time ' int2str(i) ' !'])
    end
    
    % sort eigenvalues by size
    [lambd,ix]=sort(lambd,'descend');
    U=U(:,ix);
    
    %relative eigenvalues?
    if isequal(rel,1)
        lambd=lambd/trace(Sigma_k);
    end
    
    % assign outputs
    EVal_mat(i,:)=lambd(1:nf)';
    if nargout>1
        EVec_arr(:,:,i)=U(:,1:nf);
    end
    
end




    
    




