function [rq,rqss,diagnostics] = realized_quantile_variance(price,time,timeType,samplingType,samplingInterval,quantiles,samplesperbin,symmetric,subsamples,oversamples)
% Computes the Quantile Realized Variance of Christensen, Oomen and Podolskij (2008),
% the MinRV and MedRV estimator of Andersen, Dobrev and Shaumberg and the
% general symmetrized version suggested by Sheppard in a discussion of COP
%
% USAGE:
%   [RQ] = realized_quantile_variance(PRICE,TIME,TIMETYPE,SAMPLINGTYPE,SAMPLINGINTERVAL,QUANTILES,SAMPLESPERBIN)
%   [RQ,RQSS,DIAGNOSTICS] = realized_quantile_variance(PRICE,TIME,TIMETYPE,SAMPLINGTYPE,SAMPLINGINTERVAL,QUANTILES,SAMPLESPERBIN,SYMMETRIC,SUBSAMPLES,OVERSAMPLES)
%
% INPUTS:
%   PRICE            - m by 1 vector of high frequency prices
%   TIME             - m by 1 vector of times where TIME(i) corresponds to PRICE(i)
%   TIMETYPE         - String describing the way times are measured
%                       'wall'    24-hour clock of the form HHMMSS, e.g. 101543 or 153217
%                       'seconds' Time measured in seconds past midnight on the first day.
%                       'unit'  Unit normalized date format, e.g. .1, .234, .9
%                         Unit normalized times are more general than the other types and can be
%                         applied to data from more than one calendar day
%   SAMPLINGTYPE     - String describing the type of sampling to use when
%                        filtering PRICE
%                        'CalendarTime' - Sample in calendar time using observations separated by
%                          SAMPLINGINTERVAL seconds
%                        'CalendarUniform' - Sample in calendar time using SAMPLINGINTERVAL
%                          observations spread uniformly between TIME(1) and TIME(m)
%                        'BusinessTime' - Sample in business (tick) time using observation separated
%                          by SAMPLINGINTERVALticks
%                        'BusinessUniform' - Sample in business (tick) time using observations
%                          uniformly spaced in business time.
%                        'Fixed' - Sample at specific points in time. When using fixed,
%                          SAMPLINGINTERVAL must be a n by 1 vector of times with the same TIMETYPE
%                          as TIME (i.e. seconds if TIME is in seconds)
%   SAMPLINGINTERVAL - Scalar integer or n by 1 vector whose meaning depends on the selected SAMPLINGTYPE
%   QUANTILES        - k by 1 vector of quantile values to use when computing RQ, must satisfy 0.5<QUANTILES<=1.
%                        Quantiles must produce values such that QUANTILES*SAMPLESPERBIN is an
%                        integer.  The simplest method to accomplish this is to specify QUANTILES as
%                        the ratio of the index number of the return to SAMPLESPERBIN (e.g.
%                        QUANTILES = [13 15 19]/20)
%   SAMPLESPERBIN    - Number of returns to use in each bin when computing the quantiles. NOTE: The
%                        number of returns produced by filtering according to SAMPLINGTYPE and
%                        SAMPLINGINTERVAL must be an integer multiple of SAMPLESPERBIN.
%   SUBSAMPLES       - [OPTIONAL] Integer value containing the number of subsamples to use when
%                        computing RRSS. It must be the case that SUBSAMPLE<SAMPLESPERBIN. Setting
%                        SUBSAMPLES>0 will produce an estimator based on jittering the starting
%                        point over floor(linspace(0,(SUBSAMPLES)/(SUBSAMPLES+1),SUBSAMPLES+1)*SAMPLESPERBIN)+1
%                        For example, if SUBSAMPLES = 3 and SAMPLESPERBIN = 20, then the starting
%                        points for the four RR estimators will be observations 1, 6, 11 and 16.  If
%                        omitted SUBSAMPLE = 0 and RRSS=RR.
%
% OUTPUTS:
%   RQ               - Quantile realized variance vector (k by 1) corresponding to the values in QUANTILES
%   RQSS             - Subsample based version of RQ.  RQSS = RQ if SUBSAMPLES = 0
%   DIAGNOSTICS      - Structure with fields:
%                        RQINDIV   - k by 1 vector of RQ estimates
%                        RQSSINDIV - k by 1 vector of subsample RQ estimates
%                        RQWEIGHTS - Weights used in combining the individual RQ estimates.  RQ
%                                      weights are computed from RQCOV as the minimum variance
%                                      combination
%                        RQCOV     - k by k non-scaled (integrated quarticity) covariance matrix.
%
% COMMENTS:
%   The cases where SAMPLESPERBIN is in {5 6 10 13 15 18 20 25 26 30 36 39 50 60 65 72 75 100 144} (non-symmetric) or 
%   {2 3 ... 9 10 13 15 18 20 25 26 30 36 39 50 60 65 72 75 100 144} (symmetric) the scales and non-scaled 
%   covariances have been pre-computed using Monte Carlo integration using 100,000,000 simulations. 
%   Choosing another number for SAMPLESPERBIN requires a run of realized_quantile_variance_scale which
%   can be slow if SAMPLESPERBIN is large. If using another value, 1,000,000 simulations will be used
%   in computing the weights. If repeatedly using another value, pre-computing the appropriate scales
%   and non-scaled covariance using realzed_quantile_weight_simulation is recommended.  It is also necessary 
%   to modify one line in the code marked *MODIFY*
%
% EXAMPLES:
%   % Estimate the RQ from prices available from 9:30 to 16:00 using 1 minute returns, 30 samples per
%   % bin and quantiles {.60 .7333 .90}
%   sampleTimes = seconds2wall(wall2seconds(93000):60:wall2seconds(160000))
%   RQ=realized_quantile_variance(price,time,'wall','Fixed',sampleTimes,[18 22 27]/30,30);
%
%   % The same but using 4 subsamples
%   RQ=realized_quantile_variance(price,time,'wall','Fixed',sampleTimes,[18 22 27]/30,30,4);
%
%   % Estimate the RQ from prices previously filtered using 20 samples per bin and quantiles {.65 .75 .90}
%   time = linspace(0,1,length(price));
%   [RQ,RQSS]=realized_quantile_variance(price,time,'unit','BusinessTime',1,[13 15 18]/20,20)
%
%  See also REALIZED_KERNEL, REALIZED_VARIANCE, REALIZED_PRICE_FILTER,
%  REALIZED_KERNEL_WEIGHTS, REALIZED_KERNEL_SELECT_LAG_LENGTH

% Copyright: Kevin Sheppard
% kevin.sheppard@economics.ox.ac.uk
% Revision: 1    Date: 5/1/2008

%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% InputChecking
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if nargin<7 || nargin>10
    error('Seven to ten inputs required.')
end
if size(price,2)>size(price,1)
    price=price';
end
if size(price,2)>1
    error('PRICE must be a m by 1 vector.')
end
if size(time,2)>size(time,1)
    time=time';
end
if any(diff(time)<0)
    error('TIME must be sorted and increasing')
end
if size(time,2)>1 || length(time)~=length(price)
    error('TIME must be a m by 1 vector.')
end
% Inserted to protect against inputing integer times
time = double(time);

timeType=lower(timeType);
if ~ismember(timeType,{'wall','seconds','unit'})
    error('TIMETYPE must be one of ''wall'', ''seconds'' or ''unit''.');
end
samplingType=lower(samplingType);
if ~ismember(samplingType,{'calendartime','calendaruniform','businesstime','businessuniform','fixed'})
    error('SAMPLINGTYPE must be one of ''CalendarTime'', ''CalendarUniform'', ''BusinessTime'', ''BusinessUniform'' or ''Fixed''.');
end

m=size(price,1);
t0=time(1);
tT=time(m);
if ismember(samplingType,{'calendartime','calendaruniform','businesstime','businessuniform'})
    % Must be a scalar integer
    if ~isscalar(samplingInterval) || floor(samplingInterval)~=samplingInterval || samplingInterval<1
        error('SAMPLINGINTERVAL must be a positive integer for the SAMPLINGTYPE selected.')
    end
else
    if size(samplingInterval,2)>size(samplingInterval,1)
        samplingInterval=samplingInterval';
    end
    if ~(any(samplingInterval>=t0) && any(samplingInterval<=tT))
        error('At least one sampling interval must be between min(TIME) and max(TIME) when using ''Fixed'' as SAMPLINGTYPE.')
    end
    if any(diff(samplingInterval)<=0)
        error('When using ''Fixed'' as SAMPLINGTYPE the vector of sampling times in SAMPLINGINTERVAL must be sorted and strictly increasing.')
    end
end

% SAMPLERPERBIN, positive integer >=2
if samplesperbin<1 || floor(samplesperbin)~=samplesperbin
    error('SAMPLESPERBIN must be a positive integer greater than or equal to 2 (and usually at least 20)')
end
if samplesperbin<10
    warning('oxfordRealized:realizedQuantileVariance','SAMPLESPERBIN is small and the results may be unreliable.')
end

% QUANTILES
if any(quantiles<=0.5) || any(quantiles>1) || length(quantiles)>(samplesperbin/2) || length(unique(quantiles*samplesperbin)) ~= length(quantiles)
    error('Quantiles must be unique, greater than 0.5 and less than or equal to 1, and the number of quantiles must be smaller than 0.5*SAMPLESPERBIN.')
end

% SUBSAMPLES
if nargin>=8 && ~isempty(subsamples)
    if ~isscalar(subsamples) || subsamples<0 || floor(subsamples)~=subsamples || subsamples>=samplesperbin
        error('SUBSAMPLES must be a non-negative integer less than SAMPLESPERBIN.')
    end
else
    subsamples = 0;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Input Checking
%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% 1. Compute returns
returns = realized_return_filter(price,time,timeType,samplingType,samplingInterval,subsamples);
% Check that the number of returns is compatible with SAMPLESPERBIN
n=length(returns{1});
if floor(n/samplesperbin)~=(n/samplesperbin)
    error(['The number of returns computed from the prices returned from realized_price_filter(PRICE,TIME,TIMETYPE,SAMPLINGTYPE,SAMPLINGINTERVAL) must be an integer multiple of SAMPLESPERBIN.  The number of return produced is ' num2str(n)]);
end

rqvec = zeros(subsamples+1,1);
for j=1:(subsamples+1)
    rqvec(j)=realized_quantile_variance_core(returns{j},quantiles,samplesperbin,symmetric,oversamples);
end



% If subsamples are requested repeat over the subsample bins
if nargin==8 && subsamples>0
    binstartindices = 1:samplesperbin:n;
    offset = round(linspace(0,subsamples/(subsamples+1),subsamples+1)*samplesperbin);
    [binstartindices,offset]=meshgrid(binstartindices,offset);
    binstartindices = binstartindices + offset;
    binstartindices = binstartindices(:);
    binstartindices = binstartindices((binstartindices+samplesperbin-1)<=n);
    rqindivSS = zeros(n/samplesperbin,q);
    count = 1;
    for jj=1:length(binstartindices)
        j = binstartindices(jj);
        binreturns = returns(j:j+samplesperbin-1);
        binreturns = sort(binreturns)';
        rqindivSS(count,:)=binreturns(lowerIndices).^2+binreturns(upperIndices).^2;
        count = count + 1;
    end
    rqindivSS = (mean(rqindivSS)./finalScale)';
    rqss = rqweights'*rqindivSS;
    diagnostics.rqssindiv = rqindivSS;
    % Subsample
else
    % If not just use the same values for rq and rqindiv
    rqss = rq;
    diagnostics.rqssindiv = rqindiv;
end


function rq = realized_quantile_variance_core(returns,quantiles,samplesperbin,symmetric,oversamples)

% 1. Filter the price
n = length(returns);
% Scale the returns
returns = sqrt(n) * returns;
if symmetric
    returns = abs(returns);
end
% Get the length of the quantiles and initialize a holder for the individual RQ
q = length(quantiles);
rqindiv = zeros(n/samplesperbin,q);

% Compute the indices to use
if symmetric
    % CHECK THESE
    upperIndices = round(quantiles*(2*samplesperbin)-samplesperbin);
    lowerIndices = upperIndices;
else
    upperIndices = round(quantiles*samplesperbin);
    lowerIndices = round((1-quantiles)*samplesperbin+1);
end
% Loop over the bins
count = 1;
initialBins = 1:samplesperbin:n;
bins = initialBins';
originalNumberOfBins = length(initialBins);
if oversamples>1
    skip=(samplesperbin/(oversamples+1));
    skip = skip * (0:oversamples)';
    [bins,skip]=meshgrid(initialBins,skip);
    bins = bins+skip;
    bins = bins(:);
    bins = bins((bins+samplesperbin-1)<=n);
end
actualNumberOfBins = length(bins);
finalScale = originalNumberOfBins/actualNumberOfBins;

for j=bins'
    binreturns = returns(j:j+samplesperbin-1);
    binreturns = sort(binreturns)';
    rqindiv(count,:)=binreturns(lowerIndices).^2+binreturns(upperIndices).^2;
    count = count + 1;
end
keyboard
% Compute scaling coefficients

% These are precomputed
% ******************************************************************************
% * MODIFY THE FOLLOWING LINE IF realzed_quantile_weight_simulation IS UPDATED *
MS=[5     6    10    13    15    18    20    25    26    30    36    39    50    60    65    72    75   100   144];
% * MODIFY THE PREVIOUS LINE IF realzed_quantile_weight_simulation IS UPDATED  *
% ******************************************************************************

if symmetric
    adjSamplesPerBin = 2 * samplesperbin;
else
    adjSamplesPerBin = samplesperbin;
end

if ismember(adjSamplesPerBin,MS)
    % If available load and use the pre-computed value
    load realized_quantile_scales;
    [ind,pl]=ismember(adjSamplesPerBin,cell2mat(simulationSamplerperbin));
    thisScale = expectedQuantiles{pl}; %#ok<USENS>
    thisQuantile = simulationQuantile{pl}; %#ok<USENS>
    thisCovariance = expectedCovariance{pl}; %#ok<USENS>
    indicator = zeros(1,q);
    for i = 1:q
        [temp,indicator(i)] = min(abs(thisQuantile-quantiles(i)));
    end
    finalScale = thisScale(indicator);
    % Find index
    rqcov     = thisCovariance(indicator,indicator);
    rqweights = inv(rqcov)*ones(q,1)/(ones(1,q)*inv(rqcov)*ones(q,1));
else
    % Need to simulate using 1,000,000 simulations
    if samplesperbin>100
        warning('oxfordRealized:realizedQuantileVariance','Computing the scales needed.  Since SAMPLESPERBIN is very large this may take a long time.  \nConsider pre-computing this value, especially if using this value of SAMPLESPERBIN many times.')
    end
    [finalScale,rqweights,rqcov]=realized_quantile_variance_scale(adjSamplesPerBin,quantiles,1000000);
end
% Finalize and store
rqindiv = (mean(rqindiv)./finalScale)';
diagnostics.rqindiv = rqindiv;
diagnostics.rqcov = rqcov;
diagnostics.rqweights = rqweights;
rq = rqweights'*rqindiv;
