function [MSCRCOV, M1,M2,M3] = MSCRCOV_2x2_oracle(data_a1,data_a2,Omega_mat,ICov,ICoQuart)
%% MSCRCOV_2x2_oracle
%--------------------------------------------------------------
% Multi-Scale Realized (2x2) Covariance Estimator as proposed by 
% Bibinger (2011) with distinct max subsampling scales for elements 
% of covariance matrix
% NOTE: oracle version
%--------------------------------------------------------------
% INPUT:
% data_a1 -> (n_a x 2) matrix of (synchronous) data for asset 1
%       - 1st  col: observation times 
%       - 2nd col: prices 
% startt -> start of trading (in hours)
% Omega_mat -> (2x2) noise covariance matrix
% ICov-> (3x1) vector of integrated covariances
% ICoQuart -> (6x1) vector of integrated coquarticities
%--------------------------------------------------------------
% OUTPUT:
% MSCRCOV -> multi-scale estimate of (2x2) integrated covariance matrix
% M1, M2 M3 -> largest subsampling scales for two variances and covariance
% -------------------------------------------------------------------------
%
% Peter Malec, February 1, 2012
% -------------------------------------------------------------------------
%       
%% Compute Optimal Max Scale

% number of obs
N=size(data_a1,1);

% compute c* for all elements
Vn_vec=24*[Omega_mat(1,1)^2;Omega_mat(2,2)^2;Omega_mat(1,1)*Omega_mat(2,2)];
Vd_vec=26/35*[2*ICoQuart(1);2*ICoQuart(2);ICoQuart(4)+ICoQuart(3)];
Vc_vec=12/5*[Omega_mat(1,1)^2+2*Omega_mat(1,1)*ICov(1);Omega_mat(2,2)^2+2*Omega_mat(2,2)*ICov(2);...
   Omega_mat(1,1)*Omega_mat(2,2)+Omega_mat(1,1)*ICov(2)+Omega_mat(2,2)*ICov(1)];
c_vec=([4;4;1]).*((-Vc_vec+sqrt(Vc_vec.^2+12*Vd_vec.*Vn_vec))./(6*Vn_vec)).^(-1/2);    %rescale for variances

% Vector of Ms
M_vec=sqrt(N)*c_vec;

% round
M1=min(max(ceil(M_vec(1)),2),N);       %   2<= M <= N
M2=min(max(ceil(M_vec(2)),2),N);       %   2<= M <= N
M3=min(max(ceil(M_vec(3)),2),N);       %   2<= M <= N
M=max([M1;M2;M3]);          % max of all rounded Ms
% -------------------------------------------------------------------------
%       
%% Compute Multi-Scale Estimator

% Log Prices
LogPrice=[log(data_a1(:,2)) log(data_a2(:,2))];

% array for subsampled estimates
RCOVSS_arr=NaN*zeros(2,2,M-1);

% loop to compute subsampled estimates
for i=1:M-1
    RCOVSS_arr(:,:,i)=RCovSS(LogPrice,i+1); %RCOVSS=0 for K=1
end

% weight vectors
ivec1=(2:M1)';
alpha_vec1=12*ivec1.^(2)/(M1^3-M1)-6*ivec1/(M1^2-1)-6*ivec1/(M1^3-M1);
ivec2=(2:M2)';
alpha_vec2=12*ivec2.^(2)/(M2^3-M2)-6*ivec2/(M2^2-1)-6*ivec2/(M2^3-M2);
ivec3=(2:M3)';
alpha_vec3=12*ivec3.^(2)/(M3^3-M3)-6*ivec3/(M3^2-1)-6*ivec3/(M3^3-M3);

% compute estimators
MSCRCOV=NaN*zeros(2,2);
MSCRCOV(1,1)=sum(RCOVSS_arr(1,1,1:(M1-1)).*permute(alpha_vec1,[2 3 1]),3);
MSCRCOV(2,2)=sum(RCOVSS_arr(2,2,1:(M2-1)).*permute(alpha_vec2,[2 3 1]),3);
MSCRCOV(1,2)=sum(RCOVSS_arr(1,2,1:(M3-1)).*permute(alpha_vec3,[2 3 1]),3);
MSCRCOV(2,1)=MSCRCOV(1,2);



