function [mrk_cov, mrk_corr, Hvalstar1, Hvalstar2] = MRK_2x2_sync_optb(data_a1,data_a2,startt,Omega_mat,IQ_mat)
%% MRK_2x2_sync
%--------------------------------------------------------------
% Bivariate Realized Kernel estimation as stated in BNHLS (2009) 
% for the (2x2) case with synchronous data. Adjustment of bandwidth selection
% following (LSS, 2011).
%--------------------------------------------------------------
% INPUT:
% data_a1 -> (n_a x 2) matrix of (synchronous) data for asset 1
%       - 1st  col: observation times 
%       - 2nd col: prices 
% startt -> start of trading (in hours)
% Omega_mat -> Estimate of Noise Covariance (2 x 2) for Bandwidth Choice (if =0:
% not used)
% IQ_mat -> Estimate of integrated quarticity (2 x 2) for Bandwidth Choice (if =0:
% not used)
%--------------------------------------------------------------
% OUTPUT:
% mrk_corr-> (2x2) matrix of integrated covariance using jointly optimal
% bandwidth
% Hvalstar1 -> corresponding bandwidth
% mrk_cov-> (2x2) matrix of integrated covariance average univariate
% bandwidth
% Hvalstar2 -> corresponding bandwidth
% -------------------------------------------------------------------------
%
% Peter Malec, January 6, 2012
% -------------------------------------------------------------------------
%       
%% refreshtime sampling

%price _rft is (n_rft x 2) matrix
price_rft=[data_a1(:,2) data_a2(:,2)];

%convert time in seconds after midnight
data_a1(:,1)=startt*3600+data_a1(:,1);
data_a2(:,1)=startt*3600+data_a2(:,1);
                                    
% Jittering: average over first and last two obs

price_rftJ =[mean(price_rft(1:2,:),1); price_rft(3:(end-2),:); mean(price_rft((end-1):end,:),1)]; 

% Returns (RFT)
rtn_rft = diff(log(price_rftJ)); 

% number of returns (RFT)
n_rft=size(rtn_rft,1);      
% -------------------------------------------------------------------------
%       
%% MRK based on Average univariate bandwidths

% Average univariate bandwidth

if isequal(Omega_mat,0) && isequal(IQ_mat,0)
    Hvalstar1= HvalParzenOmega_2x2_noss(data_a1,data_a2,n_rft);
elseif ~isequal(Omega_mat,0) && ~isequal(IQ_mat,0)
    Hvalstar1=HvalParzenOmega_OM_IQ(Omega_mat,IQ_mat,n_rft);
else
    error('Supply both Noise Covariance AND Int. Quarticity!!!')
end

% sanity check: Autocov of higher orders than n2-1 cannot be computed
Hvalstar1 = nanmax(1, nanmin(Hvalstar1, n_rft-1));  % mininum bandwidth =1
                                                                

% Compute MRK estimates using average univariate bandwidths                                                                          
 mrk_cov=MRK(rtn_rft,Hvalstar1);                                                                          
% -------------------------------------------------------------------------
%                                                                                  
%% MRK based on jointly optimal bandwidth

 % Compute jointly optimal bandwidth (2x2) bandwidth following LSS (2011)
 Hvalstar2 = Hval_crk(rtn_rft,mrk_cov); 
 Hvalstar2 = nanmax(1, nanmin(Hvalstar2, n_rft-1));% sanity check  
                                                                % Note: autocov of higher orders than n2-1 cannot be computed  

% Compute MRK estimates using jointly optimal bandwidth                                                                         
mrk_corr=MRK(rtn_rft,Hvalstar2);                

                                                                           
                                                                           

