function [rqv,omega2hat,output]=rv_qmle(lprice,sigma2,omega2,options)
%% rv_qmle
%--------------------------------------------------------------
% QMLE of integrated variance following Xiu (JE, 2010)
%--------------------------------------------------------------
% INPUT:
% lprice -> (n x 1) vector of log-price observations
% sigma2 -> starting value for integrated variance estimate
% omega2-> starting value for noise variance estimate
% options -> options for optimization
%--------------------------------------------------------------
% OUTPUT:
% rqv -> QML estimate of quadratic variation
% omega2hat -> estimate of noise variance
% output-> output structure from optimization procedure
% -------------------------------------------------------------------------
%
% Peter Malec, January 31, 2012
% -------------------------------------------------------------------------
%       
%% Preparations

% return series
r = diff(lprice);

% number of obs
n = size(r,1);

% distance between equidistat observations
delta = 1/n;
% -------------------------------------------------------------------------
%       
%% Maximize Quasi likelihood function


% Starting values for MA(1) parameters
%--------------------------------

% theta
v=sigma2*delta+2*omega2;
theta_s=(-v+sqrt(sigma2*delta*(4*omega2+sigma2*delta)))/(2*omega2);

% gamma2
gamma2_s= (v+sqrt(sigma2*delta*(4*omega2+sigma2*delta)))/2; % variance of white noise term

% join
theta_s=[theta_s;gamma2_s];

% Set Bounds on parameters
LB=ones(2,1)*(-inf);   
LB(2,1)=1e-9;           %lower bound of variance is zero(+eps)
UB=ones(2,1)*inf;        %no upper bounds


% Optimization of Quasi likelihood
%--------------------------------
[param,~,exitflag,output]=fmincon('LLH_MA_1_Exact',theta_s,[],[],[],[],LB,UB,[],options,r);


% retain estimates of noise variance and integrated variance
%-----------------------------------------------------------------------
if ~any(exitflag==[-1 -2 -3])      % "normal convergence"
    omega2hat=-param(2)*param(1);
    rqv=(param(2)*(1+param(1)^2)-2*omega2hat)/delta;
else
    omega2hat=omega2;
    rqv=sigma2;
    disp('The QML estimator did not converge.');
end







