function [rqv,diagnostics]=rv_qmle_ks(price,sigma2,omega2,opttol)
%% lle_cov
%--------------------------------------------------------------
% QMLE of integrated variance following Xiu (JE, 2010)
%--------------------------------------------------------------
% INPUT:
% price -> (n x 1) vector of price observations
% sigma2 -> starting value for integrated variance estimate
% omega2-> starting value for noise variance estimate
% opttol -> structure containing convergence criteria for optimization:
% opttol.par > change in parameter estimates (default: 1e-3)
% opttol.iter > max number of iterations (default: 20)
%--------------------------------------------------------------
% OUTPUT:
% rqv -> QML estimate of quadratic variation
% diagnostics -> structure containing # of iterations and noise variance
% estimate
% -------------------------------------------------------------------------
%
% Peter Malec, January 31, 2012
%
% based on code by Kevin Sheppard
% -------------------------------------------------------------------------
%       
%% Preparations

% return series
r = diff(log(price));

% number of obs
n = size(r,1);

% distance between equidistat observations
delta = 1/n;

% convergence criteria for maximization
if nargin<4
    opttol.par=1e-3;
    opttol.iter=20;
end
% -------------------------------------------------------------------------
%       
%% Maximize Quasi likelihood function


% Prepare structures
%--------------------------------
L = spalloc(n,n,4+3*(n-2));
diagInd = (0:n-1)*n+(1:n);
uDiagInd = (1:n-1)*n+(1:n-1);
lDiagInd = (0:n-2)*n+(2:n);
L(diagInd)=2;
L(uDiagInd)=-1;
L(lDiagInd)=-1;
O = spalloc(n,n,4+3*(n-2));

% Loop for maximization
%--------------------------------
theta = [sigma2;omega2];
thetaOld = [-1;-1];
iter = 0;
while max(abs(theta-thetaOld)>opttol.par) && iter<opttol.iter
    thetaOld = theta;
    O(diagInd) = sigma2 * delta + 2*omega2;
    O(lDiagInd) = -omega2;
    O(uDiagInd) = -omega2;
    Om1 = inv(O);
    Om1 = (Om1+Om1')/2;
    Om2= inv(O*O);
    Om2 = (Om2+Om2')/2;
    Om1L = Om1*L;
    Om2L = Om2*L;
    Om2L2 = Om2L*L;
    Om1LOm1 = Om1L*Om1;
    Om1LOm1 = (Om1LOm1+Om1LOm1')/2;
    
    trOm2L = diag(Om2L)'*ones(n,1);
    trOm2L2 = diag(Om2L2)'*ones(n,1);
    trOm2 = diag(Om2)'*ones(n,1);
    denom = (trOm2L)^2 - (trOm2) * (trOm2L2);
    W1 = n * trOm2L * Om1LOm1 - n * trOm2L2 * Om2;
    W1 = W1 / denom;
    W1 = (W1+W1')/2;
    W2 = trOm2L * Om2 - trOm2 * Om1LOm1;
    W2 = W2/denom;
    W2 = (W2+W2')/2;
    
    sigma2 = r'*W1*r;
    omega2 = r'*W2*r;
    omega2 = max(0,omega2);
    theta = [sigma2;omega2];
    iter = iter + 1;
    disp(iter)
    disp(theta)
 end

% if max number of iterations exceeded
if max(abs(theta-thetaOld)>opttol.par)  && iter>=opttol.iter
    warning('MFE::Convergence','The realized QMLE estimator did not converge.');
end
% -------------------------------------------------------------------------
%       
%% Prepare Output

rqv = sigma2;
diagnostics.iterations = iter;
diagnostics.noiseVariance = omega2;