function [iquart1,iquart2,icoquart,icoprod12,icoprod112,icoprod212]=integ_quart(sigma1,sigma2,sigma12)
%% integ_cov
%--------------------------------------------------------------
% Computes integrated quarticities on unit interval by numerical
% integration using simpson rule on spot (co-) variances 
%--------------------------------------------------------------
% INPUT:
% sigma1 -> vector of spot variances of asset 1 (including starting value)
% sigma2 -> vector of spot variances of asset 2 (including starting value)
% sigma12 -> vector of spot covariances of asset 1,2 (including starting value)
%--------------------------------------------------------------
% OUTPUT:
% iquart1 -> integr quart asset 1   (int sigma1^2 ) 
% iquart2 -> integr quart asset 2   (int sigma2^2 ) 
% icoquart  -> integr co quarticity (int sigma12^2 ) 
% icoprod12 -> integr co product 1 & 2 (int sigma1 sigma2 ) 
% icoprod112 -> integr co product 1 & 12 (int sigma1 sigma12 ) 
% icoprod212 -> integr co product 2 & 12 (int sigma2 sigma12 ) 
% -------------------------------------------------------------------------
%
% Peter Malec, January 25, 2012
% -------------------------------------------------------------------------
%   
%%  Preparations

% # of intervals in euler discretization = # segments * 2
N=length(sigma1)-1;

% 1/2 width of segments
h=1/N; %(1-0)/(N/2)/2

%% Compute integrated (co-quarticities)

% compute products and squared series
quart1=sigma1.^2;
quart2=sigma2.^2;
coquart=sigma12.^2;
coprod12=sigma1.*sigma2;
coprod112=sigma1.*sigma12;
coprod212=sigma2.*sigma12;

% select spot (co-)variances at segment limits and centers
quart1_t=quart1(1:2:end);       %t=0:2/N:1 (segment limits)
quart2_t=quart2(1:2:end);
coquart_t=coquart(1:2:end);
coprod12_t=coprod12(1:2:end);
coprod112_t=coprod112(1:2:end);
coprod212_t=coprod212(1:2:end);
sigma_t=[quart1_t quart2_t coquart_t coprod12_t coprod112_t coprod212_t];   %join

% select spot (co-)variances at segment centers
quart1_t_m=quart1(2:2:end-1);       %t_m=1/N:2/N:(1-1/N) (segment centers)
quart2_t_m=quart2(2:2:end-1);
coquart_m=coquart(2:2:end-1);
coprod12_t_m=coprod12(2:2:end-1);
coprod112_t_m=coprod112(2:2:end-1);
coprod212_t_m=coprod212(2:2:end-1);
sigma_t_m=[quart1_t_m quart2_t_m coquart_m coprod12_t_m coprod112_t_m coprod212_t_m];   %join


% simpson rule
isigma= sum((h/3)*(sigma_t(1:end-1,:)+4*sigma_t_m(1:end,:)+sigma_t(2:end,:)));

iquart1=isigma(:,1);
iquart2=isigma(:,2);
icoquart=isigma(:,3);
icoprod12=isigma(:,4);
icoprod112=isigma(:,5);
icoprod212=isigma(:,6);










