function [y,sigma]=sim_svol_1f_seas(mu,alpha,beta0,beta1,rho,N,phisq)
%% sim_svol_1f_seas
%--------------------------------------------------------------
% Simulates (adjusted) 1-factor stochastic volatility model following
% Chernov et al. (2003, JE) extended by deterministic 
%intraday vola pattern 
% uses euler discretization for  log-price processes and 
% spot variances
%--------------------------------------------------------------
% INPUT:
% mu        -> drift y
% alpha      -> rate in drift of logarithmic stoch. vola v
% beta0, beta1 ->parameters for exponential transformation of logarithmic stoch. vola
% rho           -> spot correlation between y and v ("leverage")
% N             -> # of intervals for euler discretization (usually 23400,
% corresponds to T=1 and 6.5 h hours of trading)
% phisq         -> ((N+1)x1) vector of deterministic seasonal spot variance
%                       factor (N+1 elements due to starting value)
%--------------------------------------------------------------
% OUTPUT:
% y  -> (Nx1) vector of simulated obs of log-prices 
% sigma  -> ((N+1)x1) vector of simulated obs of spot volatility (incl starting values)
% -------------------------------------------------------------------------
%
% Peter Malec, April 9, 2014
% -------------------------------------------------------------------------
%   
%% Preparations

%check argument
if nargin<7||isempty(phisq)
    phisq =ones(N+1,1);
end

% alpha negative
if alpha>=0
    error('Alpha needs to be negative!')
end

% check dimensions
if ~isequal(length(phisq),(N+1))
    error('phi has needs to have (N+1) elements !!!')
end

% square root of seasonality
phi=sqrt(phisq(:));

% step size
h=1/N;

%initialize vectors
y=NaN*zeros(N,1);
v=NaN*zeros(N,1);
sigma=NaN*zeros(N,1);

%starting values
y0=0;
v0 = randn/(sqrt(-2*alpha)); % sample from stationary distribution 
sigma0 =exp(beta0+beta1*v0)*phi(1);  % include intraday seasonality factor
% -------------------------------------------------------------------------
%   
%% Simulations of Brownian Increments

% generate (uncorrelated) N(0,1) pseudo-random numbers
Bmat=randn(N,2);

% assign to BM processes
dB=Bmat(:,1);
dZ=Bmat(:,2);

% derive brownian motion entering v process
dW=rho*dB+sqrt(1-rho^2)*dZ;%rho: (spot) corr between B & Z
% -------------------------------------------------------------------------
%   
%% Simulate SV Processes by euler discretization

% Add one row to vectors (for starting values)
y=[y0;y];
v=[v0;v];
sigma=[sigma0;sigma];

% Start Loop
% --------------------------------------
for i=1:N
    
    % log-vola process by euler discretization
    v(i+1)=v(i)+alpha*v(i)*h+sqrt(h)*dW(i);
    
     % compute spot volatilities
     sigma(i+1)=exp(beta0+beta1*v(i+1))*phi(i+1);% positive-valued transformation for volatilities
                                                                                 % include intraday seasonality factor
     
    % simulate sv processes
    y(i+1)=y(i)+mu*h+sigma(i)*sqrt(h)*dB(i);
    
end
% -------------------------------------------------------------------------
%   
%% Adjust Output

% remove 1st row from output vectors y1 and y2
y=y(2:end);



