function [y,sigma]=sim_svol_2f_seas_multivariate_factor(m,rhotild,mu,alpha1,alpha2,beta0,beta1,beta2,betav,rho1,rho2,N,Nst,u0,phisq,...
J_time,J_size,J_size_det,J_prob)
%% sim_svol_2f_seas_multivariate_factor
%--------------------------------------------------------------
% Simulates simple multivatiate factor model as in 
% Barndorff-Nielsen, Hansen, Lunde & Shephard (2011, JE)
% based on equicorrelation structure for signal
% with univariate volatilities following (adjusted) 2-factor
% stochastic volatility model following
% Chernov et al. (2003, JE) extended by deterministic 
%intraday vola pattern. 
% uses euler discretization for  log-price processes and 
% spot variances
%--------------------------------------------------------------
% INPUT:
% m          -> number of dimensions
% rhotild -> square root of spot correlation between signals
% mu        -> (mx1) vector or scalar, drift y
% alpha1, alpha2      -> (mx1) vector or scalar, rate in drift of logarithmic stoch. volas v1 and
%                                   v2
% beta0, beta1,beta2 -> (mx1) vector or scalar,parameters for exponential transformation of logarithmic
%                           stoch. volas v1 and v2
% betav                  -> (mx1) vector or scalar,vol of vol parameter
% rho1, rho2           -> (mx1) vector or scalar, spot correlation between y and v1 or y and v2 (two "leverage" parameters)
% N             -> # of intervals for euler discretization (usually 23400,
% corresponds to T=1 and 6.5 h hours of trading)
% Nst           -> number of observations used for initializaiton of
%                       logarithmic stoch. volas v1 and v2 (no initialization period of Nst empty)
% phisq         -> ((N+1)xm) matrix or ((N+1)x1) vector of deterministic seasonal spot variance
%                       factor (N+1 elements due to starting value)
% u0             -> threshold for s-exp function
% J_time     -> nJx1 vector of fixed jump times \in [0,1] (optional)
% J_size     -> nJx1 vector of jump sizes or expected jump sizes (optional)
% J_size_det -> if true, jump size is deterministic, else
%                           exponential with mean J_size
% J_prob    -> nJx1 vector of jump probabilities (in one given series)
%--------------------------------------------------------------
% OUTPUT:
% y  -> (Nxm) matrix of simulated obs of log-prices 
% sigma  -> ((N+1)xm) matrix of simulated obs of spot volatility (incl starting values)
% -------------------------------------------------------------------------
%
% Peter Malec, April 26, 2016
% -------------------------------------------------------------------------
%   
%% Preparations

%check argument
if nargin<15
    error('Wrong number of arguments!')
end

%check argument
if nargin<19
    J_time=[];
    J_size=J_time;
    J_size_det=J_time;
    J_prob=J_time;
end

%check argument
if isempty(phisq)
    phisq =ones(N+1,1);
end

% check Nst
if ~isempty(Nst)&&(Nst<1)
    error('Nst has to be >=1!!!')
end

% rhotilde outside bounds
if (rhotild^2>1)
    error('rhotild^2 needs to be <=1!')
end

% alpha negative
if any(alpha1>=0)||any(alpha2>=0)
    error('Alpha1 and alpha2 need to be negative!')
end

% check dimensions
if ~isequal(size(phisq,1),(N+1))
    error('phi has needs to have (N+1) rows !!!')
end

% check dimensions
if (~isequal(size(phisq,2),1))&&(~isequal(size(phisq,2),m))
    error('phi has needs to have m columns !!!')
end

%check dimensions
if ((length(mu)>1)&&(~isequal(length(mu),m)))
    error('mu needs to have 1 or m elements!')
end
if ((length(alpha1)>1)&&(~isequal(length(alpha1),m)))
    error('alpha1 needs to have 1 or m elements!')
end
if ((length(alpha2)>1)&&(~isequal(length(alpha2),m)))
    error('alpha2 needs to have 1 or m elements!')
end
if ((length(beta0)>1)&&(~isequal(length(beta0),m)))
    error('beta0 needs to have 1 or m elements!')
end
if ((length(beta1)>1)&&(~isequal(length(beta1),m)))
    error('beta1 needs to have 1 or m elements!')
end
if ((length(beta2)>1)&&(~isequal(length(beta2),m)))
    error('beta2 needs to have 1 or m elements!')
end
if ((length(betav)>1)&&(~isequal(length(betav),m)))
    error('betav needs to have 1 or m elements!')
end
if ((length(rho1)>1)&&(~isequal(length(rho1),m)))
    error('rho1 needs to have 1 or m elements!')
end
if ((length(rho2)>1)&&(~isequal(length(rho2),m)))
    error('rho2 needs to have 1 or m elements!')
end

% check dimensions
if ~isequal(size(J_time),size(J_size))
    error('J_time and J_size must have same dimension!')
end
if ~isequal(size(J_time),size(J_prob))
    error('J_time and J_prob must have same dimension!')
end

% check if vector
if (~isempty(J_time))&&(~isvector(J_time))
    error('J_time, J_size and J_prob must be vectors')
end

%check correct values of jump times
if any((J_time>1)|(J_time<0))
    error('Jump times must be between zero and one!')
end

%check correct values of jump probabilities
if any((J_prob>1)|(J_prob<0))
    error('Jump probabilities must be between zero and one!')
end

% include jumps?
if (~isempty(J_time)&&~isempty(J_size)&&~isempty(J_prob)&&~isempty(J_size_det))
    doJump=true;
else
    doJump=false;
end

%expand parameters if necessary
if isequal(length(mu),1)
    mu=mu(ones(m,1),1);
end
if isequal(length(alpha1),1)
    alpha1=alpha1(ones(m,1),1);
end
if isequal(length(alpha2),1)
    alpha2=alpha2(ones(m,1),1);
end
if isequal(length(beta0),1)
    beta0=beta0(ones(m,1),1);
end
if isequal(length(beta1),1)
    beta1=beta1(ones(m,1),1);
end
if isequal(length(beta2),1)
    beta2=beta2(ones(m,1),1);
end
if isequal(length(betav),1)
    betav=betav(ones(m,1),1);
end
if isequal(length(rho1),1)
    rho1=rho1(ones(m,1),1);
end
if isequal(length(rho2),1)
    rho2=rho2(ones(m,1),1);
end
if isequal(size(phisq,2),1)
    phisq=phisq(:,ones(m,1));
end
% -------------------------------------------------------------------------
%   
%% Simulate required brownian motions

% Simulate
if ~isempty(Nst)   
    Bmat=randn((N+Nst),3*m+1);
else
    % generate (uncorrelated) N(0,1) pseudo-random numbers
    Bmat=randn(N,3*m+1);
end

% BM for common factor
dW= Bmat(:,1);
Bmat=Bmat(:,2:end);
% -------------------------------------------------------------------------
%   
%% Simulate jump sizes if necessary

if doJump
   
    % simulate jump sizes 
     if ~J_size_det
        J_size= -log(1-rand(length(J_time),m)).*J_size(:,ones(m,1));
     else
        J_size= J_size(:,ones(m,1));
    end
    
    % simulate jump occurrence at given times
    J_size=(rand(length(J_time),m)<J_prob(:,ones(m,1))).*J_size;
   
end

% -------------------------------------------------------------------------
%   
%% Simulate each dimension

% create collection matrices
y=NaN*zeros(N,m);
sigma=NaN*zeros(N+1,m);

% loop over dimensions
for iii=1:m
    
    % create correlated brownian motion
    dV=rhotild*dW+sqrt(1-rhotild^2)*Bmat(:,3*(iii-1)+1);
    
    % select jump sizes
    if doJump
        J_size_temp=J_size(:,iii);
    else
        J_size_temp=[];
    end
    
    % simulate for given dimension
    [y_temp,sigma_temp]=sim_svol_2f_seas_univ(mu(iii),alpha1(iii),alpha2(iii),beta0(iii),beta1(iii),beta2(iii),betav(iii),...
    rho1(iii),rho2(iii),N,Nst,u0,phisq(:,iii),[Bmat(:,3*(iii-1)+(2:3)) dV],J_time,J_size_temp);

    % assign
    y(:,iii)=y_temp;
    sigma(:,iii)=sigma_temp;
end



