function [y,sigma]=sim_svol_2f_seas_univ(mu,alpha1,alpha2,beta0,beta1,beta2,betav,rho1,rho2,N,Nst,u0,phisq,Bmat,J_time,J_size)
%% sim_svol_2f_seas_univ
%--------------------------------------------------------------
% Simulates (adjusted) 2-factor stochastic volatility model following
% Chernov et al. (2003, JE) extended by deterministic 
%intraday vola pattern and jumps (at fixed times) in first factor 
% uses euler discretization for  log-price processes and 
% spot variances
%--------------------------------------------------------------
% INPUT:
% mu        -> drift y
% alpha1, alpha2      -> rate in drift of logarithmic stoch. volas v1 and
%                                   v2
% beta0, beta1,beta2 ->parameters for exponential transformation of logarithmic
%                           stoch. volas v1 and v2
% betav                  -> vol of vol parameter
% rho1, rho2           -> spot correlation between y and v1 or y and v2 (two "leverage" parameters)
% N             -> # of intervals for euler discretization (usually 23400,
% corresponds to T=1 and 6.5 h hours of trading)
% Nst           -> number of observations used for initializaiton of
%                       logarithmic stoch. volas v1 and v2 (no initialization period of Nst empty)
% phisq         -> ((N+1)x1) vector of deterministic seasonal spot variance
%                       factor (N+1 elements due to starting value)
% u0             -> threshold for s-exp function
% Bmat        -> ((N+Nst)x3) matrix of independent standard Gaussian random
%                       numbers
% J_time     -> nJx1 vector of fixed jump times \in [0,1] (optional)
% J_size     -> nJx1 vector of jump sizes (optional)
%--------------------------------------------------------------
% OUTPUT:
% y  -> (Nx1) vector of simulated obs of log-prices 
% sigma  -> ((N+1)x1) vector of simulated obs of spot volatility (incl starting values)
% -------------------------------------------------------------------------
%
% Peter Malec, May 18, 2016
% -------------------------------------------------------------------------
%   
%% Preparations

%check argument
if nargin<14
    error('Wrong number of arguments!')
end

%check argument
if nargin<16
    J_time=[];
    J_size=J_time;
end

%check argument
if isempty(phisq)
    phisq =ones(N+1,1);
end

% check Nst
if ~isempty(Nst)&&(Nst<1)
    error('Nst has to be >=1!!!')
end

% alpha negative
if (alpha1>=0)||(alpha2>=0)
    error('Alpha1 and alpha2 need to be negative!')
end

% check dimensions
if ~isequal(length(phisq),(N+1))
    error('phi has needs to have (N+1) elements !!!')
end

% check dimensions
if (~isequal(size(Bmat,1),(Nst+N)))||(~isequal(size(Bmat,2),3))
    error('Bmat needs to be a  ((N+Nst)x3) matrix !!!')
end

% check dimensions
if ~isequal(size(J_time),size(J_size))
    error('J_time and J_size must have same dimension!')
end

% check if vector
if (~isempty(J_time))&&(~isvector(J_time))
    error('J_time and J_size must be vectors')
end

%check correct values of jump times
if any((J_time>1)|(J_time<0))
    error('Jump times must be between zero and one!')
end

% include jumps?
if (~isempty(J_time)&&~isempty(J_size))
    doJump=true;
else
    doJump=false;
end

% square root of seasonality
phi=sqrt(phisq(:));

% step size
h=1/N;

%initialize vectors
y=NaN*zeros(N,1);
v1=NaN*zeros(N,1);
v2=NaN*zeros(N,1);
sigma=NaN*zeros(N,1);
% -------------------------------------------------------------------------
%   
%% Determine position of jumps in euler discr

if doJump

    % no of jumps
    nJ=length(J_time);

    % initialise vector of jump indicators
    J_ind=NaN*zeros(nJ,1);

    % loop over jumps and check where they occur on grid
    SimGrid=0:h:1;
    for iii=1:nJ
        J_ind(iii)=find((J_time(iii) <= SimGrid(2:end))&(J_time(iii) > SimGrid(1:(end-1))),1);
    end
    J_ind=J_ind+1;

end
% -------------------------------------------------------------------------
%   
%% Assignment of Brownian Increments

if ~isempty(Nst)      
    dZ1=Bmat((Nst+1):end,1);
    dZ2=Bmat((Nst+1):end,2);
    dV=Bmat((Nst+1):end,3);
else
    dZ1=Bmat(:,1);
    dZ2=Bmat(:,2);
    dV=Bmat(:,3);    
end

% derive brownian motions entering vola processes
dW1=rho1*dV+sqrt(1-rho1^2)*dZ1;
dW2=rho2*dV+sqrt(1-rho2^2)*dZ2;
if ~isempty(Nst)
    dW2_ini=rho2*Bmat(1:Nst,3)+sqrt(1-rho2^2)*Bmat(1:Nst,2);
end
% -------------------------------------------------------------------------
%   
%% Initialize v2 process

%starting value
v20=0;

if ~isempty(Nst)
    
    %initialize vector
    v2_ini=NaN*zeros(Nst,1);
    
    % Add one row to vector (for starting values)
    v2_ini=[v20;v2_ini];
    
    % Start Loop
    % --------------------------------------
    for i=1:Nst
    
        % log-vola process by euler discretization
        v2_ini(i+1)=v2_ini(i)+alpha2*v2_ini(i)*h+(1+betav*v2_ini(i))*sqrt(h)*dW2_ini(i);
    
    end
    
    %last value as actual starting value
    v20=v2_ini(end);
    
end
% -------------------------------------------------------------------------
%   
%% Simulate SV Processes by euler discretization

%remaining starting values
y0=0;
v10 = randn/(sqrt(-2*alpha1)); % sample from stationary distribution 
sigma0 =s_exp(beta0+beta1*v10+beta2*v20,u0)*phi(1);  % include intraday seasonality factor

% Add one row to vectors (for starting values)
y=[y0;y];
v1=[v10;v1];
v2=[v20;v2];
sigma=[sigma0;sigma];

% Start Loop
% --------------------------------------
for i=1:N
    
    % log-vola process by euler discretization (v1)
    v1(i+1)=v1(i)+alpha1*v1(i)*h+sqrt(h)*dW1(i);
    
    % add vola jumps to first factor
    if doJump
        for jjj=1:nJ
            if isequal(i,J_ind(jjj))
                v1(i+1)=v1(i+1)+J_size(jjj);
            end
        end
    end
    
    % log-vola process by euler discretization (v2)
    v2(i+1)=v2(i)+alpha2*v2(i)*h+(1+betav*v2(i))*sqrt(h)*dW2(i);
    
     % compute spot volatilities
     sigma(i+1)=s_exp(beta0+beta1*v1(i+1)+beta2*v2(i+1),u0)*phi(i+1);% positive-valued transformation for volatilities
                                                                                 % include intraday seasonality factor
     
    % simulate sv processes
    y(i+1)=y(i)+mu*h+sigma(i)*sqrt(h)*dV(i);
    
end
% -------------------------------------------------------------------------
%   
%% Adjust Output

% remove 1st row from output vectors y1 and y2
y=y(2:end);



