function [y1,y2,sigma1,sigma2,sigma12]=sim_svol_factor_BNHLS_seas(mu1,mu2,beta01,beta02,beta11,beta12,alpha1,alpha2,rho1,rho2,tau1,tau2,N,phisq)
%% sim_svol_factor_BNHLS_nlevg
%--------------------------------------------------------------
% Simulates 2-dimensional factor stochastic volatility model following
% Barndorff-Nielsen et al. (JE, 2011) extended by deterministic 
%intraday vola pattern and with leverage decoupled from correlation between signals
%. uses euler discretization for  log-price processes and 
% exact discreltization for OU process driving volatilities. 
%--------------------------------------------------------------
% INPUT:
% mu1:rho2 -> parameters of the model (see BNHLS(2011))
% tau1         -> spot correlation between y1 and gamma1 ("leverage" of 
%                       asset 1)
% N             -> # of intervals for euler discretization (usually 23400,
% corresponds to T=1 and 6.5 h hours of trading)
% phisq         -> ((N+1)x2) matrix of deterministic seasonal spot variance
%                       factor (N+1 elements due to starting value)
%--------------------------------------------------------------
% OUTPUT:
% y1  -> (Nx1) vector of simulated obs of log-prices of asset 1
% y2  -> (Nx1) vector of simulated obs of log-prices of asset 2
% sigma1  -> ((N+1)x1) vector of simulated obs of spot volatility of asset
% 1 (incl starting values)
% sigma2  -> ((N+1)x1) vector of simulated obs of spot volatility of asset 2
% sigma12  -> ((N+1)x1) vector of simulated obs of spot covolatility of asset 1 and 2
% -------------------------------------------------------------------------
%
% Peter Malec, October 10, 2012
% -------------------------------------------------------------------------
%   
%% Preparations

%check argument
if nargin<12
    phisq =ones(N+1,1);
end

% check dimensions
if size(phisq,1)<(N+1)
    error('phi has needs to have (N+1) rows !!!')
end

if size(phisq,2)==2
    phi1=sqrt(phisq(:,1));
    phi2=sqrt(phisq(:,2));
elseif size(phisq,2)==1
    phi1=sqrt(phisq);
    phi2=phi1;
else
     error('phi has needs to have 1 or 2 columns !!!')
end


% step size
h=1/N;

%initialize vectors
y1=NaN*zeros(N,1);
y2=NaN*zeros(N,1);
gam1=NaN*zeros(N,1);
gam2=NaN*zeros(N,1);
sigma1=NaN*zeros(N,1);
sigma2=NaN*zeros(N,1);

% adjust leverage parameter for corr between assets in model
tau1st=tau1/rho1;
tau2st=tau2/rho2;

%starting values
y10=0;
y20=0;
gam10=randn*sqrt((-2*alpha1)^(-1)); % sample from stationary distribution of OU-process
gam20=randn*sqrt((-2*alpha2)^(-1));
sigma10=exp(beta01+beta11*gam10)*phi1(1);   % positive-valued transformation for volatilities
sigma20=exp(beta02+beta12*gam20)*phi2(1);       % include intraday seasonality factor
% -------------------------------------------------------------------------
%   
%% Simulations of Brownian Increments

% generate (uncorrelated) N(0,1) pseudo-random numbers
Bmat=randn(N,5);

% assign to BM processes
dB1=Bmat(:,1);
dB2=Bmat(:,2);
dM1=Bmat(:,3);
dM2=Bmat(:,4);
dW=Bmat(:,5);

% derive BM increments entering gamma process
dZ1=tau1st*dB1+sqrt(1-tau1st^2)*dM1;
dZ2=tau2st*dB2+sqrt(1-tau2st^2)*dM2;%tau1st: (spot) corr between B1 & Z1
% -------------------------------------------------------------------------
%   
%% Simulate SV Processes by euler discretization

% Add one row to vectors (for starting values)
y1=[y10;y1];
y2=[y20;y2];
gam1=[gam10;gam1];
gam2=[gam20;gam2];
sigma1=[sigma10;sigma1];
sigma2=[sigma20;sigma2];

% Start Loop
% --------------------------------------
for i=1:N
    
    % simulate OU process for gamma by exact discretization
    gam1(i+1)=exp(alpha1*h)*gam1(i)+sqrt((-2*alpha1)^(-1)*(1-exp(2*alpha1*h)))*dZ1(i);
    gam2(i+1)=exp(alpha2*h)*gam2(i)+sqrt((-2*alpha2)^(-1)*(1-exp(2*alpha2*h)))*dZ2(i);
    
    % compute spot volatilities
    sigma1(i+1)=exp(beta01+beta11*gam1(i+1))*phi1(i+1);   % positive-valued transformation for volatilities
    sigma2(i+1)=exp(beta02+beta12*gam2(i+1))*phi2(i+1);   % include intraday seasonality factor
    
    % simulate sv processes
    y1(i+1)=y1(i)+mu1*h+rho1*sigma1(i)*sqrt(h)*dB1(i)+sqrt(1-rho1^2)*sigma1(i)*sqrt(h)*dW(i);
    y2(i+1)=y2(i)+mu2*h+rho2*sigma2(i)*sqrt(h)*dB2(i)+sqrt(1-rho2^2)*sigma2(i)*sqrt(h)*dW(i);
    
end
% -------------------------------------------------------------------------
%   
%% Adjust Output

% remove 1st row from output vectors y1 and y2
y1=y1(2:end);
y2=y2(2:end);

% compute spot covolatility
sigma12=sqrt((1-rho1^2)*(1-rho2^2))*sigma1.*sigma2;

