function [y1,y2,sigma21,sigma22,sigma12]=sim_svol_heston_seas(mu1,mu2,alpha1,alpha2,msigma21,msigma22,psi1,psi2,rho,tau1,tau2,N,phisq)
%% sim_svol_heston_seas
%--------------------------------------------------------------
% Simulates (adjusted) 2-dimensional stochastic volatility model based on heston structure extended by deterministic 
%intraday vola pattern 
% uses euler discretization for  log-price processes and 
% spot variances
%--------------------------------------------------------------
% INPUT:
% mu1        -> drift y1
% alpha1      -> speed of mean reversion spot var 1 (stoch. part)
% msigma21  -> long-run mean (stoch. part) spot var 1 (mean of stationary distribution)
% psi1           -> vol of  vol 1 (stoch. part)
% rho           -> spot correlation between y1 and y2 (corrr between signals)
% tau1         -> spot correlation between y1 and gamma1 ("leverage" of 
%                       asset 1)
% N             -> # of intervals for euler discretization (usually 23400,
% corresponds to T=1 and 6.5 h hours of trading)
% phisq         -> ((N+1)x2) matrix of deterministic seasonal spot variance
%                       factor (N+1 elements due to starting value)
%--------------------------------------------------------------
% OUTPUT:
% y1  -> (Nx1) vector of simulated obs of log-prices of asset 1
% y2  -> (Nx1) vector of simulated obs of log-prices of asset 2
% sigma21  -> ((N+1)x1) vector of simulated obs of spot variances of asset
% 1 (incl starting values)
% sigma22  -> ((N+1)x1) vector of simulated obs of spot variances of asset 2
% sigma12  -> ((N+1)x1) vector of simulated obs of spot covolatility of asset 1 and 2
% -------------------------------------------------------------------------
%
% Peter Malec, October 30, 2012
% -------------------------------------------------------------------------
%   
%% Preparations

%check argument
if nargin<13||isempty(phisq)
    phisq =ones(N+1,1);
end

% check dimensions
if size(phisq,1)<(N+1)
    error('phi has needs to have (N+1) rows !!!')
end

if size(phisq,2)==2
    phi1=sqrt(phisq(:,1));
    phi2=sqrt(phisq(:,2));
elseif size(phisq,2)==1
    phi1=sqrt(phisq);
    phi2=phi1;
else
     error('phi has needs to have 1 or 2 columns !!!')
end


% step size
h=1/N;

%initialize vectors
y1=NaN*zeros(N,1);
y2=NaN*zeros(N,1);
sigma21v=NaN*zeros(N,1);
sigma22v=NaN*zeros(N,1);
sigma21=NaN*zeros(N,1);
sigma22=NaN*zeros(N,1);

%starting values
y10=0;
y20=0;
sigma210v = max(mygamrnd((2*alpha1*msigma21/(psi1^2)),psi1^2/(2*alpha1),1),0); % sample from stationary distribution of CIR-process
sigma220v = max(mygamrnd((2*alpha2*msigma22/(psi2^2)),psi2^2/(2*alpha2),1),0);

sigma210 = sigma210v*phi1(1); 
sigma220 = sigma220v*phi2(1);% include intraday seasonality factor
% -------------------------------------------------------------------------
%   
%% Simulations of Brownian Increments

% generate (uncorrelated) N(0,1) pseudo-random numbers
Bmat=randn(N,4);

% assign to BM processes
dB1=Bmat(:,1);
dM1=Bmat(:,2);
dM2=Bmat(:,3);
dW=Bmat(:,4);

% derive brownian motion entering y2 process
dB2=rho*dB1+sqrt(1-rho^2)*dW;%rho: (spot) corr between B1 & B2

% derive BM increments entering variance process
dZ1=tau1*dB1+sqrt(1-tau1^2)*dM1;
dZ2=tau2*dB2+sqrt(1-tau2^2)*dM2;%tau: (spot) corr between B1 & Z1
% -------------------------------------------------------------------------
%   
%% Simulate SV Processes by euler discretization

% Add one row to vectors (for starting values)
y1=[y10;y1];
y2=[y20;y2];
sigma21v=[sigma210v;sigma21v];
sigma22v=[sigma220v;sigma22v];
sigma21=[sigma210;sigma21];
sigma22=[sigma220;sigma22];

% Start Loop
% --------------------------------------
for i=1:N
    
    % simulate CIR process  by euler discretization
    sigma21v(i+1)=sigma21v(i)+alpha1*(msigma21-sigma21v(i))*h+sqrt(h)*psi1*sqrt(max(sigma21v(i),0))*dZ1(i);
    sigma22v(i+1)=sigma22v(i)+alpha2*(msigma22-sigma22v(i))*h+sqrt(h)*psi2*sqrt(max(sigma22v(i),0))*dZ2(i);
    
     % compute spot variances
     sigma21(i+1)=max(sigma21v(i+1),0)*phi1(i+1);   % positive-valued transformation for volatilities
     sigma22(i+1)=max(sigma22v(i+1),0)*phi2(i+1);   % include intraday seasonality factor
     
    % simulate sv processes
    y1(i+1)=y1(i)+mu1*h+sqrt(sigma21(i)*h)*dB1(i);
    y2(i+1)=y2(i)+mu2*h+sqrt(sigma22(i)*h)*dB2(i);
    
end
% -------------------------------------------------------------------------
%   
%% Adjust Output

% remove 1st row from output vectors y1 and y2
y1=y1(2:end);
y2=y2(2:end);

% compute spot covolatility
sigma12=rho*sqrt(sigma21.*sigma22);

