function spot_beta_simp_func(ticker,market,tdate,cl_ind,input_v,startt,endt,...
    J_p,theta_h,theta_J,theta_K,K_p_thr,twost,twostc,lKb,ccorr,inpath,sourced,outpath)
%% spot_beta_simp_func
%--------------------------------------------------------------
% computes spot beta estimates based on local likelihood estimates of spot covariances
%for specified estimation points. Data is loaded. Outputs are saved directly
%--------------------------------------------------------------
% INPUT:
% ticker -> String: asset 1
% market -> String: market proxy
% tdate   -> data in matlab format
% cl_ind    -> if >0, data cleaned according to proc by BNHLS(2009) is used
% input_v -> points of estimation
% startt -> start of trading (in hours)
% endt   -> end of trading (in hours)
% J_p-> spectral cutoff for pilot estimates of spot covariances
% theta_h   -> proportionality parameter for block length (relevant if B
%                           not specified)
% theta_J     -> proportionaliy parameter for cutoff (relevant if J
%                           or J_p not specified)
% theta_K    -> proportionality parameter for smoothing bandwidth (relevant if K
%                           or K_p not specified)
% K_p_thr    -> lower threshold for pilot smoothing bandwidth (relevant if
%                           K_p not specified): default=1
% twst     -> if >0: twostep estimation is used (estimates from first step
%                   used as pilots for second step
% twstc      -> if >0: Covariance matrices of smoothed spot (co-)volatilities
%   under the local parametric experiment   are computed using 2ndstep spot
%   covariance estimates (mattters only if 8 output arguments)
% lkB         -> if >0, shorter smoothing windows at beginning and end of
%                   day are used for spot cov estim (NOT PILOT!)
% ccorr -> if >0, correlation and variance estimates are corrected
% inpath     -> path for loading data
% sourced   -> data source
%--------------------------------------------------------------
%
% Peter Malec, May 22 2013 
%
% -------------------------------------------------------------------------
%   
%% Status

%disp(['Estimating Spotbetas: ' ticker ' vs. ' market ' ...'])
% -------------------------------------------------------------------------
%   
%% Load Data

% clean?
if cl_ind==1
    cl_str='cl';            % cleaning string for output file
else
    cl_str='ncl';
end

% label lKb
if lKb>0
     lKbl='_lKb';
else
     lKbl='';
end

% label ccorr
if ccorr>0
     ccorrl='_ccorr';
else
     ccorrl='';
end

%label twostep
if twost>0
     twostl='_2st';
else
     twostl='';
end

% label: estimator covariance
if twostc>0
      twostcl='_2stc';
else
      twostcl='';
end

%clean labels
theta_hl=num2str(theta_h);
theta_hl(theta_hl=='.')='';
theta_Jl=num2str(theta_J);
theta_Jl(theta_Jl=='.')='';
theta_Kl=num2str(theta_K);
theta_Kl(theta_Kl=='.')='';

% name
basename=strcat(outpath,sourced,'_',ticker,'_',market,'_',datestr(tdate,'yyyymmdd'),'_',cl_str,...
            '_th',theta_hl,'tJ',theta_Jl,'tK',theta_Kl,'J_p',int2str(J_p),'K_pt',int2str(K_p_thr),lKbl,ccorrl,twostl,twostcl);


% check for existence of file
index=isequal(exist(strcat(basename,'_scov.mat'),'file'),2);


if ~isequal(index,1)
%---------------------------------------------------------
%
%% Compute Spot Cov estimates 

% Load Data
%------------------------------------------------------------

% clean?
if cl_ind==1
    cl_str='cl';            % cleaning string for output file
else
    cl_str='ncl';
end

% names
name1=strcat(inpath,ticker,'_',datestr(tdate,'yyyymmdd'),'_midp_',cl_str,'.mat');
name2=strcat(inpath,market,'_',datestr(tdate,'yyyymmdd'),'_midp_',cl_str,'.mat');

% check for existence of files
index1=isequal(exist(name1,'file'),2);
index2=isequal(exist(name2,'file'),2);

if index1&&index2
%---------------------------------------------------------

% load
load(name1);
data_1=midprice;
load(name2);
data_2=midprice;

% estimate noise variance
%-------------------------------------------------
eta21=noise_cov_est(data_1(:,2),'cov');
if eta21<=0
    eta21=noise_cov_est(data_1(:,2),'var');
end
eta22=noise_cov_est(data_2(:,2),'cov');
if eta22<=0
    eta22=noise_cov_est(data_2(:,2),'var');
end
Omega_mat=diag([eta21 eta22]);  % assume no cross-sectional dependence

% Estimate SpotCov
%------------------------------
[SigmaSpot_out,block_st,~,J_optv,J_p,K,K_p,CovSigmaSpot_out]=lle_spot_cov_corrd_simp(input_v,data_1,data_2,Omega_mat,startt,endt,...
    1,1,[],[],J_p,[],[],theta_h,theta_J,theta_K,[],K_p_thr,twost,twostc,lKb,ccorr);

% corr with variances of corrs
%------------------------------
[SpotCorr,SpotCorr_var]=comp_corr(SigmaSpot_out,CovSigmaSpot_out);
    
%save spot covariance matrices
scov_out.est=SigmaSpot_out;
scov_out.var=CovSigmaSpot_out;
scov_out_nout=strcat(basename,'_scov');
save(scov_out_nout,'scov_out')

%save spot correlation
scorr_out.est=SpotCorr;
scorr_out.var=SpotCorr_var;
scorr_out_nout=strcat(basename,'_scorr');
save(scorr_out_nout,'scorr_out')

%save LMM config
LMM_conf.gen=[length(block_st) max(J_optv) J_p K K_p];
LMM_conf.Jex=J_optv;
LMM_conf_nout=strcat(basename,'_LMM_conf');
save(LMM_conf_nout,'LMM_conf')

%file do not exist: do nothing
%----------------------------------------
else
    disp(['Data for ' ticker ' or ' market ' are missing. Skip.'])
end

else 
    
   index1=1;
   index2=1;
   
   % load scovs
    load(strcat(basename,'_scov.mat'));
end
%---------------------------------------------------------
%
%% Compute Spot Beta estimates and save

if index1&&index2
    
% betas with variances of betas
[SpotBeta,SpotBeta_var]=comp_beta(scov_out.est,scov_out.var);

%save spot betas
sbeta_out.est=SpotBeta;
sbeta_out.var=SpotBeta_var;
sbeta_out_nout=strcat(basename,'_sbeta');
save(sbeta_out_nout,'sbeta_out')

end






    
  
    