function spot_covm_simp_func(ticker_a,tdate,typed,cl_ind,rev_ind,input_v,startt,endt,...
    J_p,theta_h,theta_J,theta_K,K_p_thr,compcov,twost,twostc,lKb,ccorr,ccorrlast,sdnoise_in,inpath,sourced,outpath)
%% spot_covm_simp_func
%--------------------------------------------------------------
% local likelihood estimation of spot covariance matrices for specified
% estimation points. Date is loaded. Outputs are saved directly
%
%--------------------------------------------------------------
% INPUT:
% ticker_a -> (mx1) cell array of ticker symbols (strings)
% tdate   -> data in matlab format
% cl_ind    -> if ==1, data cleaned according to proc by BNHLS(2009) is used
% rev_ind    -> if ==1, only obs with price revisions are used
% input_v -> points of estimation
% startt -> start of trading (in hours)
% endt   -> end of trading (in hours)
% J_p-> spectral cutoff for pilot estimates of spot covariances
% theta_h   -> proportionality parameter for block length (relevant if B
%                           not specified)
% theta_J     -> proportionaliy parameter for cutoff (relevant if J
%                           or J_p not specified)
% theta_K    -> proportionality parameter for smoothing bandwidth (relevant if K
%                           or K_p not specified)
% K_p_thr    -> lower threshold for pilot smoothing bandwidth (relevant if
%                           K_p not specified): default=1
% compcov    -> if >0, covariances are computed
% twst     -> if >0: twostep estimation is used (estimates from first step
%                   used as pilots for second step
% twstc      -> if >0: Covariance matrices of smoothed spot (co-)volatilities
%   under the local parametric experiment   are computed using 2ndstep spot
%   covariance estimates (mattters only if 8 output arguments)
% lkB         -> if >0, shorter smoothing windows at beginning and end of
%                   day are used for spot cov estim (NOT PILOT!)
% ccorr -> if >0, correlation and variance estimates are corrected
% ccorrlast -> if >0, correlation and variance estimates are corrected on last
%                       step
% sdnoise_in -> "non iid noise" strcture with elemensts [alpha,Q_max,skips,inpathn], where
%                       alpha-> significance level for AC test
%                       Q_max-> maximum order of serial dependence + 1
%                       skips->skip sampling frequency (for correction as
%                       in H&L (2006, JBES)) in seconds, if empty, then no
%                                                       correction
%                       inpathn-> path for loading noise estimates
%                   Note: if sdnoise_in is empty, iid noise case is considered
%                       (variance estimated on the fly)
% inpath     -> path for loading data
% sourced   -> data source
% outpath  -> path for saving
%--------------------------------------------------------------
%
% Peter Malec,  February 12 2014 
%
% Modifications May 2, 2014:
% consider LR noise variance in non-iid noise case
% -------------------------------------------------------------------------
%   
%% Load Data

% clean?
if cl_ind==1
     cl_str='cl';            % cleaning string for output file
else
     cl_str='ncl';
end

% revisions?
if rev_ind==1
     rev_str='_rev';            
else
     rev_str='';     
end


% no of stocks
m=length(ticker_a);

% check for existence of files
%-----------------------------------------------------------------------------------
name_arr=cell(m,1);
index_v=NaN*zeros(m,1);

for i=1:m
    name_arr{i}=strcat(inpath,ticker_a{i},filesep,ticker_a{i},'_',datestr(tdate,'yyyymmdd'),'_',typed,'_',cl_str,'.mat');
    index_v(i)=isequal(exist(name_arr{i},'file'),2);
end

if isequal(sum(index_v),m)
%---------------------------------------------------------

%empty cell array
PRICETIME=cell(m,1);

% loop across assets
for i=1:m
    load(name_arr{i});
    if isequal(typed,'trades')
        midprice=trades;
    end
    if rev_ind==1
        midprice=keep_rev(midprice,2);
    end
    PRICETIME{i}=midprice;
end
% -------------------------------------------------------------------------
%   
%% Compute Spot Cov- and Corr estimates

% estimate noise variance
%-------------------------------------------------
%-------------------------------------------------
noisev=NaN*zeros(m,1);

% check if non-iid noise
if ~isempty(sdnoise_in)
    
    alpha=sdnoise_in.alpha;
    Q_max=sdnoise_in.Q_max;
    skips=sdnoise_in.skips;
    inpathn=sdnoise_in.inpathn;
    % alpha_label
    alpha_l=num2str(alpha);
    alpha_l(alpha_l=='.')='';
    
    % check for existence of files
    name_arrn=cell(m,1);
    index_vn=NaN*zeros(m,1);
    for i=1:m
        if ~isempty(skips)
            name_arrn{i}=strcat(inpathn,ticker_a{i},filesep,'noise_est_',ticker_a{i},'_',sourced,'_',datestr(tdate,'yyyymmdd'),typed,cl_str,rev_str,'_alpha',alpha_l,'_Q',int2str(Q_max),'_ss',int2str(skips),'.mat');
        else
            name_arrn{i}=strcat(inpathn,ticker_a{i},filesep,'noise_est_',ticker_a{i},'_',sourced,'_',datestr(tdate,'yyyymmdd'),typed,cl_str,rev_str,'_alpha',alpha_l,'_Q',int2str(Q_max),'.mat');
        end
        index_vn(i)=isequal(exist(name_arrn{i},'file'),2);
    end
    
    if isequal(sum(index_vn),m)
%---------------------------------------------------------  
    
    % loop across assets
    for i=1:m
         load(name_arrn{i});
        if eta_est<=0
            disp(['Noise estimate for: ' ticker_a{i} ' nonpositive. Use iid-based approach.'])
            data_temp=PRICETIME{i};
            eta_est=noise_cov_est(data_temp(:,2),'cov');
            if eta_est<=0
                eta_est=noise_cov_est(data_temp(:,2),'var');
            end
        end
        noisev(i)=eta_est;   
    end
    
    % Estimate Spot cov mat
    %------------------------------
    if compcov>0
    [SigmaSpot_out,block_st,~,J_optv,J_p,K,K_p,CovSigmaSpot_out]=lle_spot_cov_md(input_v,PRICETIME,...
        noisev,startt,endt,1,1,[],[],J_p,[],[],theta_h,theta_J,theta_K,[],K_p_thr,twost,twostc,lKb,ccorr,ccorrlast);
    else
        [SigmaSpot_out,block_st,~,J_optv,J_p,K,K_p]=lle_spot_cov_md(input_v,PRICETIME,...
            noisev,startt,endt,1,1,[],[],J_p,[],[],theta_h,theta_J,theta_K,[],K_p_thr,twost,twostc,lKb,ccorr,ccorrlast);
    end
  
    % Save outputs
    %----------------------------------------------
    % label lKb
    if lKb>0
        lKbl='_lKb';
    else
        lKbl='';
    end
  % label ccorr
    if ccorr>0
        ccorrl='_ccorr';
    else
        ccorrl='';
    end
        % label ccorr
    if ccorrlast>0
        ccorrlastl='_ccorrl';
    else
        ccorrlastl='';
    end
    %label twostep
    if twost>0
        twostl='_2st';
    else
        twostl='';
    end
    % label: estimator covariance
    if twostc>0
        twostcl='_2stc';
    else
        twostcl='';
    end
    %clean labels
    theta_hl=num2str(theta_h);
    theta_hl(theta_hl=='.')='';
    theta_Jl=num2str(theta_J);
    theta_Jl(theta_Jl=='.')='';
    theta_Kl=num2str(theta_K);
    theta_Kl(theta_Kl=='.')='';
    %basename
    if m>2
        basename=strcat(outpath,sourced,'_',datestr(tdate,'yyyymmdd'),typed,'_d',int2str(m),'_',cl_str,rev_str,...
                 '_th',theta_hl,'tJ',theta_Jl,'tK',theta_Kl,'J_p',int2str(J_p),'K_pt',int2str(K_p_thr),lKbl,ccorrl,ccorrlastl,twostl,twostcl,'_alpha',alpha_l,'_Q',int2str(Q_max));
             
    elseif isequal(m,2)
        basename=strcat(outpath,sourced,'_',ticker_a{1},'_',ticker_a{2},'_',datestr(tdate,'yyyymmdd'),typed,'_',cl_str,rev_str,...
                 '_th',theta_hl,'tJ',theta_Jl,'tK',theta_Kl,'J_p',int2str(J_p),'K_pt',int2str(K_p_thr),lKbl,ccorrl,ccorrlastl,twostl,twostcl,'_alpha',alpha_l,'_Q',int2str(Q_max));
             
    else
        basename=strcat(outpath,sourced,'_',ticker_a{1},'_',datestr(tdate,'yyyymmdd'),typed,'_',cl_str,rev_str,...
                 '_th',theta_hl,'tJ',theta_Jl,'tK',theta_Kl,'J_p',int2str(J_p),'K_pt',int2str(K_p_thr),lKbl,ccorrl,ccorrlastl,twostl,twostcl,'_alpha',alpha_l,'_Q',int2str(Q_max));
    end        
    if ~isempty(skips)
        basename=strcat(basename,'_ss',int2str(skips));
    end
    %save spot covariance matrices
    scov_out_nout=strcat(basename,'_scov');
    save(scov_out_nout,'SigmaSpot_out')
    if compcov>0
        %save covariance matrix of spot covariance 
        scov_out_cov_nout=strcat(basename,'_scov_cov');
        save(scov_out_cov_nout,'CovSigmaSpot_out','-v7.3')
    end
    %save LMM config
    LMM_conf.gen=[length(block_st) max(J_optv) J_p K K_p];
    LMM_conf.Jex=J_optv;
    LMM_conf_nout=strcat(basename,'_LMM_conf');
    save(LMM_conf_nout,'LMM_conf')
    
   
    %---------------------------------------------------------   
    else
        
        disp(['Noise estimates missing for the following tickers: ' ticker_a{logical(1-index_vn)} '. Skip day.'])
        
   %---------------------------------------------------------   
    end
    
else
    
    % iid noise: estimate on the fly
    %------------------------------------------------
    % loop across assets
    for i=1:m
        data_temp=PRICETIME{i};
        eta2=noise_cov_est(data_temp(:,2),'cov');
        if eta2<=0
           eta2=noise_cov_est(data_temp(:,2),'var');
        end
        noisev(i)=eta2;   
    end

    % Estimate Spot cov mat
    %------------------------------
    if compcov>0
    [SigmaSpot_out,block_st,~,J_optv,J_p,K,K_p,CovSigmaSpot_out]=lle_spot_cov_md(input_v,PRICETIME,...
        noisev,startt,endt,1,1,[],[],J_p,[],[],theta_h,theta_J,theta_K,[],K_p_thr,twost,twostc,lKb,ccorr,ccorrlast);
    else
        [SigmaSpot_out,block_st,~,J_optv,J_p,K,K_p]=lle_spot_cov_md(input_v,PRICETIME,...
            noisev,startt,endt,1,1,[],[],J_p,[],[],theta_h,theta_J,theta_K,[],K_p_thr,twost,twostc,lKb,ccorr,ccorrlast);
    end
  
    % Save outputs
    %----------------------------------------------
    % label lKb
    if lKb>0
        lKbl='_lKb';
    else
        lKbl='';
    end
  % label ccorr
    if ccorr>0
        ccorrl='_ccorr';
    else
        ccorrl='';
    end
    % label ccorr
    if ccorrlast>0
        ccorrlastl='_ccorrl';
    else
        ccorrlastl='';
    end
    %label twostep
    if twost>0
        twostl='_2st';
    else
        twostl='';
    end
    % label: estimator covariance
    if twostc>0
        twostcl='_2stc';
    else
        twostcl='';
    end
    %clean labels
    theta_hl=num2str(theta_h);
    theta_hl(theta_hl=='.')='';
    theta_Jl=num2str(theta_J);
    theta_Jl(theta_Jl=='.')='';
    theta_Kl=num2str(theta_K);
    theta_Kl(theta_Kl=='.')='';
    %basename
    if m>2
        basename=strcat(outpath,sourced,'_',datestr(tdate,'yyyymmdd'),typed,'_d',int2str(m),'_',cl_str,rev_str,...
                 '_th',theta_hl,'tJ',theta_Jl,'tK',theta_Kl,'J_p',int2str(J_p),'K_pt',int2str(K_p_thr),lKbl,ccorrl,ccorrlastl,twostl,twostcl);
             
    elseif isequal(m,2)
        basename=strcat(outpath,sourced,'_',ticker_a{1},'_',ticker_a{2},'_',datestr(tdate,'yyyymmdd'),typed,'_',cl_str,rev_str,...
                 '_th',theta_hl,'tJ',theta_Jl,'tK',theta_Kl,'J_p',int2str(J_p),'K_pt',int2str(K_p_thr),lKbl,ccorrl,ccorrlastl,twostl,twostcl);
             
    else
        basename=strcat(outpath,sourced,'_',ticker_a{1},'_',datestr(tdate,'yyyymmdd'),typed,'_',cl_str,rev_str,...
                 '_th',theta_hl,'tJ',theta_Jl,'tK',theta_Kl,'J_p',int2str(J_p),'K_pt',int2str(K_p_thr),lKbl,ccorrl,ccorrlastl,twostl,twostcl);
    end        
    %save spot covariance matrices
    scov_out_nout=strcat(basename,'_scov');
    save(scov_out_nout,'SigmaSpot_out')
    if compcov>0
        %save covariance matrix of spot covariance 
        scov_out_cov_nout=strcat(basename,'_scov_cov');
        save(scov_out_cov_nout,'CovSigmaSpot_out','-v7.3')
    end
    %save LMM config
    LMM_conf.gen=[length(block_st) max(J_optv) J_p K K_p];
    LMM_conf.Jex=J_optv;
    LMM_conf_nout=strcat(basename,'_LMM_conf');
    save(LMM_conf_nout,'LMM_conf')

end

%file do not exist: do nothing
%----------------------------------------
else
    disp(['Data missing for the following tickers: ' ticker_a{logical(1-index_v)} '. Skip day.'])
end



    
  
    