function comp_acf_est_intraday(ticker1,ticker2,asseti1,asseti2,d,momt,estimt,lags,date_v,startd,endd,typed,cl_ind,rev_ind,input_v,ngridskipv,...
    J_p,theta_h,theta_J,theta_K,K_p_thr,twost,twostc,lKb,ccorr,inpath,sourced,outpath,Q_strc)
%% comp_acf_est
%--------------------------------------------------------------
% compute acf of spot estimates in intraday time
%--------------------------------------------------------------
% INPUT:
% ticker1 -> ticker symbol
% asseti1 -> index of first asset (relevant only if estimt=full and momt=cov,corr,beta) 
% d         -> total number of assets (matters only for estimt="full")
% momt   -> moment to estimate: 'vol', 'cov', 'beta' or 'corr'
% estimt  -> type of estimator: 'full' or 'pair' ('cov',corr','beta'), 'full', 'pair' or 'uni' ('vol') 
% lags      -> maximum lag order for ACF
% date_v   -> vector of trading days in matlab format
% startd    -> first trading day of interest (Matlab format)
% cl_ind    -> if ==1, data cleaned according to proc by BNHLS(2009) is used
% rev_ind    -> if ==1, only obs with price revisions are used
% input_v   -> vector of estimation grid points 
% ngridskipv -> (m_skx1) vector containing selection frequency for grip
%                       points (ngridskipv(i)=1: every element selected, ngridskipv(i)=2: every 2nd )
% J_p-> spectral cutoff for pilot estimates of spot covariances
% theta_h   -> proportionality parameter for block length (relevant if B
%                           not specified)
% theta_J     -> proportionaliy parameter for cutoff (relevant if J
%                           or J_p not specified)
% theta_K    -> proportionality parameter for smoothing bandwidth (relevant if K
%                           or K_p not specified)
% K_p_thr    -> lower threshold for pilot smoothing bandwidth (relevant if
%                           K_p not specified): default=1
% twst     -> if >0: twostep estimation is used (estimates from first step
%                   used as pilots for second step
% twstc      -> if >0: Covariance matrices of smoothed spot (co-)volatilities
%   under the local parametric experiment   are computed using 2ndstep spot
%   covariance estimates (mattters only if 8 output arguments)
% lkB         -> if >0, shorter smoothing windows at beginning and end of
%                   day are used for spot cov estim (NOT PILOT!)
% ccorr -> if >0, correlation and variance estimates are corrected
% inpath     -> path for loading data
% sourced   -> data source
% outpath  -> path for saving
% Q_strc    -> structure containing inputs for non iid noise case
%--------------------------------------------------------------
%
% Peter Malec,  March 22 2014 
%
% -------------------------------------------------------------------------
%   
%% Preps

% input check
for i=1:length(ngridskipv)
    if abs(floor(ngridskipv(i))-ngridskipv(i))>0
        error('Number of grid points to skip must be integer!')
    end
end

% Create vector of trading days in matlab date format
tdate_v=date_v(sort(find((date_v>=startd)&(date_v<=endd))));

% label lKb
if lKb>0
     lKbl='_lKb';
else
     lKbl='';
end

% label ccorr
if ccorr>0
     ccorrl='_ccorr';
else
     ccorrl='';
end

%label twostep
if twost>0
     twostl='_2st';
else
     twostl='';
end

% label: estimator covariance
if twostc>0
      twostcl='_2stc';
else
      twostcl='';
end

% revisions?
if rev_ind==1
     rev_str='_rev';            
else
     rev_str='';     
end

% clean?
if cl_ind==1
     cl_str='cl';            % cleaning string for output file
else
     cl_str='ncl';
end

%clean labels
theta_hl=num2str(theta_h);
theta_hl(theta_hl=='.')='';
theta_Jl=num2str(theta_J);
theta_Jl(theta_Jl=='.')='';
theta_Kl=num2str(theta_K);
theta_Kl(theta_Kl=='.')='';

% inputs for non-iid noise case
if ~isempty(Q_strc)
    Q_max=Q_strc.Q_max;
    alpha=Q_strc.alpha;
    skips=Q_strc.skips;
    alpha_l=num2str(alpha);
    alpha_l(alpha_l=='.')='';
else
    Q_max=1;
end
% -------------------------------------------------------------------------
%   
%% Collect Data

% number of obs per day
Nd=length(input_v);

if isequal(estimt,'pair')
    
     % empty matrices for spot estimates estimates
     if isequal(momt,'cov')||isequal(momt,'corr')||isequal(momt,'beta')
           scov_mat=NaN*zeros(Nd*length(tdate_v),1);
     elseif isequal(momt,'vol')||isequal(momt,'var')
         scov_mat1=NaN*zeros(Nd*length(tdate_v),1);
         scov_mat2=NaN*zeros(Nd*length(tdate_v),1);
     end
                                
 % loop over days
%-------------------------------------------------
for j=1:length(tdate_v)
    
    % select date and display
    tdate=tdate_v(j);

    %name 
    basename=strcat(inpath,sourced,'_',ticker1,'_',ticker2,'_',datestr(tdate,'yyyymmdd'),typed,'_',cl_str,rev_str,...
                 '_th',theta_hl,'tJ',theta_Jl,'tK',theta_Kl,'J_p',int2str(J_p),'K_pt',int2str(K_p_thr),lKbl,ccorrl,twostl,twostcl);
        
    if Q_max>1
        basename=strcat(basename,'_alpha',alpha_l,'_Q',int2str(Q_max));
        if ~isempty(skips)
            basename=strcat(basename,'_ss',int2str(skips));
        end
    end           
             
    % check for existence of file
    if isequal(exist(strcat(basename,'_scov.mat'),'file'),2);
    %------------------------

        %load    
        indat=iload(strcat(basename,'_scov.mat'),'SigmaSpot_out');    
        SigmaS=indat.SigmaSpot_out;
        
        % assign
        if isequal(momt,'cov')
            scov_mat((Nd*(j-1)+1):(j*Nd))=SigmaS(:,2);
        elseif isequal(momt,'corr')
            [scorr_out,~]=comp_corr_md([2; 1],SigmaS);
            scov_mat((Nd*(j-1)+1):(j*Nd))=scorr_out;
        elseif isequal(momt,'beta')
            [sbeta_out,~]=comp_beta(SigmaS);
            scov_mat((Nd*(j-1)+1):(j*Nd))=sbeta_out;
         elseif ((isequal(momt,'vol'))||(isequal(momt,'var')))
             scov_mat1((Nd*(j-1)+1):(j*Nd))=SigmaS(:,1);
             scov_mat2((Nd*(j-1)+1):(j*Nd))=SigmaS(:,3);
        end
        
        
    else
        
        disp(['Estimates for ' ticker1 ' and ' ticker2 ' for date ' datestr(tdate,'yyyy--mm--dd') ' are missing. Skip.'])                
        
    end

end

elseif isequal(estimt,'full')
    
    
     % empty matrices for spot estimates estimates
      scov_mat=NaN*zeros(Nd*length(tdate_v),1);
    
 % loop over days
%-------------------------------------------------
for j=1:length(tdate_v)
    
    % select date and display
    tdate=tdate_v(j);

    %name 
    basename=strcat(inpath,sourced,'_',datestr(tdate,'yyyymmdd'),typed,'_d',int2str(d),'_',cl_str,rev_str,...
                 '_th',theta_hl,'tJ',theta_Jl,'tK',theta_Kl,'J_p',int2str(J_p),'K_pt',int2str(K_p_thr),lKbl,ccorrl,twostl,twostcl);
        
    if Q_max>1
        basename=strcat(basename,'_alpha',alpha_l,'_Q',int2str(Q_max));
        if ~isempty(skips)
            basename=strcat(basename,'_ss',int2str(skips));
        end
    end           
             
    % check for existence of file
    if isequal(exist(strcat(basename,'_scov.mat'),'file'),2);
    %------------------------

        %load    
        indat=iload(strcat(basename,'_scov.mat'),'SigmaSpot_out');   
        SigmaS=indat.SigmaSpot_out;
        
        %obtain relevant estimates and assign
        if isequal(momt,'cov')
            [SigmaSpot_out_sel,~]=retrieve_cov_el([asseti1;asseti2],SigmaS);
            scov_mat((Nd*(j-1)+1):(j*Nd))=SigmaSpot_out_sel(:,2);
        elseif isequal(momt,'corr')
            [scorr_out,~]=comp_corr_md([asseti1;asseti2],SigmaS);
            scov_mat((Nd*(j-1)+1):(j*Nd))=scorr_out;
        elseif isequal(momt,'beta')
            [SigmaSpot_out_sel,~]=retrieve_cov_el([d;asseti1],SigmaS);
            [sbeta_out,~]=comp_beta(fliplr(SigmaSpot_out_sel));
            scov_mat((Nd*(j-1)+1):(j*Nd))=sbeta_out;
        elseif ((isequal(momt,'vol'))||(isequal(momt,'var')))
             [SigmaSpot_out_sel,~]=retrieve_cov_el_univ(asseti1,SigmaS);
             scov_mat((Nd*(j-1)+1):(j*Nd))=SigmaSpot_out_sel;
        end
        
        
    else
        
        disp(['Estimates for date ' datestr(tdate,'yyyy--mm--dd') ' are missing. Skip.'])                
        
    end

end

% -------------------------------------------------------------------------
%
elseif ((isequal(momt,'vol'))||(isequal(momt,'var')))&&(isequal(estimt,'uni'))
    
     % empty matrices for spot estimates estimates
      scov_mat=NaN*zeros(Nd*length(tdate_v),1);
    
                                
 % loop over days
%-------------------------------------------------
for j=1:length(tdate_v)
    
    % select date and display
    tdate=tdate_v(j);

    %name 
    basename=strcat(inpath,sourced,'_',ticker1,'_',datestr(tdate,'yyyymmdd'),typed,'_',cl_str,rev_str,...
                 '_th',theta_hl,'tJ',theta_Jl,'tK',theta_Kl,'J_p',int2str(J_p),'K_pt',int2str(K_p_thr),lKbl,ccorrl,twostl,twostcl);
        
    
    if Q_max>1
        basename=strcat(basename,'_alpha',alpha_l,'_Q',int2str(Q_max));
        if ~isempty(skips)
            basename=strcat(basename,'_ss',int2str(skips));
        end
    end                    
             
             
    % check for existence of file
    if isequal(exist(strcat(basename,'_scov.mat'),'file'),2);
    %------------------------

        %load    
        indat=iload(strcat(basename,'_scov.mat'),'SigmaSpot_out');    
        SigmaS=indat.SigmaSpot_out;
        
        % assign
            scov_mat((Nd*(j-1)+1):(j*Nd))=SigmaS;
        
    else
        
        disp(['Estimates for ' ticker1 ' for date ' datestr(tdate,'yyyy--mm--dd') ' are missing. Skip.'])                
        
    end

end
    
else
    
    error('estimt needs to be either "full" or "pair". Only for volatilities, estimt can be "uni"! ')
    
end
% -------------------------------------------------------------------------
%   
%% Compute ACF for each point of estimation

% volatilities?
if (isequal(momt,'vol'))
    if isequal(estimt,'pair')
        scov_mat1(scov_mat1<0)=0;
        scov_mat1=sqrt(scov_mat1);
        scov_mat2(scov_mat2<0)=0;
        scov_mat2=sqrt(scov_mat2);
    else
        scov_mat(scov_mat<0)=0;
        scov_mat=sqrt(scov_mat);
    end
end

% initialize
if isequal(momt,'cov')||isequal(momt,'beta')||isequal(momt,'corr')
     acf_mat=NaN*zeros(length(ngridskipv),lags);
     std_mat=NaN*zeros(length(ngridskipv),lags);
elseif ((isequal(momt,'vol'))||(isequal(momt,'var')))
    if isequal(estimt,'pair')
        acf_mat1=NaN*zeros(length(ngridskipv),lags);
        acf_mat2=NaN*zeros(length(ngridskipv),lags);
        std_mat1=NaN*zeros(length(ngridskipv),lags);
        std_mat2=NaN*zeros(length(ngridskipv),lags);
    else
        acf_mat=NaN*zeros(length(ngridskipv),lags);
        std_mat=NaN*zeros(length(ngridskipv),lags);
    end
end

% loop across estimation points and compute ACF
for i=1:length(ngridskipv)
    
    selind_v=ngridskipv(i):ngridskipv(i):(Nd*length(tdate_v));
    
    %ACF
    if isequal(momt,'cov')||isequal(momt,'beta')||isequal(momt,'corr')
        [ac, acstd] = sacf(scov_mat(selind_v), lags, 0, 0);
        acf_mat(i,:)=ac';
        std_mat(i,:)=acstd';
    elseif ((isequal(momt,'vol'))||(isequal(momt,'var')))
        if isequal(estimt,'pair')
        [ac1, acstd1] = sacf(scov_mat1(selind_v), lags, 0, 0);
        acf_mat1(i,:)=ac1';
        std_mat1(i,:)=acstd1';
        [ac2, acstd2] = sacf(scov_mat2(selind_v), lags, 0, 0);
        acf_mat2(i,:)=ac2';
        std_mat2(i,:)=acstd2';
        else
        [ac, acstd] = sacf(scov_mat(selind_v), lags, 0, 0);
        acf_mat(i,:)=ac';
        std_mat(i,:)=acstd';
        end
    end
    
end
% -------------------------------------------------------------------------
%   
%% Save Results

%collect results
if isequal(momt,'cov')||isequal(momt,'beta')||isequal(momt,'corr')
       acf_res.acf_mat=acf_mat;
       acf_res.std_mat=std_mat;
    elseif ((isequal(momt,'vol'))||(isequal(momt,'var')))
        if isequal(estimt,'pair')
            acf_res.acf_mat1=acf_mat1;
            acf_res.std_mat1=std_mat1;
            acf_res.acf_mat2=acf_mat2;
            acf_res.std_mat2=std_mat2;
        else
            acf_res.acf_mat=acf_mat;
            acf_res.std_mat=std_mat;
        end
end


if isequal(momt,'cov')||isequal(momt,'corr')
        nout=strcat(outpath,'ACFintrad_',momt,'_',estimt,'_L',int2str(lags),'_ngr',int2str(length(ngridskipv)),sourced,'_',ticker1,'_',ticker2,'_',datestr(startd,'yyyymmdd'),'_',datestr(endd,'yyyymmdd'),typed,'_',cl_str,rev_str,...
                 '_th',theta_hl,'tJ',theta_Jl,'tK',theta_Kl,'J_p',int2str(J_p),'K_pt',int2str(K_p_thr),lKbl,ccorrl,twostl);
elseif isequal(momt,'beta')        
    nout=strcat(outpath,'ACFintrad_',momt,'_',estimt,'_L',int2str(lags),'_ngr',int2str(length(ngridskipv)),sourced,'_',ticker1,'_',datestr(startd,'yyyymmdd'),'_',datestr(endd,'yyyymmdd'),typed,'_',cl_str,rev_str,...
                 '_th',theta_hl,'tJ',theta_Jl,'tK',theta_Kl,'J_p',int2str(J_p),'K_pt',int2str(K_p_thr),lKbl,ccorrl,twostl);
elseif ((isequal(momt,'vol'))||(isequal(momt,'var')))
   if ~isequal(estimt,'pair')
        nout=strcat(outpath,'ACFintrad_',momt,'_',estimt,'_L',int2str(lags),'_ngr',int2str(length(ngridskipv)),sourced,'_',ticker1,'_',datestr(startd,'yyyymmdd'),'_',datestr(endd,'yyyymmdd'),typed,'_',cl_str,rev_str,...
                 '_th',theta_hl,'tJ',theta_Jl,'tK',theta_Kl,'J_p',int2str(J_p),'K_pt',int2str(K_p_thr),lKbl,ccorrl,twostl);
    else
        nout=strcat(outpath,'ACFintrad_',momt,'_',estimt,'_L',int2str(lags),'_ngr',int2str(length(ngridskipv)),sourced,'_',ticker1,'_',ticker2,'_',datestr(startd,'yyyymmdd'),'_',datestr(endd,'yyyymmdd'),typed,'_',cl_str,rev_str,...
                 '_th',theta_hl,'tJ',theta_Jl,'tK',theta_Kl,'J_p',int2str(J_p),'K_pt',int2str(K_p_thr),lKbl,ccorrl,twostl);
   end
end

if Q_max>1
        nout=strcat(nout,'_alpha',alpha_l,'_Q',int2str(Q_max));
        if ~isempty(skips)
            nout=strcat(nout,'_ss',int2str(skips));
        end
end

%save
save(nout,'acf_res');



             
             
             
             
             
             
    

