function comp_kde_est(ticker1,ticker2,asseti1,asseti2,momt,estimt,trans,date_v,startd,endd,typed,cl_ind,rev_ind,input_v,...
    J_p,theta_h,theta_J,theta_K,K_p_thr,twost,twostc,lKb,ccorr,inpath,sourced,outpath)
%% comp_kde_est
%--------------------------------------------------------------
% compute kdes of spot estimates across days
%--------------------------------------------------------------
% INPUT:
% ticker1 -> ticker symbol
% asseti1 -> index of first asset (relevant only if estimt=full and momt=cov) 
% momt   -> moment to estimate: 'vol' or 'cov'
% estimt  -> type of estimator: 'full' or 'pair' ('cov'), 'full', 'pair' or 'uni' ('vol') 
% trans      -> type of transformation: 'log','sqrt' or 'logsqrt', else
%                   empty
% date_v   -> vector of trading days in matlab format
% startd    -> first trading day of interest (Matlab format)
% cl_ind    -> if ==1, data cleaned according to proc by BNHLS(2009) is used
% rev_ind    -> if ==1, only obs with price revisions are used
% input_v -> points of estimation
% J_p-> spectral cutoff for pilot estimates of spot covariances
% theta_h   -> proportionality parameter for block length (relevant if B
%                           not specified)
% theta_J     -> proportionaliy parameter for cutoff (relevant if J
%                           or J_p not specified)
% theta_K    -> proportionality parameter for smoothing bandwidth (relevant if K
%                           or K_p not specified)
% K_p_thr    -> lower threshold for pilot smoothing bandwidth (relevant if
%                           K_p not specified): default=1
% twst     -> if >0: twostep estimation is used (estimates from first step
%                   used as pilots for second step
% twstc      -> if >0: Covariance matrices of smoothed spot (co-)volatilities
%   under the local parametric experiment   are computed using 2ndstep spot
%   covariance estimates (mattters only if 8 output arguments)
% lkB         -> if >0, shorter smoothing windows at beginning and end of
%                   day are used for spot cov estim (NOT PILOT!)
% ccorr -> if >0, correlation and variance estimates are corrected
% inpath     -> path for loading data
% sourced   -> data source
% outpath  -> path for saving
%--------------------------------------------------------------
%
% Peter Malec,  March 22 2014 
%
% -------------------------------------------------------------------------
%   
%% Preps

% Create vector of trading days in matlab date format
tdate_v=date_v(sort(find((date_v>=startd)&(date_v<=endd))));

% input check
if (~isempty(trans))&&(~isequal(momt,'vol'))
    error('Only spot VARIANCE estimates can be transformed!')
end
if (~isempty(trans))&&~ismember(trans,{'log','sqrt','logsqrt'})
    error('Possible transformations are: "log","sqrt","logsqrt"')
end

% label lKb
if lKb>0
     lKbl='_lKb';
else
     lKbl='';
end

% label ccorr
if ccorr>0
     ccorrl='_ccorr';
else
     ccorrl='';
end

%label twostep
if twost>0
     twostl='_2st';
else
     twostl='';
end

% label: estimator covariance
if twostc>0
      twostcl='_2stc';
else
      twostcl='';
end

% revisions?
if rev_ind==1
     rev_str='_rev';            
else
     rev_str='';     
end

% clean?
if cl_ind==1
     cl_str='cl';            % cleaning string for output file
else
     cl_str='ncl';
end

%clean labels
theta_hl=num2str(theta_h);
theta_hl(theta_hl=='.')='';
theta_Jl=num2str(theta_J);
theta_Jl(theta_Jl=='.')='';
theta_Kl=num2str(theta_K);
theta_Kl(theta_Kl=='.')='';
% -------------------------------------------------------------------------
%   
%% Collect Data

if isequal(estimt,'pair')
    
     % empty matrices for spot estimates estimates
     if isequal(momt,'cov')
           scov_mat=NaN*zeros(length(input_v),length(tdate_v));
     elseif isequal(momt,'vol')
         scov_mat1=NaN*zeros(length(input_v),length(tdate_v));
         scov_mat2=NaN*zeros(length(input_v),length(tdate_v));
     end
                                
 % loop over days
%-------------------------------------------------
for j=1:length(tdate_v)
    
    % select date and display
    tdate=tdate_v(j);

    %name 
    basename=strcat(inpath,sourced,'_',ticker1,'_',ticker2,'_',datestr(tdate,'yyyymmdd'),typed,'_',cl_str,rev_str,...
                 '_th',theta_hl,'tJ',theta_Jl,'tK',theta_Kl,'J_p',int2str(J_p),'K_pt',int2str(K_p_thr),lKbl,ccorrl,twostl,twostcl);
        
    % check for existence of file
    if isequal(exist(strcat(basename,'_scov.mat'),'file'),2);
    %------------------------

        %load    
        indat=iload(strcat(basename,'_scov.mat'),'SigmaSpot_out');    
        SigmaS=indat.SigmaSpot_out;
        
        % assign
        if isequal(momt,'cov')
            scov_mat(:,j)=SigmaS(:,2);
         elseif isequal(momt,'vol')
             scov_mat1(:,j)=SigmaS(:,1);
             scov_mat2(:,j)=SigmaS(:,3);
        end
        
        
    else
        
        disp(['Estimates for ' ticker1 ' and ' ticker2 ' for date ' datestr(tdate,'yyyy--mm--dd') ' are missing. Skip.'])                
        
    end

end

elseif isequal(estimt,'full')
    
    
     % empty matrices for spot estimates estimates
      scov_mat=NaN*zeros(length(input_v),length(tdate_v));
    
 % loop over days
%-------------------------------------------------
for j=1:length(tdate_v)
    
    % select date and display
    tdate=tdate_v(j);

    %name 
    basename=strcat(inpath,sourced,'_',datestr(tdate,'yyyymmdd'),typed,'_d',int2str(length(ticker_a)),'_',cl_str,rev_str,...
                 '_th',theta_hl,'tJ',theta_Jl,'tK',theta_Kl,'J_p',int2str(J_p),'K_pt',int2str(K_p_thr),lKbl,ccorrl,twostl,twostcl);
        
    % check for existence of file
    if isequal(exist(strcat(basename,'_scov.mat'),'file'),2);
    %------------------------

        %load    
        indat=iload(strcat(basename,'_scov.mat'),'SigmaSpot_out');   
        SigmaS=indat.SigmaSpot_out;
        
        %obtain relevant estimates and assign
        if isequal(momt,'cov')
            [SigmaSpot_out_sel,~]=retrieve_cov_el([asseti1;asseti2],SigmaS);
            scov_mat(:,j)=SigmaSpot_out_sel(:,2);
         elseif isequal(momt,'vol')
             [SigmaSpot_out_sel,~]=retrieve_cov_el_univ(asseti1,SigmaS);
             scov_mat(:,j)=SigmaSpot_out_sel;
        end
        
        
    else
        
        disp(['Estimates for date ' datestr(tdate,'yyyy--mm--dd') ' are missing. Skip.'])                
        
    end

end

% -------------------------------------------------------------------------
%
elseif (isequal(momt,'vol'))&&(isequal(estimt,'uni'))
    
     % empty matrices for spot estimates estimates
      scov_mat=NaN*zeros(length(input_v),length(tdate_v));
    
                                
 % loop over days
%-------------------------------------------------
for j=1:length(tdate_v)
    
    % select date and display
    tdate=tdate_v(j);

    %name 
    basename=strcat(inpath,sourced,'_',ticker1,'_',datestr(tdate,'yyyymmdd'),typed,'_',cl_str,rev_str,...
                 '_th',theta_hl,'tJ',theta_Jl,'tK',theta_Kl,'J_p',int2str(J_p),'K_pt',int2str(K_p_thr),lKbl,ccorrl,twostl,twostcl);
        
    % check for existence of file
    if isequal(exist(strcat(basename,'_scov.mat'),'file'),2);
    %------------------------

        %load    
        indat=iload(strcat(basename,'_scov.mat'),'SigmaSpot_out');    
        SigmaS=indat.SigmaSpot_out;
        
        % assign
            scov_mat(:,j)=SigmaS;
        
    else
        
        disp(['Estimates for ' ticker1 ' for date ' datestr(tdate,'yyyy--mm--dd') ' are missing. Skip.'])                
        
    end

end
    
else
    
    error('estimt needs to be either "full" or "pair". Only for volatilities, estimt can be "uni"! ')
    
end
% -------------------------------------------------------------------------
%   
%% Compute KDEs for each point of estimation 

% estimation points for KDE 
if isequal(momt,'cov')
    min_inp_kde=-1;
    max_inp_kde=1;
    n=2000;
    inp_kde=[];
elseif isequal(momt,'vol')
    if isempty(trans)||isequal(trans,'sqrt')
        inp_kde=([eps (0.001:0.001:2)])';
        n=length(inp_kde);
    elseif isequal(trans,'log')||isequal(trans,'logsqrt')
        min_inp_kde=-1;
        max_inp_kde=1;
        n=2000;
        inp_kde=[];
    end
end
if isempty(inp_kde)
    % round up n to the next power of 2
    n=2^ceil(log2(n)); 
end

%transformations
if isequal(momt,'vol')
    if isequal(trans,'sqrt')
        scov_mat=sqrt(scov_mat);
    elseif isequal(trans,'log')
        scov_mat=log(scov_mat);
    elseif isequal(trans,'logsqrt')
        scov_mat=log(sqrt(scov_mat));
    end
end

% initialize
if isequal(momt,'cov')
     KDE_mat=NaN*zeros(length(input_v),n);
     bw_vec=NaN*zeros(length(input_v),1);
elseif isequal(momt,'vol')
    if isequal(estimt,'pair')
        KDE_mat1=NaN*zeros(length(input_v),n);
        bw_vec1=NaN*zeros(length(input_v),1);
        KDE_mat2=NaN*zeros(length(input_v),n);   
        bw_vec2=NaN*zeros(length(input_v),1);
    else
        KDE_mat=NaN*zeros(length(input_v),n);
        bw_vec=NaN*zeros(length(input_v),1);
    end
end

% loop across estimation points and compute KDE
for i=1:length(input_v)
    
    %KDE
    if isequal(momt,'cov')
        data_in=scov_mat(i,:)';
        [bw,dens,~,~]=kde(data_in,n,min_inp_kde,max_inp_kde);
        KDE_mat(i,:)=dens';
        bw_vec(i)=bw;
    elseif isequal(momt,'vol')
        if isequal(estimt,'pair')
            if isempty(trans)||isequal(trans,'sqrt')
                 data_in=scov_mat1(i,:)';
                 data_in=data_in(imag(data_in)==0);
                 [dens_est,bw] = dens_est_gamma_kernel(data_in/std(data_in),inp_kde/std(data_in),[],1);
                 KDE_mat1(i,:)=dens_est'/std(data_in);
                 bw_vec1(i)=bw;
                 data_in=scov_mat2(i,:)';
                 data_in=data_in(imag(data_in)==0);
                 [dens_est,bw] = dens_est_gamma_kernel(data_in/std(data_in),inp_kde/std(data_in),[],1);
                 KDE_mat2(i,:)=dens_est'/std(data_in);
                 bw_vec2(i)=bw;
            elseif isequal(trans,'log')||isequal(trans,'logsqrt')
                data_in=scov_mat1(i,:)';
                data_in=data_in(imag(data_in)==0);
                [bw,dens,~,~]=kde(data_in,n,min_inp_kde,max_inp_kde);
                KDE_mat1(i,:)=dens';
                bw_vec1(i)=bw;
                data_in=scov_mat2(i,:)';
                data_in=data_in(imag(data_in)==0);
                [bw,dens,~,~]=kde(data_in,n,min_inp_kde,max_inp_kde);
                KDE_mat2(i,:)=dens';
                bw_vec2(i)=bw;
            end
        else
            if isempty(trans)||isequal(trans,'sqrt')
                 data_in=scov_mat(i,:)';
                 data_in=data_in(imag(data_in)==0);
                 [dens_est,bw] = dens_est_gamma_kernel(data_in/std(data_in),inp_kde/std(data_in),[],1);
                 KDE_mat(i,:)=dens_est'/std(data_in);
                 bw_vec(i)=bw;
            elseif isequal(trans,'log')||isequal(trans,'logsqrt')
                data_in=scov_mat(i,:)';
                data_in=data_in(imag(data_in)==0);
                [bw,dens,~,~]=kde(data_in,n,min_inp_kde,max_inp_kde);mean(dens)
                KDE_mat(i,:)=dens';
                bw_vec(i)=bw;
            end
        end
    end
    
end
% -------------------------------------------------------------------------
%   
%% Save Results

%collect results
if isequal(momt,'cov')
       kde_res.kde_mat=KDE_mat;
       kde_res.bw_vec=bw_vec;
    elseif isequal(momt,'vol')
        if isequal(estimt,'pair')
            kde_res.kde_mat1=KDE_mat1;
            kde_res.bw_vec1=bw_vec1;
            kde_res.kde_mat2=KDE_mat2;
            kde_res.bw_vec2=bw_vec2;
        else
            kde_res.kde_mat=KDE_mat;
            kde_res.bw_vec=bw_vec;
        end
end


%trans label
if isempty(trans)
    transl='';
else
    transl=strcat('_',trans);
end

%save
if isequal(momt,'cov')
        nout=strcat(outpath,'KDE_',momt,'_',estimt,transl,'_',sourced,'_',ticker1,'_',ticker2,'_',datestr(startd,'yyyymmdd'),'_',datestr(endd,'yyyymmdd'),typed,'_',cl_str,rev_str,...
                 '_th',theta_hl,'tJ',theta_Jl,'tK',theta_Kl,'J_p',int2str(J_p),'K_pt',int2str(K_p_thr),lKbl,ccorrl,twostl);
elseif isequal(momt,'vol')
   if ~isequal(estimt,'pair')
        nout=strcat(outpath,'KDE_',momt,'_',estimt,transl,'_',sourced,'_',ticker1,'_',datestr(startd,'yyyymmdd'),'_',datestr(endd,'yyyymmdd'),typed,'_',cl_str,rev_str,...
                 '_th',theta_hl,'tJ',theta_Jl,'tK',theta_Kl,'J_p',int2str(J_p),'K_pt',int2str(K_p_thr),lKbl,ccorrl,twostl);
    else
        nout=strcat(outpath,'KDE_',momt,'_',estimt,transl,'_',sourced,'_',ticker1,'_',ticker2,'_',datestr(startd,'yyyymmdd'),'_',datestr(endd,'yyyymmdd'),typed,'_',cl_str,rev_str,...
                 '_th',theta_hl,'tJ',theta_Jl,'tK',theta_Kl,'J_p',int2str(J_p),'K_pt',int2str(K_p_thr),lKbl,ccorrl,twostl);
   end
end

%save
save(nout,'kde_res');



             
             
             
             
             
             
    

